/***********
% -------------------------------------------------------
% C++ Classes for Clifford Product 
%
% --------------------------------------------------------
% Original Author: www.nklein.com
% nklein Software - Patrick Stein  
% --------------------------------------------------------
% Extended and Modified by: Mijail Guillemard - 2010
% 
% This program is distributed under the terms of the GNU General Public License.
% --------------------------------------------------------
**********/

namespace nklein {
  namespace nklein_priv {
	 
// =================================================================
// =================================================================
class GeomGradeTable {
 private:

  int N; 
  int *gradeTable;

 protected:

 GeomGradeTable(const GeomGradeTable & b)
	:N(b.N)
	{
	  // *this = b; 
	  initializeGradeTable();
	}


  // --------------------------	
  GeomGradeTable(const unsigned int Nn)
    {
		N = Nn; 
		initializeGradeTable();
    }

  void initializeGradeTable()
	 {
		// mexPrintf("Initializing gradeTable \n");
		gradeTable = new int[ 1U << (N) ];
  
		for ( unsigned ii=0; ii < ( 1U << (N) ); ++ii ) {
		  unsigned int iiBits = 0;
		  
		  char* ptr = (char*)&ii;

		  for ( unsigned int kk=0; kk < sizeof(unsigned int); ++kk ) {
			 static const unsigned int lut[] = {
				0, 1, 1, 2, 1, 2, 2, 3,
				1, 2, 2, 3, 2, 3, 3, 4
			 };
			 iiBits += lut[ ( ptr[kk] >> 0 ) & 0x0F ];
			 iiBits += lut[ ( ptr[kk] >> 4 ) & 0x0F ];
		  }
		  gradeTable[ii] = iiBits;
		}
	 }


 public:  
  // --------------------------	
  inline unsigned int GetGrade( unsigned int index )
  {
	 if ( index >= ( 1U << (N) ) ) {
		throw std::out_of_range( "index" );
	 }
	 return gradeTable[index];
  }

 ~GeomGradeTable()
	{
	  delete [] gradeTable;
	}
 };
	    
// =================================================================
// =================================================================
class GeomMultTable : public GeomGradeTable {

 private:

    static GeomMultTable* pInstance;

 public:

	unsigned  int upq, up, uq; 

	static GeomMultTable* Instance(const unsigned int P, const unsigned int Q)
	  {
		 if(pInstance == 0){
			pInstance = new GeomMultTable(P,Q);
		 }
		 return pInstance;
	  }
 
	 //  static int multTable[ 1U << (P+Q) ][ 1U << (P+Q) ];
	int *multTable;

  protected:

 GeomMultTable(const GeomMultTable & b)
	:GeomGradeTable(b), upq(b.upq), up(b.up), uq(b.uq)
	{
	  // *this = b; 
	  initializeMultTable();
	}

 // --------------------------	
 GeomMultTable(const unsigned int P, const unsigned int Q)
	:GeomGradeTable(P+Q)
 {

	upq = 1U << (P+Q); 
	up = P; uq = Q;

	initializeMultTable();
 }

	~GeomMultTable()
	  {
		 if(pInstance != 0) 
			{
			  delete [] multTable;
			  delete pInstance;
			}
	  }

void initializeMultTable()
  {

	 multTable = new int[ upq * upq ];

	for ( unsigned ii=0; ii < upq; ++ii ) {
	  unsigned int iiInitialTopBits
		 // = nklein_priv::GeomGradeTable< P+Q >::GetGrade(ii);
		 // = nklein_priv::GeomGradeTable::GetGrade(ii);
		 = GetGrade(ii);
	  
	  // for ( unsigned jj=0; jj < ( 1U << (P+Q) ); ++jj ) {
	  for ( unsigned jj=0; jj < upq; ++jj ) {
		 unsigned int iiTopBits = iiInitialTopBits;
		 
		 
		 int sign = 1;
		 
		 for ( unsigned int kk=0; kk < up+uq; ++kk ) {
			unsigned int bit = ( 1U << kk );

			
			if ( ( ii & bit ) != 0 ) {
			  --iiTopBits;
			}
			
			if ( ( jj & bit ) != 0 ) {
			  sign *= ( iiTopBits & 1 ) ? -1 : 1;
			  
			  if ( ( ii & bit ) != 0 ) {
		
				 if ( kk >= up ) {
					sign *= -1;
				 }
			  }
			}
		 }
		 
		 // multTable[ii][jj] = sign;
		 multTable[ii*upq+jj] = sign;
	  }
	}

  }

 // --------------------------	
 public:
 inline bool IsPositive( unsigned int ii, unsigned int jj )
 {
	if ( ii >= upq ) {
	  throw std::out_of_range( "first index" );
	}
	if ( jj >= upq ) {
	  throw std::out_of_range( "second index" );
	}
	return ( multTable[ii*upq + jj] >= 0 );
 }

 // --------------------------	
  public:
 inline int getMultTable( unsigned int ii, unsigned int jj )
	{
	  return ( multTable[ii*upq + jj] );
	}

 // private: 

 // ~GeomMultTable()
 //	{
 //	  delete [] multTable;
 //	}
  };

  GeomMultTable* GeomMultTable::pInstance = 0;

};


// =================================================================
// =================================================================
template <class Type> class GeometricAlgebra// : public nklein_priv::GeomMultTable
  {
  protected:
	 
	 // Type coef[ 1U << (P+Q) ];
	 Type *coef;
	nklein_priv::GeomMultTable *geomMultTable;

  public:  

	unsigned  int upq, up, uq; 

	 // --------------------------	
	 GeometricAlgebra( bool init = true, const unsigned int P=0, const unsigned int Q=2)
		// :GeomMultTable(P,Q)
		{

		  // geomMultTable = new nklein_priv::GeomMultTable(P,Q);
		  geomMultTable = nklein_priv::GeomMultTable::Instance(P,Q);

		  upq = geomMultTable->upq;
		  up = geomMultTable->up;
		  uq = geomMultTable->uq;
		  
	 // coef  = new Type[ 1U << (P+Q) ];
	 coef  = new Type[ upq ];

	 if ( init ) {
		// for ( unsigned int ii=0; ii < ( 1U << (P+Q) ); ++ii ) {
		for ( unsigned int ii=0; ii < upq; ++ii ) {
		  coef[ii] = 0;
		}
	 }
  }

	 ~GeometricAlgebra()
		{
		  delete [] coef;
		  // delete geomMultTable; 
		}

	 inline unsigned int GetGrade( unsigned int index )
	 {
		return geomMultTable->GetGrade(index);
	 }

	 inline bool IsPositive( unsigned int ii, unsigned int jj )
	 {
		return geomMultTable->IsPositive(ii,jj);	
	 }
	 
  // --------------------------	
  inline
	 // GeometricAlgebra( const GeometricAlgebra<Type,P,Q>& b )
  GeometricAlgebra( const GeometricAlgebra<Type>& b )
	 // :GeomMultTable(b)
  {
	 *this = b;
  }

  // --------------------------	
  inline
  void SetCoefs(double *outCoefs)
  {
    // memcpy(coef, outCoefs,  (1U << (P+Q)) *sizeof(Type));
    memcpy(coef, outCoefs,  upq *sizeof(Type));
  }


  // --------------------------	
  inline
  void SetCoefs(double *outCoefs, int copySize)
  {
    // memcpy(coef, outCoefs,  (1U << (P+Q)) *sizeof(Type));
    memcpy(coef, outCoefs,  copySize *sizeof(Type));
  }

  // --------------------------	
  inline
  void SetCoefsVector(double *outCoefs)
  {

	 coef[0] = outCoefs[0];

	 for ( unsigned int ii=0; ii < up+uq-1; ++ii ) {
		unsigned int index = 1U << ii; 

		// mexPrintf("ii=%i, index=%i, uq=%i, val=%f \n",ii,index,uq,outCoefs[ii+1]);

		coef[index] = outCoefs[ii+1];
	 }
  }

  // --------------------------	
  inline
  void GetCoefs(double *outCoefs)
  {
    // memcpy(outCoefs, coef, (1U << (P+Q)) *sizeof(Type));
    memcpy(outCoefs, coef, upq *sizeof(Type));
  }

  // --------------------------	
  // inline GeometricAlgebra<Type,P,Q>&
  inline GeometricAlgebra<Type>&
  operator =( const GeometricAlgebra<Type>& b )
  {
	 // for ( unsigned int ii=0; ii < ( 1U << (P+Q) ); ++ii ) {
	 for ( unsigned int ii=0; ii < upq; ++ii ) {
		coef[ii] = b.coef[ii];
	 }
	 return *this;
  }

  // --------------------------	
  inline Type&
  operator [] ( const unsigned int index )
  {
	 // if ( index >= ( 1U << (P+Q) ) ) {
	 if ( index >= upq ) {
		throw std::out_of_range( "index" );
	 }
	 return coef[ index ];
  }
  // --------------------------	
  inline const Type&
  operator [] ( const unsigned int index ) const
  {
	 // if ( index >= ( 1U << (P+Q) ) ) {
	 if ( index >= upq ) {
		throw std::out_of_range( "index" );
	 }
	 return coef[ index ];
  }
  // --------------------------	
  inline
	 // GeometricAlgebra<Type,P,Q>
  GeometricAlgebra<Type>
  GradeInvolution( void ) const
  {
	 // GeometricAlgebra<Type,P,Q> a(false);
	 GeometricAlgebra<Type> a(false,up,uq);
	 Type zero = 0;
	 
	 // for ( unsigned int ii=0; ii < ( 1U << (P+Q) ); ++ii ) {
	 //	if ( ( nklein_priv::GeomGradeTable< P+Q >::GetGrade( ii ) & 1 )
	 for ( unsigned int ii=0; ii < upq; ++ii ) {
		// if ( ( nklein_priv::GeomGradeTable::GetGrade( ii ) & 1 )
		if ( ( GetGrade( ii ) & 1 )

			  != 0 ) {
		  a[ii] = zero - coef[ii];
		} else {
		  a[ii] = coef[ii];
		}
	 }
	 
	 return a;
  }
  // --------------------------	
  inline
	 // GeometricAlgebra<Type,P,Q>
  GeometricAlgebra<Type>
  Reversion( void ) 
  {
	 // GeometricAlgebra<Type,P,Q> a(false);
	 GeometricAlgebra<Type> a(true,up,uq);
	 Type zero = 0;
	 
	 // for ( unsigned int ii=0; ii < ( 1U << (P+Q) ); ++ii ) {
	 //	if ( ( nklein_priv::GeomGradeTable< P+Q >::GetGrade( ii ) & 2 )
	 for ( unsigned int ii=0; ii < upq; ++ii ) {
		// if ( ( nklein_priv::GeomGradeTable::GetGrade( ii ) & 2 )
		if ( ( GetGrade( ii ) & 2 )
			  != 0 ) {
		  a[ii] = zero - coef[ii];
		} else {
		  a[ii] = coef[ii];
		}
	 }
	 
	 return a;
  }

  // --------------------------	
  inline
	 // GeometricAlgebra<Type,P,Q>
  GeometricAlgebra<Type>
  Conjugation( void )
  {
	 GeometricAlgebra<Type> a(true,up,uq);
	 Type zero = 0;
	 
	 for ( unsigned int ii=0; ii < upq; ++ii ) {
		// switch ( nklein_priv::GeomGradeTable::GetGrade( ii ) & 3 ) {
		switch ( GetGrade( ii ) & 3 ) {
		case 0:
		case 3:
		  a[ii] = coef[ii];
		  break;
		case 1:
		case 2:
		  a[ii] = zero - coef[ii];
		  break;
		}
	 }	 
	 return a;
  }
  };

// =================================================================
// =================================================================
// different operators


// --------------------------	 
template <class Type>
  inline
  GeometricAlgebra<Type>
  operator + ( const GeometricAlgebra<Type>& a,
					const Type& b )
  {
	 // GeometricAlgebra<Type> c = a;
	 GeometricAlgebra<Type> c(true,a.up,a.uq);
	 c = a; 
	 
	 c[0] = a[0] + b;
	 
	 return c;
  }
template <class Type>
    inline
    GeometricAlgebra<Type>
    operator + ( const Type& a,
		    const GeometricAlgebra<Type>& b )
    {
	GeometricAlgebra<Type> c(false,b.p,b.q);
	// GeometricAlgebra<Type> c = b;

	c = b;

	c[0] = a + b[0];

	return c;
    }
template <class Type>
    inline
    GeometricAlgebra<Type>
    operator + ( const GeometricAlgebra<Type>& a,
		    const GeometricAlgebra<Type>& b )
  {
	 int upq = a.upq;  int up = a.up;  int uq = a.uq;
	 if (upq != b.upq)
		{
		  std::cerr << "Error : UNKNOWN STD EXCEPTION" << std::endl;
		}

	GeometricAlgebra<Type> c(false,up,uq);

	for ( unsigned int ii=0; ii < upq; ++ii ) {
	    c[ii] = a[ii] + b[ii];
	}

	return c;
    }
template <class Type>
    inline
    GeometricAlgebra<Type>
    operator - ( const GeometricAlgebra<Type>& a,
		    const Type& b )
    {

	GeometricAlgebra<Type> c = a;

	c[0] = a[0] - b;

	return c;
    }
template <class Type>
    inline
    GeometricAlgebra<Type>
    operator - ( const GeometricAlgebra<Type>& a,
		    const GeometricAlgebra<Type>& b )
    {
		int upq = a.upq;  int up = a.up;  int uq = a.uq;
	 if (upq != b.upq)
		{
		  std::cerr << "Error : UNKNOWN STD EXCEPTION" << std::endl;
		}

	GeometricAlgebra<Type> c(false,up,uq);

	// for ( unsigned int ii=0; ii < ( 1U << (P+Q) ); ++ii ) {
	for ( unsigned int ii=0; ii < upq; ++ii ) {
	    c[ii] = a[ii] - b[ii];
	}

	return c;
    }
template <class Type>
    inline
    GeometricAlgebra<Type>
    operator - ( const GeometricAlgebra<Type>& a )
    {
		int upq = a.upq;  int up = a.up;  int uq = a.uq;

	 GeometricAlgebra<Type> b(false,up,uq);
	 Type zero = 0;

	for ( unsigned int ii=0; ii < upq; ++ii ) {
	    b[ii] = zero - a[ii];
	}

	return b;
    }
template <class Type>
    inline
    GeometricAlgebra<Type>
    operator * ( const GeometricAlgebra<Type>& a,
		    const Type& b )
    {
		int upq = a.upq;  int up = a.up;  int uq = a.uq;

	GeometricAlgebra<Type> c(false,up,uq);

	for ( unsigned int ii=0; ii < upq; ++ii ) {
	    c[ ii ] = a[ ii ] * b;
	}

	return c;
    }
template <class Type>
    inline
    GeometricAlgebra<Type>
    operator * ( const Type& a,
		    const GeometricAlgebra<Type>& b )
    {
		int upq = b.upq;  int up = b.up;  int uq = b.uq;

	GeometricAlgebra<Type> c(true,up,uq);

	for ( unsigned int ii=0; ii < upq; ++ii ) {
	    c[ ii ] = a * b[ ii ];
	}

	return c;
    }

template <class Type>
    inline
    GeometricAlgebra<Type>
    operator * ( const GeometricAlgebra<Type>& a,
		    const GeometricAlgebra<Type>& b )
    {
		int upq = a.upq;  int up = a.up;  int uq = a.uq;
	 if (upq != b.upq)
		{
		  std::cerr << "Error : UNKNOWN STD EXCEPTION" << std::endl;
		}

	GeometricAlgebra<Type> c(true,up,uq);
	// mexPrintf("\n c.upq: %i\n",c.upq);

	for ( unsigned int ii=0; ii < upq; ++ii ) {
	    for ( unsigned int jj=0; jj < upq; ++jj ) {
		unsigned int index = ( ii ^ jj );

		// 		if (a[ii] != 0 & b[jj] != 0)
		//  mexPrintf("\n index: %i\n",index);

		if ( c.IsPositive( ii, jj ) ) {
		    c[ index ] = c[ index ] + ( a[ ii ] * b[ jj ] );
		} else {
		    c[ index ] = c[ index ] - ( a[ ii ] * b[ jj ] );
		}
	    }
	}

	return c;
    }

template <class Type>
  inline
  GeometricAlgebra<Type>
  operator ^ ( const GeometricAlgebra<Type>& a,
					const GeometricAlgebra<Type>& b )
  {
	 int upq = a.upq;  int up = a.up;  int uq = a.uq;
	 if (upq != b.upq)
		{
		  std::cerr << "Error : UNKNOWN STD EXCEPTION" << std::endl;
		}

	 GeometricAlgebra<Type> c(true,up,uq);
	 
	 for ( unsigned int ii=0; ii < upq; ++ii ) {
		for ( unsigned int jj=0; jj < upq; ++jj ) {
		  unsigned int mask = ( ii & jj );
		  if ( mask == 0 ) {
		    unsigned int index = ( ii ^ jj );
		    if ( c.IsPositive( ii, jj ) ) {
				c[ index ] = c[ index ] + ( a[ ii ] * b[ jj ] );
		    } else {
				c[ index ] = c[ index ] - ( a[ ii ] * b[ jj ] );
		    }
		  }
		}
	 }
	 
	 return c;
  }
};
