/* BasicColumnIterator.h
 * ---------------------
 * This is a quick-and-dirty column iterator valid for any matrix
 * type.
 *
 * Written by Patrick Perry
 * Original Version on December 1, 2003
 *
 * $Id: BasicColumnIterator.h,v 1.2 2005/11/02 01:59:52 patperry Exp $
 * $Log: BasicColumnIterator.h,v $
 * Revision 1.2  2005/11/02 01:59:52  patperry
 * The last (?) gcc4.0 fixes\!\!\!
 *
 * Revision 1.1.1.1  2005/03/18 23:36:05  silva
 * Imported Sources
 *
 * Revision 1.1.1.1  2004/09/25 22:04:08  patperry
 * Imported sources
 *
 * Revision 1.1  2004/01/15 07:13:36  patperry
 * Changed file extensions.
 *
 * Revision 1.1  2003/12/02 11:05:00  patperry
 * Added BasicColumnIterator
 *
 */

#ifndef _BASICCOLUMNITERATOR_H
#define _BASICCOLUMNITERATOR_H

#include "Matrix.h"
#include "RowColumnIterator.h"

template <class Element_t, class Alloc>
class BasicColumnIterator : public RowColumnIterator<Element_t, Alloc>
{
  //! the number of rows in the column
  int numRows;

  //! the column data
  Element_t * columnData;

  //! Memory allocator for elements
  static typename Alloc::template rebind<Element_t>::other ElementAlloc;

protected:

  using RowColumnIterator<Element_t, Alloc>::logicalPosition;

public:

  /**
   * Default constructor.
   */
  BasicColumnIterator ();

  /**
   * Copy constructor.
   */
  BasicColumnIterator (const BasicColumnIterator& other);

  /**
   * Constructor.  Creates a BasicColumnIterator for the given matrix and
   * column, positioned at the given position.
   *
   * \param matrix the matrix to iterate over.
   * \param column the column in the matrix to iterate over.
   * \param position the position of the iterator.
   */
  template <class ColumnIterator_t, class Alloc1>
  BasicColumnIterator (const Matrix<Element_t, ColumnIterator_t, Alloc1>& matrix,
				   int column, int position);

  /**
   * Destructor.
   */
  virtual ~BasicColumnIterator ();

  /**
   * Assignment operator.
   */
  BasicColumnIterator& operator= (const BasicColumnIterator& other);

  /**
   * De-refernce operator.
   *
   * \returns the value pointed to by the iterator.
   */
  Element_t operator* ();

  /**
   * Pre-increment operator.  Increments the iterator.
   *
   * \returns the iterator at (this + 1).
   */
  BasicColumnIterator& operator++ ();

  /**
   * Post-increment operator.
   *
   * \param post dummy argument.
   * \returns this iterator before incrementing.
   */
  BasicColumnIterator operator++ (int post);

  /**
   * Pre-decrement operator.  Decrements the iterator.
   *
   * \returns the iterator at (this - 1).
   */
  BasicColumnIterator& operator-- ();

  /**
   * Post-decrement operator.
   *
   * \param post dummy argument.
   * \returns this iterator before decrementing.
   */
  BasicColumnIterator operator-- (int post);

  /**
   * Forward-advance operator.  Increments the iterator by n.
   *
   * \param n an integer.
   * \returns this after it has been incremented.
   */
  BasicColumnIterator& operator+= (int n);

  /**
   * Backward-reverse operator.  Decrements the iterator by n.
   *
   * \param n an integer.
   * \returns this after it has been decremented.
   */
  BasicColumnIterator& operator-= (int n);

  /**
   * Distance operator.
   *
   * \param x another iterator.
   * \returns the distance between this and x.
   */
  int operator- (const BasicColumnIterator& x) const;

  /**
   * Index operator.
   *
   * \param n an integer.
   * \returns the value at this[n].
   */
  Element_t operator[] (int n) const;

  /**
   * Equals comparator.
   *
   * \param x iterator to compare this to.
   * \returns (*this) == x.
   */
  bool operator== (const BasicColumnIterator& x) const;

  /**
   * Not-Equals comparator.
   *
   * \param x iterator to compare this to.
   * \returns (*this) != x.
   */
  bool operator!= (const BasicColumnIterator& x) const;

  /**
   * Less-than comparator.
   *
   * \param x iterator to compare this to.
   * \returns (*this) < x.
   */
  bool operator< (const BasicColumnIterator& x) const;

};


/**
 * Iterator addition.
 *
 * \param n an integer.
 * \param i an iterator.
 * \returns the iterator at i + n.
 */
template <class Element_t, class Alloc>
BasicColumnIterator<Element_t, Alloc>
operator+ (int n, const BasicColumnIterator<Element_t, Alloc>& i);


/**
 * Iterator addition.
 *
 * \param i an iterator.
 * \param n an integer.
 * \returns the iterator at n + i.
 */
template <class Element_t, class Alloc>
BasicColumnIterator<Element_t, Alloc>
operator+ (const BasicColumnIterator<Element_t, Alloc>& i, int n);


/**
 * Iterator subtraction.
 *
 * \param i an iterator.
 * \param n an integer.
 * \returns the iterator at i - n.
 */
template <class Element_t, class Alloc>
BasicColumnIterator<Element_t, Alloc>
operator- (const BasicColumnIterator<Element_t, Alloc>& i, int n);


// get function definitions
#include "BasicColumnIterator.cc"

#endif //  _BASICCOLUMNITERATOR_H
