#ifndef _BLOCKMATRIX_H
#define _BLOCKMATRIX_H

#include <stdio.h>
#include "BlockableMatrix.h"

/*
class for 
*/

template<class Element_t>
class BlockMatrix : public BlockableMatrix<Element_t> 
{
private:
  SubmatrixIndices pivotSubmatrix;
  int maxNumBlockings;
  int numBlockings;
  Blocking* blockings;
  bool reduced;

  void initialize();

protected:

  using BlockableMatrix<Element_t>::rowPerm;
  using BlockableMatrix<Element_t>::colPerm;
  using BlockableMatrix<Element_t>::rowPermInverse;
  using BlockableMatrix<Element_t>::colPermInverse;
  using BlockableMatrix<Element_t>::pivotRows;
  using BlockableMatrix<Element_t>::pivotCols;

  void reduceOneStep(const Blocking& blocking, int blockingIndex, Blocking& unreducedBlocking, SubmatrixIndices& pivotMatrix);
  void reduce(const SubmatrixIndices& submatrixIndices);
  void reduce(const Blocking& blocking, int blockingIndex, SubmatrixIndices& pivotMatrix);
  void naiveReduce(const SubmatrixIndices& submat, SubmatrixIndices& pivotIndices);
  void reduceRowsUsingReducedSubmatrix(const SubmatrixIndices& reducedSubmat, const IndexList& rows);
  void applyPermutationAccordingToPivots();

public:
  typedef SubmatrixIndicesIterator const_iterator;

  template<class ColumnIterator_t, class Alloc>
  BlockMatrix(const Matrix<Element_t, ColumnIterator_t, Alloc>& M, 
		    int maxNumBlockings);

  BlockMatrix(Element_t** M, int numRows, int numCols, int maxNumBlockings);
  ~BlockMatrix();
  void addBlocking(const Blocking& blocking);
  void reduce();
  
  void display(const SubmatrixIndices& submat);
  void display() {BlockableMatrix<Element_t>::display(); rowPermInverse.display(); colPermInverse.display();};
  int getNumBlockings();
  bool isReduced() const {return reduced;};
  bool isFullRank() const;
  int getRank() const;
  bool rowContainsPivot(int row) const;
  bool colContainsPivot(int col) const;
  int getNumPivots() const;
};


//////////////////////////////////////////////////////////////////////////////////////////////////
/////////////////////////////////////definitions//////////////////////////////////////////////////
//////////////////////////////////////////////////////////////////////////////////////////////////


template<class Element_t>
template<class ColumnIterator_t, class Alloc>
BlockMatrix<Element_t>::
BlockMatrix(const Matrix<Element_t, ColumnIterator_t, Alloc>& M, 
		  int maxNumBlockings) 
  : BlockableMatrix<Element_t>(M), maxNumBlockings(maxNumBlockings)
{
  initialize();
}

template<class Element_t>
BlockMatrix<Element_t>::BlockMatrix(Element_t** M, int numRows, int numCols, int maxNumBlockings) : BlockableMatrix<Element_t>(M, numRows, numCols), maxNumBlockings(maxNumBlockings)
{
  initialize();
}

template<class Element_t>
BlockMatrix<Element_t>::~BlockMatrix() {
  delete[] blockings;
}

template<class Element_t>
void BlockMatrix<Element_t>::initialize() {
  blockings = new Blocking[maxNumBlockings];
  numBlockings = 0;
  reduced = false;
}

template<class Element_t>
void BlockMatrix<Element_t>::addBlocking(const Blocking& blocking) {
  if(numBlockings == maxNumBlockings) { printf("cannot add new blocking. Maximum number reached.\n"); return; }
  blockings[numBlockings].setVals(blocking);
  numBlockings++;
}

/*
BlockMatrix::reduce
*******************
Reduces the matrix taking full advantage of every available blocking.
*/

template<class Element_t>
void BlockMatrix<Element_t>::reduce() {
  if(reduced) {printf("Matrix is already reduced. Exiting.\n"); return;}
  if(numBlockings > 0) reduce(blockings[0], 1, pivotSubmatrix);
  else {SubmatrixIndices full(rowPerm, colPerm); naiveReduce(full, pivotSubmatrix);}
  applyPermutationAccordingToPivots();
  reduced = true;
}


/*
BlockMatrix<Element_t>::reduceOneStep
*************************************
Reduces all blocks along the diagonal of 'blocking', and converts them all to pivots using
'BlockMatrix<Element_t>::reduceRowsUsingReducedSubmatrix'. Does not then reduce the remaining blocks.

preconditions:
'blocking' must block 'M' such that all blocks below the diagonal are 0. 'blocking' must be square.
*/

template<class Element_t>
void BlockMatrix<Element_t>::reduceOneStep(const Blocking& blocking, int blockingIndex, Blocking& unreducedBlocking, SubmatrixIndices& pivotMatrix) {
  int numBlocks = blocking.rowPartition.getNumGroups();
  IndexList reducedRows[numBlocks], reducedCols[numBlocks], unreducedRows[numBlocks], unreducedCols[numBlocks];
  SubmatrixIndices* pivots = new SubmatrixIndices[numBlocks];
  SubmatrixIndices block;

  //reduce all blocks, (i, i) along the diagonal, storing the submatrix of the pivots in pivots[i].
  //reducedRows[i], reducedCols[i] hold the indices of the reduced rows, cols in sorted order.
  //similarly for unreducedRows[i], unreducedCols[i].

  reducedRows[numBlocks - 1].setVals(blocking.rowPartition.getElemsInGroup(numBlocks - 1)); reducedRows[numBlocks-1].sortList();
  reducedCols[0].setVals(blocking.colPartition.getElemsInGroup(0)); reducedCols[0].sortList();

  for(int i = 0; i < numBlocks; i++) {
    block.setVals(blocking.rowPartition.getElemsInGroup(i), blocking.colPartition.getElemsInGroup(i));

    if(blockingIndex >= numBlockings) naiveReduce(block, pivots[i]);
    else {
      Blocking nextBlockingRestricted(blockings[blockingIndex]);
      nextBlockingRestricted.restrict(block);
      reduce(nextBlockingRestricted, blockingIndex + 1, pivots[i]);
    }
    
    if(i < numBlocks - 1) {reducedRows[i].setVals(pivots[i].rowIndices); reducedRows[i].sortList();}
    if(i > 0) {reducedCols[i].setVals(pivots[i].colIndices); reducedCols[i].sortList();}
    
    unreducedRows[i].setDiff(block.rowIndices, reducedRows[i]);
    unreducedCols[i].setDiff(block.colIndices, reducedCols[i]);
  }

  //use the pivot[i] matrices to cancel unreducedrows to make them pivots for the entire matrix, not just
  //within each subblock.
  
  for(int blocksAbove = 1; blocksAbove < numBlocks; blocksAbove++) {
    for(int i = blocksAbove; i < numBlocks; i++) {
      reduceRowsUsingReducedSubmatrix(pivots[i], unreducedRows[i - blocksAbove]);
    }
  }
  
  //set unreducedBlocking as the original blocking, eliminating all reduced rows. What remains
  //is a blocking of only non-pivot rows and columns.

  unreducedBlocking.setVals(blocking);
  IndexList temp;
  temp.disjointUnion(reducedRows, numBlocks); unreducedBlocking.rowPartition.setDiff(temp); unreducedBlocking.rowPartition.elimEmptyGroup(numBlocks-1);
  temp.disjointUnion(reducedCols, numBlocks); unreducedBlocking.colPartition.setDiff(temp); unreducedBlocking.colPartition.elimEmptyGroup(0);

  pivotMatrix.concat(pivots, numBlocks);
  delete[] pivots;
}

/*
BlockMatrix<Element_t>::naiveReduce
************************
Reduce submatrix without using any blocking information. Store pivot indices in 'pivots'
*/

template<class Element_t>
void BlockMatrix<Element_t>::naiveReduce(const SubmatrixIndices& submat, SubmatrixIndices& pivots) {
  BlockableMatrix<Element_t>::reduce(submat, pivots);
}

/*
BlockMatrix<Element_t>::reduce
*******************
Reduces the submatrix of 'M' contained in 'blocking'.

preconditions:
same as 'BlockMatrix<Element_t>::reduceOneStep.
*/

template<class Element_t>
void BlockMatrix<Element_t>::reduce(const Blocking& blocking, int blockingIndex, SubmatrixIndices& pivotMatrix) {
  Blocking b1(blocking);
  Blocking b2;
  int numBlocks = b1.rowPartition.getNumGroups();
  if(numBlocks != b1.colPartition.getNumGroups()) {printf("error: non-square blocking used\n");}
  int count = 0;
  while(b1.rowPartition.getNumGroups() > 0) {
    SubmatrixIndices temp;
    if(count == 0) reduceOneStep(b1, blockingIndex, b2, temp);
    else reduceOneStep(b1, maxNumBlockings, b2, temp);
    b1.setVals(b2);
    if(count == 0) pivotMatrix.setVals(temp);
    else pivotMatrix.adjoin(temp);
    count++;
  }
}

/*
BlockMatrix<Element_t>::reduceRowsUsingReducedSubmatrix
********************************************
This function uses the reduced square matrix 'reducedSubmat' to convert the submatrix
with rows 'rows' and columns 'reducedSubmat.colIndices' to the zero matrix.

preconditions:
'reducedSubmat' is a permutation matrix. 
'reducedSubmat.rowIndices' is not necessarily in sorted order--in fact, pivot i
must occur at 'reducedSubmat.rowIndices[i]', 'reducedSubmat.colIndices[i]'.
The indexList 'rows' should also be sorted.
*/

template<class Element_t>
void BlockMatrix<Element_t>::reduceRowsUsingReducedSubmatrix(const SubmatrixIndices& reducedSubmat, const IndexList& rows) {
  int submatSize = reducedSubmat.rowIndices.length;
  if(submatSize != reducedSubmat.colIndices.length) {printf("error: non-square submatrix\n");}
  
  for(int col = 0; col < submatSize; col++) {
    int i = reducedSubmat.rowIndices.list[col];
    int j = reducedSubmat.colIndices.list[col];
    for(int rowIndex = 0; rowIndex < rows.length; rowIndex++) {
      this->cancelRowWith(rows.list[rowIndex], i, j);
    }
  }
}

/*
void BlockMatrix<Element_t>::cancelRowWith
****************************************
Eliminates the element in spot 'row', 'pivot' by subtracting a multiple of row 'cancelRow'.
*/



template<class Element_t>
int BlockMatrix<Element_t>::getRank() const {
  if(!reduced) {printf("cannot call 'getRank()' on unreduced matrix.\n"); return -1; }
  return getNumPivots();
}

template<class Element_t>
bool BlockMatrix<Element_t>::isFullRank() const {
  if(!reduced) {printf("cannot call 'isFullRank()' on unreduced matrix.\n"); return false; }
  return  (getNumPivots() == this->getNumRows() 
		 || getNumPivots() == this->getNumCols());
}

template<class Element_t>
bool BlockMatrix<Element_t>::rowContainsPivot(int row) const {
  if(!reduced) {printf("cannot call 'rowContainsPivot()' on unreduced matrix.\n"); return false; }
  return row < getRank();
}

template<class Element_t>
bool BlockMatrix<Element_t>::colContainsPivot(int col) const {
  if(!reduced) {printf("cannot call 'colContainsPivot()' on unreduced matrix.\n"); return false; }
  return col < getRank();
}

template<class Element_t>
int BlockMatrix<Element_t>::getNumPivots() const {
  if(!reduced) {printf("cannot call 'getNumPivots()' on unreduced matrix.\n"); return false; }
  return pivotSubmatrix.rowIndices.length;
}


template<class Element_t>
void BlockMatrix<Element_t>::applyPermutationAccordingToPivots() {
  pivotRows.markWith(pivotSubmatrix.rowIndices, this->getNumRows()); 
  pivotCols.markWith(pivotSubmatrix.colIndices, this->getNumCols());
  applyBackwardsPartitionInducedRowPermutation(pivotRows);
  applyBackwardsPartitionInducedColPermutation(pivotCols);
  this->applyRowPermutation(pivotSubmatrix.rowIndices);
  this->applyColPermutation(pivotSubmatrix.colIndices);
}


#endif
