#ifndef _BLOCKABLEMATRIX_H
#define _BLOCKABLEMATRIX_H

#include "Matrix.h"
#include "GeomSparse.h"
#include "BasicColumnIterator.h"
#include <iostream>
#include <memory>

template<class Element_t>
class BlockableMatrix : public Matrix<Element_t, 
							   BasicColumnIterator<Element_t, 
											   std::allocator<void> >,
							   std::allocator<void> > {
  Element_t** M;
  void copyMat(Element_t** M);
  int firstNonZero(int i, const IndexList& colIndices);
  void killRowWith(int i, int j, int pivot);
protected:
  IndexList rowPerm, colPerm, rowPermInverse, colPermInverse;
  Partition pivotRows, pivotCols;
  void initializePerms();

public:
  template<class ColumnIterator_t, class Alloc>
  BlockableMatrix(const Matrix<Element_t, ColumnIterator_t, Alloc>& M);
  BlockableMatrix(Element_t** M, int numRows, int numCols);
  ~BlockableMatrix();
  void makeIdentity();
  void eraseMatrix();
  void display();
  void display(const IndexList& rowPerm, const IndexList& colPerm);
  //void display(Blocking& blocking);
  void display(const SubmatrixIndices& submat);
  void displayMatlab(std::ostream& os);
  void reduce(const SubmatrixIndices& submat, SubmatrixIndices& pivotMatrix);
  void reduce();
  virtual void mRowAdd(Element_t pivVal, int row1, int row2);
  virtual void mRowAdd(Element_t pivVal, int row1, int row2, int startCol);
  void cancelRowWith(int row, int cancelRow, int pivotCol);
  template<class InputIterator> void addVectorToRow(InputIterator vectorBegin, int row, const Element_t& k = static_cast<Element_t>(1.0));
  int getNumCols() const {return this->getNumColumns();};
  virtual Element_t getElement(int i, int j) const;
  void virtual setElement(int i, int j, Element_t k);

  void applyPermutationAccordingToPivots();
  void applyBlockingInducedPermutation(const Blocking& blocking);
  void applyRowPermutation(const IndexList& permutedIndices);
  void applyColPermutation(const IndexList& permutedIndices);
  void applyPartitionInducedRowPermutation(const Partition& partition);
  void applyBackwardsPartitionInducedRowPermutation(const Partition& partition);
  void applyBackwardsPartitionInducedColPermutation(const Partition& partition);
  void getP(IndexList& P) {P.setVals(rowPermInverse);};
  void getQ(IndexList& Q) {Q.setVals(colPermInverse);};
  void getPinv(IndexList& Pinv) {Pinv.setVals(rowPerm);};
  void getQinv(IndexList& Qinv) {Qinv.setVals(colPerm);};
  //const IndexList& getP() {return rowPermInverse;};
  const int * getP () { return rowPermInverse.list; };
  //const IndexList& getQ() {return colPermInverse;};
  const int * getQ () { return colPermInverse.list; };
  const IndexList& getPinv() {return rowPerm;};
  const IndexList& getQinv() {return colPerm;};
  void mult(BlockableMatrix<Element_t> A, BlockableMatrix<Element_t> B);
  bool operator== (BlockableMatrix& b);

  BasicColumnIterator<Element_t, std::allocator<void> > 
  getColumnBegin (int column) const
  {
    BasicColumnIterator<Element_t, std::allocator<void> > 
	 result(*this, column, 0);
	 
    return result;
  }


  BasicColumnIterator<Element_t, std::allocator<void> > 
  getColumnEnd (int column) const
  {
    BasicColumnIterator<Element_t, std::allocator<void> >
	 result(*this, column, this->getNumRows());
	 
    return result;
  }

};


void applyPartitionInducedPermutation(const Partition& partition, IndexList& target, IndexList& targetInverse);
void applyBackwardsPartitionInducedPermutation(const Partition& partition, IndexList& target, IndexList& targetInverse);
void applyPartitionInducedPermutation(const Partition& partition, IndexList& target, IndexList& targetInverse, bool lowIndicesFirst);
void applyPermutation(const IndexList& unpermutedIndices, const IndexList& permutedIndices, IndexList& target, IndexList& targetInverse);
void applyPartitionInducedPermutation(const Partition& partition, IndexList& target, IndexList& targetInverse);

int abs(int x);
void readIntMatrixFromAsciiMatlabFile(int** M, int& numRows, int& numColumns);

//////////////////////////////////////////////////////////////////////////////////////////////////
/////////////////////////////////////definitions//////////////////////////////////////////////////
//////////////////////////////////////////////////////////////////////////////////////////////////

template<class Element_t>
void BlockableMatrix<Element_t>::copyMat(Element_t** M) {
  this->M = new Element_t*[this->getNumRows()];
  for(int i = 0; i < this->getNumRows(); i++) {
    this->M[i] = new Element_t[this->getNumCols()];
    if(M != NULL) 
      for(int j = 0; j < this->getNumCols(); j++) this->M[i][j] = M[i][j];
    else
      for(int j = 0; j < this->getNumCols(); j++) this->M[i][j] = 0;
  }
}

template<class Element_t>
template<class ColumnIterator_t, class Alloc>
BlockableMatrix<Element_t>::
BlockableMatrix(const Matrix<Element_t, ColumnIterator_t, Alloc>& matrix) 
  : Matrix<Element_t, BasicColumnIterator<Element_t, std::allocator<void> >,
		 std::allocator<void> >(matrix.getNumRows(), matrix.getNumColumns()) {
  initializePerms();
  this->M = new Element_t*[this->getNumRows()];
  for(int i = 0; i < this->getNumRows(); i++) {
    this->M[i] = new Element_t[this->getNumColumns()];
    for(int j = 0; j < this->getNumColumns(); j++) {
      this->M[i][j] = matrix.getElement(i, j);
    }
  }
}

template<class Element_t>
BlockableMatrix<Element_t>::
BlockableMatrix(Element_t** M, int numRows, int numColumns) 
  : Matrix<Element_t, BasicColumnIterator<Element_t, std::allocator<void> >,
		 std::allocator<void> >(numRows, numColumns)
{
  initializePerms();
  copyMat(M);
}

template<class Element_t>
void BlockableMatrix<Element_t>::initializePerms() {
  rowPerm.makeFull(this->getNumRows()); 
  colPerm.makeFull(this->getNumCols());
  rowPermInverse.makeFull(this->getNumRows()); 
  colPermInverse.makeFull(this->getNumCols());
}

template<class Element_t>
void BlockableMatrix<Element_t>::makeIdentity() {
  for(int i = 0; i < this->getNumRows(); i++) {
    for(int j = 0; j < this->getNumCols(); j++) {
      if(i == j) M[i][j] = 1;
      else M[i][j] = 0;
    }
  }
}

template<class Element_t>
BlockableMatrix<Element_t>::~BlockableMatrix() {
  eraseMatrix();
}

template<class Element_t>
void BlockableMatrix<Element_t>::eraseMatrix() {
  for(int i = 0; i < this->getNumRows(); i++)
    delete[] M[i];
  delete[] M;
  M = NULL;
}


template<class Element_t>
Element_t BlockableMatrix<Element_t>::getElement(int i, int j) const {
  return M[rowPerm.list[i]][colPerm.list[j]];
}

template<class Element_t>
void BlockableMatrix<Element_t>::display() {
  int i, j;
  for(i = 0; i < this->getNumRows(); i++) {
    for( j = 0; j < this->getNumColumns(); j++) {
      std::cout << getElement(i, j);
    }
    std::cout << "\n";
  }
}


template<class Element_t>
void BlockableMatrix<Element_t>::displayMatlab(std::ostream& os) {
  os << "[";
  for(int i = 0; i < this->getNumRows(); i++) {
    for(int j = 0; j < this->getNumCols(); j++) {
      os << getElement(i, j);
      if(j < this->getNumCols() - 1) os << ","; else if(i < this->getNumRows()-1) os << ";"; else os << "]";
    }
  }
  os << "\n";
}

template<class Element_t>
void BlockableMatrix<Element_t>::setElement(int i, int j, Element_t k) {
  M[rowPerm.list[i]][colPerm.list[j]] = k;
}

template<class Element_t>
void BlockableMatrix<Element_t>::display(const SubmatrixIndices& submat) {
  int i, j;
  for(i = 0; i < submat.rowIndices.length; i++) {
    std::cout << "[";
    for( j = 0; j < submat.colIndices.length; j++) {
      std::cout << M[submat.rowIndices.list[i]][submat.colIndices.list[j]];
    }
    std::cout << "]\n";
  }
}

/*
template<class Element_t>
void BlockableMatrix<Element_t>::display(Blocking& blocking) {
  int i, j, k, l;
  for(i = 0; i < blocking.rowPartition.getNumGroups(); i++) {
    IndexList rowGroupElems = blocking.rowPartition.getElemsInGroup(i);
    for(j = 0; j < rowGroupElems.length; j++) {
      printf("[");
      for(k = 0; k < blocking.colPartition.getNumGroups(); k++) {
	IndexList colGroupElems = blocking.colPartition.getElemsInGroup(k);
	for(l = 0; l < colGroupElems.length; l++) {
	  printf("%d ", (int) M[rowGroupElems.list[j]]
		 [colGroupElems.list[l]]);
	}
	if(k != blocking.colPartition.getNumGroups()-1) printf("| ");
      }
      printf("]\n");
    }
    if(i != blocking.rowPartition.getNumGroups()-1) {
      printf("[");
      for(k = 0; k < blocking.colPartition.getNumElems() + blocking.colPartition.getNumGroups()-1; k++) {
	printf("--");
      }
      printf("]\n");
    }
  }
  printf("\n");
}
*/


/*void BlockableMatrix<Element_t>::reduce(SubmatrixIndices submat) {
  int i, j, pivot;
  IndexList rowIndices(submat.rowIndices);
  IndexList colIndices(submat.colIndices);
  for(j = 0; j < colIndices.length; j++) {
    pivot = -1;
    for(i = 0; i < rowIndices.length; i++) {
      if(firstNonZero(i, colIndices) == j) {
	pivot = i; break;
      }
    }
    if(pivot != -1) {
      for(i = 0; i < rowIndices.length; i++) {
	if(i != pivot && firstNonZero(i, colIndices) == j)
	  killRowWith(rowIndices.list[i], rowIndices.list[pivot], colIndices.list[j]);
      }
    }
  }
  }*/

template<class Element_t>
void BlockableMatrix<Element_t>::reduce() {
  int l1[this->getNumRows()], l2[this->getNumColumns()];
  for(int i = 0; i < this->getNumRows(); i++) l1[i] = i;
  for(int i = 0; i < this->getNumColumns(); i++) l2[i] = i;
  IndexList allRows(l1, this->getNumRows()), allCols(l2, this->getNumColumns());
  SubmatrixIndices fullMatrix(allRows, allCols);
  SubmatrixIndices dummy;
  reduce(fullMatrix, dummy);
}

template<class Element_t>
void BlockableMatrix<Element_t>::reduce(const SubmatrixIndices& submat, SubmatrixIndices& pivotMatrix) {
  int rows = 0;
  int rowPivots[submat.rowIndices.length], colPivots[submat.colIndices.length];
  int i, j, pivot;
  IndexList rowIndices(submat.rowIndices);
  IndexList colIndices(submat.colIndices);
  for(j = 0; j < colIndices.length; j++) {
    pivot = -1;
    for(i = 0; i < rowIndices.length; i++) {
      if(firstNonZero(rowIndices.list[i], colIndices) == j) {
	pivot = i; 
	rowPivots[rows] = rowIndices.list[i];
	colPivots[rows] = colIndices.list[j];
	rows++;
	break;
      }
    }
    if(pivot != -1) {
      for(i = 0; i < rowIndices.length; i++) {
	if(i != pivot && firstNonZero(rowIndices.list[i], colIndices) == j)
	  cancelRowWith(rowIndices.list[i], rowIndices.list[pivot], colIndices.list[j]);
      }
    }
  }
  IndexList rIndices(rowPivots, rows);
  IndexList cIndices(colPivots, rows);
  pivotMatrix.setVals(rIndices, cIndices);
}

/*SubmatrixIndices BlockableMatrix<Element_t>::reduce(SubmatrixIndices submat) {
  int i, j, pivot, rows = 0;
  int rowPivots[submat.rowIndices.length], colPivots[submat.colIndices.length];
  IndexList rowIndices(submat.rowIndices);
  IndexList colIndices(submat.colIndices);
  for(j = 0; j < colIndices.length; j++) {
    pivot = -1;
    for(i = 0; i < rowIndices.length; i++) {
      if(M[i][j] != 0) {
	pivot = i; 
	rowPivots[rows] = rowIndices.list[i];
	colPivots[rows] = colIndices.list[j];
	rows++;
	break;
      }
      if(pivot != -1) {
	
      }
    }
    
  }
  }*/

template<class Element_t>
void BlockableMatrix<Element_t>::cancelRowWith(int row, int cancelRow, int pivotCol) {
  Element_t a = getElement(row, pivotCol), b = getElement(cancelRow, pivotCol);
  if(a == 0) {return;}
  if(b == 0) {std::cout << "can't cancel row " << row << "with row "
				    << cancelRow << "using pivot column "
				    << pivotCol << ". Pivot has value 0.\n"; return;}
  mRowAdd(-a/b, row, cancelRow, pivotCol);
}

/*template<class Element_t>
void BlockableMatrix<Element_t>::killRowWith(int i, int j, int pivot) {
  Element_t c = M[i][pivot], d = M[j][pivot];
  if(i < 0 || j < 0 || i >= getNumRows() || j >= getNumColumns()) { printf("can't cancel row: range error.\n"); return; }
  if(c == 0 || d == 0) return;
  //printf("adding %d times row %d to %d times row %d\n", -c, j, d, i);
  mRowAdd(-c/d, i, j);
}*/

template<class Element_t>
void BlockableMatrix<Element_t>::mRowAdd(Element_t pivVal, int row1, int row2) {
  mRowAdd(pivVal, row1, row2, 0);
}

template<class Element_t>
void BlockableMatrix<Element_t>::mRowAdd(Element_t pivVal, int row1, int row2, int startCol) {
  for(int k = startCol; k < this->getNumColumns(); k++) {
    setElement(row1, k,  getElement(row1, k) + pivVal*getElement(row2,k));
  }
}

template<class Element_t>
int BlockableMatrix<Element_t>::firstNonZero(int i, const IndexList& colIndices) {
  int k;
  for(k = 0; k < colIndices.length; k++) {
    if(getElement(i, colIndices.list[k]) != 0) return k;
  }

  return -1;
}

template<class Element_t>
template<class InputIterator> 
void BlockableMatrix<Element_t>::addVectorToRow(InputIterator vectorBegin, int row, const Element_t& k) {
  int j = 0;
  InputIterator vectorIterator = vectorBegin;
  while(j < this->getNumCols()) {
    M[row][j] += *vectorIterator * k;
    j++; vectorIterator++;
  }
}

int abs(int x) {
  if(x >= 0) return x;
  return -x;
}

template<class Element_t>
void BlockableMatrix<Element_t>::mult(BlockableMatrix<Element_t> A, BlockableMatrix<Element_t> B) {
  if(this->getNumRows() != A.getNumRows() 
	|| A.getNumCols() != B.getNumRows() 
	|| B.getNumCols() != this->getNumCols()) {
    std::cout << "can't multiply matrices: wrong size.\n"; return;
  }
  for(int i = 0; i < this->getNumRows(); i++) {
    for(int j = 0; j < this->getNumCols(); j++) {
      Element_t sum = 0;
      for(int k = 0; k < A.getNumCols(); k++) {
	sum += A.getElement(i, k) * B.getElement(k, j);
      }
      setElement(i, j, sum);
    }
  }
}

template<class Element_t>
bool BlockableMatrix<Element_t>::operator== (BlockableMatrix& b) 
{ 
  if(this->getNumRows() != b.getNumRows() 
	|| this->getNumCols() != b.getNumCols()) return false;
  for(int i = 0; i < this->getNumRows(); i++) {
    for(int j = 0; j < this->getNumRows(); j++) {
      if(getElement(i, j) != b.getElement(i, j)) return false;
    }
  }
  return true;
}

/*
These functions are for keeping track of the permutation matrices stored in rowPerm, colPerm, rowPermInverse, colPermInverse.
Most of them are not used.
*/

void applyPermutation(const IndexList& permutedIndices, IndexList& target, IndexList& targetInverse) {
  IndexList unpermutedIndices(permutedIndices);
  unpermutedIndices.sortList();
  applyPermutation(unpermutedIndices, permutedIndices, target, targetInverse);
}

void applyPermutation(const IndexList& unpermutedIndices, const IndexList& permutedIndices, IndexList& target, IndexList& targetInverse) {
  IndexList temp(unpermutedIndices);
  for(int i = 0; i < temp.length; i++) temp.list[i] = targetInverse.list[temp.list[i]];
  for(int i = 0; i < unpermutedIndices.length; i++) {
    target.list[temp.list[i]] = permutedIndices.list[i];
    targetInverse.list[permutedIndices.list[i]] = temp.list[i];   
  }
}
 
void applyPartitionInducedPermutation(const Partition& partition, IndexList& target, IndexList& targetInverse) {
  applyPartitionInducedPermutation(partition, target, targetInverse, true);
}

void applyBackwardsPartitionInducedPermutation(const Partition& partition, IndexList& target, IndexList& targetInverse) {
  applyPartitionInducedPermutation(partition, target, targetInverse, false);
}

void applyPartitionInducedPermutation(const Partition& partition, IndexList& target, IndexList& targetInverse, bool lowIndicesFirst) {
  int numGroups = partition.getNumGroups();
  IndexList* groupElems = new IndexList[numGroups];

  if(lowIndicesFirst) for(int i = 0; i < numGroups; i++) groupElems[i].setVals(partition.getElemsInGroup(i));
  else for(int i = 0; i < numGroups; i++) groupElems[i].setVals(partition.getElemsInGroup(numGroups - i - 1));

  IndexList unpermutedIndices, permutedIndices;
  unpermutedIndices.disjointUnion(groupElems, numGroups);
  permutedIndices.concat(groupElems, numGroups);
  applyPermutation(unpermutedIndices, permutedIndices, target, targetInverse);
  delete[] groupElems;
}

template<class Element_t> 
void BlockableMatrix<Element_t>::applyPartitionInducedRowPermutation(const Partition& partition) {
  applyPartitionInducedPermutation(partition, rowPerm, rowPermInverse);
}

template<class Element_t>
void BlockableMatrix<Element_t>::applyBackwardsPartitionInducedRowPermutation(const Partition& partition) {
  applyBackwardsPartitionInducedPermutation(partition, rowPerm, rowPermInverse);
}

template<class Element_t>
void BlockableMatrix<Element_t>::applyBackwardsPartitionInducedColPermutation(const Partition& partition) {
  applyBackwardsPartitionInducedPermutation(partition, colPerm, colPermInverse);
}

template<class Element_t>
void BlockableMatrix<Element_t>::applyRowPermutation(const IndexList& permutedIndices) {
  applyPermutation(permutedIndices, rowPerm, rowPermInverse);
}

template<class Element_t>
void BlockableMatrix<Element_t>::applyColPermutation(const IndexList& permutedIndices) {
  applyPermutation(permutedIndices, colPerm, colPermInverse);
}

template<class Element_t>
void BlockableMatrix<Element_t>::applyBlockingInducedPermutation(const Blocking& blocking) {
  applyPartitionInducedPermutation(blocking.colPartition, colPerm, colPermInverse);
  blocking.rowPartition.display();  blocking.colPartition.display();
}


#endif

