/* CRCovering.h
 * ------------
 * Interface defining a CR-covering.
 *
 * Written by Patrick Perry
 * Original Version on July 2, 2003
 *
 * $Id: CRCovering.h,v 1.2 2005/10/24 16:20:08 patperry Exp $
 * $Log: CRCovering.h,v $
 * Revision 1.2  2005/10/24 16:20:08  patperry
 * Hells (literally) of compile fixes for gcc4.0
 *
 * Revision 1.1.1.1  2005/03/18 23:36:08  silva
 * Imported Sources
 *
 * Revision 1.1.1.1  2004/09/25 22:04:08  patperry
 * Imported sources
 *
 * Revision 1.2  2004/01/23 04:48:54  patperry
 * Generalized constructor.
 *
 * Revision 1.1  2004/01/15 07:13:36  patperry
 * Changed file extensions.
 *
 * Revision 1.3  2003/12/12 20:13:46  patperry
 * bugfixes and debug code galore
 *
 * Revision 1.2  2003/10/17 09:51:10  patperry
 * Updated CRCovering and CSCSparseMatrix, (probably) breaking GeomSparse
 * in the process.
 *
 * 
 */

#ifndef _CRCOVERING_H
#define _CRCOVERING_H

#include <cstdlib>
#include <iostream>
#include <set>
#include "CSCSparseMatrix.h"

/**
 * Class for defining a CR-covering, as described in Carlsson and 
 * V. De Silva, "A geometric framework for sparse matrix problems".
 */
class CRCovering 
{
public:  

  //! Type for iterating through the row or column index sets.
  typedef const std::set<int> * const_iterator;

private:

  //! An array of column sets
  std::set<int> * const columnSets;

  //! An array of row sets
  std::set<int> * const rowSets;

  //! The number of sets
  int numSets;

  //! The number of distinct rows in the partition 
  const int numRows;

  //! The number of distinct columns in the partition 
  const int numColumns;

public:

  /**
   * Constructor. Builds a new CR-covering from the column cover data 
   * and the matrix.  
   *
   * \param columnCoverIndices an (int) iterator to the indices to put
   * in the column cover.
   * \param columnCoverSets an (int) iterator to the corresponding set
   * identifiers of the column indices.
   * \param numIndices the number of indices.
   * \param numSets the number of sets in the cover.
   * \param matrix a matrix to derive the row cover from.
   */
  template <class InputIterator1, class InputIterator2,
		  class Element_t, class ColumnIterator_t, class Alloc>
  CRCovering (InputIterator1 columnCoverIndices, 
		    InputIterator2 columnCoverSets, int numIndices, int numSets,
		    const Matrix<Element_t, ColumnIterator_t, Alloc>& matrix)
    : columnSets(new std::set<int>[numSets + 1]),
	 rowSets(new std::set<int>[numSets + 1]),
	 numSets(numSets), 
	 numRows(matrix.getNumRows()),
	 numColumns(matrix.getNumColumns())
  {
    std::vector<int> usedColumns(numColumns, 0);
    std::vector<int> usedRows(numRows, 0);

    // first build all the column sets
    for (int i = 0; i < numIndices; ++i) {
	 columnSets[*columnCoverSets].insert(*columnCoverIndices);
	 usedColumns[*columnCoverIndices] = 1;

	 ++columnCoverIndices;
	 ++columnCoverSets;
    }

    // then build the row sets
    for (int i = 0; i < numSets; ++i) {
	 buildRowSet(columnSets[i], matrix, rowSets[i], usedRows);
    }

    // if necessary add extra sets for the unused rows and columns
    if (std::count(usedColumns.begin(), usedColumns.end(), 0)
	   || std::count(usedRows.begin(), usedRows.end(), 0)) {

	 std::vector<int>::const_iterator usedColumnsBegin = usedColumns.begin();
	 std::vector<int>::const_iterator unusedColumnIt = usedColumnsBegin;
	 std::vector<int>::const_iterator usedColumnsEnd = usedColumns.end();

	 while ((unusedColumnIt = std::find(unusedColumnIt, usedColumnsEnd, 0))
		    != usedColumnsEnd) {
	   columnSets[numSets].insert(unusedColumnIt - usedColumnsBegin);
	   ++unusedColumnIt;
	 }

	 std::vector<int>::const_iterator usedRowsBegin = usedRows.begin();
	 std::vector<int>::const_iterator unusedRowIt = usedRowsBegin;
	 std::vector<int>::const_iterator usedRowsEnd = usedRows.end();

	 while ((unusedRowIt = std::find(unusedRowIt, usedRowsEnd, 0))
		   != usedRowsEnd) {
	   rowSets[numSets].insert(unusedRowIt - usedRowsBegin);
	   ++unusedRowIt;
	 }

	 ++(this->numSets);
    }
    
  }

  
  /**
   * Constructor.  Takes full descriptions of both the column and the 
   * row sets and builds a covering.
   * 
   * \param columnCoverIndices an (int) iterator to the indices to put
   * in the column cover.
   * \param columnCoverSets an (int) iterator to the corresponding set
   * identifiers of the column indices.
   * \param numColumnCoverIndices the number of column indices.
   * \param rowCoverIndices an (int) iterator to the indices to put
   * in the row cover.
   * \param rowCoverSets an (int) iterator to the corresponding set
   * identifiers of the row indices.
   * \param numRowCoverIndices the number of row indices.
   * \param numSets the number of sets in the cover.
   * \param numRows the number of unique rows in the partition.
   * \param numColumns the number of unique columns in the partition.
   */
  template <class InputIterator1, class InputIterator2,
		  class InputIterator3, class InputIterator4>
  CRCovering (InputIterator1 columnCoverIndices,
		    InputIterator2 columnCoverSets,
		    int numColumnCoverIndices,
		    InputIterator3 rowCoverIndices,
		    InputIterator4 rowCoverSets,
		    int numRowCoverIndices,
		    int numSets, int numRows, int numColumns)
    : columnSets(new std::set<int>[numSets + 1]),
	 rowSets(new std::set<int>[numSets + 1]),
	 numSets(numSets), 
	 numRows(numRows),
	 numColumns(numColumns)
  {
    std::vector<int> usedColumns(numColumns, 0);
    std::vector<int> usedRows(numRows, 0);

    // first build all the column sets
    for (int i = 0; i < numColumnCoverIndices; ++i) {
	 columnSets[*columnCoverSets].insert(*columnCoverIndices);
	 usedColumns[*columnCoverIndices] = 1;

	 ++columnCoverIndices;
	 ++columnCoverSets;
    }

    // then build the row sets
    for (int i = 0; i < numRowCoverIndices; ++i) {
	 rowSets[*rowCoverSets].insert(*rowCoverIndices);
	 usedRows[*rowCoverIndices] = 1;

	 ++rowCoverIndices;
	 ++rowCoverSets;
    }

    // if necessary add extra sets for the unused rows and columns
    if (std::count(usedColumns.begin(), usedColumns.end(), 0)
	   || std::count(usedRows.begin(), usedRows.end(), 0)) {

	 std::vector<int>::const_iterator usedColumnsBegin = usedColumns.begin();
	 std::vector<int>::const_iterator unusedColumnIt = usedColumnsBegin;
	 std::vector<int>::const_iterator usedColumnsEnd = usedColumns.end();

	 while ((unusedColumnIt = std::find(unusedColumnIt, usedColumnsEnd, 0))
		    != usedColumnsEnd) {
	   columnSets[numSets].insert(unusedColumnIt - usedColumnsBegin);
	   ++unusedColumnIt;
	 }

	 std::vector<int>::const_iterator usedRowsBegin = usedRows.begin();
	 std::vector<int>::const_iterator unusedRowIt = usedRowsBegin;
	 std::vector<int>::const_iterator usedRowsEnd = usedRows.end();

	 while ((unusedRowIt = std::find(unusedRowIt, usedRowsEnd, 0))
		   != usedRowsEnd) {
	   rowSets[numSets].insert(unusedRowIt - usedRowsBegin);
	   ++unusedRowIt;
	 }

	 ++(this->numSets);
    }
  }
    

  /**
   * Destructor.
   */
  virtual ~CRCovering ();

  /**
   * Returns the number of sets in the cover.
   * 
   * \returns number of sets.
   */
  int getNumSets () const;

  /** 
   * Returns the number of rows in the cover.
   *
   * \returns the number of rows.
   */
  int getNumRows () const;

  /**
   * Returns the number of columns in the cover.
   *
   * \returns the number of columns.
   */
  int getNumColumns () const;

  /**
   * Get the column set of a given index.
   *
   * \param i the index.
   * \returns a reference to the column set with index i.
   */
  const std::set<int>& getColumnSet (int i) const;

  /**
   * Get the row set of a given index.
   *
   * \param i the index.
   * \returns a reference to the row set with index i.
   */
  const std::set<int>& getRowSet (int i) const;

  /**
   * Returns an iterator to the first column set.
   *
   * \returns the begin of the column sets.
   */
  const_iterator getColumnSetsBegin () const;

  /**
   * Returns an iterator to the end of the column sets.
   *
   * \returns the end of the column sets.
   */
  const_iterator getColumnSetsEnd () const;

  /**
   * Returns an iterator to the first row set.
   *
   * \returns the begin of the row sets.
   */
  const_iterator getRowSetsBegin () const;

  /**
   * Returns an iterator to the end of the row sets.
   *
   * \returns the end of the row sets.
   */
  const_iterator getRowSetsEnd () const;

  /**
   * Writes the covering to an output stream.
   * 
   * \param s an output stream.
   * \returns the output stream.
   */
  virtual std::ostream& put (std::ostream& s) const;

private:

  /**
   * Inserts the necessary row indices to the given row set
   * based on the corresponding column set and the matrix.
   *
   * \param the corresponding column set of the row set to build.
   * \param matrix the matrix to infer from.
   * \param rowSet the row set to build.
   * \param usedRows a logical vector indicating whether or not
   *        a row appears in a row set.
   * \returns void.
   */
  template <class Element_t, class ColumnIterator_t, class Alloc>
  static void buildRowSet (const std::set<int>& columnSet, 
					  const Matrix<Element_t, ColumnIterator_t, Alloc>& matrix,
					  std::set<int>& rowSet, std::vector<int>& usedRows)
  {
    for (typename std::set<int>::const_iterator column = columnSet.begin();
	    column != columnSet.end(); ++column) {
	 for (int row = 0; row < matrix.getNumRows(); ++row) {
	   if (matrix.getElement(row, *column) != 0) {
		rowSet.insert(row);
		usedRows[row] = 1;
	   }
	 }
    }
  }

};


/**
 * Overloaded output stream operator for CR-covering objects.
 *
 * \param s an output stream.
 * \param covering a CRCovering.
 * \returns the output stream.
 */
std::ostream& operator<< (std::ostream& s, 
					 const CRCovering& covering);


// get function definitions
//#include "CRCovering.cc"

#endif // _CRCOVERING_H
