/* CSCSparseColumnIterator.cc
 * --------------------------
 * Implements CSCSparseColumnIterator.h
 *
 * Written by Patrick Perry
 * Original Version on October 22, 2003
 *
 * $Id: CSCSparseColumnIterator.cc,v 1.1.1.1 2005/03/18 23:36:08 silva Exp $
 * $Log: CSCSparseColumnIterator.cc,v $
 * Revision 1.1.1.1  2005/03/18 23:36:08  silva
 * Imported Sources
 *
 * Revision 1.1.1.1  2004/09/25 22:04:08  patperry
 * Imported sources
 *
 * Revision 1.5  2004/01/15 07:13:36  patperry
 * Changed file extensions.
 *
 * Revision 1.4  2003/12/12 20:13:46  patperry
 * bugfixes and debug code galore
 *
 * Revision 1.3  2003/10/25 00:49:32  patperry
 * Added RowIterators.  Make RowColumnIterator have virtual functions.
 *
 * Revision 1.2  2003/10/22 10:52:36  patperry
 * Implemented and tested ColumnIterator for sparse matrices.
 *
 * Revision 1.1  2003/10/22 07:37:18  patperry
 * More work on column iterators.
 *
 *
 */

#include <algorithm>

template <class Element_t, class Alloc>
CSCSparseColumnIterator<Element_t, Alloc>::
CSCSparseColumnIterator ()
  : RowColumnIterator<Element_t, Alloc>(0),
    valueArray(NULL), rowIndexArray(NULL), numNonzeroElements(0),
    rowPermutationInverse(NULL), numRows(0),
    actualPosition(0)
{
}


template <class Element_t, class Alloc>
CSCSparseColumnIterator<Element_t, Alloc>::
CSCSparseColumnIterator (const Element_t * valueArray, 
					const int * rowIndexArray, int numNonzeroElements,
					const int * rowPermutationInverse, int numRows,
					int logicalPosition)
  : RowColumnIterator<Element_t, Alloc>(logicalPosition),
    valueArray(valueArray), rowIndexArray(rowIndexArray),
    numNonzeroElements(numNonzeroElements), 
    rowPermutationInverse(rowPermutationInverse),
    numRows(numRows)
{
  if (logicalPosition >= numRows) {
    actualPosition = numNonzeroElements;
  } else if (!rowPermutationInverse) {
    actualPosition = std::lower_bound(rowIndexArray, 
							   rowIndexArray + numNonzeroElements,
							   logicalPosition) - rowIndexArray;
  } else {
    actualPosition = (std::lower_bound(rowIndexArray, 
							    rowIndexArray + numNonzeroElements,
							    rowPermutationInverse[logicalPosition])
				  - rowIndexArray);
  }
}


template <class Element_t, class Alloc>
CSCSparseColumnIterator<Element_t, Alloc>::
~CSCSparseColumnIterator ()
{
}


template <class Element_t, class Alloc>
Element_t 
CSCSparseColumnIterator<Element_t, Alloc>::
operator* ()
{
  Element_t result = 0;

  if (!rowPermutationInverse) {
    if ((actualPosition < numNonzeroElements)
	   && (rowIndexArray[actualPosition] == logicalPosition)) {
	 result = valueArray[actualPosition];
    }
  } else {
    if ((actualPosition < numNonzeroElements)
	   && (rowIndexArray[actualPosition] 
		  == rowPermutationInverse[logicalPosition])) {
	 result = valueArray[actualPosition];
    }
  }

  return result;
}

template <class Element_t, class Alloc>
CSCSparseColumnIterator<Element_t, Alloc>&
CSCSparseColumnIterator<Element_t, Alloc>::
operator++ ()
{
  ++logicalPosition;

  if (!rowPermutationInverse) {
    if ((actualPosition < numNonzeroElements)
	   && (rowIndexArray[actualPosition] < logicalPosition)) {
	 ++actualPosition;
    }
  } else if (logicalPosition >= numRows) {
	 actualPosition = numNonzeroElements;
  } else {
    actualPosition = (std::lower_bound(rowIndexArray, 
							    rowIndexArray + numNonzeroElements,
							    rowPermutationInverse[logicalPosition])
				  - rowIndexArray);
  }

  return *this;
}


template <class Element_t, class Alloc>
CSCSparseColumnIterator<Element_t, Alloc>
CSCSparseColumnIterator<Element_t, Alloc>::
operator++ (int post)
{
  CSCSparseColumnIterator result = *this;
  ++(*this);
  return result;
}


template <class Element_t, class Alloc>
CSCSparseColumnIterator<Element_t, Alloc>&
CSCSparseColumnIterator<Element_t, Alloc>::
operator-- ()
{
  --logicalPosition;

  if (!rowPermutationInverse) {
    if ((actualPosition > 0)
	   && (rowIndexArray[actualPosition - 1] >= logicalPosition)) {
	 --actualPosition;
    }
  } else {
    actualPosition = (std::lower_bound(rowIndexArray, 
							    rowIndexArray + numNonzeroElements,
							    rowPermutationInverse[logicalPosition])
				  - rowIndexArray);
  }

  return *this;
}


template <class Element_t, class Alloc>
CSCSparseColumnIterator<Element_t, Alloc>
CSCSparseColumnIterator<Element_t, Alloc>::
operator-- (int post)
{
  CSCSparseColumnIterator result = *this;
  --(*this);
  return result;
}


template <class Element_t, class Alloc>
CSCSparseColumnIterator<Element_t, Alloc>&
CSCSparseColumnIterator<Element_t, Alloc>::
operator+= (int n)
{
  if (n > 0) {
    logicalPosition += n;

    if (!rowPermutationInverse) {
	 actualPosition = (std::lower_bound(rowIndexArray + actualPosition,
								 rowIndexArray + 
								 std::min(actualPosition + n, 
										numNonzeroElements),
								 logicalPosition)
				    - rowIndexArray);
    } else {
	 actualPosition 
	   = (std::lower_bound(rowIndexArray, rowIndexArray + numNonzeroElements,
					   rowPermutationInverse[logicalPosition])
		 - rowIndexArray);
    }
  } else if (n < 0) {
    (*this)-=(-n);
  }
	 
  return *this;
}

template <class Element_t, class Alloc>
CSCSparseColumnIterator<Element_t, Alloc>&
CSCSparseColumnIterator<Element_t, Alloc>::
operator-= (int n)
{
  if (n > 0) {
    logicalPosition -= n;
    
    if (!rowPermutationInverse) {
	 actualPosition = (std::lower_bound(rowIndexArray + 
								 std::max(actualPosition - n, 0), 
								 rowIndexArray + actualPosition,
								 logicalPosition)
				    - rowIndexArray);
    } else {
	 actualPosition 
	   = (std::lower_bound(rowIndexArray, rowIndexArray + numNonzeroElements,
					   rowPermutationInverse[logicalPosition])
		 - rowIndexArray);
    }
  } else if (n < 0) {
    (*this)+=(-n);
  }

  return *this;
}


template <class Element_t, class Alloc>
int 
CSCSparseColumnIterator<Element_t, Alloc>::
operator- (const CSCSparseColumnIterator& x) const
{
  return (logicalPosition - x.logicalPosition);
}


template <class Element_t, class Alloc>
Element_t 
CSCSparseColumnIterator<Element_t, Alloc>::
operator[] (int n) const
{
  CSCSparseColumnIterator resultPosition = *this;
  resultPosition += n;
  return *resultPosition;
}


template <class Element_t, class Alloc>
bool
CSCSparseColumnIterator<Element_t, Alloc>::
operator== (const CSCSparseColumnIterator& x) const
{
  return (logicalPosition == x.logicalPosition);
}


template <class Element_t, class Alloc>
bool
CSCSparseColumnIterator<Element_t, Alloc>::
operator!= (const CSCSparseColumnIterator& x) const
{
  return (logicalPosition != x.logicalPosition);
}

 
template <class Element_t, class Alloc>
bool
CSCSparseColumnIterator<Element_t, Alloc>::
operator< (const CSCSparseColumnIterator& x) const
{
  return (logicalPosition < x.logicalPosition);
}

