/* CSCSparseColumnIterator.h
 * -------------------------
 * Written by Patrick Perry
 * Initial Version on October 18, 2003
 *
 * $Id: CSCSparseColumnIterator.h,v 1.2 2005/10/24 16:20:08 patperry Exp $
 * $Log: CSCSparseColumnIterator.h,v $
 * Revision 1.2  2005/10/24 16:20:08  patperry
 * Hells (literally) of compile fixes for gcc4.0
 *
 * Revision 1.1.1.1  2005/03/18 23:36:09  silva
 * Imported Sources
 *
 * Revision 1.1.1.1  2004/09/25 22:04:08  patperry
 * Imported sources
 *
 * Revision 1.1  2004/01/15 07:13:36  patperry
 * Changed file extensions.
 *
 * Revision 1.6  2003/12/12 20:13:46  patperry
 * bugfixes and debug code galore
 *
 * Revision 1.5  2003/11/22 06:54:14  patperry
 * lotsa fixes.
 *
 * Revision 1.4  2003/10/25 00:49:32  patperry
 * Added RowIterators.  Make RowColumnIterator have virtual functions.
 *
 * Revision 1.3  2003/10/24 02:16:48  patperry
 * Add FullMatrix and FullColumnIterator.
 *
 * Revision 1.2  2003/10/22 10:52:36  patperry
 * Implemented and tested ColumnIterator for sparse matrices.
 *
 * Revision 1.1  2003/10/22 07:37:18  patperry
 * More work on column iterators.
 *
 *
 */

#ifndef _CSCSPARSECOLUMNITERATOR_H
#define _CSCSPARSECOLUMNITERATOR_H

#include "RowColumnIterator.h"


template <class Element_t, class Alloc>
class CSCSparseColumnIterator;
#include "CSCSparseMatrix.h"


/**
 * Column iterator for CSCSparseMatrices.
 */
template <class Element_t, class Alloc>
class CSCSparseColumnIterator : public RowColumnIterator<Element_t, Alloc>
{
  //! an array of the nonzero values in the column.
  const Element_t * valueArray;

  //! an array of the row indices of the nonzero values.
  const int * rowIndexArray;

  //! the number of nonzero elements in the column
  int numNonzeroElements;

  //! the row permutation
  const int * rowPermutationInverse;

  //! the number of rows in the matrix
  const int numRows;

  //! the index into the valueArray and rowIndexArray for the current position.
  int actualPosition;

protected:

  using RowColumnIterator<Element_t, Alloc>::logicalPosition;

public:

  /**
   * Constructor.  Creates an iterator pointing to nowhere.
   */
  CSCSparseColumnIterator ();

private:
  /**
   * Constructor.  Creates an iterator with the given value and rowIndex
   * arrays, at the given position.  This is a private function.
   *
   * \param valueArray the valueArray for the column.
   * \param rowIndexArray the rowIndexArray for the column.
   * \param numNonzeroElements the number of nonzero elements in
   * the column.
   * \param rowPermutationInverse the row permutation for the column.
   * \param numRows the number of rows in the matrix.
   * \param logicalPosition the logical position of the iterator.
   */
  CSCSparseColumnIterator (const Element_t * valueArray, 
					  const int * rowIndexArray, int numNonzeroElements,
					  const int * rowPermutationInverse, int numRows,
					  int logicalPosition);

public:

  /**
   * Destructor.
   */
  virtual ~CSCSparseColumnIterator ();

  /**
   * De-refernce operator.
   *
   * \returns the value pointed to by the iterator.
   */
  Element_t operator* ();

  /**
   * Pre-increment operator.  Increments the iterator.
   *
   * \returns the iterator at (this + 1).
   */
  CSCSparseColumnIterator& operator++ ();

  /**
   * Post-increment operator.
   *
   * \param post dummy argument.
   * \returns this iterator before incrementing.
   */
  CSCSparseColumnIterator operator++ (int post);

  /**
   * Pre-decrement operator.  Decrements the iterator.
   *
   * \returns the iterator at (this - 1).
   */
  CSCSparseColumnIterator& operator-- ();

  /**
   * Post-decrement operator.
   *
   * \param post dummy argument.
   * \returns this iterator before decrementing.
   */
  CSCSparseColumnIterator operator-- (int post);

  /**
   * Forward-advance operator.  Increments the iterator by n.
   *
   * \param n an integer.
   * \returns this after it has been incremented.
   */
  CSCSparseColumnIterator& operator+= (int n);

  /**
   * Backward-reverse operator.  Decrements the iterator by n.
   *
   * \param n an integer.
   * \returns this after it has been decremented.
   */
  CSCSparseColumnIterator& operator-= (int n);

  /**
   * Distance operator.
   *
   * \param x another iterator.
   * \returns the distance between this and x.
   */
  int operator- (const CSCSparseColumnIterator& x) const;

  /**
   * Index operator.
   *
   * \param n an integer.
   * \returns the value at this[n].
   */
  Element_t operator[] (int n) const;

  /**
   * Equals comparator.
   *
   * \param x iterator to compare this to.
   * \returns (*this) == x.
   */
  bool operator== (const CSCSparseColumnIterator& x) const;

  /**
   * Not-Equals comparator.
   *
   * \param x iterator to compare this to.
   * \returns (*this) != x.
   */
  bool operator!= (const CSCSparseColumnIterator& x) const;

  /**
   * Less-than comparator.
   *
   * \param x iterator to compare this to.
   * \returns (*this) < x.
   */
  bool operator< (const CSCSparseColumnIterator& x) const;

  //! this is a friend of it's corresponding matrix type.
  friend class CSCSparseMatrix<Element_t, Alloc>;
};


/**
 * Iterator addition.
 *
 * \param n an integer.
 * \param i an iterator.
 * \returns the iterator at i + n.
 */
template<class Element_t, class Alloc>
CSCSparseColumnIterator<Element_t, Alloc> 
operator+ (int n, const CSCSparseColumnIterator<Element_t, Alloc>& i)
{
  CSCSparseColumnIterator<Element_t, Alloc> result = i;
  result += n;
  return result;
}


/**
 * Iterator addition.
 *
 * \param i an iterator.
 * \param n an integer.
 * \returns the iterator at n + i.
 */
template<class Element_t, class Alloc>
CSCSparseColumnIterator<Element_t, Alloc> 
operator+ (const CSCSparseColumnIterator<Element_t, Alloc>& i, int n)
{
  CSCSparseColumnIterator<Element_t, Alloc> result = i;
  result += n;
  return result;
}


/**
 * Iterator subtraction.
 *
 * \param i an iterator.
 * \param n an integer.
 * \returns the iterator at i - n.
 */
template<class Element_t, class Alloc>
CSCSparseColumnIterator<Element_t, Alloc> 
operator- (const CSCSparseColumnIterator<Element_t, Alloc>& i, int n)
{
  CSCSparseColumnIterator<Element_t, Alloc> result = i;
  result -= n;
  return result;
}


// get function definitions.
#include "CSCSparseColumnIterator.cc"


#endif // _CSCSPARSECOLUMNITERATOR_H
