/* CSCSparseMatrix.cc
 * ------------------
 * Implements CSCSparseMatrix.h
 *
 * Written by Patrick Perry
 * Original Version on October 16, 2002
 *
 * $Id: CSCSparseMatrix.cc,v 1.2 2005/10/24 16:20:08 patperry Exp $
 * $Log: CSCSparseMatrix.cc,v $
 * Revision 1.2  2005/10/24 16:20:08  patperry
 * Hells (literally) of compile fixes for gcc4.0
 *
 * Revision 1.1.1.1  2005/03/18 23:36:10  silva
 * Imported Sources
 *
 * Revision 1.1.1.1  2004/09/25 22:04:08  patperry
 * Imported sources
 *
 * Revision 1.13  2004/02/09 06:31:14  patperry
 * Replaced C arrays with std::vectors.
 *
 * Revision 1.12  2004/01/15 07:13:36  patperry
 * Changed file extensions.
 *
 * Revision 1.11  2003/12/13 00:09:18  patperry
 * bugfixes
 *
 * Revision 1.10  2003/12/12 20:13:46  patperry
 * bugfixes and debug code galore
 *
 * Revision 1.9  2003/12/04 14:25:54  patperry
 * added data pointer accessors
 *
 * Revision 1.8  2003/11/22 06:54:14  patperry
 * lotsa fixes.
 *
 * Revision 1.7  2003/11/15 04:46:06  patperry
 * fixed bug in alternate constructor.  changed size_t to int
 *
 * Revision 1.6  2003/11/14 10:18:10  patperry
 * Fixed variable name bug.
 *
 * Revision 1.5  2003/10/24 02:16:48  patperry
 * Add FullMatrix and FullColumnIterator.
 *
 * Revision 1.4  2003/10/22 10:52:36  patperry
 * Implemented and tested ColumnIterator for sparse matrices.
 *
 * Revision 1.3  2003/10/22 07:37:18  patperry
 * More work on column iterators.
 *
 * Revision 1.2  2003/10/17 09:51:10  patperry
 * Updated CRCovering and CSCSparseMatrix, (probably) breaking GeomSparse
 * in the process.
 *
 *
 */

#include <algorithm>
#include <iterator>
#include <map>
#include <memory>
#include <utility>
#include <vector>

template <class Element_t, class Alloc>
template <class InputIterator1, class InputIterator2, class InputIterator3>
CSCSparseMatrix<Element_t, Alloc>::
CSCSparseMatrix(InputIterator1 rowIndices, InputIterator2 columnIndices,
			 InputIterator3 values, int numNonzeroElements,
			 int numRows, int numColumns)
  : Matrix<Element_t, CSCSparseColumnIterator<Element_t, Alloc>, Alloc>
      (numRows, numColumns),
    maxNonzeroElements(numNonzeroElements), numNonzeroElements(0),
    valueArray(maxNonzeroElements),
    rowIndexArray(maxNonzeroElements),
    columnPointerArray(numColumns + 1)
{
  std::map<int, Element_t> columns[numColumns];

  // sort the elements into rows and columns
  for (int i = 0; i < numNonzeroElements; ++i) {
    Element_t value = *values;

    if (value != 0) {
	 int row = *rowIndices;
	 int column = *columnIndices;
	 
	 typename std::map<int, Element_t>::iterator position 
	   = columns[column].lower_bound(row);
	 
	 if ((position != columns[column].end())
		&& (position->first == row)) {
	   position->second += value;
	   
	   if (position->second == 0) {
		columns[column].erase(position);
	   }
	 } else {
	   columns[column].insert(position, 
						 std::pair<int, Element_t>(row, value));
	 }
    }

    ++rowIndices;
    ++columnIndices;
    ++values;
  }

  // copy the elements over, one column at a time
  numNonzeroElements = 0;
  for (int column = 0; column < numColumns; ++column) {
    columnPointerArray[column] = numNonzeroElements;
    
    for (typename std::map<int, Element_t>::iterator it 
		 = columns[column].begin(); it != columns[column].end(); ++it) {
	 rowIndexArray[numNonzeroElements] = it->first;
	 valueArray[numNonzeroElements] = it->second;
	 ++numNonzeroElements;
    }
  }

  columnPointerArray[numColumns] = numNonzeroElements;
}


template <class Element_t, class Alloc>
CSCSparseMatrix<Element_t, Alloc>::
CSCSparseMatrix (int maxNonzeroElements, const Element_t * dataPointer, 
			  const int * rowIndexPointer, 
			  const int * columnOffsetPointer,
			  int numRows, int numColumns)
  : Matrix<Element_t, CSCSparseColumnIterator<Element_t, Alloc>, Alloc>
      (numRows, numColumns),
    maxNonzeroElements(maxNonzeroElements), numNonzeroElements(0),
    valueArray(maxNonzeroElements),
    rowIndexArray(maxNonzeroElements),
    columnPointerArray(numColumns + 1)
{
  //std::cerr << "inputPr: [ ";
  //std::copy(dataPointer, dataPointer + maxNonzeroElements, 
  //		  std::ostream_iterator<int>(std::cerr, " "));
  //std::cerr << "]\n";

  //std::cerr << "inputIr: [ ";
  //std::copy(rowIndexPointer, rowIndexPointer + maxNonzeroElements,
  //		  std::ostream_iterator<int>(std::cerr, " "));
  //std::cerr << "]\n";

  //std::cerr << "inputJc: [ ";
  //std::copy(columnOffsetPointer, columnOffsetPointer + numColumns + 1,
  //		  std::ostream_iterator<int>(std::cerr, " "));
  //std::cerr << "]\n";

  int nextColumn = 1;

  columnPointerArray[0] = 0;
  for (int i = 0; i < maxNonzeroElements; ++i) {
    while (i == columnOffsetPointer[nextColumn]) {
	 columnPointerArray[nextColumn++] = numNonzeroElements;
    }

    if (dataPointer[i] != 0) {
	 valueArray[numNonzeroElements] = dataPointer[i];
	 rowIndexArray[numNonzeroElements] = rowIndexPointer[i];
	 numNonzeroElements++;
    }
  }


  for (int i = nextColumn; i <= numColumns; ++i) {
    columnPointerArray[i] = numNonzeroElements;
  }


  //std::cerr << "outputPr: [ ";
  //std::copy(valueArray, valueArray + numNonzeroElements,
  //		  std::ostream_iterator<int>(std::cerr, " "));
  //std::cerr << "]\n";

  //std::cerr << "outputIr: [ ";
  //std::copy(rowIndexArray, rowIndexArray + numNonzeroElements,
  //		  std::ostream_iterator<int>(std::cerr, " "));
  //std::cerr << "]\n";

  //std::cerr << "outputJc: [ ";
  //std::copy(columnPointerArray, columnPointerArray + numColumns + 1,
  //		  std::ostream_iterator<int>(std::cerr, " "));
  //std::cerr << "]\n";

}


template <class Element_t, class Alloc>
CSCSparseMatrix<Element_t, Alloc>::
CSCSparseMatrix(int maxNonzeroElements, int numRows, int numColumns)
  : Matrix<Element_t, CSCSparseColumnIterator<Element_t, Alloc>, Alloc>
      (numRows, numColumns),
    maxNonzeroElements(maxNonzeroElements), numNonzeroElements(0),
    valueArray(maxNonzeroElements),
    rowIndexArray(maxNonzeroElements),
    columnPointerArray(numColumns + 1)
{
  // initially, there is no data, so set columnPointerArray appropriately
  std::uninitialized_fill(columnPointerArray.begin(), 
					 columnPointerArray.begin() + numColumns + 1, 0);
}


template<class Element_t, class Alloc>
CSCSparseMatrix<Element_t, Alloc>::
~CSCSparseMatrix ()
{
}


template<class Element_t, class Alloc>
int
CSCSparseMatrix<Element_t, Alloc>::
getNumRows () const
{
  return numRows;
}


template<class Element_t, class Alloc>
int
CSCSparseMatrix<Element_t, Alloc>::
getNumColumns () const
{
  return numColumns;
}


template<class Element_t, class Alloc>
int
CSCSparseMatrix<Element_t, Alloc>::
getNumNonzeroElements () const
{
  return numNonzeroElements;
}


template<class Element_t, class Alloc>
const Element_t * 
CSCSparseMatrix<Element_t, Alloc>::
getDataPointer () const
{
  return &(valueArray[0]);
}


template<class Element_t, class Alloc>
const int * 
CSCSparseMatrix<Element_t, Alloc>::
getRowIndexPointer () const
{
  return &(rowIndexArray[0]);
}


template<class Element_t, class Alloc>
const int * 
CSCSparseMatrix<Element_t, Alloc>::
getColumnOffsetPointer () const
{
  return &(columnPointerArray[0]);
}


template<class Element_t, class Alloc>
Element_t
CSCSparseMatrix<Element_t, Alloc>::
getElement (int i, int j) const
{
  if (rowPermutationInverse) {
    i = rowPermutationInverse[i];
  }
  if (columnPermutationInverse) {
    j = columnPermutationInverse[j];
  }

  Element_t result = 0;
  typename std::vector<int, typename Alloc::template rebind<int>::other>
    ::const_iterator position 
    = std::lower_bound(rowIndexArray.begin() + columnPointerArray[j],
				   rowIndexArray.begin() + columnPointerArray[j + 1], i);

  if ((position < rowIndexArray.begin() + columnPointerArray[j + 1])
	 && (*position == i)) {
    result = valueArray[position - rowIndexArray.begin()];
  } 

  return result;
}


template<class Element_t, class Alloc>
typename CSCSparseMatrix<Element_t, Alloc>::ColumnIterator 
CSCSparseMatrix<Element_t, Alloc>::
getColumnBegin (int column) const
{
  if (columnPermutationInverse) {
    column = columnPermutationInverse[column];
  }

  int columnBegin = columnPointerArray[column];
  int columnEnd = columnPointerArray[column + 1];

  return ColumnIterator(getDataPointer() + columnBegin,
				    getRowIndexPointer() + columnBegin,
				    columnEnd - columnBegin, 
				    rowPermutationInverse, numRows, 0);
}


template<class Element_t, class Alloc>
typename CSCSparseMatrix<Element_t, Alloc>::ColumnIterator 
CSCSparseMatrix<Element_t, Alloc>::
getColumnEnd (int column) const
{
  if (columnPermutationInverse) {
    column = columnPermutationInverse[column];
  }

  int columnBegin = columnPointerArray[column];
  int columnEnd = columnPointerArray[column + 1];

  return ColumnIterator(getDataPointer() + columnBegin,
				    getRowIndexPointer() + columnBegin,
				    columnEnd - columnBegin, 
				    rowPermutationInverse, numRows, numRows);
}


template<class Element_t, class Alloc>
template <class InputIterator>
void 
CSCSparseMatrix<Element_t, Alloc>::
fillColumn (int column, InputIterator values)
{
  typename std::vector<std::pair<int, Element_t> > nonzeroValues;

  // find the nonzero values
  for (int row = 0; row < numRows; ++row) {
    Element_t value = *values;

    if (value != 0) {
	 nonzeroValues.push_back(std::pair<int, Element_t>(row, value));
    }

    ++values;
  }

  int columnSize = nonzeroValues.size();
  int oldColumnSize = (columnPointerArray[column + 1] 
				   - columnPointerArray[column]);
  int numNewElements = columnSize - oldColumnSize;

  // make sure the column fits exactly in memory
  if (numNewElements > 0) {
    valueArray.insert(valueArray.begin() + columnPointerArray[column + 1],
				  numNewElements, static_cast<Element_t>(0));
    rowIndexArray.insert(rowIndexArray.begin() 
					+ columnPointerArray[column + 1], numNewElements, 0);
  } else if (numNewElements < 0) {
    valueArray.erase(valueArray.begin() + columnPointerArray[column + 1]
				 + numNewElements,
				 valueArray.begin() + columnPointerArray[column + 1]);
    rowIndexArray.erase(rowIndexArray.begin() + columnPointerArray[column + 1]
				    + numNewElements,
				    rowIndexArray.begin() 
				    + columnPointerArray[column + 1]);
  }

  // update numNonzeroValues
  numNonzeroElements += numNewElements;

  // update columnPointerArray
  for (int j = column + 1; j <= numColumns; ++j) {
    columnPointerArray[j] += numNewElements;
  }

  // copy over the new column data
  int offset = columnPointerArray[column];
  for (typename std::vector<std::pair<int, Element_t> >::iterator 
	    it = nonzeroValues.begin(); it != nonzeroValues.end(); ++it) {
    rowIndexArray[offset] = it->first;
    valueArray[offset] = it->second;

    ++offset;
  }
}


template<class Element_t, class Alloc>
std::ostream& 
CSCSparseMatrix<Element_t, Alloc>::
put (std::ostream& s) const
{
  // print out each column
  for (int column = 0; column < numColumns; ++column) {
    for (int i = columnPointerArray[column]; 
	    i < columnPointerArray[column + 1]; ++i) {
	 s << "(" << rowIndexArray[i] << "," << column << ")\t" 
	   << valueArray[i] << "\n";
    }
  }

  if (rowPermutationInverse) {
    s << "P inverse: [ ";
    std::copy(rowPermutationInverse, rowPermutationInverse + numRows,
		    std::ostream_iterator<int>(s, " "));
    s << "]\n";
  }

  if (columnPermutationInverse) {
    s << "Q inverse: [ ";
    std::copy(columnPermutationInverse, columnPermutationInverse + numColumns,
		    std::ostream_iterator<int>(s, " "));
    s << "]\n";
  }

  return s;
}


template<class Element_t, class Alloc>
std::ostream& 
operator<< (std::ostream& s, 
		  const CSCSparseMatrix<Element_t, Alloc>& matrix)
{
  return matrix.put(s);
}

