/* CSCSparseMatrix.h
 * -----------------
 * Written by Patrick Perry
 * Original Version on October 16, 2002
 *
 * $Id: CSCSparseMatrix.h,v 1.3 2005/11/02 01:59:52 patperry Exp $
 * $Log: CSCSparseMatrix.h,v $
 * Revision 1.3  2005/11/02 01:59:52  patperry
 * The last (?) gcc4.0 fixes\!\!\!
 *
 * Revision 1.2  2005/10/24 16:20:08  patperry
 * Hells (literally) of compile fixes for gcc4.0
 *
 * Revision 1.1.1.1  2005/03/18 23:36:11  silva
 * Imported Sources
 *
 * Revision 1.1.1.1  2004/09/25 22:04:08  patperry
 * Imported sources
 *
 * Revision 1.2  2004/02/09 06:31:14  patperry
 * Replaced C arrays with std::vectors.
 *
 * Revision 1.1  2004/01/15 07:13:36  patperry
 * Changed file extensions.
 *
 * Revision 1.8  2003/12/12 20:13:46  patperry
 * bugfixes and debug code galore
 *
 * Revision 1.7  2003/12/04 14:25:54  patperry
 * added data pointer accessors
 *
 * Revision 1.6  2003/11/15 04:46:06  patperry
 * fixed bug in alternate constructor.  changed size_t to int
 *
 * Revision 1.5  2003/10/24 21:15:51  patperry
 * Documentation fixes.
 *
 * Revision 1.4  2003/10/24 02:16:48  patperry
 * Add FullMatrix and FullColumnIterator.
 *
 * Revision 1.3  2003/10/22 07:37:18  patperry
 * More work on column iterators.
 *
 * Revision 1.2  2003/10/17 09:51:10  patperry
 * Updated CRCovering and CSCSparseMatrix, (probably) breaking GeomSparse
 * in the process.
 *
 *
 */

#ifndef _CSCSPARSEMATRIX_H
#define _CSCSPARSEMATRIX_H


#include <cstdlib>
#include <memory>
#include <iostream>
#include <vector>
#include "Matrix.h"

template <class Element_t, class Alloc>
class CSCSparseMatrix;
#include "CSCSparseColumnIterator.h"


/**
 * Interface for a "Compressed Sparse Column" matrix defined
 * in Carney, S. et al. "A Revised Proposal for a Sparse BLAS
 * Toolkit".  This interface is also inspired by the MATLAB
 * notion of how a sparse matrix should work.
 */
template<class Element_t, class Alloc = std::allocator<Element_t> >
class CSCSparseMatrix 
  : public Matrix<Element_t, CSCSparseColumnIterator<Element_t, Alloc>, Alloc> 
{
public:

  //! column iterator type
  typedef CSCSparseColumnIterator<Element_t, Alloc> ColumnIterator;

private:

  //! maximum storable nonzero elements.
  int maxNonzeroElements;

  //! actual number of nonzero elements.
  int numNonzeroElements;

  //! the actual nontrivial elements
  std::vector<Element_t, typename Alloc::template rebind<Element_t>::other> 
    valueArray;

  //! the row indices of the nontrivial elements
  std::vector<int, typename Alloc::template rebind<int>::other> rowIndexArray;

  //! offsets to the begin and end of each column
  std::vector<int, typename Alloc::template rebind<int>::other> columnPointerArray;

protected:
  
  using Matrix<Element_t, CSCSparseColumnIterator<Element_t, Alloc>, Alloc>
    ::numRows;
  using Matrix<Element_t, CSCSparseColumnIterator<Element_t, Alloc>, Alloc>
    ::numColumns;
  using Matrix<Element_t, CSCSparseColumnIterator<Element_t, Alloc>, Alloc>
    ::rowPermutationInverse;
  using Matrix<Element_t, CSCSparseColumnIterator<Element_t, Alloc>, Alloc>
    ::columnPermutationInverse;

public:

  /**
   * Constructor.  Creates a new matrix by assuming there are 
   * numNozeroElements nonzero elements.  The element with at
   * row and columns (rowIndices(i), columnIndices(i)) is to have
   * a value of values(i).  If the same (row, column) position is
   * given for more than one value, the values are added.
   *
   * \param rowIndices (int) itetator to the row indices of the values.
   * \param columnIndices (int) iterator to the column indices of the 
   * values.
   * \param values (Element_t) iterator to the data values.
   * \param numNonzeroElements the number of nonzero elements.
   * \param numRows the number of rows in the matrix.
   * \param numColumns the number of columns in the matrix.
   */
  template <class InputIterator1, class InputIterator2, class InputIterator3>
  CSCSparseMatrix(InputIterator1 rowIndices, InputIterator2 columnIndices,
			   InputIterator3 values, int numNonzeroElements,
			   int numRows, int numColumns);

  /**
   * Constructor.  Create new CSCSparse matrix using the given internal
   * representatin.  This constructor is very dangerous, as the soundness
   * of the resulting object depends on good parameters being passed in.
   *
   * \param maxNonzeroElements the number of elements to allocate 
   *        space for.
   * \param dataPointer an array of length maxNonzeroElements containing
   *        the nonzero data values.
   * \param rowIndexPointer an array of length maxNonzeoElements containing
   *        the row indeces.
   * \param columnOffsetPointer an array of length numColumns + 1 containing
   *        offsets for the begin and end of every column.
   * \param numRows the number of rows in the matrix.
   * \param numColumns the number of columns in the matrix.
   */
  CSCSparseMatrix (int maxNonzeroElements, const Element_t * dataPointer, 
			    const int * rowIndexPointer, 
			    const int * columnOffsetPointer,
			    int numRows, int numColumns);

  /**
   * Constructor.  Create new CSCSparse matrix of the given dimensions
   * with enough space allocated to store a certain number of elements.
   * After constructing a matrix with this method, the client should use
   * the fillColumn method to input the data values.
   *
   * \param maxNonzeroElements the number of elements to allocate 
   * space for.
   * \param numRows the number of rows in the matrix.
   * \param numColumns the number of columns in the matrix.
   */
  CSCSparseMatrix (int maxNonzeroElements, int numRows, int numColumns);
			    
  /**
   *	Destructor.  Frees resources associated with this object.
   */
  virtual ~CSCSparseMatrix ();
  
  /** 
   * Returns the number of rows in the matrix.
   *
   * \returns the number of rows.
   */
  int getNumRows () const;

  /** 
   * Returns the number of columns in the matrix.
   *
   * \returns the number of columns.
   */
  int getNumColumns () const;

  /**
   * Returns number of nonzero elements in the matrix.
   *
   * \returns the number of nonzero elements.
   */
  int getNumNonzeroElements () const;

  /**
   * Get the pointer to the data.
   *
   * \returns the data pointer (known as "pr" in MATLAB).
   */
  const Element_t * getDataPointer () const;

  /**
   * Get the pointer to the row index array.
   * 
   * \returns the row index pointer (known as "ir" in MATLAB).
   */
  const int * getRowIndexPointer () const;

  /**
   * Get the column offset array.
   *
   * \returns the column offset array (known as "jc" in MATLAB).
   */
  const int * getColumnOffsetPointer () const;

  /**
   * Returns the value at the given position.
   * 
   * \param i the row index.
   * \param j the column index.
   * \returns the value at (i, j).
   */
  Element_t getElement (int i, int j) const;

  /**
   * Get a column iterator to the begin of a column.  Note that the
   * iterator is read only.
   *
   * \param column the index of the column to get.
   * \returns a ColumnIterator to the begin of the requested column.
   */
  ColumnIterator getColumnBegin (int column) const;

  /**
   * Get a column iterator to the end of a column.
   *
   * \param column the index of the column to get.
   * \returns a ColumnIterator to the end of the requested column.
   */
  ColumnIterator getColumnEnd (int column) const;

  /**
   * Fills a column with the given values.  If uncareful, this function
   * can be very costly.  The "correct" way to call it is to construct
   * an empty matrix with the correct number of elements and then to
   * successively call fillColumn on each column, starting with column 0.
   *
   * \param column the column to fill.
   * \param values an (Element) iterator to numRows values.
   * \returns void.
   */
  template <class InputIterator>
  void fillColumn (int column, InputIterator values);

  /**
   * Writes the matrix to an output stream.
   *
   * \param s the output stream.
   * \returns the output stream.
   */
  virtual std::ostream& put (std::ostream& s) const;
};


/**
 * Overloaded output stream operator for CSCSparseMatrices.
 *
 * \param s an output stream.
 * \param matrix a sparse matrix.
 * \returns the output stream.
 */
template<class Element_t, class Alloc>
std::ostream& 
operator<< (std::ostream& s, 
		  const CSCSparseMatrix<Element_t, Alloc>& matrix);



// get function definitions
#include "CSCSparseMatrix.cc"

#endif // _CSCSPARSEMATRIX_H
