/* DiagonalBlock.h
 * ---------------
 * Written by Patrick Perry
 * Original Version on October 18, 2003
 *
 * $Id: DiagonalBlock.h,v 1.1.1.1 2005/03/18 23:36:12 silva Exp $
 * $Log: DiagonalBlock.h,v $
 * Revision 1.1.1.1  2005/03/18 23:36:12  silva
 * Imported Sources
 *
 * Revision 1.1.1.1  2004/09/25 22:04:08  patperry
 * Imported sources
 *
 * Revision 1.1  2004/01/15 07:13:36  patperry
 * Changed file extensions.
 *
 * Revision 1.6  2003/11/22 06:54:14  patperry
 * lotsa fixes.
 *
 * Revision 1.5  2003/10/28 04:43:51  patperry
 * Preliminary version of SelectionMaskColumnIterator.
 * Copy constructors for all full matrix types.
 *
 * Revision 1.4  2003/10/26 00:24:51  patperry
 * Preliminary Version of OffDiagonalBlock.  Deleted unnecessary files.
 *
 * Revision 1.3  2003/10/24 21:15:51  patperry
 * Documentation fixes.
 *
 * Revision 1.2  2003/10/24 08:56:06  patperry
 * Filled in diagonal block and got rid of LUMatrixBlock
 *
 * Revision 1.1  2003/10/18 22:52:15  patperry
 * Added DiagonalBlock.
 * More breaking compatibility with GeomSparseMatrix and
 * MatrixSelectionMask.
 *
 *
 */

#ifndef _DIAGONALBLOCK_H
#define _DIAGONALBLOCK_H

#include <memory>
#include <vector>
#include "LUFullMatrix.h"

template <class Element_t, class Alloc>
class DiagonalBlock;
#include "OffDiagonalBlock.h"


/**
 * Class for representing a diagonal block in a geometrically sparse
 * matrix.  The class has pointers to the blocks above it (cofaces) and
 * the blocks to the right of it (faces).
 */
template <class Element_t, class Alloc = std::allocator<void> >
class DiagonalBlock : public LUFullMatrix<Element_t, Alloc>
{
public:

  //! the off-diagonal block type corresponding to this class.
  typedef OffDiagonalBlock<Element_t, Alloc> OffDiagonalBlock;

private:

  //! each LUMatrixBlock has a unique ID.
  const int id;  

  //! the cofaces of the block.  A coface is in the same column and above.
  std::vector<OffDiagonalBlock *> cofaces;

  //! the faces of the block.  A face is in the same row and to the right.
  std::vector<OffDiagonalBlock *> faces;

public:

  /**
   * Constructor.  Creates an empty DiagonalBlock with the given id and
   * dimensions.
   *
   * \param id the id to assign the block.
   * \param numRows the number of rows in the block.
   * \param numColumns the number of columns in the block.
   */
  DiagonalBlock (int id, int numRows, int numColumns);

  /**
   * Constructor.  Creates an empty DiagonalBlock with the given id and
   * matrix data.
   *
   * \param id the id to assign the block.
   * \param matrix a matrix to copy.
   */
  template <class ColumnIterator_t, class Alloc1>
  DiagonalBlock (int id, 
			  const Matrix<Element_t, ColumnIterator_t, Alloc1>& matrix);

  /**
   * Destructor.
   */
  virtual ~DiagonalBlock ();

  /**
   * Get the block ID.
   *
   * \returns the ID.
   */
  int getID () const;

  /**
   * Adds the coface to the cofaces collection.
   * 
   * \param coface the coface to be added.
   * \returns void.
   */
  void addCoface (OffDiagonalBlock * coface);
			   
  /**
   * Adds the given range to the collection of cofaces.
   *
   * \param begin an InputIterator to a pointer to the begin of the
   * coface collection.
   * \param end an InputIterator to a pointer to the end of the coface
   * collection.
   * \returns void.
   */
  template <class InputIterator>
  void addCofaces (InputIterator begin, InputIterator end);

  /**
   * Adds the given face to the faces collection.
   *
   * \param face the face to be added.
   * \returns void.
   */
  void addFace (OffDiagonalBlock * face);

  /**
   * Adds the given range to the collection of faces.
   *
   * \param begin an InputIterator to a pointer to the begin of the
   * face collection.
   * \param end an InputIterator to a pointer to the end of the face
   * collection.
   * \returns void.
   */
  template <class InputIterator>
  void addFaces (InputIterator begin, InputIterator end);

  /**
   * Permute the rows of the block according the given permutation,
   * and apply the permutation to all faces of the block.
   *        
   * \param permutation an integer of length numRows.  Row i
   * is sent to permutation[i] under the represented permutation.
   * \returns void.
   */
  void permuteRows (const int * permutation);

  /**
   * Swap two rows in the block, and swap the same rows in all faces.
   *
   * \param i the index of the first row.
   * \param j the index of the second row.
   * \returns void.
   */
  void swapRows (int i, int j);

  /**
   * Permute the columns of the block according the given permutation,
   * and apply the permutation to all cofaces of the block.
   *
   * \param permutation an integer of length numColumns.  Column i
   * is sent to permutation[i] under the represented permutation.
   * \returns void.
   */
  void permuteColumns (const int * permutation);

  /**
   * Swap two columns in the block, and swap the same columns in all cofaces.
   *
   * \param i the index of the first column.
   * \param j the index of the second column.
   * \returns void.
   */
  void swapColumns (int i, int j);

protected:

  /**
   * Assuming there is a pivot in the given row, add a scalar multiple of
   * that row to another row.  This also performs the given operation on
   * all faces of the block.
   *
   * \param scalar the scalar to multiply the pivot row by.
   * \param pivot the index of the pivot row.
   * \param row the index of the row to add the pivot row to.
   * \returns void.
   */
  void addScalarTimesPivotToRow (Element_t scalar, int pivot, int row);

};


// get function definitions
#include "DiagonalBlock.cc"


#endif // _DIAGONALBLOCK_H
