/* FullColumnIterator.h
 * --------------------
 * Written by Patrick Perry
 * Original Version on October 23, 2003
 *
 * $Id: FullColumnIterator.h,v 1.2 2005/10/24 16:20:08 patperry Exp $
 * $Log: FullColumnIterator.h,v $
 * Revision 1.2  2005/10/24 16:20:08  patperry
 * Hells (literally) of compile fixes for gcc4.0
 *
 * Revision 1.1.1.1  2005/03/18 23:36:14  silva
 * Imported Sources
 *
 * Revision 1.1.1.1  2004/09/25 22:04:08  patperry
 * Imported sources
 *
 * Revision 1.1  2004/01/15 07:13:36  patperry
 * Changed file extensions.
 *
 * Revision 1.3  2003/11/22 06:54:14  patperry
 * lotsa fixes.
 *
 * Revision 1.2  2003/10/25 00:49:32  patperry
 * Added RowIterators.  Make RowColumnIterator have virtual functions.
 *
 * Revision 1.1  2003/10/24 02:16:48  patperry
 * Add FullMatrix and FullColumnIterator.
 *
 *
 */

#ifndef _FULLCOLUMNITERATOR_H
#define _FULLCOLUMNITERATOR_H

#include "RowColumnIterator.h"


template <class Element_t, class Alloc>
class FullColumnIterator;
#include "FullMatrix.h"


/**
 * Column iterator for FullMatrices.
 */
template <class Element_t, class Alloc>
class FullColumnIterator : public RowColumnIterator<Element_t, Alloc>
{
  //! an array of the column data
  const Element_t * data;

  //! the row permutation
  const int * rowPermutationInverse;

protected:

  using RowColumnIterator<Element_t, Alloc>::logicalPosition;

public:

  /**
   * Constructor.  Creates an iterator pointing to nowhere.
   */
  FullColumnIterator ();

private:
  /**
   * Constructor.  Creates an iterator with the given data and 
   * row permutation, pointing to the given position.  This is a private 
   * function.
   * 
   * \param data the column data, stored in an array.
   * \param rowPermutationInverse the row permutation for the column.
   * \param logicalPosition the logical position of the iterator.
   */
  FullColumnIterator (const Element_t * data,
				  const int * rowPermutationInverse,
				  int logicalPosition);

public:

  /**
   * Destructor.
   */
  ~FullColumnIterator ();

  /**
   * De-refernce operator.
   *
   * \returns the value pointed to by the iterator.
   */
  Element_t operator* ();

  /**
   * Pre-increment operator.  Increments the iterator.
   *
   * \returns the iterator at (this + 1).
   */
  FullColumnIterator& operator++ ();

  /**
   * Post-increment operator.
   *
   * \param post dummy argument.
   * \returns this iterator before incrementing.
   */
  FullColumnIterator operator++ (int post);

  /**
   * Pre-decrement operator.  Decrements the iterator.
   *
   * \returns the iterator at (this - 1).
   */
  FullColumnIterator& operator-- ();

  /**
   * Post-decrement operator.
   *
   * \param post dummy argument.
   * \returns this iterator before decrementing.
   */
  FullColumnIterator operator-- (int post);

  /**
   * Forward-advance operator.  Increments the iterator by n.
   *
   * \param n an integer.
   * \returns this after it has been incremented.
   */
  FullColumnIterator& operator+= (int n);

  /**
   * Backward-reverse operator.  Decrements the iterator by n.
   *
   * \param n an integer.
   * \returns this after it has been decremented.
   */
  FullColumnIterator& operator-= (int n);

  /**
   * Distance operator.
   *
   * \param x another iterator.
   * \returns the distance between this and x.
   */
  int operator- (const FullColumnIterator& x) const;

  /**
   * Index operator.
   *
   * \param n an integer.
   * \returns the value at this[n].
   */
  Element_t operator[] (int n) const;

  /**
   * Equals comparator.
   *
   * \param x iterator to compare this to.
   * \returns (*this) == x.
   */
  bool operator== (const FullColumnIterator& x) const;

  /**
   * Not-Equals comparator.
   *
   * \param x iterator to compare this to.
   * \returns (*this) != x.
   */
  bool operator!= (const FullColumnIterator& x) const;

  /**
   * Less-than comparator.
   *
   * \param x iterator to compare this to.
   * \returns (*this) < x.
   */
  bool operator< (const FullColumnIterator& x) const;


  //! this is a friend of it's corresponding matrix type.
  friend class FullMatrix<Element_t, Alloc>;
};



/**
 * Iterator addition.
 *
 * \param n an integer.
 * \param i an iterator.
 * \returns the iterator at i + n.
 */
template<class Element_t, class Alloc>
FullColumnIterator<Element_t, Alloc> 
operator+ (int n, const FullColumnIterator<Element_t, Alloc>& i)
{
  FullColumnIterator<Element_t, Alloc> result = i;
  result += n;
  return result;
}


/**
 * Iterator addition.
 *
 * \param i an iterator.
 * \param n an integer.
 * \returns the iterator at n + i.
 */
template<class Element_t, class Alloc>
FullColumnIterator<Element_t, Alloc> 
operator+ (const FullColumnIterator<Element_t, Alloc>& i, int n)
{
  FullColumnIterator<Element_t, Alloc> result = i;
  result += n;
  return result;
}


/**
 * Iterator subtraction.
 *
 * \param i an iterator.
 * \param n an integer.
 * \returns the iterator at i - n.
 */
template<class Element_t, class Alloc>
FullColumnIterator<Element_t, Alloc> 
operator- (const FullColumnIterator<Element_t, Alloc>& i, int n)
{
  FullColumnIterator<Element_t, Alloc> result = i;
  result -= n;
  return result;
}



// get function definitions
#include "FullColumnIterator.cc"


#endif // _FULLCOLUMNITERATOR_H
