/* FullMatrix.cc
 * -------------
 * Implements FullMatrix.h
 *
 * Written by Patrick Perry
 * Original Version on August 13, 2003
 *
 * $Id: FullMatrix.cc,v 1.2 2005/10/24 16:20:08 patperry Exp $
 * $Log: FullMatrix.cc,v $
 * Revision 1.2  2005/10/24 16:20:08  patperry
 * Hells (literally) of compile fixes for gcc4.0
 *
 * Revision 1.1.1.1  2005/03/18 23:36:14  silva
 * Imported Sources
 *
 * Revision 1.1.1.1  2004/09/25 22:04:08  patperry
 * Imported sources
 *
 * Revision 1.14  2004/01/23 04:36:42  patperry
 * Cleaned up MATLAB support code.
 *
 * Revision 1.13  2004/01/15 07:13:36  patperry
 * Changed file extensions.
 *
 * Revision 1.12  2003/12/12 20:13:46  patperry
 * bugfixes and debug code galore
 *
 * Revision 1.11  2003/12/04 14:25:54  patperry
 * added data pointer accessors
 *
 * Revision 1.10  2003/12/02 08:55:57  patperry
 * Added Destructor
 *
 * Revision 1.9  2003/11/30 05:36:29  patperry
 * more bugfixes
 *
 * Revision 1.8  2003/11/30 00:24:43  patperry
 * fixed reduction bugs.  inserted printfs
 *
 * Revision 1.7  2003/11/22 06:54:14  patperry
 * lotsa fixes.
 *
 * Revision 1.6  2003/10/28 04:43:51  patperry
 * Preliminary version of SelectionMaskColumnIterator.
 * Copy constructors for all full matrix types.
 *
 * Revision 1.5  2003/10/25 00:49:32  patperry
 * Added RowIterators.  Make RowColumnIterator have virtual functions.
 *
 * Revision 1.4  2003/10/24 08:16:05  patperry
 * LUFullMatrix works better now.
 *
 * Revision 1.3  2003/10/24 04:46:02  patperry
 * Overhaul of LUFullMatrix.  It may not compile/function yet.
 *
 * Revision 1.2  2003/10/24 02:16:48  patperry
 * Add FullMatrix and FullColumnIterator.
 *
 *
 */

#include <set>
#include <memory>
#include <algorithm>
#include <iostream>


template<class Element_t, class Alloc>
FullMatrix<Element_t, Alloc>::
FullMatrix (int numRows, int numColumns)
  : Matrix<Element_t, FullColumnIterator<Element_t, Alloc>, Alloc>
      (numRows, numColumns),
    data(ElementAlloc.allocate(numRows * numColumns))
{ 
  std::uninitialized_fill(data, data + numRows * numColumns, 0);
}

template <class Element_t, class Alloc>
FullMatrix<Element_t, Alloc>::
FullMatrix (const Element_t * data, int numRows, int numColumns)
	: Matrix<Element_t, FullColumnIterator<Element_t, Alloc>, Alloc>
			(numRows, numColumns),
		data(ElementAlloc.allocate(numRows * numColumns))
{
			std::copy(data, data + numRows * numColumns, this->data);
}

template<class Element_t, class Alloc>
template <class ColumnIterator_t, class Alloc1>
FullMatrix<Element_t, Alloc>::
FullMatrix (const Matrix<Element_t, ColumnIterator_t, Alloc1>& matrix)
  : Matrix<Element_t, FullColumnIterator<Element_t, Alloc>, Alloc>
      (matrix.getNumRows(), matrix.getNumColumns()),
    data(ElementAlloc.allocate(matrix.getNumRows() * matrix.getNumColumns()))
{
  for (int column = 0; column < matrix.getNumColumns(); ++column) {
    ColumnIterator_t begin = matrix.getColumnBegin(column);
    fillColumn(column, begin);
  }

}


template<class Element_t, class Alloc>
FullMatrix<Element_t, Alloc>::
~FullMatrix ()
{ 
  for (int i = 0; i < numRows * numColumns; ++i) {
    ElementAlloc.destroy(data + i);
  }
  
  if (numRows * numColumns) {
    ElementAlloc.deallocate(data, numRows * numColumns);
  }
}


template<class Element_t, class Alloc>
const Element_t * 
FullMatrix<Element_t, Alloc>::
getDataPointer () const
{
  return data;
}


template<class Element_t, class Alloc>
Element_t 
FullMatrix<Element_t, Alloc>::
getElement (int i, int j) const
{
  if (rowPermutationInverse) {
    i = rowPermutationInverse[i];
  } 
  if (columnPermutationInverse) {
    j = columnPermutationInverse[j];
  }

  return data[i + j * numRows];
}


template<class Element_t, class Alloc>
void
FullMatrix<Element_t, Alloc>::
setElement (int i, int j, Element_t value)
{
  if (rowPermutationInverse) {
    i = rowPermutationInverse[i];
  } 
  if (columnPermutationInverse) {
    j = columnPermutationInverse[j];
  }

  data[i + j * numRows] = value;
}


template<class Element_t, class Alloc>
typename FullMatrix<Element_t, Alloc>::RowIterator 
FullMatrix<Element_t, Alloc>::
getRowBegin (int row) const
{
  if (rowPermutationInverse) {
    row = rowPermutationInverse[row];
  }

  return RowIterator(data, row, numRows, columnPermutationInverse, 0);
}


template<class Element_t, class Alloc>
typename FullMatrix<Element_t, Alloc>::RowIterator 
FullMatrix<Element_t, Alloc>::
getRowEnd (int row) const
{
  if (rowPermutationInverse) {
    row = rowPermutationInverse[row];
  }

  return RowIterator(data, row, numRows, columnPermutationInverse, numColumns);
}



template<class Element_t, class Alloc>
typename FullMatrix<Element_t, Alloc>::ColumnIterator 
FullMatrix<Element_t, Alloc>::
getColumnBegin (int column) const
{
  if (columnPermutationInverse) {
    column = columnPermutationInverse[column];
  }

  return ColumnIterator(data + column * numRows, rowPermutationInverse, 0);
}


template<class Element_t, class Alloc>
typename FullMatrix<Element_t, Alloc>::ColumnIterator 
FullMatrix<Element_t, Alloc>::
getColumnEnd (int column) const
{
  if (columnPermutationInverse) {
    column = columnPermutationInverse[column];
  }

  return ColumnIterator(data + column * numRows, rowPermutationInverse, 
				    numRows);
}


template<class Element_t, class Alloc>
template <class InputIterator>
void 
FullMatrix<Element_t, Alloc>::
fillColumn (int column, InputIterator values)
{
  Element_t * columnData = data + column * numRows;

  for (int i = 0; i < numRows; ++i) {
    *columnData = *values;
    ++columnData;
    ++values;
  }
}


template<class Element_t, class Alloc>
std::ostream& 
FullMatrix<Element_t, Alloc>::
put (std::ostream& s) const
{
  for (int i = 0; i < numRows; ++i) {
    for (RowIterator row = getRowBegin(i); row != getRowEnd(i); ++row) {
	 s << *row << "\t";
    }
    s << std::endl;
  }
  s << std::endl;

  return s;
}


template<class Element_t, class Alloc>
void 
FullMatrix<Element_t, Alloc>::
swapRows (int i, int j)
{
  if (i != j) {
    Element_t * columnData = data;
    
    for (int column = 0; column < this->getNumColumns(); ++column) {
	 std::swap(columnData[i], columnData[j]);
	 columnData += this->getNumRows();
    }
  }
}


template<class Element_t, class Alloc>
void 
FullMatrix<Element_t, Alloc>::
swapColumns (int i, int j)
{
  if (i != j) {
    std::swap_ranges(data + i * numRows, data + (i + 1) * numRows,
				 data + j * numRows);
  }
}


template<class Element_t, class Alloc>
void 
FullMatrix<Element_t, Alloc>::
reducePermutations ()
{
  reduceRowPermutation();
  reduceColumnPermutation();
}


template<class Element_t, class Alloc>
void 
FullMatrix<Element_t, Alloc>::
reduceRowPermutation ()
{
  if (rowPermutationInverse) {
    Element_t oldColumnData[numRows];
    Element_t * columnData = data;
    
    for (int j = 0; j < numColumns; ++j) {
	 std::copy(columnData, columnData + numRows, oldColumnData);
	 
	 for (int i = 0; i < numRows; ++i) {
	   columnData[i] = oldColumnData[rowPermutationInverse[i]];
	 }
	 
	 columnData += numRows;
    }
    
    IntAlloc.deallocate(rowPermutationInverse, numRows);
    rowPermutationInverse = NULL;
  }
}


template<class Element_t, class Alloc>
void 
FullMatrix<Element_t, Alloc>::
reduceColumnPermutation ()
{
  if (columnPermutationInverse) {
    Element_t oldData[numRows * numColumns];
    Element_t * columnData = data;
    
    std::copy(data, data + numRows * numColumns, oldData);
    
    for (int j = 0; j < numColumns; ++j) {
	 Element_t * oldColumnData = oldData 
	   + numRows * columnPermutationInverse[j];

	 std::copy(oldColumnData, oldColumnData + numRows, columnData);
	 columnData += numRows;
    }

    IntAlloc.deallocate(columnPermutationInverse, numColumns);
    columnPermutationInverse = NULL;
  }
}


