/* FullMatrix.h
 * ------------
 * Full matrix interface.
 *
 * Written by Patrick Perry
 * Original Version on August 13, 2003
 *
 * $Id: FullMatrix.h,v 1.2 2005/10/24 16:20:08 patperry Exp $
 * $Log: FullMatrix.h,v $
 * Revision 1.2  2005/10/24 16:20:08  patperry
 * Hells (literally) of compile fixes for gcc4.0
 *
 * Revision 1.1.1.1  2005/03/18 23:36:14  silva
 * Imported Sources
 *
 * Revision 1.1.1.1  2004/09/25 22:04:08  patperry
 * Imported sources
 *
 * Revision 1.2  2004/01/23 04:36:42  patperry
 * Cleaned up MATLAB support code.
 *
 * Revision 1.1  2004/01/15 07:13:36  patperry
 * Changed file extensions.
 *
 * Revision 1.7  2003/12/04 14:25:54  patperry
 * added data pointer accessors
 *
 * Revision 1.6  2003/10/28 04:43:51  patperry
 * Preliminary version of SelectionMaskColumnIterator.
 * Copy constructors for all full matrix types.
 *
 * Revision 1.5  2003/10/25 00:49:32  patperry
 * Added RowIterators.  Make RowColumnIterator have virtual functions.
 *
 * Revision 1.4  2003/10/24 08:16:05  patperry
 * LUFullMatrix works better now.
 *
 * Revision 1.3  2003/10/24 04:46:02  patperry
 * Overhaul of LUFullMatrix.  It may not compile/function yet.
 *
 * Revision 1.2  2003/10/24 02:16:48  patperry
 * Add FullMatrix and FullColumnIterator.
 *
 *
 */

#ifndef _FULLMATRIX_H
#define _FULLMATRIX_H

#include <memory>
#include "Matrix.h"

template <class Element_t, class Alloc>
class FullMatrix;
#include "FullRowIterator.h"
#include "FullColumnIterator.h"


/**
 * Class for representing a full matrix.  "Full" means that every
 * element has space allocated for it.  Element access is constant time.
 */
template<class Element_t, class Alloc = std::allocator<void> >
class FullMatrix 
  : public Matrix<Element_t, FullColumnIterator<Element_t, Alloc>, Alloc>
{
public:

  //! row iterator type
  typedef FullRowIterator<Element_t, Alloc> RowIterator;

  //! column iterator type
  typedef FullColumnIterator<Element_t, Alloc> ColumnIterator;

protected:

  /**
   * The data is stored as an array of length numRows * numColumns,
   * where the columns are stored contiguously.  This means that
   * element (i, j) is at data[i + j * numRows]
   */
  Element_t * const data;

  using Matrix<Element_t, FullColumnIterator<Element_t, Alloc>, Alloc>
    ::IntAlloc;
  using Matrix<Element_t, FullColumnIterator<Element_t, Alloc>, Alloc>
    ::ElementAlloc;
  using Matrix<Element_t, FullColumnIterator<Element_t, Alloc>, Alloc>
    ::numRows;
  using Matrix<Element_t, FullColumnIterator<Element_t, Alloc>, Alloc>
    ::numColumns;
  using Matrix<Element_t, FullColumnIterator<Element_t, Alloc>, Alloc>
    ::rowPermutationInverse;
  using Matrix<Element_t, FullColumnIterator<Element_t, Alloc>, Alloc>
    ::columnPermutationInverse;

public:

  /**
   * Constructor.  Constructs a matrix of zeros of the given
   * dimensions.
   *
   * \param numRows the number of rows in the matrix.
   * \param numColumns the number of columns in the matrix.
   */
  FullMatrix (int numRows, int numColumns);

	/**
	 * Constructor.  Constructs of full matrix of the given size using
	 * the passed in data array.  The passed-in data is assumed to store
	 * columns in contiguous order.
	 *
	 * \param data the matrix data.  This is an array of length
	 *        numRows * numColumns, with columns stored contiguously.
   * \param numRows the number of rows in the matrix.
   * \param numColumns the number of columns in the matrix.
	 */
	FullMatrix (const Element_t * data, int numRows, int numColumns);
	
  /**
   * Constructor.  Constructs a matrix out of the data in the
   * passed-in matrix.
   *
   * \param matrix a matrix to copy.
   */
  template <class ColumnIterator_t, class Alloc1>
  FullMatrix (const Matrix<Element_t, ColumnIterator_t, Alloc1>& matrix);

  /**
   * Destructor.
   */
  virtual ~FullMatrix ();

  /** 
   * Returns a pointer to the internal data, stored column-by-column.
   *
   * \returns the data poitner.
   */
  const Element_t * getDataPointer () const;

  /**
   * Returns the element at the given position.
   *
   * \param i the row index of the element.
   * \param j the column index of the element.
   * \returns the element stored at (i, j).
   */
   Element_t getElement (int i, int j) const;

  /**
   * Set the element at the specified position to the given value.
   *
   * \param i the row index of the element.
   * \param j the column index of the element.
   * \param value the value to set the element to.
   * \returns void.
   */
  void setElement (int i, int j, Element_t value);

  /**
   * Get a row iterator to the begin of a row.  This iterator
   * is read-only.
   *
   * \param row the index of the row to get.
   * \returns a RowIterator to the begin of the requested row.
   */
  RowIterator getRowBegin (int row) const;

  /**
   * Get a row iterator to the end of a row.  This iterator
   * is read-only.
   *
   * \param row the index of the row to get.
   * \returns a RowIterator to the end of the requested row.
   */
  RowIterator getRowEnd (int row) const;

  /**
   * Get a column iterator to the begin of a column.  This iterator
   * is read-only.
   *
   * \param column the index of the column to get.
   * \returns a ColumnIterator to the begin of the requested column.
   */
  ColumnIterator getColumnBegin (int column) const;

  /**
   * Get a column iterator to the end of a column.
   *
   * \param column the index of the column to get.
   * \returns a ColumnIterator to the end of the requested column.
   */
  ColumnIterator getColumnEnd (int column) const;

  /**
   * Fills a column with the given values.  
   *
   * \param column the column to fill.
   * \param values an (Element) iterator to numRows values.
   * \returns void.
   */
  template <class InputIterator>
  void fillColumn (int column, InputIterator values);

  /**
   *  Write the matrix to an output stream.
   *
   * \param s an output stream.
   * \returns the output stream.
   */
  virtual std::ostream& put (std::ostream& s) const;

  /**
   * Swap the given rows in the matrix.  Unlike the definition in
   * the class parent, this actually moves the data around.
   *
   * \param i the index of the first row to swap.
   * \param j the index of the second row to swap.
   * \returns void.
   */
  void swapRows (int i, int j);

  /**
   * Swap the given columns in the matrix.  Unlike the definition in
   * the class parent, this actually moves the data around.
   *
   * \param i the index of the first column to swap.
   * \param j the index of the second column to swap.
   * \returns void.
   */
  void swapColumns (int i, int j);

  /**
   * Carry out the logical permutations stored as part of the matrix
   * be rearranging the data.  This invalidates iterators.
   *
   * \returns void.
   */
  void reducePermutations ();

  /**
   * Carry out the logical row permutations stored as part of the matrix
   * be rearranging the data.  This invalidates iterators.
   *
   * \returns void.
   */
  void reduceRowPermutation ();

  /**
   * Carry out the logical column permutations stored as part of the matrix
   * be rearranging the data.  This invalidates iterators.
   *
   * \returns void.
   */
  void reduceColumnPermutation ();

};


// get template function definitions
#include "FullMatrix.cc"

#endif // _FULLMATRIX_H
