/* FullRowIterator.h
 * -----------------
 * Written by Patrick Perry
 * Original Version on October 24, 2003
 *
 * $Id: FullRowIterator.h,v 1.2 2005/10/24 16:20:08 patperry Exp $
 * $Log: FullRowIterator.h,v $
 * Revision 1.2  2005/10/24 16:20:08  patperry
 * Hells (literally) of compile fixes for gcc4.0
 *
 * Revision 1.1.1.1  2005/03/18 23:36:14  silva
 * Imported Sources
 *
 * Revision 1.1.1.1  2004/09/25 22:04:08  patperry
 * Imported sources
 *
 * Revision 1.1  2004/01/15 07:13:36  patperry
 * Changed file extensions.
 *
 * Revision 1.1  2003/10/25 00:49:32  patperry
 * Added RowIterators.  Make RowColumnIterator have virtual functions.
 *
 *
 */

#ifndef _FULLROWITERATOR_H
#define _FULLROWITERATOR_H

#include "RowColumnIterator.h"


template <class Element_t, class Alloc>
class FullRowIterator;
#include "FullMatrix.h"


/**
 * Column iterator for FullMatrices.
 */
template <class Element_t, class Alloc>
class FullRowIterator : public RowColumnIterator<Element_t, Alloc>
{
  //! an array of the matrix data (not just the row)
  const Element_t * data;

  //! the row index of this iterator
  const int row;

  //! the number of rows in the matrix
  const int numRows;

  //! the row permutation
  const int * columnPermutationInverse;

  //! the actual pointer into the data array
  int actualPosition;

protected:

  using RowColumnIterator<Element_t, Alloc>::logicalPosition;

public:

  /**
   * Constructor.  Creates an iterator pointing to nowhere.
   */
  FullRowIterator ();

private:
  /**
   * Constructor.  Creates an iterator with the given data and 
   * row permutation, pointing to the given position.  This is a private 
   * function.
   * 
   * \param data the column data, stored in an array.
   * \param row the row index of this iterator.
   * \param numRows the number of rows in the matrix.
   * \param columnPermutationInverse the column permutation for the row.
   * \param logicalPosition the logical position of the iterator.
   */
  FullRowIterator (const Element_t * data, int row, int numRows,
			    const int * columnPermutationInverse,
			    int logicalPosition);

public:

  /**
   * Destructor.
   */
  ~FullRowIterator ();

  /**
   * De-refernce operator.
   *
   * \returns the value pointed to by the iterator.
   */
  Element_t operator* ();

  /**
   * Pre-increment operator.  Increments the iterator.
   *
   * \returns the iterator at (this + 1).
   */
  FullRowIterator& operator++ ();

  /**
   * Post-increment operator.
   *
   * \param post dummy argument.
   * \returns this iterator before incrementing.
   */
  FullRowIterator operator++ (int post);

  /**
   * Pre-decrement operator.  Decrements the iterator.
   *
   * \returns the iterator at (this - 1).
   */
  FullRowIterator& operator-- ();

  /**
   * Post-decrement operator.
   *
   * \param post dummy argument.
   * \returns this iterator before decrementing.
   */
  FullRowIterator operator-- (int post);

  /**
   * Forward-advance operator.  Increments the iterator by n.
   *
   * \param n an integer.
   * \returns this after it has been incremented.
   */
  FullRowIterator& operator+= (int n);

  /**
   * Backward-reverse operator.  Decrements the iterator by n.
   *
   * \param n an integer.
   * \returns this after it has been decremented.
   */
  FullRowIterator& operator-= (int n);

  /**
   * Distance operator.
   *
   * \param x another iterator.
   * \returns the distance between this and x.
   */
  int operator- (const FullRowIterator& x) const;

  /**
   * Index operator.
   *
   * \param n an integer.
   * \returns the value at this[n].
   */
  Element_t operator[] (int n) const;

  /**
   * Equals comparator.
   *
   * \param x iterator to compare this to.
   * \returns (*this) == x.
   */
  bool operator== (const FullRowIterator& x) const;

  /**
   * Not-Equals comparator.
   *
   * \param x iterator to compare this to.
   * \returns (*this) != x.
   */
  bool operator!= (const FullRowIterator& x) const;

  /**
   * Less-than comparator.
   *
   * \param x iterator to compare this to.
   * \returns (*this) < x.
   */
  bool operator< (const FullRowIterator& x) const;


  //! this is a friend of it's corresponding matrix type.
  friend class FullMatrix<Element_t, Alloc>;
};


// get function definitions
#include "FullRowIterator.cc"


#endif // _FULLROWITERATOR_H
