/*
'GeomSparse.h'
Contains helpful classes for reducing geometrically sparse matrices.
*/

#ifndef _GEOMSPARSE_H
#define _GEOMSPARSE_H


#include <iostream>
#include <algorithm>
#include <utility>
#include <map>
#include "IndexList.h"
#include "Partition.h"

/*
Contains Indices specifying a submatrix with indices in rowIndices and
colIndices
*/
class SubmatrixIndicesIterator;

struct SubmatrixIndices
{
  IndexList rowIndices;
  IndexList colIndices;
  SubmatrixIndices();
  SubmatrixIndices(const IndexList& rowIndices, const IndexList& colIndices);
  ~SubmatrixIndices();
  void display() const;
  void setVals(const IndexList& rowIndices, const IndexList& colIndices);
  void setVals(const SubmatrixIndices& submat);
  void disjointUnion(SubmatrixIndices* submatrices, int length);
  void adjoin(const SubmatrixIndices& submat);
  void adjoinMaintainingSortedColumns(const SubmatrixIndices& submat);
  void concat(SubmatrixIndices* submatrices, int numSubmatrices);
  SubmatrixIndicesIterator begin();
  SubmatrixIndicesIterator end();
};


/*
A dividing of a matrix into blocks. Basically just a partition of
the rows and columns
*/
struct Blocking 
{
  Partition rowPartition;
  Partition colPartition;
  Blocking();
  Blocking(const Partition& rowPartition, const Partition& colPartition);
  void setVals(const Partition& rowPartition, const Partition& colPartition);
  void setVals(const Blocking& blocking);
  void restrict(SubmatrixIndices submat);
};

//////////////////////////////////////////////////////////////////////////
/////////////////////////definitions//////////////////////////////////////
//////////////////////////////////////////////////////////////////////////


SubmatrixIndices::SubmatrixIndices() : rowIndices(), colIndices()
{
}

SubmatrixIndices::SubmatrixIndices(const IndexList& rowIndices, const IndexList& colIndices) :
  rowIndices(rowIndices), colIndices(colIndices)
{
}

void SubmatrixIndices::setVals(const SubmatrixIndices& submat) {
  setVals(submat.rowIndices, submat.colIndices);
}

void SubmatrixIndices::setVals(const IndexList& rowIndices, const IndexList& colIndices) {
  this->rowIndices.setVals(rowIndices);
  this->colIndices.setVals(colIndices);
}

void SubmatrixIndices::display() const {
  std::cout << "displaying submatrix:\n";
  rowIndices.display();
  colIndices.display();
}

SubmatrixIndices::~SubmatrixIndices() {

}
/*
void SubmatrixIndices::disjointUnion(SubmatrixIndices* submatrices, int length) {
  IndexList temp[length];
  for(int i = 0; i < length; i++) {
    temp[i].setVals(submatrices[i].rowIndices);
  }
  rowIndices.disjointUnion(temp, length);
  for(int i = 0; i < length; i++) {
    temp[i].setVals(submatrices[i].colIndices);
  }
  colIndices.disjointUnion(temp, length);
  }*/

void SubmatrixIndices::disjointUnion(SubmatrixIndices* submatrices, int numSubmatrices) {
  rowIndices.length = 0;
  for(int i = 0; i < numSubmatrices; i++) {
    rowIndices.length += submatrices[i].rowIndices.length;
  }
  colIndices.length = rowIndices.length;
  delete[] rowIndices.list; delete[] colIndices.list;
  if(colIndices.length == 0) {rowIndices.list = NULL; colIndices.list = NULL; return;}
  rowIndices.list = new int[rowIndices.length]; colIndices.list = new int[colIndices.length];
  int listIndices[numSubmatrices];
  for(int i = 0; i < numSubmatrices; i++) {
    listIndices[i] = 0;
  }
  int mincolval = 0, rowval = 0, index;
  for(int j = 0; j < colIndices.length; j++) {
    index = -1;
    for(int i = 0; i < numSubmatrices; i++) {
      if(listIndices[i] < submatrices[i].colIndices.length) {
	if(submatrices[i].colIndices.list[listIndices[i]] < mincolval || index == -1) {
	  index = i;
	  mincolval = submatrices[i].colIndices.list[listIndices[i]];
	  rowval = submatrices[i].rowIndices.list[listIndices[i]];
	  
	}
      }
    }
    listIndices[index]++;
    rowIndices.list[j] = rowval;
    colIndices.list[j] = mincolval;
  }
}

void SubmatrixIndices::adjoin(const SubmatrixIndices& submat) {
  rowIndices.adjoin(submat.rowIndices);
  colIndices.adjoin(submat.colIndices);
}

void SubmatrixIndices::adjoinMaintainingSortedColumns(const SubmatrixIndices& submat) {
  int newRowLength = rowIndices.length + submat.rowIndices.length, 
    newColLength = colIndices.length + submat.colIndices.length;
  if(newRowLength != newColLength) {std::cout << "adjoinMaintainingSortedColumns error: non-square matrix.\n"; return;}
  int* newRowList = new int[newRowLength];
  int* newColList = new int[newColLength];
  
  int k = 0, l = 0;
  for(int i = 0; i < newColLength; i++) {
    if(k != colIndices.length && (l == submat.colIndices.length || colIndices.list[k] <= submat.colIndices.list[l])) {
      newRowList[i] = rowIndices.list[k];
      newColList[i] = colIndices.list[k];
      k++;
    } else {
      newRowList[i] = submat.rowIndices.list[l];
      newColList[i] = submat.colIndices.list[l];
      l++;
    }
  }
  delete[] rowIndices.list;
  delete[] colIndices.list;
  rowIndices.list = newRowList; rowIndices.length = newRowLength;
  colIndices.list = newColList; colIndices.length = newColLength;
}

class SubmatrixIndicesIterator {
  int i;
  SubmatrixIndices* submat;
public:
  SubmatrixIndicesIterator() : i(0), submat(NULL) {}

  SubmatrixIndicesIterator(SubmatrixIndices& submat) : i(0), submat(&submat) {}

  SubmatrixIndicesIterator(SubmatrixIndices& submat, int index) : i(index), submat(&submat) {}

  

  friend bool operator==(const SubmatrixIndicesIterator& a, const SubmatrixIndicesIterator& b) {
    return (a.i == b.i && a.submat == b.submat);
  }
  
  std::pair<int, int> operator*() const {
    if(i >= submat->rowIndices.length || i >= submat->colIndices.length) throw -1;
    std::pair<int, int> pivotPair(submat->rowIndices.list[i], submat->colIndices.list[i]);
    return pivotPair;
  }

  SubmatrixIndicesIterator& operator++() {
    if(i >= submat->rowIndices.length || i >= submat->colIndices.length) throw -1;
    i++;
    return *this;
  }

  SubmatrixIndicesIterator operator++(int) {
    SubmatrixIndicesIterator temp = *this;
    ++*this;
    return temp;
  }  
};

SubmatrixIndicesIterator SubmatrixIndices::begin() {
  SubmatrixIndicesIterator iterator(*this);
  return iterator;
}

SubmatrixIndicesIterator SubmatrixIndices::end() {
  int index = rowIndices.length;
  if(colIndices.length < index) index = colIndices.length;
  SubmatrixIndicesIterator iterator(*this, index);
  return iterator;
}



Blocking::Blocking() : rowPartition(), colPartition() {}

Blocking::Blocking(const Partition& rowPartition, const Partition& colPartition) :
  rowPartition(rowPartition), colPartition(colPartition) 
{  
}

void Blocking::setVals(const Partition& rowPartition, const Partition& colPartition) {
  this->rowPartition.setVals(rowPartition);
  this->colPartition.setVals(colPartition);
}

void Blocking::setVals(const Blocking& blocking) {
  this->rowPartition.setVals(blocking.rowPartition);
  this->colPartition.setVals(blocking.colPartition);
}

void Blocking::restrict(SubmatrixIndices submat) {
  rowPartition.restrict(submat.rowIndices);
  colPartition.restrict(submat.colIndices);
}

/*void Blocking::setVals(const Partition& rowPartition, const Partition& colPartition) {
  rowPartition.setVals(rowPartition);
  colPartition.setVals(colPartition);
  }*/

void IndexList::disjointUnion(IndexList* indexLists, int numLists) {
  length = 0;
  for(int i = 0; i < numLists; i++) {
    length += indexLists[i].length;
  }
  delete[] list;
  if(length == 0) {list = NULL; return;}
  list = new int[length];
  int listIndices[numLists];
  for(int i = 0; i < numLists; i++) {
    listIndices[i] = 0;
  }
  int minval = 0, index;
  for(int j = 0; j < length; j++) {
    index = -1;
    for(int i = 0; i < numLists; i++) {
      if(listIndices[i] < indexLists[i].length) {
	if(indexLists[i].list[listIndices[i]] < minval || index == -1) {
	  index = i;
	  minval = indexLists[i].list[listIndices[i]];
	}
      }
    }
    listIndices[index]++;
    list[j] = minval;
  }
}

void SubmatrixIndices::concat(SubmatrixIndices* submatrices, int numSubmatrices) {
  IndexList* temp = new IndexList[numSubmatrices];
  for(int i = 0; i < numSubmatrices; i++) {
    temp[i] = submatrices[i].rowIndices;
  }
  rowIndices.concat(temp, numSubmatrices);
  for(int i = 0; i < numSubmatrices; i++) {
    temp[i] = submatrices[i].colIndices;
  }
  colIndices.concat(temp, numSubmatrices);
}

void IndexList::concat(IndexList* indexLists, int numLists) {
  length = 0;
  for(int i = 0; i < numLists; i++) {
    length += indexLists[i].length;
  }
  delete[] list;
  if(length == 0) {list = NULL; return;}
  list = new int[length];

  int k = 0, listIndex = 0;
  for(int i = 0; i < length; i++) {
    if(k == indexLists[listIndex].length) {
      do {
	listIndex++;
      } while(indexLists[listIndex].length == 0);
      k = 0;
    }
    list[i] = indexLists[listIndex].list[k];
    k++;
  }
}

#endif
