/* GeomSparseMatrix.cc
 * -------------------
 * Implements GeomSparseMatrix.h
 *
 * Written by Patrick Perry
 * Original Version on August 21, 2003
 *
 * $Id: GeomSparseMatrix.cc,v 1.1.1.1 2005/03/18 23:36:16 silva Exp $
 * $Log: GeomSparseMatrix.cc,v $
 * Revision 1.1.1.1  2005/03/18 23:36:16  silva
 * Imported Sources
 *
 * Revision 1.1.1.1  2004/09/25 22:04:08  patperry
 * Imported sources
 *
 * Revision 1.19  2004/01/23 04:36:42  patperry
 * Cleaned up MATLAB support code.
 *
 * Revision 1.18  2004/01/15 07:13:36  patperry
 * Changed file extensions.
 *
 * Revision 1.17  2003/12/13 00:09:18  patperry
 * bugfixes
 *
 * Revision 1.16  2003/12/12 20:13:46  patperry
 * bugfixes and debug code galore
 *
 * Revision 1.15  2003/12/02 12:39:11  patperry
 * added code to combine the four blocks
 *
 * Revision 1.14  2003/12/02 11:05:00  patperry
 * Added BasicColumnIterator
 *
 * Revision 1.13  2003/12/02 08:55:57  patperry
 * Added Destructor
 *
 * Revision 1.12  2003/12/01 23:12:01  patperry
 * Added functionality and documentation.
 *
 */

#include <set>
#include "CSCSparseMatrix.h"
#include "MatrixSelectionMask.h"


template <class Element_t, class Alloc>
template <class Alloc1>
GeomSparseMatrix<Element_t, Alloc>::
GeomSparseMatrix (const CSCSparseMatrix<Element_t, Alloc1>& matrix, 
			   const NerveComplex& nerve)
  : numRows(matrix.getNumRows()), numColumns(matrix.getNumColumns()),
    nerve(nerve),
    offDiagonalBlocks(calculateNumOffDiagonalBlocks(nerve),
		      nerve.getNumSimplices(), nerve.getNumSimplices()),
    upperLeftBlock(NULL), upperRightBlock(NULL), 
    lowerLeftBlock(NULL), lowerRightBlock(NULL),
    numPivots(0), rowPermutation(NULL), columnPermutation(NULL),
    reduced(false)
{
  // initialize the diagonal blocks
  for (NerveComplex::const_iterator simplex = nerve.simplicesBegin();
			 simplex != nerve.simplicesEnd(); ++simplex) {
			MatrixSelectionMask<Element_t, 
		typename CSCSparseMatrix<Element_t, Alloc1>::ColumnIterator,
														 Alloc1>
		mask(matrix, simplex->getRows().begin(), simplex->getRows().end(),
				 simplex->getColumns().begin(), simplex->getColumns().end());

    //std::cerr << "Block (" << simplex->getID() << ", " 
    //	    << simplex->getID() << "):\n" << mask;

    diagonalBlocks.push_back(new DiagonalBlock(simplex->getID(), mask));
  }
  
  // initialize the off-diagonal blocks, one column at a time
  for (NerveComplex::const_iterator simplex = nerve.simplicesBegin();
       simplex != nerve.simplicesEnd(); ++simplex) {
    // start with a column of null pointers
    OffDiagonalBlock * offDiagonalBlockColumn[nerve.getNumSimplices()];
    std::uninitialized_fill(offDiagonalBlockColumn, offDiagonalBlockColumn
			    + nerve.getNumSimplices(), 
			    static_cast<OffDiagonalBlock *>(NULL));
  
    // get the common information for the column
    const std::set<int>& columns = simplex->getColumns();
    const int columnID = simplex->getID();
    const DiagonalBlock * columnBlock = diagonalBlocks[columnID];
    
    // the cofaces of the simplex correspond to blocks in this column
    for (NerveSimplex::const_iterator cofacePointer = simplex->cofacesBegin();
	 cofacePointer != simplex->cofacesEnd(); ++cofacePointer) {
      // get the row information
      const std::set<int>& rows = (*cofacePointer)->getRows();
      const int rowID = (*cofacePointer)->getID();
      const DiagonalBlock * rowBlock = diagonalBlocks[rowID];
      
      // make a mask of the data
      MatrixSelectionMask<Element_t, 
	   typename CSCSparseMatrix<Element_t, Alloc1>::ColumnIterator, Alloc1> 
	   mask(matrix, rows.begin(), rows.end(), columns.begin(), columns.end());

	 //std::cerr << "Block (" << rowID << ", " << columnID << "):\n" << mask;
	 
      // initialize the off-diagonal block
      offDiagonalBlockColumn[rowID] 
	   = new OffDiagonalBlock(mask, rowBlock, columnBlock);
    }
    
    // copy the column of off-diagonal blocks
    offDiagonalBlocks.fillColumn(columnID, offDiagonalBlockColumn);
  }

  // At this point, the off-diagonal blocks have pointers to their
  // corresponding row and column diagonal blocks.  

  std::vector<OffDiagonalBlock *> rowBlocks[nerve.getNumSimplices()];

  // tie everything together
  for (int column = 0; column < nerve.getNumSimplices(); ++column) {
    typename CSCSparseMatrix<OffDiagonalBlock *, Alloc>::
      ColumnIterator columnIt = offDiagonalBlocks.getColumnBegin(column);
    typename CSCSparseMatrix<OffDiagonalBlock *, Alloc>::
      ColumnIterator columnEnd = offDiagonalBlocks.getColumnEnd(column); 
    
    diagonalBlocks[column]->addCofaces(columnIt, columnEnd);
    while (columnIt != columnEnd) {
      if ((*columnIt) != NULL) {
	(*columnIt)->addColumnFaces(columnIt + 1, columnEnd);
	rowBlocks[(*columnIt)->getRowID()].push_back(*columnIt);
      }
      ++columnIt;
    }
  }
  
  for (int row = 0; row < nerve.getNumSimplices(); ++row) {
    typename std::vector<OffDiagonalBlock *>::iterator rowIt 
      = rowBlocks[row].begin();
    typename std::vector<OffDiagonalBlock *>::iterator rowEnd
      = rowBlocks[row].end();
    
    diagonalBlocks[row]->addFaces(rowIt, rowEnd);
    while (rowIt != rowEnd) {
      (*rowIt)->addRowFaces(rowIt + 1, rowEnd);
      ++rowIt;
    }
  }
}


template <class Element_t, class Alloc>
GeomSparseMatrix<Element_t, Alloc>::
~GeomSparseMatrix ()
{
  if (!reduced) {
    freeBlockStructure ();
  } else {
    delete upperLeftBlock;
    delete lowerLeftBlock;
    delete upperRightBlock;
    delete lowerRightBlock;
    delete[] rowPermutation;
    delete[] columnPermutation;
  }
}


template <class Element_t, class Alloc>
Element_t
GeomSparseMatrix<Element_t, Alloc>::
getElement (int i, int j) const
{
  Element_t result = 0;
  
  if (!reduced) {
    const_cast<GeomSparseMatrix *>(this)->reduce();
  }
  
  if (i < numPivots) {
    if (j < numPivots) {
	 result = upperLeftBlock->getElement(i, j);
    } else {
	 result = upperRightBlock->getElement(i, j - numPivots);
    }
  } else {
    if (j < numPivots) {
	 result = lowerLeftBlock->getElement(i - numPivots, j);
    } else {
	 result = lowerRightBlock->getElement(i - numPivots, j - numPivots);
    }
  }

  return result;
}


template <class Element_t, class Alloc>
void
GeomSparseMatrix<Element_t, Alloc>::
reduce ()
{
  if (!reduced) {
    int numUpperLeftElements = 0;
    int numLowerLeftElements = 0;
    int numUpperRightElements = 0;
    int numLowerRightElements = 0;
    
    // first, reduce all the diagonal blocks
    for (int i = 0; i < nerve.getNumSimplices(); ++i) {
      int numBlockPivots;
      int numBlockRows = diagonalBlocks[i]->getNumRows();
      int numBlockColumns = diagonalBlocks[i]->getNumColumns();
      
      diagonalBlocks[i]->reduce();
      numBlockPivots = diagonalBlocks[i]->getNumPivots();
      numPivots += numBlockPivots;
      
      numUpperLeftElements += (numBlockPivots * numBlockPivots);
      numLowerLeftElements += ((numBlockRows - numBlockPivots)
			       * numBlockPivots);
      numUpperRightElements += (numBlockPivots 
				* (numBlockColumns - numBlockPivots));
      numLowerRightElements += ((numBlockRows - numBlockPivots)
				* (numBlockColumns - numBlockPivots));

	 //std::cerr << "Block(" << i << ", " << i << "):\n"
	 //		 << *(diagonalBlocks[i]);
    }
    
    // now, cancel everything above the pivots
    for (int column = 0; column < nerve.getNumSimplices(); ++column) {
      typename CSCSparseMatrix<OffDiagonalBlock *, Alloc>::
	   ColumnIterator columnIt = offDiagonalBlocks.getColumnBegin(column);
      typename CSCSparseMatrix<OffDiagonalBlock *, Alloc>::
	   ColumnIterator columnEnd = offDiagonalBlocks.getColumnEnd(column); 
      
      while (columnIt != columnEnd) {
	   if (*columnIt) {
		int numBlockRows = (*columnIt)->getNumRows();
		int numBlockColumns = (*columnIt)->getNumColumns();
		int numRowPivots = (*columnIt)->getNumRowPivots();
		int numColumnPivots = (*columnIt)->getNumColumnPivots();
		
		(*columnIt)->cancel();
		
		numUpperLeftElements += (numRowPivots * numColumnPivots);
		numLowerLeftElements += ((numBlockRows - numRowPivots)
							* numColumnPivots);
		numUpperRightElements += (numRowPivots 
							 * (numBlockColumns - numColumnPivots));
		numLowerRightElements += ((numBlockRows - numRowPivots)
							 * (numBlockColumns - numColumnPivots));
		
		//std::cerr << "Block(" << (*columnIt)->getRowID() << ", "
		//		<< (*columnIt)->getColumnID() << "):\n" << **columnIt;
	   }
	   ++columnIt;
      }
    }
    
    upperLeftBlock = new CSCSparseMatrix<Element_t, Alloc>
      (numUpperLeftElements, numPivots, numPivots);
    lowerLeftBlock = new CSCSparseMatrix<Element_t, Alloc>
      (numLowerLeftElements, numRows - numPivots, numPivots);
    upperRightBlock = new CSCSparseMatrix<Element_t, Alloc>
      (numUpperRightElements, numPivots, numColumns - numPivots);
    lowerRightBlock = new CSCSparseMatrix<Element_t, Alloc>
      (numLowerRightElements, numRows - numPivots, numColumns - numPivots);

    rearrangeData();
    
    rowPermutation = new int[numRows];
    columnPermutation = new int[numColumns];
    
    computePermutations();

    freeBlockStructure ();

    reduced = true;
  }
}


template <class Element_t, class Alloc>
const int 
GeomSparseMatrix<Element_t, Alloc>::
getNumPivots () const
{
  return numPivots;
}


template <class Element_t, class Alloc>
const int *
GeomSparseMatrix<Element_t, Alloc>::
getP () const
{
  return rowPermutation;
}


template <class Element_t, class Alloc>
const int *
GeomSparseMatrix<Element_t, Alloc>::
getQ () const
{
  return columnPermutation;
}


template <class Element_t, class Alloc>
CSCSparseMatrix<Element_t, Alloc> * 
GeomSparseMatrix<Element_t, Alloc>::
getUpperLeftBlock () const
{
  return upperLeftBlock;
}


template <class Element_t, class Alloc>
CSCSparseMatrix<Element_t, Alloc> * 
GeomSparseMatrix<Element_t, Alloc>::
getLowerLeftBlock () const
{
  return lowerLeftBlock;
}


template <class Element_t, class Alloc>
CSCSparseMatrix<Element_t, Alloc> * 
GeomSparseMatrix<Element_t, Alloc>::
getUpperRightBlock () const
{
  return upperRightBlock;
}


template <class Element_t, class Alloc>
CSCSparseMatrix<Element_t, Alloc> * 
GeomSparseMatrix<Element_t, Alloc>::
getLowerRightBlock () const
{
  return lowerRightBlock;
}


template <class Element_t, class Alloc>
const Blocking&
GeomSparseMatrix<Element_t, Alloc>::
getLowerRightBlocking () const
{
  return lowerRightBlocking;
}


template <class Element_t, class Alloc>
template <class ColumnIterator_t, class Alloc1>
CSCSparseMatrix<Element_t, Alloc> * 
GeomSparseMatrix<Element_t, Alloc>::
combineBlocks (const Matrix<Element_t, ColumnIterator_t, Alloc1>&
			reducedLowerRight, const int * lowerRightP,
			const int * lowerRightQ)
{
  CSCSparseMatrix<Element_t, Alloc> * result 
    = new CSCSparseMatrix<Element_t, Alloc>
    (upperLeftBlock->getNumNonzeroElements()
	+ lowerLeftBlock->getNumNonzeroElements()
	+ upperRightBlock->getNumNonzeroElements()
	+ reducedLowerRight.getNumNonzeroElements(),
	numRows, numColumns);
  Element_t newColumn[numRows];
  
  // apply the permutations to the lower-left and upper-right blocks
  lowerLeftBlock->permuteRows(lowerRightP);
  upperRightBlock->permuteColumns(lowerRightQ);
  
  // copy the data from the pivot columns over
  for (int column = 0; column < numPivots; ++column) {
    std::copy(upperLeftBlock->getColumnBegin(column),
		    upperLeftBlock->getColumnEnd(column), newColumn);
    std::copy(lowerLeftBlock->getColumnBegin(column),
		    lowerLeftBlock->getColumnEnd(column), 
		    newColumn + numPivots);

    result->fillColumn(column, newColumn);
  } 
  
  // copy the data from the nonpivot columns over
  for (int column = 0; column < numColumns - numPivots; ++column) {
    std::copy(upperRightBlock->getColumnBegin(column),
		    upperRightBlock->getColumnEnd(column), newColumn);
    std::copy(reducedLowerRight.getColumnBegin(column),
		    reducedLowerRight.getColumnEnd(column),
		    newColumn + numPivots);

    result->fillColumn(column + numPivots, newColumn);
  }

  // update the row permutation
  for (int i = 0; i < numRows; ++i) {
    if (rowPermutation[i] >= numPivots) {
	 rowPermutation[i] 
	   = lowerRightP[rowPermutation[i] - numPivots] + numPivots;
    }
  }

  // update the column permutation
  for (int i = 0; i < numColumns; ++i) {
    if (columnPermutation[i] >= numPivots) {
	 columnPermutation[i]
	   = lowerRightQ[columnPermutation[i] - numPivots] + numPivots;
    }
  }

  return result;
}


template <class Element_t, class Alloc>
int 
GeomSparseMatrix<Element_t, Alloc>::
calculateNumOffDiagonalBlocks (const NerveComplex& nerve) 
{
  int result = 0;

  for (NerveComplex::const_iterator simplex = nerve.simplicesBegin();
	  simplex != nerve.simplicesEnd(); ++simplex) {
    result += simplex->getNumFaces();
  }

  return result;
}


template <class Element_t, class Alloc>
void
GeomSparseMatrix<Element_t, Alloc>::
rearrangeData ()
{
  // we copy over one column at a time
  Element_t columnData[numRows];
  int pivotRowIndex = 0;
  int nonpivotRowIndex = numPivots;
  int pivotColumnIndex = 0;
  int nonpivotColumnIndex = 0;
  
  // for each column of blocks
  for (int simplexID = 0; simplexID < nerve.getNumSimplices(); ++simplexID) {
    const NerveSimplex& simplex = nerve.getSimplex(simplexID);
    typename CSCSparseMatrix<OffDiagonalBlock *, Alloc>::
      ColumnIterator columnBegin = offDiagonalBlocks.getColumnBegin(simplexID);
    typename CSCSparseMatrix<OffDiagonalBlock *, Alloc>::
      ColumnIterator columnEnd = columnBegin + simplexID;
    
    // for each data column in the block column
    for (unsigned int blockColumn = 0; 
	    blockColumn < simplex.getColumns().size(); ++blockColumn) {
      int rowSimplexID = 0;
      
      // initialize the column to all zeros
      std::uninitialized_fill(columnData, columnData + numRows, 0);
      pivotRowIndex = 0;
      nonpivotRowIndex = numPivots;
      
      for (typename CSCSparseMatrix<OffDiagonalBlock *, Alloc>::ColumnIterator
		   columnIt = columnBegin; columnIt != columnEnd; ++columnIt) {
	   int numRowSimplexPivots = diagonalBlocks[rowSimplexID]->getNumPivots();
	   
	   // copy the data in the nonzero column blocks
	   if ((*columnIt != NULL)
		  && nerve.getSimplex(rowSimplexID).getRows().size() != 0) {	
		typename OffDiagonalBlock::ColumnIterator columnBegin
		  = (*columnIt)->getColumnBegin(blockColumn);
		typename OffDiagonalBlock::ColumnIterator columnMid
		  = columnBegin + numRowSimplexPivots;
		typename OffDiagonalBlock::ColumnIterator columnEnd
		  = (*columnIt)->getColumnEnd(blockColumn);
		
		if (columnBegin != columnMid) {
		  std::copy(columnBegin, columnMid, columnData + pivotRowIndex);
		}
		if (columnMid != columnEnd) {
		  std::copy(columnMid, columnEnd, columnData + nonpivotRowIndex);
		}
	   } 
	   
	   pivotRowIndex += numRowSimplexPivots;
	   nonpivotRowIndex += (diagonalBlocks[rowSimplexID]->getNumRows()
					    - numRowSimplexPivots);
	   ++rowSimplexID;
      }
      
      // copy the data in the diagonal block
      if (diagonalBlocks[simplexID]->getNumRows() != 0) {
	   typename DiagonalBlock::ColumnIterator columnBegin
		= diagonalBlocks[simplexID]->getColumnBegin(blockColumn);
	   typename DiagonalBlock::ColumnIterator columnMid
		= columnBegin + diagonalBlocks[simplexID]->getNumPivots();
	   typename DiagonalBlock::ColumnIterator columnEnd
		= diagonalBlocks[simplexID]->getColumnEnd(blockColumn);
	   
	   if (columnBegin != columnMid) {
		std::copy(columnBegin, columnMid, columnData + pivotRowIndex);
	   }
	   if (columnMid != columnEnd) {
		std::copy(columnMid, columnEnd, columnData + nonpivotRowIndex);
	   }
      }
      
      // now we have the whole column in memory.  we need to copy the
      // column into the appropriate blocks
      if (blockColumn < static_cast<unsigned int>
		(diagonalBlocks[simplexID]->getNumPivots())) {
	   upperLeftBlock->fillColumn(pivotColumnIndex, columnData);
	   lowerLeftBlock->fillColumn(pivotColumnIndex, columnData + numPivots);
	   
	   ++pivotColumnIndex;
      } else {
	   upperRightBlock->fillColumn(nonpivotColumnIndex, columnData);
	   lowerRightBlock->fillColumn(nonpivotColumnIndex, 
							 columnData + numPivots);
	   
	   ++nonpivotColumnIndex;
      }
    }
  }
}


template <class Element_t, class Alloc>
void
GeomSparseMatrix<Element_t, Alloc>::
computePermutations ()
{
  int pivotRow = 0;
  int nonpivotRow = numPivots;
  int pivotColumn = 0;
  int nonpivotColumn = numPivots;
  int nonpivotRowPartition[numRows - numPivots];
  int nonpivotColumnPartition[numColumns - numPivots];
  int numNonpivotRows = 0;
  int numNonpivotColumns = 0;

  for (int i = 0; i < nerve.getNumSimplices(); ++i) {
    const DiagonalBlock& block = *(diagonalBlocks[i]);
    const int numSimplexPivots = block.getNumPivots();
    const int * blockP = block.getP();
    const int * blockQ = block.getQ();

    const NerveSimplex& simplex = nerve.getSimplex(i);
    std::set<int>::const_iterator rowsIt = simplex.getRows().begin();
    std::set<int>::const_iterator columnsIt = simplex.getColumns().begin();

    for (int row = 0; row < block.getNumRows(); ++row) {
      if (blockP[row] < numSimplexPivots) {
	   rowPermutation[*rowsIt++] = pivotRow + blockP[row];
      } else {
	   rowPermutation[*rowsIt++] = nonpivotRow 
		+ blockP[row] - numSimplexPivots;
	   nonpivotRowPartition[numNonpivotRows++] = i;
      }
    }
    
    for (int column = 0; column < block.getNumColumns(); ++column) {
      if (blockQ[column] < numSimplexPivots) {
	   columnPermutation[*columnsIt++] = pivotColumn + blockQ[column];
      } else {
	   columnPermutation[*columnsIt++] = nonpivotColumn
		+ blockQ[column] - numSimplexPivots;
	   nonpivotColumnPartition[numNonpivotColumns++] = i;
      }
    }
    
    pivotRow += numSimplexPivots;
    nonpivotRow += block.getNumRows() - numSimplexPivots;
    pivotColumn += numSimplexPivots;
    nonpivotColumn += block.getNumColumns() - numSimplexPivots;
  }

  IndexList nonpivotRowPartitionList(nonpivotRowPartition, numNonpivotRows);
  IndexList nonpivotColumnPartitionList(nonpivotColumnPartition,
								numNonpivotColumns);
  Partition rowPartition(nerve.getNumSimplices(), nonpivotRowPartitionList);
  Partition columnPartition(nerve.getNumSimplices(), 
					   nonpivotColumnPartitionList);

  lowerRightBlocking.setVals(rowPartition, columnPartition);
}


template <class Element_t, class Alloc>
void
GeomSparseMatrix<Element_t, Alloc>::
freeBlockStructure ()
{
  // for each column
  for (int blockColumn = 0; blockColumn < nerve.getNumSimplices();
       ++blockColumn) {
    // free the off-diagonal blocks
    typename CSCSparseMatrix<OffDiagonalBlock *, Alloc>::ColumnIterator it 
      = offDiagonalBlocks.getColumnBegin(blockColumn);
    typename CSCSparseMatrix<OffDiagonalBlock *, Alloc>::ColumnIterator end
      = offDiagonalBlocks.getColumnEnd(blockColumn);

    while (it != end) {
      delete *it++;
    }

    // free the diagonal block
    delete diagonalBlocks[blockColumn];
  }

  diagonalBlocks.clear();
}


