/* GeomSparseMatrix.h
 * ------------------
 * Interface for a geometrically sparse matrix.
 *
 * Written by Patrick Perry
 * Original Version on August 20, 2003
 *
 * $Id: GeomSparseMatrix.h,v 1.3 2005/11/02 01:59:52 patperry Exp $
 * $Log: GeomSparseMatrix.h,v $
 * Revision 1.3  2005/11/02 01:59:52  patperry
 * The last (?) gcc4.0 fixes\!\!\!
 *
 * Revision 1.2  2005/10/24 16:20:08  patperry
 * Hells (literally) of compile fixes for gcc4.0
 *
 * Revision 1.1.1.1  2005/03/18 23:36:17  silva
 * Imported Sources
 *
 * Revision 1.1.1.1  2004/09/25 22:04:08  patperry
 * Imported sources
 *
 * Revision 1.1  2004/01/15 07:13:36  patperry
 * Changed file extensions.
 *
 * Revision 1.11  2003/12/12 20:13:46  patperry
 * bugfixes and debug code galore
 *
 * Revision 1.10  2003/12/02 12:39:11  patperry
 * added code to combine the four blocks
 *
 * Revision 1.9  2003/12/02 11:05:00  patperry
 * Added BasicColumnIterator
 *
 * Revision 1.8  2003/12/02 08:55:57  patperry
 * Added Destructor
 *
 * Revision 1.7  2003/12/01 23:12:01  patperry
 * Added functionality and documentation.
 *
 */

#ifndef _GEOMSPARSEMATRIX_H
#define _GEOMSPARSEMATRIX_H

#include <cstdlib>
#include "Matrix.h"
#include "CSCSparseMatrix.h"
#include "NerveComplex.h"
#include "DiagonalBlock.h"
#include "OffDiagonalBlock.h"
#include "GeomSparse.h"


template <class Element_t, class Alloc = std::allocator<Element_t> >
class GeomSparseMatrix
{
  //! the number of rows of data
  const int numRows;

  //! the number of columns of data
  const int numColumns;

  //! diagonal block type
  typedef DiagonalBlock<Element_t, Alloc> DiagonalBlock;

  //! off-diagonal block type
  typedef OffDiagonalBlock<Element_t, Alloc> OffDiagonalBlock;

  //! underlying nerve complex
  const NerveComplex& nerve;

  //! blocks along the diagonal, stored in an array
  std::vector<DiagonalBlock *, 
		    typename Alloc::template rebind<DiagonalBlock *>::other> 
    diagonalBlocks;

  //! blocks off the diagnonal, stored in sparse CSC format
  CSCSparseMatrix<OffDiagonalBlock *, Alloc> offDiagonalBlocks;

  //! the upper-left portion of the LU-decomposition, corresponding
  //! to the rows and columns with pivots in them
  CSCSparseMatrix<Element_t, Alloc> * upperLeftBlock;

  //! the upper-right portion of the LU-decomposition, corresponding
  //! to the rows with pivots and columns without pivots
  CSCSparseMatrix<Element_t, Alloc> * upperRightBlock;

  //! the lower-left portion of the LU-decomposition, corresponding 
  //! to the rows without pivots and the columns with pivots
  CSCSparseMatrix<Element_t, Alloc> * lowerLeftBlock;

  //! the lower-right portion of the LU-decomposition, corresponding
  //! to the rows and columns without pivots
  CSCSparseMatrix<Element_t, Alloc> * lowerRightBlock;

  //! the number of pivots after the reduction
  int numPivots;

  //! stores the row permutation
  int * rowPermutation;

  //! stores the column permutation
  int * columnPermutation;

  //! the blocking for the lower right (nonpivot) block
  Blocking lowerRightBlocking;

  //! indicates whether or not matrix has been reduced
  bool reduced;

public:

  /**
   * Constructor.  Create a geometric-sparse matrix from the passed-in
   * matrix and nerve complex.
   *
   * \param matrix the matrix to reduce.
   * \param nerve the nerve complex.
   */
  template <class Alloc1>
  GeomSparseMatrix (const CSCSparseMatrix<Element_t, Alloc1>& matrix, 
				const NerveComplex& nerve);

  /**
   * Destructor.
   */
  virtual ~GeomSparseMatrix ();

  /**
   * Element accessor.  Returns the element at position (i, j) after
   * the LU-decomposition.
   *
   * \param i the row index.
   * \param j the column index.
   * \returns the element at position (i, j).
   */
  virtual Element_t getElement (int i, int j) const;

  /**
   * LU-decompose the matrix.
   */
  void reduce ();

  /**
   * Get the number of pivots after the reduction.
   *
   * \returns the number of pivots.
   */
  const int getNumPivots () const;

  /** 
   * Get the row permutation.
   *
   * \returns the row permutation in the form of an integer 
   *          array of length numRows.
   */
  const int * getP () const;

  /** 
   * Get the column permutation.
   *
   * \returns the column permutation in the form of an integer 
   *          array of length numColumns.
   */
  const int * getQ () const;

  /**
   * Get the upper left block.  This block consists of the data in rows
   * and columns of pivots.  Below the diagonal, this block is data for
   * L.  Above and including the diagonal, the block contains the data
   * for U.
   *
   * \returns a pointer the the upper left block.
   */
  CSCSparseMatrix<Element_t, Alloc> * getUpperLeftBlock () const;

  /**
   * Get the lower left block.  This block consists of the data in columns
   * of pivots and rows of nonpivots.  It is entirely data for the L part
   * of the LU-decomposition.
   *
   * \returns a pointer the the lower left block.
   */
  CSCSparseMatrix<Element_t, Alloc> * getLowerLeftBlock () const;

  /**
   * Get the upper right block.  This block consists of data in the rows
   * of pivots and columns of nonpivots.  All data in this block is for
   * the U part of the LU-decomposition.
   *
   * \returns a pointer the the upper right block.
   */
  CSCSparseMatrix<Element_t, Alloc> * getUpperRightBlock () const;

  /**
   * Get the lower right block.  This block is that data in the rows and
   * columns of nonpivots.  For a complete LU-decomposition, the data in
   * this block needs to be reduced, and the appropriate permutations from
   * its reduction need to be applied to the lower left and the upper right
   * blocks.
   *
   * \returns a pointer the the lower right block.
   */
  CSCSparseMatrix<Element_t, Alloc> * getLowerRightBlock () const;

  /**
   * Get the blocking for the lower right block.  This blocking can be
   * reflects the block structure of the lower right matrix.
   *
   * \returns a blocking for the lower right block.
   */
  const Blocking& getLowerRightBlocking () const;

  /**
   * Using a reduced lower right block, combines the four blocks into
   * a signle matrix and returns the matrix.  This also applies the
   * lowerRightP permutation to the lower-left block, applies the
   * lowerRightQ permutation to the upper-right block, and updates
   * the row and column permutations.
   *
   * \param reducedLowerRight an LU-reduced version of the lower-right
   *        block
   * \param lowerRightP the P of the lower-right LU-reduction.
   * \param lowerRightQ the Q of the lower-=right LU-reduction.
   * \returns a new CSCSparse matrix representing a fully-reduced
   *          matrix.  It is the clients responsibility to free this
   *          matrix (using delete).
   */
  template <class ColumnIterator_t, class Alloc1>
  CSCSparseMatrix<Element_t, Alloc> * 
  combineBlocks (const Matrix<Element_t, ColumnIterator_t, Alloc1>&
			  reducedLowerRight, const int * lowerRightP,
			  const int * lowerRightQ);

private:

  /**
   * Calculates the number of off-diagonal blocks to store based on 
   * the given nerve complex.
   *
   * \param nerve the nerve complex to perform the calculation on.
   * \returns the number of off-diagonal blocks in the geometrically
   *          sparse matrix.
   */
  static int calculateNumOffDiagonalBlocks (const NerveComplex& nerve);

  /**
   * Moves the data from block form to the four sparse components.
   */
  void rearrangeData ();

  /**
   * Computes the final row and column permutations after the reduction.
   */
  void computePermutations ();

  /**
   * Frees the data stored in the blocks, and deletes the block structure.
   */
  void freeBlockStructure ();

};

// get function definitions
#include "GeomSparseMatrix.cc"


#endif // _GEOMSPARSEMATRIX_H
