#include "BlockMatrix.h"

template<class Element_t>
class LUBlockMatrix : public BlockMatrix<Element_t> {
  BlockableMatrix<Element_t> L;
  BlockMatrix<Element_t>** rowNeighbors;
  int numRowNeighbors;

protected:
  using BlockMatrix<Element_t>::rowPerm;

private:
  void initializeLU();
public:
  template<class ColumnIterator_t, class Alloc>
  LUBlockMatrix(const Matrix<Element_t, ColumnIterator_t, Alloc>& M, 
			 int maxNumBlockings);
  LUBlockMatrix(Element_t** M, int numRows, int numCols, int maxNumBlockings);
  ~LUBlockMatrix();
  void getL(BlockableMatrix<Element_t>& L);
  void computeLinv(BlockableMatrix<Element_t>& L);
  virtual void mRowAdd(Element_t k, int i, int j);
  virtual void mRowAdd(Element_t k, int i, int j, int startCol);
  template<class InputIterator> void reduce(InputIterator blocksBegin, InputIterator blocksEnd);
  void reduce(BlockMatrix<Element_t>** rowNeighbors, int numRowNeighbors) {setRowNeighbors(rowNeighbors, numRowNeighbors); 
  reduce(); setRowNeighbors(NULL, 0);}
  void setRowNeighbors(BlockMatrix<Element_t>** rowNeighbors, int numRowNeighbors);
  void reduce() {
    BlockMatrix<Element_t>::reduce();   
    L.applyRowPermutation(this->getPinv()); 
    L.applyColPermutation(this->getPinv());
  };
  virtual Element_t getElement(int i, int j) const;
  void display();
  void displayMatlab(std::ostream& os);
};

template<class Element_t>
Element_t LUBlockMatrix<Element_t>::getElement(int i, int j) const {
  if(this->isReduced() && (i > j)) {
    return L.getElement(i, j);
  } else {
    return BlockMatrix<Element_t>::getElement(i, j);
  }
}

template<class Element_t>
template<class ColumnIterator_t, class Alloc>
LUBlockMatrix<Element_t>::
LUBlockMatrix(const Matrix<Element_t, ColumnIterator_t, Alloc>& M, 
		    int maxNumBlockings) 
  : BlockMatrix<Element_t>(M, maxNumBlockings), 
    L(NULL, M.getNumRows(), M.getNumRows()) {
  initializeLU();
}

template<class Element_t>
LUBlockMatrix<Element_t>::LUBlockMatrix(Element_t** M, int numRows, int numCols, int maxNumBlockings) :
  BlockMatrix<Element_t>(M, numRows, numCols, maxNumBlockings), L(M, numRows, numRows) {
  initializeLU();
}

template<class Element_t>
LUBlockMatrix<Element_t>::~LUBlockMatrix() {
  delete[] rowNeighbors;
}

template<class Element_t>
void LUBlockMatrix<Element_t>::initializeLU() {
  L.makeIdentity();
  rowNeighbors = NULL;
  numRowNeighbors = 0;
}


template<class Element_t>
void LUBlockMatrix<Element_t>::mRowAdd(Element_t k, int row, int cancelRow) {
  mRowAdd(k, row, cancelRow, 0);
}

template<class Element_t>
void LUBlockMatrix<Element_t>::mRowAdd(Element_t k, int row, int cancelRow, int startCol) {
  L.setElement(row, cancelRow, -k);
  BlockMatrix<Element_t>::mRowAdd(k, row, cancelRow, 0);
  for(int i = 0; i < numRowNeighbors; i++) {
    rowNeighbors[i]->BlockableMatrix<Element_t>::mRowAdd(k, row, cancelRow);
  }
}

template<class Element_t>
void LUBlockMatrix<Element_t>::getL(BlockableMatrix<Element_t>& L) {
  int size = this->getNumRows();
  for(int i = 0; i < size; i++)
    for(int j = 0; j < size; j++)
      L.setElement(i, j, this->L.getElement(i, j));
}

template<class Element_t>
void LUBlockMatrix<Element_t>::computeLinv(BlockableMatrix<Element_t>& Linv) {
  int size = this->getNumRows();
  Linv.makeIdentity();
  for(int j = 0; j < size; j++) {
    for(int i = j + 1; i < size; i++) {
      Element_t sum = 0;
      for(int k = j; k < i; k++) {
       sum += L.getElement(rowPerm.list[i], rowPerm.list[k]) * Linv.getElement(rowPerm.list[k], rowPerm.list[j]);
      }
      Linv.setElement(rowPerm.list[i], rowPerm.list[j], -sum);
    } 
  }
}

template<class Element_t>
void LUBlockMatrix<Element_t>::setRowNeighbors(BlockMatrix<Element_t>** rowNeighbors, int numRowNeighbors) {
  delete[] this->rowNeighbors;
  this->rowNeighbors = new BlockMatrix<Element_t>*[numRowNeighbors];
  for(int i = 0; i < numRowNeighbors; i++) this->rowNeighbors[i] = rowNeighbors[i];
  this->numRowNeighbors = numRowNeighbors;
}


template<class Element_t>
template<class InputIterator> 
void LUBlockMatrix<Element_t>::reduce(InputIterator blocksBegin, InputIterator blocksEnd) {
  delete[] rowNeighbors; numRowNeighbors = 0;
  InputIterator i = blocksBegin;
  while(i != blocksEnd) {i++; numRowNeighbors++;}
  rowNeighbors = new BlockMatrix<Element_t>*[numRowNeighbors];
  i = blocksBegin; int index = 0;
  while(i != blocksEnd) {rowNeighbors[index] = *i; i++; index++;}
  reduce();
  delete[] rowNeighbors; rowNeighbors = NULL; numRowNeighbors = 0;
}

template<class Element_t>
void LUBlockMatrix<Element_t>::display() {
  int i, j;
  for(i = 0; i < this->getNumRows(); i++) {
    for( j = 0; j < this->getNumColumns(); j++) {
	 std::cout << getElement(i, j);
    }
    std::cout << "\n";
  }
}

template<class Element_t>
void LUBlockMatrix<Element_t>::displayMatlab(std::ostream& os) {
  os << "[";
  for(int i = 0; i < this->getNumRows(); i++) {
    for(int j = 0; j < this->getNumCols(); j++) {
      os << getElement(i, j);
      if(j < this->getNumCols() - 1) os << ","; else if(i < this->getNumRows()-1) os << ";"; else os << "]";
    }
  }
  os << "\n";
}
