/* LUCancelFullMatrix.h
 * --------------------
 * Defines a full off-diagonal matrix used in block LU-decomposition.
 *
 * Written by Patrick Perry
 * Original Version on August 14, 2003
 *
 * $Id: LUCancelFullMatrix.h,v 1.2 2005/11/02 01:59:52 patperry Exp $
 * $Log
 *
 */

#ifndef _LUCANCELFULLMATRIX_H
#define _LUCANCELFULLMATRIX_H

#include <memory>
#include "FullMatrix.h"

template <class Element_t, class Alloc>
class LUCancelFullMatrix;
#include "LUFullMatrix.h"


/**
 * An LUCancelFullMatrix is a class to "cancel" rows of one matrix with
 * rows of another.  It needs information from two other matrices.  
 * pivotMatrix is a matrix with the same number of rows, and pivots in 
 * the first N positions.  cancelor is a matrix of the same number of
 * columns, and pivots in the first M position.  
 *
 * When cancel is called, this cancels out the rows that aren't pivots 
 * of pivotMatrix using pivots from cancelor.  All the moves performed 
 * are then stored in the lower-left corner of size (numRows - N ) x M.  
 * That is, if k * row i from cancelor was added to row j, element (i, j)
 * will be (-k).
 */
template <class Element_t, class Alloc = std::allocator<void> >
class LUCancelFullMatrix : public FullMatrix<Element_t, Alloc>
{
public:

  //! row iterator type
  typedef typename FullMatrix<Element_t, Alloc>::RowIterator RowIterator;

  //! column iterator type
  typedef typename FullMatrix<Element_t, Alloc>::ColumnIterator ColumnIterator;

private:

  //! indicates which rows have pivots
  const LUFullMatrix<Element_t, Alloc> * pivotMatrix;

  //! used to cancel the nonpivot rows
  const LUFullMatrix<Element_t, Alloc> * cancelor;
  
  //! indicates whether or not the matrix has been canceled
  bool hasBeenCanceled;

protected:

  using FullMatrix<Element_t, Alloc>::IntAlloc;
  using FullMatrix<Element_t, Alloc>::ElementAlloc;
  using FullMatrix<Element_t, Alloc>::numRows;
  using FullMatrix<Element_t, Alloc>::numColumns;
  using FullMatrix<Element_t, Alloc>::rowPermutationInverse;
  using FullMatrix<Element_t, Alloc>::columnPermutationInverse;
  using FullMatrix<Element_t, Alloc>::data;
  
public:
  
  /**
   * Constructor.  Initializes the cancel matrix with the given pivotMatrix
   * and cancelor.
   *
   * \param pivotMatrix the matrix to get pivot information from.
   * \param cancelor the matrix to cancel this one with.
   */
  LUCancelFullMatrix (const LUFullMatrix<Element_t, Alloc> * pivotMatrix,
				  const LUFullMatrix<Element_t, Alloc> * cancelor);

  /**
   * Constructor.  Initializes the cancel matrix with the given pivotMatrix
   * and cancelor.  The matrix data is taken from the passed-in matrix.
   *
   * \param the matrix whose data is used to initialize this.
   * \param pivotMatrix the matrix to get pivot information from.
   * \param cancelor the matrix to cancel this one with.
   */
  template <class ColumnIterator_t, class Alloc1>
  LUCancelFullMatrix (const Matrix<Element_t, 
				               ColumnIterator_t, Alloc1>& matrix,
				  const LUFullMatrix<Element_t, Alloc> * pivotMatrix,
				  const LUFullMatrix<Element_t, Alloc> * cancelor);

  /**
   * Destructor.
   */
  virtual ~LUCancelFullMatrix ();

  /**
   * Indicates whether or not the matrix has been canceled yet.
   *
   * \returns a bool indicating the cancel status.
   */
  bool isCanceled () const;

  /**
   * Cancels the matrix using the pivots in the cancelor.  See class
   * description for more details as to what this does.
   *
   * \returns void.
   */
  void cancel ();

  /**
   * Get the number of pivots in the pivot matrix.
   *
   * \returns the number of pivots.
   */
  int getNumRowPivots () const;

  /**
   * Get the number of pivots in the cancelor.
   *
   * \returns the number of pivots.
   */
  int getNumColumnPivots () const;

  /**
   * Returns the element at the given position.  Calling this function
   * cancels the matrix if it has not already been canceled.
   *
   * \param i the row index of the element.
   * \param j the column index of the element.
   * \returns the element stored at (i, j).
   */
  Element_t getElement (int i, int j) const;

  /**
   * Set the element at the specified position to the given value.  It
   * is an error to call this function after the matrix has been canceled.
   *
   * \param i the row index of the element.
   * \param j the column index of the element.
   * \param value the value to set the element to.
   * \returns void.
   */
  void setElement (int i, int j, Element_t value);

  /**
   * Get a row iterator to the begin of a row.  This iterator
   * is read-only.  Calling this function causes the matrix to cancel
   * itself if it has not already done so.
   *
   * \param row the index of the row to get.
   * \returns a RowIterator to the begin of the requested row.
   */
  RowIterator getRowBegin (int row) const;

  /**
   * Get a row iterator to the end of a row.  This iterator
   * is read-only.  Calling this function causes the matrix to cancel
   * itself if it has not already done so.
   *
   * \param row the index of the row to get.
   * \returns a RowIterator to the end of the requested row.
   */
  RowIterator getRowEnd (int row) const;

  /**
   * Get a column iterator to the begin of a column.  This iterator
   * is read-only.  Calling this function causes the matrix to cancel
   * itself if it has not already done so.
   *
   * \param column the index of the column to get.
   * \returns a ColumnIterator to the begin of the requested column.
   */
  ColumnIterator getColumnBegin (int column) const;

  /**
   * Get a column iterator to the end of a column.  Calling this function
   * causes the matrix to cancel itself if it has not already done so.
   *
   * \param column the index of the column to get.
   * \returns a ColumnIterator to the end of the requested column.
   */
  ColumnIterator getColumnEnd (int column) const;

  /**
   * Fills a column with the given values.  It is an error to call this
   * function if the matrix has already been canceled.
   *
   * \param column the column to fill.
   * \param values an (Element) iterator to numRows values.
   * \returns void.
   */
  template <class InputIterator>
  void fillColumn (int column, InputIterator values);

  /**
   * Write the matrix to an output stream.  Calling this function causes
   * the matrix to cancel itself if it has not already done so.
   *
   * \param s an output stream.
   * \returns the output stream.
   */
  virtual std::ostream& put (std::ostream& s) const;

  /**
   * Applies a permutation to the rows of the matrix.  It is an error
   * to call this function if the matrix has already been canceled.
   * 
   * \param permutation an int array of length numRows representing
   * the permutation where row i is sent to permutation[i].
   * \returns void.
   */
  virtual void permuteRows (const int * permutation);

  /**
   * Swap the given rows in the matrix.  Unlike the definition in
   * the class parent, this actually moves the data around.  It is an
   * error to call this function if the matrix has already been canceled.
   *
   * \param i the index of the first row to swap.
   * \param j the index of the second row to swap.
   * \returns void.
   */
  virtual void swapRows (int i, int j);

  /**
   * Applies a permutation to the colums of the matrix. It is an error
   * to call this function if the matrix has already been canceled.
   * 
   * \param permutation an int array of length numColumns representing
   * the permutation where column i is sent to permutation[i].
   * \returns void.
   */
  virtual void permuteColumns (const int * permutation);

  /**
   * Swap the given columns in the matrix.  Unlike the definition in
   * the class parent, this actually moves the data around.  It is an
   * error to call this function if the matrix has already been canceled.
   *
   * \param i the index of the first column to swap.
   * \param j the index of the second column to swap.
   * \returns void.
   */
  virtual void swapColumns (int i, int j);

  /**
   * Adds a scalar multiple of an external vector to a row.  It is an error
   * to call this function after the matrix has been canceled.
   *
   * \param scalar the scalar to multiply the vector by.
   * \param vectorBegin an iterator pointing to the begin if the vector.
   * \param row the index of the row to add the vector to.
   * \returns void.
   */
  template <class InputIterator>
  void addScalarTimesVectorToRow (Element_t scalar, InputIterator vectorBegin,
						    int row);

protected:

  /**
   * Performs the row operation (row2 := row2 + scalar * row1).  This 
   * function should only be called before the matrix is actually cancelled.
   *
   * \param scalar the number to multiply the first row by.
   * \param row1 the row to use for the scalar multiple.
   * \param row2 the target of the operation.
   * \returns void.
   */
  virtual void addScalarTimesRowToRow (Element_t scalar, int row1, int row2);

  /**
   * Cancels the given row by multiplying the pivotRow in the
   * cancelor by scalar and adding it to the row.
   *
   * \param row the row to cancel.
   * \param pivotRow the row in the cancelor to use.
   * \param scalar the scalar to multiply pivotRow by when adding it to row.
   */
  virtual void cancelRow (int row, int pivotRow, Element_t scalar);

};


// get function definitions
#include "LUCancelFullMatrix.cc"


#endif // _LUCANCELFULLMATRIX_H

