/* LUFullMatrix.cc
 * ---------------
 * Implements LUFullMatrix.h
 *
 * Written by Patrick Perry
 * Original Version on August 13, 2003
 *
 * $Id: LUFullMatrix.cc,v 1.2 2005/11/02 01:59:52 patperry Exp $
 * $Log: LUFullMatrix.cc,v $
 * Revision 1.2  2005/11/02 01:59:52  patperry
 * The last (?) gcc4.0 fixes\!\!\!
 *
 * Revision 1.1.1.1  2005/03/18 23:36:19  silva
 * Imported Sources
 *
 * Revision 1.1.1.1  2004/09/25 22:04:08  patperry
 * Imported sources
 *
 * Revision 1.10  2004/01/15 07:13:36  patperry
 * Changed file extensions.
 *
 * Revision 1.9  2003/12/02 00:54:02  patperry
 * Mod2 pivot specialization
 *
 * Revision 1.8  2003/11/30 22:45:15  patperry
 * changed reduction to do complete pivoting
 *
 * Revision 1.7  2003/11/22 06:54:14  patperry
 * lotsa fixes.
 *
 * Revision 1.6  2003/10/28 04:43:51  patperry
 * Preliminary version of SelectionMaskColumnIterator.
 * Copy constructors for all full matrix types.
 *
 * Revision 1.5  2003/10/25 00:49:32  patperry
 * Added RowIterators.  Make RowColumnIterator have virtual functions.
 *
 * Revision 1.4  2003/10/24 08:56:06  patperry
 * Filled in diagonal block and got rid of LUMatrixBlock
 *
 * Revision 1.3  2003/10/24 08:16:05  patperry
 * LUFullMatrix works better now.
 *
 * Revision 1.2  2003/10/24 04:46:02  patperry
 * Overhaul of LUFullMatrix.  It may not compile/function yet.
 *
 *
 */

#include <cassert>
#include <numeric>
#include <cmath>
#include "Mod2.h"


template <class Element_t, class Alloc>
LUFullMatrix<Element_t, Alloc>::
LUFullMatrix (int numRows, int numColumns)
  : FullMatrix<Element_t, Alloc>(numRows, numColumns),
    P(NULL), Q(NULL), hasBeenReduced(false)
{
}


template<class Element_t, class Alloc>
template <class ColumnIterator_t, class Alloc1>
LUFullMatrix<Element_t, Alloc>::
LUFullMatrix (const Matrix<Element_t, ColumnIterator_t, Alloc1>& matrix)
  : FullMatrix<Element_t, Alloc>(matrix),
    P(NULL), Q(NULL), hasBeenReduced(false)
{
}


template <class Element_t, class Alloc>
LUFullMatrix<Element_t, Alloc>::
~LUFullMatrix ()
{
  if (P) {
    IntAlloc.deallocate(P, numRows);
  }

  if (Q) {
    IntAlloc.deallocate(Q, numColumns);
  }
}


template <class Element_t, class Alloc>
bool
LUFullMatrix<Element_t, Alloc>::
isReduced () const
{
  return hasBeenReduced;
}


/*
 * This code is based on the algorithm in Golub and Van Loan's
 * _Matrix Computation_, 3rd Ed.
 */
template <class Element_t, class Alloc>
void 
LUFullMatrix<Element_t, Alloc>::
reduce ()
{
  if (!hasBeenReduced) {
    this->reducePermutations();

    int maxNumPivots = std::min(numRows, numColumns);
    int rowTranspositions[maxNumPivots];
    int columnTranspositions[maxNumPivots];
    int numPivots = 0;
    Element_t pivotValue;
    int pivotRow;
    int pivotColumn;

    for (numPivots = 0; numPivots < maxNumPivots; ++numPivots) {
	 if (!(pivotValue = findPivot(numPivots, pivotRow, pivotColumn))) {
	   break;
	 }

	 swapRows(numPivots, pivotRow);
	 swapColumns(numPivots, pivotColumn);
	 rowTranspositions[numPivots] = pivotRow;
	 columnTranspositions[numPivots] = pivotColumn;

	 // cancel the rows below the pivot
	 for (int row = numPivots + 1; row < numRows; ++row) {
	   Element_t rowValue 
		= FullMatrix<Element_t, Alloc>::getElement(row, numPivots);
	   
	   if (rowValue != static_cast<Element_t>(0)) {
		Element_t scalar = -rowValue / pivotValue;
		
		addScalarTimesPivotToRow(scalar, numPivots, row);
	   }
	 }
    }

    // set the rank
    this->numPivots = numPivots;

    // calculate P and Q
    P = IntAlloc.allocate(numRows);
    for (int i = 0; i < numRows; ++i) {
	 P[i] = i;
    }

    Q = IntAlloc.allocate(numColumns);
    for (int i = 0; i < numColumns; ++i) {
	 Q[i] = i;
    }

    for (int i = numPivots - 1; i >= 0; --i) {
	 std::swap(P[i], P[rowTranspositions[i]]);
	 std::swap(Q[i], Q[columnTranspositions[i]]);
    }

    hasBeenReduced = true;
  }
}


template <class Element_t, class Alloc>
int
LUFullMatrix<Element_t, Alloc>::
getNumPivots () const
{
  if (!hasBeenReduced) {
    const_cast<LUFullMatrix *>(this)->reduce();
  }
  
  return numPivots;
}


template <class Element_t, class Alloc>
const int * 
LUFullMatrix<Element_t, Alloc>::
getP () const
{
  if (!hasBeenReduced) {
    const_cast<LUFullMatrix<Element_t, Alloc> *>(this)->reduce();
  }

  return P;
}


template <class Element_t, class Alloc>
const int * 
LUFullMatrix<Element_t, Alloc>::
getQ () const
{
  if (!hasBeenReduced) {
    const_cast<LUFullMatrix<Element_t, Alloc> *>(this)->reduce();
  }

  return Q;
}


template <class Element_t, class Alloc>
Element_t 
LUFullMatrix<Element_t, Alloc>::
getElement (int i, int j) const
{
  if (!hasBeenReduced) {
    const_cast<LUFullMatrix<Element_t, Alloc> *>(this)->reduce();
  }

  return FullMatrix<Element_t, Alloc>::getElement(i, j);
}


template <class Element_t, class Alloc>
void
LUFullMatrix<Element_t, Alloc>::
setElement (int i, int j, Element_t value) 
{
  assert(!hasBeenReduced);
  FullMatrix<Element_t, Alloc>::setElement(i, j, value);
}


template <class Element_t, class Alloc>
typename LUFullMatrix<Element_t, Alloc>::RowIterator 
LUFullMatrix<Element_t, Alloc>::
getRowBegin (int row) const
{
  if (!hasBeenReduced) {
    const_cast<LUFullMatrix<Element_t, Alloc> *>(this)->reduce();
  }
  
  return FullMatrix<Element_t, Alloc>::getRowBegin(row);
}


template <class Element_t, class Alloc>
typename LUFullMatrix<Element_t, Alloc>::RowIterator 
LUFullMatrix<Element_t, Alloc>::
getRowEnd (int row) const
{
  if (!hasBeenReduced) {
    const_cast<LUFullMatrix<Element_t, Alloc> *>(this)->reduce();
  }
  
  return FullMatrix<Element_t, Alloc>::getRowEnd(row);
}


template <class Element_t, class Alloc>
typename LUFullMatrix<Element_t, Alloc>::ColumnIterator 
LUFullMatrix<Element_t, Alloc>::
getColumnBegin (int column) const
{
  if (!hasBeenReduced) {
    const_cast<LUFullMatrix<Element_t, Alloc> *>(this)->reduce();
  }
  
  return FullMatrix<Element_t, Alloc>::getColumnBegin(column);
}


template <class Element_t, class Alloc>
typename LUFullMatrix<Element_t, Alloc>::ColumnIterator 
LUFullMatrix<Element_t, Alloc>::
getColumnEnd (int column) const
{
  if (!hasBeenReduced) {
    const_cast<LUFullMatrix<Element_t, Alloc> *>(this)->reduce();
  }
  
  return FullMatrix<Element_t, Alloc>::getColumnEnd(column);
}


template <class Element_t, class Alloc>
template <class InputIterator>
void 
LUFullMatrix<Element_t, Alloc>::
fillColumn (int column, InputIterator values)
{
  assert(!hasBeenReduced);
  FullMatrix<Element_t, Alloc>::fillColumn(column, values);
}


template <class Element_t, class Alloc>
std::ostream& 
LUFullMatrix<Element_t, Alloc>::
put (std::ostream& s) const
{
  if (!hasBeenReduced) {
    const_cast<LUFullMatrix<Element_t, Alloc> *>(this)->reduce();
  }
  
  return FullMatrix<Element_t, Alloc>::put(s);
}


template <class Element_t, class Alloc>
void
LUFullMatrix<Element_t, Alloc>::
permuteRows (const int * permutation)
{
  assert(!hasBeenReduced);
  FullMatrix<Element_t, Alloc>::permuteRows(permutation);
}


template <class Element_t, class Alloc>
void
LUFullMatrix<Element_t, Alloc>::
swapRows (int i, int j)
{
  assert(!hasBeenReduced);
  FullMatrix<Element_t, Alloc>::swapRows(i, j);
}


template <class Element_t, class Alloc>
void
LUFullMatrix<Element_t, Alloc>::
permuteColumns (const int * permutation)
{
  assert(!hasBeenReduced);
  FullMatrix<Element_t, Alloc>::permuteColumns(permutation);
}


template <class Element_t, class Alloc>
void
LUFullMatrix<Element_t, Alloc>::
swapColumns (int i, int j)
{
  assert(!hasBeenReduced);
  FullMatrix<Element_t, Alloc>::swapColumns(i, j);
}


template <class Element_t, class Alloc>
Element_t
LUFullMatrix<Element_t, Alloc>::
findPivot (int start, int& row, int& column) const
{
  const Element_t * currentElement = data + start + start * numRows;
  Element_t value = 0;
  Element_t norm = 0;
  Element_t newValue;
  Element_t newNorm;

  for (int j = start; j < numColumns; ++j) {
    for (int i = start; i < numRows; ++i) {
	 newValue = *currentElement++;
	 newNorm = std::abs(newValue);

	 if (newNorm > norm) {
	   value = newValue;
	   norm = newNorm;
	   row = i;
	   column = j;
	 }
    }
    currentElement += start;
  }

  return value;
}


// template <class Alloc>
// Mod2
// LUFullMatrix<Mod2, Alloc>::
// findPivot (int start, int& row, int& column) const
// {
//   const Mod2 * currentElement = data + start + start * numRows;
//   Mod2 newValue;
//
//   for (int j = start; j < numColumns; ++j) {
//     for (int i = start; i < numRows; ++i) {
// 	 newValue = *currentElement++;
// 	 if (newValue != 0) {
// 	   row = i;
// 	   column = j;
// 	   return newValue;
// 	 }
//     }
//     currentElement += start;
//   }
//
//   // no nonzero entry found
//   return 0;
// }


template <class Element_t, class Alloc>
void
LUFullMatrix<Element_t, Alloc>::
addScalarTimesPivotToRow (Element_t scalar, int pivot, int row)
{
  for (int j = pivot; j < this->getNumColumns(); ++j) {
    setElement(row, j, FullMatrix<Element_t, Alloc>::getElement(row, j)
			+ scalar * FullMatrix<Element_t, Alloc>::getElement(pivot, j));
  }

  FullMatrix<Element_t, Alloc>::setElement(row, pivot, -scalar);
}

