/* LUFullMatrix.h
 * --------------
 * Written by Patrick Perry
 * Original Version on August 13, 2003
 *
 * $Id: LUFullMatrix.h,v 1.2 2005/11/02 01:59:52 patperry Exp $
 * $Log: LUFullMatrix.h,v $
 * Revision 1.2  2005/11/02 01:59:52  patperry
 * The last (?) gcc4.0 fixes\!\!\!
 *
 * Revision 1.1.1.1  2005/03/18 23:36:20  silva
 * Imported Sources
 *
 * Revision 1.1.1.1  2004/09/25 22:04:08  patperry
 * Imported sources
 *
 * Revision 1.1  2004/01/15 07:13:36  patperry
 * Changed file extensions.
 *
 * Revision 1.8  2003/11/30 22:45:15  patperry
 * changed reduction to do complete pivoting
 *
 * Revision 1.7  2003/11/22 06:54:14  patperry
 * lotsa fixes.
 *
 * Revision 1.6  2003/10/28 04:43:51  patperry
 * Preliminary version of SelectionMaskColumnIterator.
 * Copy constructors for all full matrix types.
 *
 * Revision 1.5  2003/10/25 00:49:32  patperry
 * Added RowIterators.  Make RowColumnIterator have virtual functions.
 *
 * Revision 1.4  2003/10/24 08:56:06  patperry
 * Filled in diagonal block and got rid of LUMatrixBlock
 *
 * Revision 1.3  2003/10/24 08:16:05  patperry
 * LUFullMatrix works better now.
 *
 * Revision 1.2  2003/10/24 04:46:02  patperry
 * Overhaul of LUFullMatrix.  It may not compile/function yet.
 *
 *
 */

#ifndef _LUFULLMATRIX_H
#define _LUFULLMATRIX_H

#include <memory>
#include "FullMatrix.h"

template <class Element_t, class Alloc>
class LUFullMatrix;
#include "LUCancelFullMatrix.h"


/** 
 * Class for finding the an LU-decomposition of a matrix.
 * After calling reduce, or through lazy evaluation, this stores
 * L and U of a matrix decomposition L * U = P * X * Q in place.
 * The class also allows access to the permutations P and Q.
 */
template <class Element_t, class Alloc = std::allocator<void> >
class LUFullMatrix : public FullMatrix<Element_t, Alloc>
{
public:
  
  //! row iterator type
  typedef typename FullMatrix<Element_t, Alloc>::RowIterator RowIterator;

  //! column iterator type
  typedef typename FullMatrix<Element_t, Alloc>::ColumnIterator ColumnIterator;

private:

  //! Value for indicating no pivot found in a row.
  static const int NO_PIVOT = -1;

  //! the number of pivots in the LU-decomposition.
  int numPivots;

  //! the row permutation.
  int * P;

  //! the columnPermutation.
  int * Q;

  //! indicates whether or not the matrix has been reduced.
  bool hasBeenReduced;

protected:

  using FullMatrix<Element_t, Alloc>::IntAlloc;
  using FullMatrix<Element_t, Alloc>::ElementAlloc;
  using FullMatrix<Element_t, Alloc>::numRows;
  using FullMatrix<Element_t, Alloc>::numColumns;
  using FullMatrix<Element_t, Alloc>::rowPermutationInverse;
  using FullMatrix<Element_t, Alloc>::columnPermutationInverse;
  using FullMatrix<Element_t, Alloc>::data;

public:

  /**
   * Constructor.  Constructs a zero matrix of the given dimensions.
   */
  LUFullMatrix (int numRows, int numColumns);

  /**
   * Constructor.  Constructs a matrix out of the data in the
   * passed-in matrix.
   *
   * \param matrix a matrix to copy.
   */
  template <class ColumnIterator_t, class Alloc1>
  LUFullMatrix (const Matrix<Element_t, ColumnIterator_t, Alloc1>& matrix);

  /**
   * Destructor.
   */
  virtual ~LUFullMatrix ();

  /**
   * Indicates whether or not the matrix has been reduced.
   *
   * \returns an indicator as to whether or not the matrix has been reduced.
   */
  bool isReduced () const;

  /**
   * Uses Gaussian elimination to put the matrix into 
   * LU-decompostion form.
   *
   * \returns void.
   */
  void reduce ();

  /**
   * Get the number of pivots in the decomposition.  If the matrix has
   * not already been reduced, calling this function causes the matrix
   * to reduce itself.
   *
   * \returns the number of pivots.
   */
  int getNumPivots () const;

  /**
   * Get the row permutation associated with the decomposition.  The 
   * permuation is an array of length numRows.  It sends element i
   * to element P[i].
   *
   * \returns the permutation array, P.
   */
  const int * getP () const;

  /**
   * Get the column permutation associated with the decomposition.  The 
   * permuation is an array of length numColumns.  It sends element i
   * to element Q[i].
   *
   * \returns the permutation array, Q.
   */
  const int * getQ () const;

  /**
   * Returns the element at the given position.  Calling this function
   * reduces the matrix if it has not already been reduced.
   *
   * \param i the row index of the element.
   * \param j the column index of the element.
   * \returns the element stored at (i, j).
   */
  Element_t getElement (int i, int j) const;

  /**
   * Set the element at the specified position to the given value.  It
   * is an error to call this function after the matrix has been reduced.
   *
   * \param i the row index of the element.
   * \param j the column index of the element.
   * \param value the value to set the element to.
   * \returns void.
   */
  void setElement (int i, int j, Element_t value);

  /**
   * Get a row iterator to the begin of a row.  This iterator
   * is read-only.  Calling this function causes the matrix to reduce
   * itself if it has not already done so.
   *
   * \param row the index of the row to get.
   * \returns a RowIterator to the begin of the requested row.
   */
  RowIterator getRowBegin (int row) const;

  /**
   * Get a row iterator to the end of a row.  This iterator
   * is read-only.  Calling this function causes the matrix to reduce
   * itself if it has not already done so.
   *
   * \param row the index of the row to get.
   * \returns a RowIterator to the end of the requested row.
   */
  RowIterator getRowEnd (int row) const;

  /**
   * Get a column iterator to the begin of a column.  This iterator
   * is read-only.  Calling this function causes the matrix to reduce
   * itself if it has not already done so.
   *
   * \param column the index of the column to get.
   * \returns a ColumnIterator to the begin of the requested column.
   */
  ColumnIterator getColumnBegin (int column) const;

  /**
   * Get a column iterator to the end of a column.  Calling this function
   * causes the matrix to reduce itself if it has not already done so.
   *
   * \param column the index of the column to get.
   * \returns a ColumnIterator to the end of the requested column.
   */
  ColumnIterator getColumnEnd (int column) const;

  /**
   * Fills a column with the given values.  It is an error to call this
   * function if the matrix has already been reduced.
   *
   * \param column the column to fill.
   * \param values an (Element) iterator to numRows values.
   * \returns void.
   */
  template <class InputIterator>
  void fillColumn (int column, InputIterator values);

  /**
   * Write the matrix to an output stream.  Calling this function causes
   * the matrix to reduce itself if it has not already done so.
   *
   * \param s an output stream.
   * \returns the output stream.
   */
  virtual std::ostream& put (std::ostream& s) const;

  /**
   * Applies a permutation to the rows of the matrix.  It is an error
   * to call this function if the matrix has already been reduced.
   * 
   * \param permutation an int array of length numRows representing
   * the permutation where row i is sent to permutation[i].
   * \returns void.
   */
  virtual void permuteRows (const int * permutation);

  /**
   * Swap the given rows in the matrix.  Unlike the definition in
   * the class parent, this actually moves the data around.  It is an
   * error to call this function if the matrix has already been reduced.
   *
   * \param i the index of the first row to swap.
   * \param j the index of the second row to swap.
   * \returns void.
   */
  virtual void swapRows (int i, int j);

  /**
   * Applies a permutation to the colums of the matrix. It is an error
   * to call this function if the matrix has already been reduced.
   * 
   * \param permutation an int array of length numColumns representing
   * the permutation where column i is sent to permutation[i].
   * \returns void.
   */
  virtual void permuteColumns (const int * permutation);

  /**
   * Swap the given columns in the matrix.  Unlike the definition in
   * the class parent, this actually moves the data around.  It is an
   * error to call this function if the matrix has already been reduced.
   *
   * \param i the index of the first column to swap.
   * \param j the index of the second column to swap.
   * \returns void.
   */
  virtual void swapColumns (int i, int j);

protected:

  /**
   * Finds a pivot in a submatrix of the matrix.
   *
   * \param start the starting row and column to look in
   * \param row the row of the pivot
   * \param column the column of the pivot
   * \returns the value at the pivot
   */
  Element_t findPivot (int start, int& row, int& column) const;

  /**
   * Assuming there is a pivot in the given row, add a scalar multiple
   * of that row to another row.
   *
   * \param scalar the scalar to multiply the pivot row by.
   * \param pivot the index of the pivot row.
   * \param row the index of the row to add the pivot row to.
   * \returns void.
   */
  virtual void addScalarTimesPivotToRow (Element_t scalar, int pivot, int row);

  // friend classes
  friend class LUCancelFullMatrix<Element_t, Alloc>;

};

// let compiler see definitions
#include "LUFullMatrix.cc"


#endif // _LUFULLMATRIX_H
