/* Matrix.cc
 * ---------
 * Implements Matrix.h.
 *
 * Written by Patrick Perry
 * Original Version on August 12, 2003
 *
 * $Id: Matrix.cc,v 1.2 2005/10/24 16:20:08 patperry Exp $
 * $Log: Matrix.cc,v $
 * Revision 1.2  2005/10/24 16:20:08  patperry
 * Hells (literally) of compile fixes for gcc4.0
 *
 * Revision 1.1.1.1  2005/03/18 23:36:22  silva
 * Imported Sources
 *
 * Revision 1.1.1.1  2004/09/25 22:04:08  patperry
 * Imported sources
 *
 * Revision 1.8  2004/01/15 07:13:36  patperry
 * Changed file extensions.
 *
 * Revision 1.7  2003/12/02 12:39:11  patperry
 * added code to combine the four blocks
 *
 * Revision 1.6  2003/11/22 06:54:14  patperry
 * lotsa fixes.
 *
 * Revision 1.5  2003/10/24 02:16:48  patperry
 * Add FullMatrix and FullColumnIterator.
 *
 * Revision 1.4  2003/10/22 10:52:36  patperry
 * Implemented and tested ColumnIterator for sparse matrices.
 *
 * Revision 1.3  2003/10/22 07:37:18  patperry
 * More work on column iterators.
 *
 * Revision 1.2  2003/10/19 02:30:23  patperry
 * New and improved matrix class.
 *
 *
 */

#include <algorithm>
#include <cstddef>

// static member instantiation
template <class Element_t, class ColumnIterator_t, class Alloc>
typename Alloc::template rebind<int>::other 
Matrix<Element_t, ColumnIterator_t, Alloc>::IntAlloc;
	  

template <class Element_t, class ColumnIterator_t, class Alloc>
typename Alloc::template rebind<Element_t>::other 
Matrix<Element_t, ColumnIterator_t, Alloc>::ElementAlloc;


template <class Element_t, class ColumnIterator_t, class Alloc>
Matrix<Element_t, ColumnIterator_t, Alloc>::
Matrix (int numRows, int numColumns)
  : numRows(numRows), numColumns(numColumns), 
    rowPermutationInverse(NULL), columnPermutationInverse(NULL)
{ 
}


template <class Element_t, class ColumnIterator_t, class Alloc>
Matrix<Element_t, ColumnIterator_t, Alloc>::
~Matrix ()
{ 
  if (rowPermutationInverse) {
    IntAlloc.deallocate(rowPermutationInverse, numRows);
  }

  if (columnPermutationInverse) {
    IntAlloc.deallocate(columnPermutationInverse, numColumns);
  }
}


template <class Element_t, class ColumnIterator_t, class Alloc>
int
Matrix<Element_t, ColumnIterator_t, Alloc>::
getNumRows () const
{
  return numRows;
}


template <class Element_t, class ColumnIterator_t, class Alloc>
int
Matrix<Element_t, ColumnIterator_t, Alloc>::
getNumColumns () const
{
  return numColumns;
}


template <class Element_t, class ColumnIterator_t, class Alloc>
int
Matrix<Element_t, ColumnIterator_t, Alloc>::
getNumNonzeroElements () const
{
  int result = 0;
  
  for (int column = 0; column < numColumns; ++column) {
    ColumnIterator_t it = getColumnBegin(column);
    ColumnIterator_t end = getColumnEnd(column);

    while (it != end) {
	 if (*it++) {
	   ++result;
	 }
    }
  }

  return result;
}


template <class Element_t, class ColumnIterator_t, class Alloc>
std::ostream&
Matrix<Element_t, ColumnIterator_t, Alloc>::
put (std::ostream& s) const
{
  for (int i = 0; i < getNumRows(); ++i) {
    if (getNumColumns() > 0) {
	 s << getElement(i, 0);
	 for (int j = 1; j < getNumColumns(); ++j) {
	   s << "\t" << getElement(i, j);
	 }
	 s << std::endl;
    }
  }
  
  return s;
}


template <class Element_t, class ColumnIterator_t, class Alloc>
void 
Matrix<Element_t, ColumnIterator_t, Alloc>::
permuteRows (const int * permutation)
{
  // if there is no permutation, allocate one and set it to be
  // the passed-in one
  if (!rowPermutationInverse) {
    rowPermutationInverse = IntAlloc.allocate(numRows);

    for (int i = 0; i < numRows; ++i) {
	 rowPermutationInverse[permutation[i]] = i;
    }

  // otherwise, compose the existing permutation with the passed-in one.
  } else {
    int newRowPermutationInverse[numRows];

    for (int i = 0; i < numRows; ++i) {
	 newRowPermutationInverse[permutation[i]] = rowPermutationInverse[i];
    }

    std::copy(newRowPermutationInverse, newRowPermutationInverse + numRows,
		    rowPermutationInverse);
  }
}


template <class Element_t, class ColumnIterator_t, class Alloc>
void 
Matrix<Element_t, ColumnIterator_t, Alloc>::
swapRows (int i, int j)
{
  if (i != j) {
    // if there is no permutation, allocate one and initialize it
    // to be the trivial permutation.
    if (!rowPermutationInverse) {
	 rowPermutationInverse = IntAlloc.allocate(numRows);
	 
	 for (int k = 0; k < numRows; ++k) {
	   rowPermutationInverse[k] = k;
	 }
    } 
    
    // apply the swap
    std::swap(rowPermutationInverse[i], rowPermutationInverse[j]);
  }
}


template <class Element_t, class ColumnIterator_t, class Alloc>
void 
Matrix<Element_t, ColumnIterator_t, Alloc>::
permuteColumns (const int * permutation)
{
  // if there is no permutation, allocate one and set it to be
  // the passed-in one
  if (!columnPermutationInverse) {
    columnPermutationInverse = IntAlloc.allocate(numColumns);

    for (int i = 0; i < numColumns; ++i) {
	 columnPermutationInverse[permutation[i]] = i;
    }

  // otherwise, compose the existing permutation with the passed-in one.
  } else {
    int newColumnPermutationInverse[numColumns];

    for (int i = 0; i < numColumns; ++i) {
	 newColumnPermutationInverse[permutation[i]] 
	   = columnPermutationInverse[i];
    }

    std::copy(newColumnPermutationInverse, 
		    newColumnPermutationInverse + numColumns, 
		    columnPermutationInverse);
  }
}


template <class Element_t, class ColumnIterator_t, class Alloc>
void 
Matrix<Element_t, ColumnIterator_t, Alloc>::
swapColumns (int i, int j)
{
  if (i != j) {
    // if there is no permutation, allocate one and initialize it
    // to be the trivial permutation.
    if (!columnPermutationInverse) {
	 columnPermutationInverse = IntAlloc.allocate(numColumns);
	 
	 for (int k = 0; k < numColumns; ++k) {
	   columnPermutationInverse[k] = k;
	 }
    } 
    
    // apply the swap
    std::swap(columnPermutationInverse[i], columnPermutationInverse[j]);
  }
}


template <class Element_t, class ColumnIterator_t, class Alloc>
std::ostream& 
operator<< (std::ostream& s, const Matrix<Element_t, 
		                                ColumnIterator_t, Alloc>& m)
{
  return m.put(s);
}

