/* Matrix.h
 * --------
 * Abstract matrix interface.
 *
 * Written by Patrick Perry
 * Original Version on August 12, 2003
 *
 * $Id: Matrix.h,v 1.2 2005/10/24 16:20:08 patperry Exp $
 * $Log: Matrix.h,v $
 * Revision 1.2  2005/10/24 16:20:08  patperry
 * Hells (literally) of compile fixes for gcc4.0
 *
 * Revision 1.1.1.1  2005/03/18 23:36:22  silva
 * Imported Sources
 *
 * Revision 1.1.1.1  2004/09/25 22:04:08  patperry
 * Imported sources
 *
 * Revision 1.2  2004/01/23 04:36:42  patperry
 * Cleaned up MATLAB support code.
 *
 * Revision 1.1  2004/01/15 07:13:36  patperry
 * Changed file extensions.
 *
 * Revision 1.9  2003/12/02 12:39:11  patperry
 * added code to combine the four blocks
 *
 * Revision 1.8  2003/11/22 06:54:14  patperry
 * lotsa fixes.
 *
 * Revision 1.7  2003/10/25 00:49:32  patperry
 * Added RowIterators.  Make RowColumnIterator have virtual functions.
 *
 * Revision 1.6  2003/10/24 02:16:48  patperry
 * Add FullMatrix and FullColumnIterator.
 *
 * Revision 1.5  2003/10/22 10:52:36  patperry
 * Implemented and tested ColumnIterator for sparse matrices.
 *
 * Revision 1.4  2003/10/22 07:37:18  patperry
 * More work on column iterators.
 *
 * Revision 1.3  2003/10/19 02:42:31  patperry
 * Added a ColumnIterator.
 *
 * Revision 1.2  2003/10/19 02:30:23  patperry
 * New and improved matrix class.
 *
 *
 */

#ifndef _MATRIX_H
#define _MATRIX_H

#include <utility>
#include <cstdlib>
#include <iostream>


/**
 * Class for representing of matrix holding elements of arbitrary
 * type.  This is an abstract class, implementing commen code like date
 * queries, along with common code for row and column permutations.
 */
template<class Element_t, class ColumnIterator_t, class Alloc>
class Matrix
{

public:
  
  //! iterator type for accessing the elements of a column.  
  typedef ColumnIterator_t ColumnIterator;

protected:

  //! Memory allocator for ints.
  static typename Alloc::template rebind<int>::other IntAlloc;

  //! Memory allocators for elements
  static typename Alloc::template rebind<Element_t>::other ElementAlloc;

  //! the number of rows in the matrix.
  const int numRows;

  //! the number of columns in the matrix.
  const int numColumns;

  //! the inverse of the row permutation associated with the matrix
  int * rowPermutationInverse;

  //! the inverse of the column permutation assocated witht the matrix
  int * columnPermutationInverse;

public:

  /**
   * Constructor.  Creates a new matrix of the given dimensions.
   */
  Matrix (int numRows, int numColumns);

  /**
   * Destructor.
   */
  virtual ~Matrix ();

  /**
   * Returns the number of rows in the matrix.
   *
   * \returns the number of rows.
   */
  int getNumRows () const;

  /**
   * Returns the number of columns in the matrix.
   *
   * \returns the number of columns.
   */
  int getNumColumns () const;

  /**
   * Returns number of nonzero elements in the matrix.
   *
   * \returns the number of nonzero elements.
   */
  virtual int getNumNonzeroElements () const;

  /**
   * Get the value of a particular element. 
   *
   * \param i the row index of the element.
   * \param j the column index of the element.
   * \returns the element at position (i, j).
   */
  virtual Element_t getElement (int i, int j) const = 0;

  /**
   * Get a column iterator to the begin of a column.  Note that the
   * iterator is read only.
   *
   * \param column the index of the column to get.
   * \returns a ColumnIterator to the begin of the requested column.
   */
  virtual ColumnIterator getColumnBegin (int column) const = 0;

  /**
   * Get a column iterator to the end of a column.
   *
   * \param column the index of the column to get.
   * \returns a ColumnIterator to the end of the requested column.
   */
  virtual ColumnIterator getColumnEnd (int column) const = 0;

  /**
   *  Write the matrix to an output stream.
   *
   * \param s an output stream.
   * \returns the output stream.
   */
  virtual std::ostream& put (std::ostream& s) const;

  /**
   * Applies a permutation to the rows of the matrix. 
   * 
   * \param permutation an int array of length numRows representing
   * the permutation where row i is sent to permutation[i].
   * \returns void.
   */
  void permuteRows (const int * permutation);

  /**
   * Swap two rows in the matrix.
   *
   * \param i the first row index.
   * \param j the second row index.
   * \returns void.
   */
  void swapRows (int i, int j);

  /**
   * Applies a permutation to the colums of the matrix. 
   * 
   * \param permutation an int array of length numColumns representing
   * the permutation where column i is sent to permutation[i].
   * \returns void.
   */
  void permuteColumns (const int * permutation);

  /**
   * Swap two columns in the matrix.
   *
   * \param i the first column index.
   * \param j the second column index.
   * \returns void.
   */
  void swapColumns (int i, int j);

};


/**
 * Output stream operator for matrix objects.
 *
 * \param s an output stream.
 * \param m a matrix.
 * \returns the output stream.
 */
template<class Element_t, class ColumnIterator_t, class Alloc>
std::ostream& 
operator<< (std::ostream& s, 
		  const Matrix<Element_t, ColumnIterator_t, Alloc>& m);
					 

// get template function definitions
#include "Matrix.cc"

#endif // _MATRIX_H
