// NerveComplex.cc
// ---------------
// Implements NerveComplex.h
//
// Author: Patrick Perry
// Date:   July 2, 2003
//

#ifndef _NERVECOMPLEX_CC
#define _NERVECOMPLEX_CC

#include <algorithm>
#include <iterator>
#include "NerveComplex.h"


NerveComplex::
NerveComplex (const CRCovering& crCovering)
{
  int numRows = crCovering.getNumRows();
  int numColumns = crCovering.getNumColumns();
  SimplexSet * rowSets = new SimplexSet[numRows];
  SimplexSet * columnSets = new SimplexSet[numColumns];

  computeSimplexSets(rowSets, columnSets, highestSimplexDimension,
				 crCovering);
  
  // sort the simplex sets
  std::stable_sort(rowSets, rowSets + numRows, compareSimplexSets);
  std::stable_sort(columnSets, columnSets + numColumns, 
			    compareSimplexSets);

  // allocate space for the simplices
  simplices = new NerveSimplex[numRows + numColumns];
  numSimplices = 0;
  simplexDimensionOffsets = new int[highestSimplexDimension + 2];
  rowSimplexIndices = new int[numRows];
  columnSimplexIndices = new int[numColumns];

  createSimplices(rowSets, rowSets + numRows, 
			   columnSets, columnSets + numColumns);

  computeSimplexFaces ();

  delete[] rowSets;
  delete[] columnSets;

}


NerveComplex::
~NerveComplex ()
{
  delete[] simplices;
  delete[] simplexDimensionOffsets;
  delete[] rowSimplexIndices;
  delete[] columnSimplexIndices;
}


int
NerveComplex::
getHighestSimplexDimension () const
{
  return highestSimplexDimension;
}


int
NerveComplex::
getNumSimplices () const
{
  return numSimplices;
}


const NerveSimplex& 
NerveComplex::
getRowSimplex (int row) const
{
  return simplices[rowSimplexIndices[row]];
}


const NerveSimplex& 
NerveComplex::
getColumnSimplex (int column) const
{
  return simplices[columnSimplexIndices[column]];
}


const NerveSimplex& 
NerveComplex::
getSimplex (int index) const
{
  return simplices[index];
}


NerveComplex::const_iterator 
NerveComplex::
simplicesBegin () const
{
  return simplices;
}


NerveComplex::const_iterator 
NerveComplex::
simplicesEnd () const
{
  return simplices + numSimplices;
}


int
NerveComplex::
getNumDimensionNSimplices (int n) const
{
  return (simplexDimensionOffsets[highestSimplexDimension - n + 1]
		- simplexDimensionOffsets[highestSimplexDimension - n]);
}


NerveComplex::const_iterator 
NerveComplex::
dimensionNSimplicesBegin (int n) const
{
  return simplices + simplexDimensionOffsets[highestSimplexDimension - n];
								    
}


NerveComplex::const_iterator 
NerveComplex::
dimensionNSimplicesEnd (int n) const
{
  return simplices + simplexDimensionOffsets[highestSimplexDimension - n + 1];
}


std::ostream&
NerveComplex::
put (std::ostream& s) const
{
  s << getNumSimplices() << " total simplices.\n";
  for (int dimension = getHighestSimplexDimension();
	  dimension >= 0; --dimension) {
    s << "\nDimension " << dimension << " simplices:\n";
    std::copy(dimensionNSimplicesBegin(dimension), 
		    dimensionNSimplicesEnd(dimension),
		    std::ostream_iterator<NerveSimplex>(s));
  }

  return s;
}


bool
NerveComplex::
compareSimplexSets (const SimplexSet& x, const SimplexSet& y)
{
  bool result = false;
  
  if (x.sets.size() > y.sets.size()) {
    result = true;
  } else if (x.sets.size() == y.sets.size()) {
    result = std::lexicographical_compare(x.sets.begin(), x.sets.end(),
								  y.sets.begin(), y.sets.end());
  } else {
    result = false;
  }

  return result;
}


void 
NerveComplex::
computeSimplexSets (SimplexSet * rowSets, SimplexSet * columnSets,
				int& highestSimplexDimension,
				const CRCovering& crCovering)
{
  int numRows = crCovering.getNumRows();
  int numColumns = crCovering.getNumColumns();
  
  highestSimplexDimension = 0;
  
  for (int row = 0; row < numRows; ++row) {
    rowSets[row].index = row;
    
    for (int index = 0; index < crCovering.getNumSets (); ++index) {
	 if (crCovering.getRowSet(index).find(row) 
		!= crCovering.getRowSet(index).end()) {
	   rowSets[row].sets.insert(index);
	 }
    }
    
    if (static_cast<int>(rowSets[row].sets.size() - 1)
	   > highestSimplexDimension) {
	 highestSimplexDimension = rowSets[row].sets.size() - 1;
    }
  }
  
  for (int column = 0; column < numColumns; ++column) {
    columnSets[column].index = column;
    
    for (int index = 0; index < crCovering.getNumSets (); ++index) {
	 if (crCovering.getColumnSet(index).find(column)
		!= crCovering.getColumnSet(index).end()) {
	   columnSets[column].sets.insert(index);
	 }
    }

    if (static_cast<int>(columnSets[column].sets.size() - 1) 
	 > highestSimplexDimension) {
	 highestSimplexDimension = columnSets[column].sets.size() - 1;
    }
  }
}


void 
NerveComplex::
createSimplices (SimplexSet * rowSetsBegin, SimplexSet * rowSetsEnd,
			  SimplexSet * columnSetsBegin, SimplexSet * columnSetsEnd)
{
  SimplexSet * currentRowSimplex = rowSetsBegin;
  SimplexSet * currentColumnSimplex = columnSetsBegin;
  
  SimplexSet * currentSimplex;  
  std::vector<int> currentSimplexRows;
  std::vector<int> currentSimplexColumns;
  int currentSimplexDimension = rowSetsBegin->sets.size();
  int * currentDimensionOffset = simplexDimensionOffsets;

  *currentDimensionOffset = 0;
  ++currentDimensionOffset;

  // while there are still more simplices
  while ((currentRowSimplex != rowSetsEnd)
	    || (currentColumnSimplex != columnSetsEnd)) {

    // clear the row and column index sets
    currentSimplexRows.clear();
    currentSimplexColumns.clear();

    // determine whether to choose a simplex from the row or the
    // column set
    if (currentRowSimplex != rowSetsEnd) {
	 if (currentColumnSimplex != columnSetsEnd) {
	   if (compareSimplexSets(*currentRowSimplex, *currentColumnSimplex)) {
		currentSimplex = currentRowSimplex;
	   } else {
		currentSimplex = currentColumnSimplex;
	   }
	 } else {
	   currentSimplex = currentRowSimplex;
	 }
    } else {
	 currentSimplex = currentColumnSimplex;
    }

    while (currentSimplexDimension > 
		 static_cast<int>(currentSimplex->sets.size())) {
	 *currentDimensionOffset = numSimplices;
	 ++currentDimensionOffset;
	 --currentSimplexDimension;
    }

    // determine what rows are contained in this simplex
    while (currentRowSimplex != rowSetsEnd
		 && !compareSimplexSets(*currentRowSimplex,
						    *currentSimplex)
		 && !compareSimplexSets(*currentSimplex,
						    *currentRowSimplex)) {
	 currentSimplexRows.push_back(currentRowSimplex->index);
	 ++currentRowSimplex;
    }

    // determine what columns are contained in this simplex
    while (currentColumnSimplex != columnSetsEnd
		 && !compareSimplexSets(*currentColumnSimplex,
						    *currentSimplex)
		 && !compareSimplexSets(*currentSimplex,
						    *currentColumnSimplex)) {
	 currentSimplexColumns.push_back(currentColumnSimplex->index);
	 ++currentColumnSimplex;
    }

    // create a new simplex and insert it onto the simplices list
    NerveSimplex * newSimplex = simplices + numSimplices;

    newSimplex->id = numSimplices;
    newSimplex->simplexLabel = std::set<int>(currentSimplex->sets.begin(),
									currentSimplex->sets.end());
    newSimplex->rows = std::set<int>(currentSimplexRows.begin(),
							  currentSimplexRows.end());
    newSimplex->columns = std::set<int>(currentSimplexColumns.begin(),
								currentSimplexColumns.end());
    newSimplex->dimension = currentSimplex->sets.size() - 1;
    newSimplex->highestSimplexDimension = highestSimplexDimension;

    newSimplex->faceDimensionOffsets = new int[highestSimplexDimension + 2];
    newSimplex->cofaceDimensionOffsets = new int[highestSimplexDimension + 2];
    std::uninitialized_fill(newSimplex->cofaceDimensionOffsets,
					   newSimplex->cofaceDimensionOffsets 
					   + (highestSimplexDimension + 2), 0);


    // update row and column simplex indices
    for (std::set<int>::const_iterator it = newSimplex->rows.begin();
	    it != newSimplex->rows.end(); ++it) {
	 rowSimplexIndices[*it] = numSimplices;
    }
    for (std::set<int>::const_iterator it = newSimplex->columns.begin();
	    it != newSimplex->columns.end(); ++it) {
	 columnSimplexIndices[*it] = numSimplices;
    }

    // increment the number of simplices
    ++numSimplices;
  }

  // finish remaining simplex offsets
  while (currentDimensionOffset < (simplexDimensionOffsets 
  							+ highestSimplexDimension + 2)) {
    *currentDimensionOffset = numSimplices;
    ++currentDimensionOffset;
  }

  // set remaining simplex space to null values
  int maxNumSimplices = ((rowSetsEnd - rowSetsBegin)
					+ (columnSetsEnd - columnSetsBegin));
  for (int i = numSimplices; i < maxNumSimplices; ++i) {
    simplices[i].faceDimensionOffsets = 0;
    simplices[i].cofaceDimensionOffsets = 0;
  }

}


void 
NerveComplex::computeSimplexFaces ()
{
  for (int codimension = 0; codimension < highestSimplexDimension;
	  ++codimension) {
    for (NerveSimplex * simplex 
		 = (simplices + simplexDimensionOffsets[codimension]);
	    simplex < (simplices + simplexDimensionOffsets[codimension + 1]);
	    ++simplex) {
	 int currentFaceCodimension = 0;
	 int numFaces = 0;

	 // iterate through all potential faces
	 for (NerveSimplex * potentialFace 
		   = (simplices + simplexDimensionOffsets[codimension + 1]);
		 potentialFace < (simplices + numSimplices);
		 ++potentialFace) {
	   // if a simplex is a face..
	   if (std::includes(simplex->simplexLabel.begin(),
					 simplex->simplexLabel.end(),
					 potentialFace->simplexLabel.begin(),
					 potentialFace->simplexLabel.end())) {
		int potentialFaceCodimension = (highestSimplexDimension
								  - potentialFace->dimension);

		// update the faceDimensionOffsets if necessary
		if (currentFaceCodimension != potentialFaceCodimension) {
		  while (currentFaceCodimension <= (highestSimplexDimension 
									 - potentialFace->dimension)) {
		    simplex->faceDimensionOffsets[currentFaceCodimension] = numFaces;
		    ++currentFaceCodimension;
		  }
		}

		// add the face to the faces list
		simplex->faces.push_back(potentialFace);
		++numFaces;

		// update the cofaceDimensionOffsets if necessary
		int lastCofaceCodimension = -1;
		int lastCofaceOffset = 0;

		if (!potentialFace->cofaces.empty()) {
		  lastCofaceCodimension = (highestSimplexDimension
							  - (*(potentialFace->cofaces.rbegin()))
							  ->dimension);
		  lastCofaceOffset = 
		    potentialFace->cofaceDimensionOffsets[lastCofaceCodimension + 1];
		}

		if (lastCofaceCodimension != codimension) {
		  for (int i = lastCofaceCodimension + 1; i <= codimension; ++i) {
		    potentialFace->cofaceDimensionOffsets[i] = lastCofaceOffset;
		  }
		  potentialFace->cofaceDimensionOffsets[codimension + 1]
		    = lastCofaceOffset + 1;

		} else {
		  ++(potentialFace->cofaceDimensionOffsets[codimension + 1]);
		}

		// add the coface to the faces list
		potentialFace->cofaces.push_back(simplex);
	   }
	 }

	 // finish the rest of the faceDimensionOffsets array
	 while (currentFaceCodimension <= highestSimplexDimension + 1) {
	   simplex->faceDimensionOffsets[currentFaceCodimension] = numFaces;
	   ++currentFaceCodimension;
	 }
    }
  }
}


std::ostream& operator<< (std::ostream& s, 
					 const NerveComplex& nerve)
{
  return nerve.put(s);
}



#endif // _NERVECOMPLEX_CC
