// NerveComplex.h
// --------------
// Defines an interface for the never complex associated with
// a given matrix and CR-covering of that matrix.
//
// Author: Patrick Perry
// Date:   July 2, 2003
//

#ifndef _NERVECOMPLEX_H
#define _NERVECOMPLEX_H

#include <cstdlib>
#include <set>
#include <functional>
#include <iostream>
#include "NerveSimplex.h"
#include "CRCovering.h"


class NerveComplex
{
  NerveSimplex * simplices;
  int numSimplices;

  int * simplexDimensionOffsets;
  int highestSimplexDimension;
  int * rowSimplexIndices;
  int * columnSimplexIndices;

public:

  // type: const_iterator
  // --------------------
  // Allows iteration through NerveSimplices.
  typedef const NerveSimplex * const_iterator;

  // Constructor
  // -----------
  // Given a matrix and a CR-covering for that matrix, constructs
  // the nerve complex associated with the two.
  NerveComplex (const CRCovering& crCovering);
			 
  // Destructor
  // ----------
  // Frees all resources associated with the nerve complex
  virtual ~NerveComplex ();

  // getHighestSimplexDimension
  // --------------------------
  // Returns the dimension of the highest-dimensional simplex in
  // the nerve complex.
  int getHighestSimplexDimension () const;

  // getNumSimplices
  // ---------------
  // Returns the number of simplices in the nerve complex
  int getNumSimplices () const;

  // getRowSimplex
  // -------------
  // Returns the simplex corresponding to the given row.
  const NerveSimplex& getRowSimplex (int row) const;

  // getColumnSimplex
  // ----------------
  // Returns the simplex corresponding to the given column.
  const NerveSimplex& getColumnSimplex (int column) const;

  // getSimplex
  // ----------
  // Returns the simplex at a given index.
  const NerveSimplex& getSimplex (int index) const;

  // simplicesBegin
  // --------------
  // Returns an iterator to the first simplex.
  const_iterator simplicesBegin () const;

  // simplicesEnd
  // ------------
  // Returns an iterator to the end of the simplices.
  const_iterator simplicesEnd () const;

  // getNumDimensionNSimplices
  // -------------------------
  // Returns the number of simplices of dimension n.
  int getNumDimensionNSimplices (int n) const;

  // dimensionNSimplicesBegin 
  // ------------------------
  // Returns an iterator to the first n-simplex
  const_iterator dimensionNSimplicesBegin (int n) const;

  // dimensionNSimplicesEnd
  // ----------------------
  // Returns an iterator to the end of the n-simplices
  const_iterator dimensionNSimplicesEnd (int n) const;
  
  // put
  // ---
  // Writes the nerve complex to an output stream
  virtual std::ostream& put (std::ostream& s) const;

private:

  // type: SimplexSet
  // ----------------
  // Utility class for an intermediate simplex corresponding
  // to a row or column with the given index.
  struct SimplexSet {
    int index;
    std::set<int, std::less<int> > sets;
  };

  // compareSimplexSets
  // ------------------
  // Comparator for simplex sets such that is dim(x) > dim(y) 
  // then x < y, and if dim(x) = dim(y) then performs a 
  // lexicographical compare.
  static bool compareSimplexSets (const SimplexSet& x,
						    const SimplexSet& y);


  // computeSimplexSets
  // ------------------
  // Given pre-allocated arrays for the row sets and the column
  // sets, computes the corresponding simplex set for each row
  // and column from the matrix and the CR-covering, and also
  // sets the highestSimplexDimension variable.
  static void computeSimplexSets (SimplexSet * rowSets, 
						    SimplexSet * columnSets,
						    int& highestSimplexDimension,
						    const CRCovering& crCovering);

  // createSimplices
  // ---------------
  // Takes the row and column sets created by computeSimplexSets,
  // and computes and stores the simplices for the nerve complex,
  // as well as sets the numSimplices member and the 
  // simplexDimensionOffsets array.
  void createSimplices (SimplexSet * rowSimplicesBegin, 
				    SimplexSet * rowSimplicesEnd,
				    SimplexSet * columnSimplicesBegin,
				    SimplexSet * columnSimplicesEnd);

  // computeSimplexFaces
  // -------------------
  // For each simplex, determines the faces of the simplex.
  void computeSimplexFaces ();

};


// operator<<
// ----------
// Pretty-print for nerve complex objects
std::ostream& operator<< (std::ostream& s, 
					 const NerveComplex& nerve);


// get function definitions
//#include "NerveComplex.cc"


#endif // _NERVECOMPLEX_H
