// NerveSimplex.h
// --------------
// Defines an interface for a simplex in the Nerve Complex
// associated with a matrix.
//
// Author: Patrick Perry
// Date:   July 3, 2003
//

#ifndef _NERVESIMPLEX_H
#define _NERVESIMPLEX_H


#include <cstdlib>
#include <set>
#include <vector>
#include <iostream>


class NerveSimplex
{
  friend class NerveComplex;

  int id;
  std::set<int> simplexLabel;
  std::set<int> rows;
  std::set<int> columns;
  int dimension;
  int highestSimplexDimension;

  std::vector<const NerveSimplex *> faces;
  int * faceDimensionOffsets;

  std::vector<const NerveSimplex *> cofaces;
  int * cofaceDimensionOffsets;

public:
  // Constructor
  // -----------
  // Creates an empty simplex.
  NerveSimplex ();

  // Destructor
  // ----------
  // Frees all resources associated with the simplex.
  virtual ~NerveSimplex ();

  // type: const_iterator
  // --------------------
  // Type for iterating through pointers to the faces.
  typedef std::vector<const NerveSimplex *>::const_iterator const_iterator;

  // getRows
  // -------
  // Returns a reference to the set of rows in the simplex
  const std::set<int>& getRows () const;
  
  // getColumns
  // ----------
  // Returns a reference to the set of columns in the simplex
  const std::set<int>& getColumns () const;
  
  // getDimension
  // ------------
  // Returns the dimension of the simplex
  int getDimension () const;

  // getID 
  // -----
  // Returns the simplex ID.
  int getID () const;

  // getSimplexLabel
  // ---------------
  // Returns the simplex id in the form of what sets it is the
  // intersection of.
  const std::set<int>& getSimplexLabel () const;

  // getNumFaces
  // -----------
  // Returns the number of faces of the simplex
  int getNumFaces () const;

  // facesBegin
  // ----------
  // Get an iterator to a pointer to the first face.
  const_iterator facesBegin () const;

  // facesEnd
  // --------
  // Get an iterator to the end of the faces
  const_iterator facesEnd () const;

  // getNumDimensionNFaces 
  // ---------------------
  // Returns the number of faces of the given dimension.
  int getNumDimensionNFaces (int n) const;

  // dimensionNFacesBegin
  // --------------------
  // Returns an iterator to a pointer to the first face
  // of the given dimension.
  const_iterator dimensionNFacesBegin (int n) const;

  // dimensionNFacesEnd
  // ------------------
  // Returns an iterator to the end of the faces of the
  // given dimension.
  const_iterator dimensionNFacesEnd (int n) const;

  // getNumCofaces
  // -------------
  // Returns the number of cofaces of the simplex
  int getNumCofaces () const;

  // cofacesBegin
  // ------------
  // Get an iterator to a pointer to the first coface.
  const_iterator cofacesBegin () const;

  // cofacesEnd
  // ----------
  // Get an iterator to the end of the cofaces
  const_iterator cofacesEnd () const;

  // getNumDimensionNCofaces 
  // -----------------------
  // Returns the number of cofaces of the given dimension.
  int getNumDimensionNCofaces (int n) const;

  // dimensionNCofacesBegin
  // ----------------------
  // Returns an iterator to a pointer to the first coface
  // of the given dimension.
  const_iterator dimensionNCofacesBegin (int n) const;

  // dimensionNCofacesEnd
  // --------------------
  // Returns an iterator to the end of the cofaces of the
  // given dimension.
  const_iterator dimensionNCofacesEnd (int n) const;

  // put
  // ---
  // Writes the simplex to an output stream
  virtual std::ostream& put (std::ostream& s) const;


};


// operator<
// ---------
// Compare simplices based on their ordering.
bool operator< (const NerveSimplex& x, const NerveSimplex& y);


// operator<<
// ----------
// Pretty-print for nerve simplex objects
std::ostream& operator<< (std::ostream& s, const NerveSimplex& simplex);
					 

#endif // _NERVESIMPLEX_H
