/* OffDiagonalBlock.h
 * ------------------
 * Written by Patrick Perry
 * Original Version on October 24, 2003
 *
 * $Id: OffDiagonalBlock.h,v 1.1.1.1 2005/03/18 23:36:25 silva Exp $
 * $Log: OffDiagonalBlock.h,v $
 * Revision 1.1.1.1  2005/03/18 23:36:25  silva
 * Imported Sources
 *
 * Revision 1.1.1.1  2004/09/25 22:04:08  patperry
 * Imported sources
 *
 * Revision 1.1  2004/01/15 07:13:36  patperry
 * Changed file extensions.
 *
 * Revision 1.5  2003/12/12 20:13:46  patperry
 * bugfixes and debug code galore
 *
 * Revision 1.4  2003/11/22 06:54:14  patperry
 * lotsa fixes.
 *
 * Revision 1.3  2003/10/28 04:43:51  patperry
 * Preliminary version of SelectionMaskColumnIterator.
 * Copy constructors for all full matrix types.
 *
 * Revision 1.2  2003/10/26 00:24:51  patperry
 * Preliminary Version of OffDiagonalBlock.  Deleted unnecessary files.
 *
 * Revision 1.1  2003/10/24 09:30:26  patperry
 * Adding some files I forgot.
 *
 *
 */

#ifndef _OFFDIAGONALBLOCK_H
#define _OFFDIAGONALBLOCK_H

#include <map>
#include <vector>
#include "LUCancelFullMatrix.h"

template <class Element_t, class Alloc>
class OffDiagonalBlock;
#include "DiagonalBlock.h"


/**
 * This class defines an off-diagonal block in a geometric-sparse 
 * decomposition.  The block has access to the leading diagonal block,
 * the diagonal block in the same column, all off diagonal blocks to the
 * right of this, and all off-diagonal blocks below this.
 */
template <class Element_t, class Alloc>
class OffDiagonalBlock : public LUCancelFullMatrix<Element_t, Alloc>
{
  friend class DiagonalBlock<Element_t, Alloc>;

public:

  //! the diagonal block type correspoding to this block
  typedef DiagonalBlock<Element_t, Alloc> DiagonalBlock;

private:

  //! the row id (id of the diagonal block in the same row as this)
  const int rowID;  

  //! the column id (id of the diagonal block in the same column as this)
  const int columnID;

  //! the off-diagonal blocks in the same column that are below this,
  //! indexed by their rowIDs
  std::map<int, OffDiagonalBlock *> columnFaces;

  //! the off-diagonal blocks in the same row that are to the left of this.
  std::vector<OffDiagonalBlock *> rowFaces;

public:
  
  /**
   * Constructor.   Initialize the off-diagonal block with the given 
   * diagonal blocks corresponding to the blocks in the same row and
   * column.  The matrix is initially all zero.
   *
   * \param rowBlock the DiagonalBlock in the same row as this.
   * \param columnBlock the DiagonalBlock in the same column as this.
   */
  OffDiagonalBlock (const DiagonalBlock * rowBlock, 
				const DiagonalBlock * columnBlock);

  /**
   * Constructor.   Initialize the off-diagonal block with the given 
   * diagonal blocks corresponding to the blocks in the same row and
   * column.  The matrix data is taken from the passed-in matrix.
   *
   * \param the matrix whose data is used to initialize this.
   * \param rowBlock the DiagonalBlock in the same row as this.
   * \param columnBlock the DiagonalBlock in the same column as this.
   */
  template <class ColumnIterator_t, class Alloc1>
  OffDiagonalBlock (const Matrix<Element_t, ColumnIterator_t, Alloc1>& matrix,
				const DiagonalBlock * rowBlock, 
				const DiagonalBlock * columnBlock);

  /**
   * Destructor.
   */
  virtual ~OffDiagonalBlock ();

  /**
   * Get the row id of this block.
   *
   * \returns the row id.
   */
  int getRowID () const;

  /**
   * Get the column id of this block.
   *
   * \returns the column id.
   */
  int getColumnID () const;

  /**
   * Adds a block to the column faces collection.
   *
   * \param face the block to add.  face is assumed to be in the same 
   * column and below this.
   * \returns void.
   */
  void addColumnFace (OffDiagonalBlock * face);

  /**
   *  Adds the given range to the collection of column faces.
   *
   * \param begin a (OffDiagoanalBlock *) iterator to the begin of the 
   * collection.
   * \param end an iterator to the end of the collection.
   * \returns void.
   */
  template <class InputIterator>
  void addColumnFaces (InputIterator begin, InputIterator end);

  /**
   * Adds a block to the row faces collection.
   *
   * \param face the block to add.  face is assumed to be in the same 
   * row and to the right of this.
   * \returns void.
   */
  void addRowFace (OffDiagonalBlock * face);

  /**
   *  Adds the given range to the collection of row faces.
   *
   * \param begin a (OffDiagoanalBlock *) iterator to the begin of the 
   * collection.
   * \param end an iterator to the end of the collection.
   * \returns void.
   */
  template <class InputIterator>
  void addRowFaces (InputIterator begin, InputIterator end);

protected:

  /**
   * Cancels the given row by multiplying the pivotRow in the
   * cancelor by scalar and adding it to the row.
   *
   * \param row the row to cancel.
   * \param pivotRow the index of the pivotRow.
   * \param scalar the scalar to multiply the pivot row by.
   * \returns void.
   */
  void cancelRow (int row, int pivotRow, Element_t scalar);  

  /**
   * Adds a scalar multiple of the row from the block with the
   * given rowID to the given row.  If the blockID does not correspond
   * to the ID of a column coface, this does nothing.
   *
   * \param scalar the scalar to multiply the foreign row by.
   * \param the row index of the block row to add.
   * \param blockID the rowID of the block.
   * \param row the index of the row to add the scalar multiple to.
   * \returns void.
   */
  void addScalarTimesRowFromBlockToRow (Element_t scalar, int blockRow,
								int blockID, int row);
};

// get function definitions
#include "OffDiagonalBlock.cc"

#endif // _OFFDIAGONALBLOCK_H
