// Partition.cc
// ------------
// Implements Partition.h
//
// Author: Erik Carlsson
// Date:   August 27, 2003
//
// Slight modification by Patrick Perry
//


#include <iostream>
#include <algorithm>
#include "Partition.h"


Partition::
Partition () 
  : numGroups(0), groupings(), groupList((IndexList *) 0)
{
}


Partition::
Partition (const Partition& partition) 
  : numGroups(partition.numGroups), groupings(partition.groupings)
{
  buildGroups();
}


Partition::
Partition (int numGroups, const IndexList& groupings) 
  : numGroups(numGroups), groupings(groupings) 
{
  buildGroups();
}


Partition::
Partition (int numGroups, const IndexList& groupings, IndexList* groupList) 
  : numGroups(numGroups), groupings(groupings)
{
  int i;
  this->groupList = new IndexList[numGroups];
  for(i = 0; i < numGroups; i++) {
    this->groupList[i].length = groupList[i].length;
    std::copy(groupList[i].list, groupList[i].list + groupList[i].length, 
		    this->groupList[i].list);
  }
}


Partition::
~Partition () 
{
  delete[] groupList;
}


void 
Partition::
setVals (int numGroups, const IndexList& groupings) 
{
  this->groupings.setVals(groupings);
  this->numGroups = numGroups;
  updateGroups();
}


void 
Partition::
setVals (const Partition& partition) 
{
  setVals(partition.numGroups, partition.groupings);
}


void 
Partition::
setElementAt(int index, int val) 
{
  groupings.list[index] = val;
}


void 
Partition::
updateGroups () 
{
  delete[] groupList;
  buildGroups();
}


void 
Partition::
display () const 
{
  int i;
  std::cout << "displaying groupings:\n";
  groupings.display();
  std::cout << "displaying groups:\n";
  for(i = 0; i < numGroups; i++) {
    groupList[i].display();
  }
}


IndexList 
Partition::
getElemsInGroup (int group) const 
{
  return groupList[group];
}


int 
Partition::
getGroupFromElem (int elem) 
{
  return groupings.list[elem];
}


int
Partition::
getNumGroups () const 
{
  return numGroups;
}


int 
Partition::
getNumElems () const 
{
  return numElems;
}


int 
Partition::
getMaxIndex () const 
{
  return groupings.length;
}


void 
Partition::
markWith (const IndexList& markerGroup, int length) 
{
  groupings.makeZeros(length);
  for(int i = 0; i < markerGroup.length; i++) {
    groupings.list[markerGroup.list[i]] = 1;
  }
  numGroups = 2;
  updateGroups();
}


void 
Partition::
intersect (IndexList indexList) 
{
  int i, j = 0;
  for(i = 0; i < groupings.length; i++) {
    if(j < indexList.length && i == indexList.list[j]) j++;
    else groupings.list[i] = -1;
  }
  updateGroups();
}


void 
Partition::
setDiff(IndexList indexList) 
{
  int i, j = 0;
  for(i = 0; i < groupings.length; i++) {
    if(j < indexList.length && i == indexList.list[j]) {j++; groupings.list[i] = -1;}
  }
  updateGroups();
}


void 
Partition::
elimEmptyGroup (int groupNum) 
{
  for(int i = 0; i < groupings.length; i++) {
    if(groupings.list[i] >= groupNum) groupings.list[i]--;
  }
  numGroups--;
  updateGroups();
}


void 
Partition::
restrict (IndexList indexList) 
{
  intersect(indexList);
}


void 
Partition::
buildGroups() 
{
  int i; 
  int groupSizes[numGroups];
  numElems = 0;
  for(i = 0; i < numGroups; i++) {
    groupSizes[i] = 0;
  }  
  for(i = 0; i < groupings.length; i++) {
    if(groupings.list[i] >= 0 && groupings.list[i] < numGroups) {
      groupSizes[groupings.list[i]]++; 
      numElems++;
    }
  }
  groupList = new IndexList[numGroups];
  for(i = 0; i < numGroups; i++) {
    groupList[i].makeNewList(groupSizes[i]);
  }
  for(i = 0; i < numGroups; i++) {
    groupSizes[i] = 0;
  }  
  for(i = 0; i < groupings.length; i++) {
    if(groupings.list[i] >= 0 && groupings.list[i] < numGroups) {
      groupList[groupings.list[i]].list[groupSizes[groupings.list[i]]] = i;
      groupSizes[groupings.list[i]]++;  
    }
  }
}


void 
Partition::
refineGroupings () 
{
  int emptyGroups[numGroups];
  for(int i = 0; i < numGroups; i++) emptyGroups[i] = 1;
  for(int i = 0; i < groupings.length; i++) {
    emptyGroups[groupings.list[i]] = 0;
  }
  int count = 0;
  for(int i = 0; i < numGroups; i++) {
    if(emptyGroups[i] != 0) count++;
    emptyGroups[i] = count;
  }
  numGroups -= count;
  for(int i = 0; i < groupings.length; i++) {
    groupings.list[i] -= emptyGroups[groupings.list[i]];
  }
}

