/* RowColumnIterator.h
 * -------------------
 * Written by Patrick Perry
 * Initial Version on October 18, 2003
 *
 * $Id: RowColumnIterator.h,v 1.2 2005/10/24 16:20:08 patperry Exp $
 * $Log: RowColumnIterator.h,v $
 * Revision 1.2  2005/10/24 16:20:08  patperry
 * Hells (literally) of compile fixes for gcc4.0
 *
 * Revision 1.1.1.1  2005/03/18 23:36:26  silva
 * Imported Sources
 *
 * Revision 1.1.1.1  2004/09/25 22:04:08  patperry
 * Imported sources
 *
 * Revision 1.2  2004/07/12 18:24:43  patperry
 * Fixed bug in operator--
 *
 * Revision 1.1  2004/01/15 07:13:36  patperry
 * Changed file extensions.
 *
 * Revision 1.2  2003/10/25 01:00:08  patperry
 * Documentation fix.
 *
 * Revision 1.1  2003/10/25 00:49:32  patperry
 * Added RowIterators.  Make RowColumnIterator have virtual functions.
 *
 * Revision 1.3  2003/10/22 10:52:36  patperry
 * Implemented and tested ColumnIterator for sparse matrices.
 *
 * Revision 1.2  2003/10/22 07:37:18  patperry
 * More work on column iterators.
 *
 * Revision 1.1  2003/10/19 02:42:31  patperry
 * Added a ColumnIterator.
 *
 *
 */

#ifndef _ROWCOLUMNITERATOR_H
#define _ROWCOLUMNITERATOR_H

#include <iterator>

/**
 * Iterator class for accessing a row or column in a matrix.
 */
template <class Element_t, class Alloc>
class RowColumnIterator 
  : public std::iterator<std::random_access_iterator_tag, Element_t>
{

protected:

  //! the logical position in the row or column
  int logicalPosition;

protected:

  /**
   * Constructor.  Create an iterator starting at the given logical 
   * position.
   * 
   * \param logicalPosition the logicalPosition to start at.
   */
  RowColumnIterator (int logicalPosition)
    : logicalPosition(logicalPosition)
  { }

public:

  /**
   * Default constructor.
   */
  RowColumnIterator ()
    : logicalPosition(0)
  { }

  /**
   * Destructor.
   */
  virtual ~RowColumnIterator ()
  { }

  /**
   * De-refernce operator.
   *
   * \returns the value pointed to by the iterator.
   */
  virtual Element_t operator* () = 0;

  /**
   * Pre-increment operator.  Increments the iterator.
   *
   * \returns the iterator at (this + 1).
   */
  virtual RowColumnIterator& operator++ () = 0;

  /**
   * Post-increment operator.
   *
   * \param post dummy argument.
   * \returns this iterator before incrementing.
   */
  RowColumnIterator operator++ (int post)
  {
    RowColumnIterator result = *this;
    ++(*this);
    return result;
  }

  /**
   * Pre-decrement operator.  Decrements the iterator.
   *
   * \returns the iterator at (this - 1).
   */
  virtual RowColumnIterator& operator-- () = 0;

  /**
   * Post-decrement operator.
   *
   * \param post dummy argument.
   * \returns this iterator before decrementing.
   */
  RowColumnIterator operator-- (int post)
  {
    RowColumnIterator result = *this;
    --(*this);
    return result;
  }

  /**
   * Forward-advance operator.  Increments the iterator by n.
   *
   * \param n an integer.
   * \returns this after it has been incremented.
   */
  virtual RowColumnIterator& operator+= (int n) = 0;

  /**
   * Backward-reverse operator.  Decrements the iterator by n.
   *
   * \param n an integer.
   * \returns this after it has been decremented.
   */
  virtual RowColumnIterator& operator-= (int n) = 0;

  /**
   * Distance operator.
   *
   * \param x another iterator.
   * \returns the distance between this and x.
   */
  int operator- (const RowColumnIterator& x) const
  {
    return (logicalPosition - x.logicalPosition);
  }

  /**
   * Index operator.
   *
   * \param n an integer.
   * \returns the value at this[n].
   */
  virtual Element_t operator[] (int n) const = 0;

  /**
   * Equals comparator.
   *
   * \param x iterator to compare this to.
   * \returns (*this) == x.
   */
  bool operator== (const RowColumnIterator& x) const
  {
    return (logicalPosition == x.logicalPosition);
  }

  /**
   * Not-Equals comparator.
   *
   * \param x iterator to compare this to.
   * \returns (*this) != x.
   */
  bool operator!= (const RowColumnIterator& x) const
  {
    return (logicalPosition != x.logicalPosition);
  }

  /**
   * Less-than comparator.
   *
   * \param x iterator to compare this to.
   * \returns (*this) < x.
   */
  bool operator< (const RowColumnIterator& x) const
  {
    return (logicalPosition < x.logicalPosition);
  }

};


/**
 * Iterator addition.
 *
 * \param n an integer.
 * \param i an iterator.
 * \returns the iterator at i + n.
 */
template<class Element_t, class Alloc>
RowColumnIterator<Element_t, Alloc> 
operator+ (int n, const RowColumnIterator<Element_t, Alloc>& i)
{
  RowColumnIterator<Element_t, Alloc> result = i;
  result += n;
  return result;
}


/**
 * Iterator addition.
 *
 * \param i an iterator.
 * \param n an integer.
 * \returns the iterator at n + i.
 */
template<class Element_t, class Alloc>
RowColumnIterator<Element_t, Alloc> 
operator+ (const RowColumnIterator<Element_t, Alloc>& i, int n)
{
  RowColumnIterator<Element_t, Alloc> result = i;
  result += n;
  return result;
}


/**
 * Iterator subtraction.
 *
 * \param i an iterator.
 * \param n an integer.
 * \returns the iterator at i - n.
 */
template<class Element_t, class Alloc>
RowColumnIterator<Element_t, Alloc> 
operator- (const RowColumnIterator<Element_t, Alloc>& i, int n)
{
  RowColumnIterator<Element_t, Alloc> result = i;
  result -= n;
  return result;
}


#endif // _ROWCOLUMNITERATOR_H
