/* chmapping.cc
 * -------------
 * Constructs a chain mappoing out of a vertex map.
 *
 * $Id: chmapping.cc,v 1.1.1.1 2005/03/18 23:36:48 silva Exp $
 * $Log: chmapping.cc,v $
 * Revision 1.1.1.1  2005/03/18 23:36:48  silva
 * Imported Sources
 *
 * Revision 1.1.1.1  2004/09/25 22:04:09  patperry
 * Imported sources
 *
 * Revision 1.7  2004/05/19 05:05:52  patperry
 * Added doc and output option for chmapping
 *
 * Revision 1.6  2004/02/26 06:49:24  patperry
 * Converted mexfiles to use new scomplex interface
 *
 * Revision 1.5  2004/02/26 02:34:04  patperry
 * Fixed bugs in PlexObject
 *
 * Revision 1.4  2004/02/09 06:30:44  patperry
 * Got rid of MatlabAllocator.  A few small bugfixes.
 *
 * Revision 1.3  2004/01/28 09:56:23  patperry
 * Fixed bugs in chmappoing
 *
 * Revision 1.2  2004/01/25 14:13:54  patperry
 * Finished code for chmapping.
 *
 * Revision 1.1  2004/01/24 01:39:59  patperry
 * Development version.
 *
 *
 */

#include <algorithm>
#include <iterator>
#include <map>
#include <sstream>
#include <string>

#include "PlexObject.h"
#include "MxUtility.h"

extern "C" {
#include "mex.h"
#include "matrix.h"
}

extern "C" {


void initializeVertexMap (const mxArray * rawMap, std::map<int, int>& map);
void validateVertexMap(const std::map<int, int>& map, 
				   const SimplicialComplex_t& from,
				   const SimplicialComplex_t& to);
mxArray * getSimplexMap (const std::map<int, int>& vertexMap, 
					int simplexDimension,
					const SimplicialComplex_t& from,
					const SimplicialComplex_t& to);
  

/*   C_f = CHMAPPING(C,D,f) returns the chain-map induced
 *			by a vertex mapping f between two (simplicial?)
 *      complexes, up to the highest compatible dimension.
 * 
 *   C_f = CHMAPPING(C,D,f,k) does not compute beyond
 *			dimension k, and may stop earlier.
 */
	
void 
mexFunction (int nlhs, mxArray *plhs[], int nrhs, const mxArray *prhs[]) {
  SimplicialComplex_t * from;
  SimplicialComplex_t * to;
  std::map<int, int> vertexMap;
  int highestPossibleDimension;
  int highestDimension;
  
  if ((nrhs != 3) && (nrhs != 4)) {
    mexErrMsgTxt("Function takes 3 or 4 arguments.");
  } else if (nlhs > 2) {
    mexErrMsgTxt("Zero, One, or Two output arguments only");
  } else if (!IsPlexObject(prhs[0])) {
    mexErrMsgTxt("First argument must be a valid plex object.");
  } else if (!IsPlexObject(prhs[1])) {
    mexErrMsgTxt("Second argument must be a valid plex object.");
  } else if (mxGetN(prhs[2]) != 2) {
    mexErrMsgTxt("Third argument must have exactly two columns.");
  } else if (!mxIsDouble(prhs[2]) || mxIsComplex(prhs[2])) {
    mexErrMsgTxt("Third argument must be non-complex real.");
  } else if ((nrhs == 4) && !IsScalarMxArray(prhs[3])) {
    mexErrMsgTxt("Fourth argument must be a scalar.");
  } else {
    from = GetSimplicialComplex(prhs[0]);
    to = GetSimplicialComplex(prhs[1]);

    initializeVertexMap(prhs[2], vertexMap);
    validateVertexMap(vertexMap, *from, *to);
  
    if (nrhs == 4) {
	 highestPossibleDimension 
	   = std::min(GetIntFromMxArray(prhs[3]), from->getDimension());
    } else {
	 highestPossibleDimension = from->getDimension();
    }

    highestDimension = std::min(highestPossibleDimension, to->getDimension());
  }
  
  plhs[0] = mxCreateCellMatrix(highestDimension + 1, 1);
  for (int i = 0; i <= highestDimension; ++i) {
    mxSetCell(plhs[0], i, getSimplexMap(vertexMap, i, *from, *to));
		    
  }

  if (nlhs == 2) {
    plhs[1] = mxCreateDoubleScalar(static_cast<double>(highestDimension));
  }
}


void initializeVertexMap (const mxArray * rawMap, std::map<int, int>& map)
{
  double * fromValue = mxGetPr(rawMap);
  double * toValue = fromValue + mxGetM(rawMap);
  double * fromEnd = toValue;
  
  while (fromValue != fromEnd) {
    map[static_cast<int>(*fromValue++)] = static_cast<int>(*toValue++);
  }
}


void validateVertexMap(const std::map<int, int>& map, 
				   const SimplicialComplex_t& from,
				   const SimplicialComplex_t& to)
{
  SimplicialComplex_t::ConstSimplexIterator from_it;
  SimplicialComplex_t::ConstSimplexIterator from_end
    = from.simplicesEnd(0);
  SimplicialComplex_t::ConstSimplexIterator to_end
    = to.simplicesEnd(0);

  for (from_it = from.simplicesBegin(0); from_it != from_end; ++from_it) {
    int fromVertex = *((*from_it)->getLabel());
    std::map<int, int>::const_iterator mapping 
	 = map.find(fromVertex);
    
    // check to see that the map is defined
    if (mapping == map.end()) {
	 std::ostringstream errorMessage;

	 errorMessage << "No mapping defined from vertex " << fromVertex;
	 mexErrMsgTxt(errorMessage.str().c_str());

    // check if the to vertex is defined
    } else {
	 int toVertex = mapping->second;
	 SimplicialComplex<>::ConstSimplexIterator to_it;
	 to_it = to.findSimplex(0, &toVertex);

	 if (to_it == to_end) {
	   std::ostringstream errorMessage;

	   errorMessage << "Target vertex [" << toVertex << "] does not exist.";
	   mexErrMsgTxt(errorMessage.str().c_str());
	 }
    }
  }
}


mxArray *
getSimplexMap (const std::map<int, int>& vertexMap, 
			int simplexDimension,
			const SimplicialComplex_t& from,
			const SimplicialComplex_t& to)
{
  int numFromSimplices = from.getNumSimplices(simplexDimension);
  int numToSimplices = to.getNumSimplices(simplexDimension);

  mxArray * result = mxCreateSparse(numToSimplices, numFromSimplices,
							 numFromSimplices, mxREAL);
  double * pr = mxGetPr(result);
  int * ir = mxGetIr(result);
  int * jc = mxGetJc(result);

  int fromSimplexIndex = 0;
  int numDegenerateSimplices = 0;
  int toSimplexIndex;
  int toSimplexSign;
  Simplex_t::LabelIterator fromSimplexLabel;

  int toSimplexLabel[simplexDimension + 1];
  SimplicialComplex_t::ConstSimplexIterator from_it;
  SimplicialComplex_t::ConstSimplexIterator from_end
    = from.simplicesEnd(simplexDimension);
  SimplicialComplex_t::ConstSimplexIterator to_begin
    = to.simplicesBegin(simplexDimension);
  SimplicialComplex_t::ConstSimplexIterator to_end
    = to.simplicesEnd(simplexDimension);
  SimplicialComplex_t::ConstSimplexIterator to_it;

  for (from_it = from.simplicesBegin(simplexDimension); 
	  from_it != from_end; ++from_it) {
    jc[fromSimplexIndex] = fromSimplexIndex - numDegenerateSimplices;

    // get the label of the from simplex
    fromSimplexLabel = (*from_it)->getLabel();

    // compute the label of the to simplex
    for (int i = 0; i <= simplexDimension; ++i) {
	 toSimplexLabel[i] = (vertexMap.find(fromSimplexLabel[i]))->second;
    }
    toSimplexSign = Simplex<>::sortLabel(simplexDimension, toSimplexLabel);

    // find the to simplex
    to_it = to.findSimplex(simplexDimension, toSimplexLabel);
    if (to_it != to_end) {
	 // set the appropriate entry in the matrix
	 toSimplexIndex = std::distance(to_begin, to_it);

	 pr[fromSimplexIndex - numDegenerateSimplices] 
	   = static_cast<double>(toSimplexSign);
	 ir[fromSimplexIndex - numDegenerateSimplices] = toSimplexIndex;

    } else {
	 ++numDegenerateSimplices;
    }

    ++fromSimplexIndex;
  }

  jc[fromSimplexIndex] = fromSimplexIndex - numDegenerateSimplices;

  return result;
}


}
