/* copersistence.cc
 * ----------------
 * Compute copersistence of s filtered simplicial complex.
 *
 * Written by Patrick Perry
 * Original Version on March 4, 2005
 *  (modified from persistence.cc)
 *
 * $Id: copersistence.cc,v 1.3 2005/03/30 23:55:45 silva Exp $
 * $Log: copersistence.cc,v $
 * Revision 1.3  2005/03/30 23:55:45  silva
 *
 *
 * Maximum dimension of homology computation is at most (and defaults to) one
 * less than the dimension of the filtration.
 *
 * Revision 1.2  2005/03/20 01:00:12  patperry
 * Added simplex pair functionality.
 *
 *
 * Revision 1.1.1.1  2005/03/18 23:36:49  silva
 * Imported Sources
 *
 * Revision 1.2  2005/03/15 22:14:35  silva
 *
 *
 * Corrected PERSISTENCE input conditions.
 *
 * Revision 1.1  2005/03/04 23:46:29  patperry
 * Added copersistence.
 *
 *
 */


#include <iostream>
#include <limits>
#include "PlexFiltration.h"
#include "PlexObject.h"
#include "PlexPersistence.h"
#include "MxUtility.h"

extern "C" {
#include "mex.h"
#include "matrix.h"
}


extern "C" {

/**
 * pairs = copersistence (P, F)
 */
void 
mexFunction (int nlhs, mxArray *plhs[], int nrhs, const mxArray *prhs[])
{
  const SimplicialComplex_t * scomplex;
  int maxDim;
  int filtrationDim;

  // check input and output
  if ((nrhs != 2) && (nrhs != 3)) {
    mexErrMsgTxt("Two or three inputs required.");
  } else if (nlhs > 2) {
    mexErrMsgTxt("Too many output arguments.");
  } else if (!IsPlexObject(prhs[0])) {
    mexErrMsgTxt("First argument must be a valid plex object.");
  } else if (!mxIsCell(prhs[1])) {
    mexErrMsgTxt("Second argument must be a cell array.");
  } else if ((nrhs == 3) && !IsScalarMxArray(prhs[2])) {
    mexErrMsgTxt("Third argument must be a scalar.");
  } else {
    scomplex = GetSimplicialComplex(prhs[0]);
    filtrationDim = mxGetNumberOfElements(prhs[1]) - 1;

    if (nrhs == 3) {
	 maxDim = GetIntFromMxArray(prhs[2]);
	 if (maxDim >= filtrationDim) {
	   mexErrMsgTxt("Third argument must be less than filtration dimension.");
	 }
    } else {
	 maxDim = filtrationDim - 1;
    }
    
    if (!IsValidFiltration(*scomplex, prhs[1])) {
	 mexErrMsgTxt("Second argument is not a valid filtration.");
    }
  }
  
  // import the filtration
  Filtration_t * filtration = ImportFiltration(*scomplex, prhs[1], maxDim + 1);
  
  // compute the persistence pairs
  Pivot_space_t * pairs = ComputeZ2Copersistence (*filtration);
  
  // allocate space to hold the pairs cell array
  plhs[0] = ExportIntervalPairs(*scomplex, *filtration, *pairs, 
						  -mxGetInf(), maxDim, true);

  // export the simplex pairs if requested
  if (nlhs > 1) {
    plhs[1] = ExportSimplexPairs(*scomplex, *pairs, maxDim);
  }

  // free the filtration and the pairs objects
  delete filtration;
  delete pairs;
}
  
}

