function [I, J] = copersistence(varargin)

%COPERSISTENCE   compute persistent cohomology
%   I = COPERSISTENCE(C, F, k) returns the persistent cohomology intervals
%   up to dimension k for a simplicial complex C with a filtration F.
%
%   I = COPERSISTENCE(C, F) takes the default value k = dim(F)-1.
%
%   [I, J] = COPERSISTENCE(...) additionally returns the matching J
%   between cycle-creating and cycle-destroying simplices.
%
%   T = COPERSISTENCE(..., 'table') returns more detailed information
%   in the form of a persistable object. See PERSISTABLE for details.
%
%   Input arguments:
%      C : plex object
%      F : valid filtration on C
%      k : integer in the range {0, 1, ..., dim(F)-1}.
%
%   Default output arguments:
%      I : persistence intervals
%      J : simplex pairing
%
%   Alternate output argument:
%      T : persistable object (see PERSISTABLE)
%
%   See PERSISTENCE for a full description of the format of the input and
%   default output arguments, and for notation used below.
%
%   COPERSISTENCE encodes the results of the following cohomological
%   calculation: for any R < R', the rank of the map
%
%      H^p(C,C_R) <- H^p(C,C_R')
%
%   in p-dimensional relative cohomology induced by the inclusion
%
%      C_R -> C_R'
%
%   is equal to the number of intervals in dimension p which span a range
%   containing [R,R'].
%
%   Persistence interval data can be displayed using PX_HOMOLOGYPLOT.
%
%   Persistent homology library by Lutz Kettner and Afra Zomorodian.
%
%##PLEXTAG##

if isequal(varargin{nargin}, 'table')
  [p, q, r, s] = copersistence_cc(varargin{1: nargin-1});
  I = persistable(-1, p, q, r, s, varargin{2});
  if (nargout == 2)
    error('Too many output arguments.')
  end
  return

else
  [p, q] = copersistence_cc(varargin{:});
  [I, J] = local_intervals(-1, p, q, varargin{2});
  return

end

%--------------------------------------------
function [I, J] = local_intervals(o, p, q, F)

% o = sense (+1 or -1)
% p = partner dimension
% q = partner index
% F = filtration 

I = cell(size(p));
J = cell(size(p));

ix = +1; % shift from dimension a to index
for a = (0: length(p)-1)
  J{a +ix} = [p{a +ix}; q{a +ix}];
  I{a +ix} = [F{a +ix}; repmat([o*inf], size(p{a +ix}))];

  pairedup = (p{a +ix} == a+1);
  paireddown = (p{a +ix} == a-1);

  if ((a+1 +ix) <= length(p)+1) & (any(pairedup))
    I{a +ix}(2, pairedup) = F{a+1 +ix}(q{a +ix}(pairedup));
  end
  if ((a-1 +ix) >= 1) & (any(paireddown))
    I{a +ix}(2, paireddown) = F{a-1 +ix}(q{a +ix}(paireddown));
  end

  if (o == -1)
    % reverse order of rows so that cohomology intervals come out
    % positively oriented
    I{a +ix} = flipud(I{a +ix});
  end
end
