/* cover.cc
 * --------
 * Get a simplex cover from a vertex cover.
 *
 * Written by Patrick Perry
 * Original Version on March 15, 2004
 *
 * $Id: cover.cc,v 1.1.1.1 2005/03/18 23:36:49 silva Exp $
 * $Log: cover.cc,v $
 * Revision 1.1.1.1  2005/03/18 23:36:49  silva
 * Imported Sources
 *
 * Revision 1.1.1.1  2004/09/25 22:04:09  patperry
 * Imported sources
 *
 * Revision 1.1  2004/03/16 07:25:47  patperry
 * Initial version of cover
 *
 */

#include "PlexObject.h"
#include "MxUtility.h"

extern "C" {
#include "mex.h"
#include "matrix.h"
}

extern "C" {

/*
 * scover = cover(P, vcover, dim)
 */
void 
mexFunction (int nlhs, mxArray *plhs[], int nrhs, const mxArray *prhs[])
{
  const SimplicialComplex_t * scomplex;
  int k;
  int numNondegenerateVertices;

  // Check for proper number of arguments.
  if (nrhs != 3) {
    mexErrMsgTxt("Three inputs required.");
  } else if (nlhs > 1) {
    mexErrMsgTxt("Too many output arguments.");
  } else if (!IsPlexObject(prhs[0])) {
    mexErrMsgTxt("First arugment must be a valid plex object.");
  } else {
    scomplex = GetSimplicialComplex(prhs[0]);
    k = GetIntFromMxArray(prhs[2]);
    numNondegenerateVertices = (scomplex->getNumSimplices(0)
						  - scomplex->getNumDegenerateSimplices(0));

    if (mxIsSparse(prhs[1]) 
	   || (mxGetClassID(prhs[1]) != mxLOGICAL_CLASS)
	   || (mxGetM(prhs[1]) != numNondegenerateVertices)) {
	 mexErrMsgTxt("Vertex cover in incorrect format.");
    } else if (k < 0) {
	 mexErrMsgTxt("Third argument must be a non-negative integer.");
    } 
  }

  // extract the labels of the vertices in the complex
  std::vector<int> vertexLabels;
  SimplicialComplex_t::ConstSimplexIterator vertex_it 
    = scomplex->simplicesBegin(0);
  SimplicialComplex_t::ConstSimplexIterator vertex_end
    = scomplex->simplicesEnd(0);

  vertexLabels.reserve(numNondegenerateVertices);
  while (vertex_it != vertex_end) {
    const Simplex_t * vertex = *vertex_it++;

    if (!vertex->isDegenerate()) {
	 vertexLabels.push_back(*(vertex->getLabel()));
    }
  }

  // extract the labels of the vertices in the cover sets
  int numSets = mxGetN(prhs[1]);
  mxLogical * coverPr = reinterpret_cast<mxLogical *>(mxGetData(prhs[1]));
  std::vector<int> vertexCovers[numSets];

  for (int i = 0; i < numSets; ++i) {
    for (int vertex = 0; vertex < numNondegenerateVertices; ++vertex) {
	 if (*coverPr++) {
	   vertexCovers[i].push_back(vertexLabels[vertex]);
	 }
    }
  }

  // compute the resulting cover
  int numNondegenerateSimplices = (scomplex->getNumSimplices(k) 
							- scomplex->getNumDegenerateSimplices(k));
  SimplicialComplex_t::ConstSimplexIterator it = scomplex->simplicesBegin(k);
  SimplicialComplex_t::ConstSimplexIterator end = scomplex->simplicesEnd(k);
  int simplexID = 0;

  // allocate the result matrix
  plhs[0] = mxCreateNumericMatrix(numNondegenerateSimplices, numSets,
						   mxLOGICAL_CLASS, mxREAL);
  mxLogical * resultPr = reinterpret_cast<mxLogical *>(mxGetData(plhs[0]));

  while (it != end) {
    const Simplex_t * simplex = *it++;

    // for each nondegenerate simplex
    if (!simplex->isDegenerate()) {
	 Simplex_t::LabelIterator label = simplex->getLabel();

	 // for each set
	 for (int setID = 0; setID < numSets; ++setID) {
	   bool isInSet = false;

	   // determine if the simpelx is in the set
	   for (int i = 0; i <= k; ++i) {
		if (std::binary_search(vertexCovers[setID].begin(), 
						   vertexCovers[setID].end(), label[i])) {
		  isInSet = true;
		  break;
		}
	   }

	   // if it is in the set
	   if (isInSet) {
		// set the appropriate entry in the output cover
		resultPr[setID * numNondegenerateSimplices + simplexID] = true;
	   }
	 }
	 
	 ++simplexID;
    }
  }
}

}
