/* extend.cc
 * ---------
 * Extend the filtration of a simplicial complex to higher dimensions
 *
 * Written by Patrick Perry
 * Original Version on April 20, 2004
 *
 * $Id: extend.cc,v 1.2 2005/10/24 16:20:25 patperry Exp $
 * $Log: extend.cc,v $
 * Revision 1.2  2005/10/24 16:20:25  patperry
 * Hells (literally) of compile fixes for gcc4.0
 *
 * Revision 1.1.1.1  2005/03/18 23:36:50  silva
 * Imported Sources
 *
 * Revision 1.2  2005/02/11 22:45:22  patperry
 * Handle boundary cases in extend
 *
 * Revision 1.1.1.1  2004/09/25 22:04:09  patperry
 * Imported sources
 *
 * Revision 1.3  2004/09/16 06:23:45  patperry
 * Added trivial option
 *
 * Revision 1.2  2004/06/21 16:47:53  patperry
 * Added check for empty filtratin in extend.
 *
 * Revision 1.1  2004/04/21 04:15:30  patperry
 * Added extend
 *
 *
 */


#include <iostream>
#include "PlexObject.h"
#include "MxUtility.h"

// Lots of trouble just to get a hashtable.  Code taken from
// http://gcc.gnu.org/onlinedocs/libstdc++/faq/index.html#5_4
#ifdef __GNUC__
#if __GNUC__ < 3
  #include <hash_map.h>
  namespace Sgi { using ::hash_map }; 
#else
  #include <ext/hash_map>
  #if ((__GNUC__ == 3) && (__GNUC_MINOR__ == 0))
    namespace Sgi = std;
  #else 
    namespace Sgi = ::__gnu_cxx;
  #endif
#endif
#else
  #include <hash_map>
  namespace Sgi = std;
#endif 

template <class Handle>
class HandleHash
{
 public:

  std::size_t operator() (const Handle& key) const
  {
    return reinterpret_cast<std::size_t>(&(*key));
  }
};


extern "C" {
#include "mex.h"
#include "matrix.h"
}


extern "C" {

/**
 * F_new = extend (P, F, maxdim = dim(P))
 */
void 
mexFunction (int nlhs, mxArray *plhs[], int nrhs, const mxArray *prhs[])
{
  const SimplicialComplex_t * scomplex;
  int scomplexDim;
  int filtrationDim;
  int nontrivialFiltrationDim;
  int extendToDim;

  // check input and output
  if ((nrhs != 2) && (nrhs != 3)) {
    mexErrMsgTxt("Two or three inputs required.");
  } else if (nlhs > 1) {
    mexErrMsgTxt("Too many output arguments.");
  } else if (!IsPlexObject(prhs[0])) {
    mexErrMsgTxt("First argument must be a valid plex object.");
  } else if (!mxIsCell(prhs[1])) {
    mexErrMsgTxt("Second argument must be a cell array.");
  } else if ((nrhs == 3) && !IsScalarMxArray(prhs[2])) {
    mexErrMsgTxt("Third argument must be a scalar.");
  } else {
    scomplex = GetSimplicialComplex(prhs[0]);
    scomplexDim = scomplex->getDimension();
    filtrationDim = mxGetNumberOfElements(prhs[1]) - 1;
    nontrivialFiltrationDim = filtrationDim;
    while ((nontrivialFiltrationDim >= 0)
		 && mxIsEmpty(mxGetCell(prhs[1], nontrivialFiltrationDim))) {
	 --nontrivialFiltrationDim;
    }

    if (nrhs == 3) {
	 extendToDim = GetIntFromMxArray(prhs[2]);
	 if (extendToDim < -1 ) {
	   mexErrMsgTxt("Third argument must be greater or equal to -1.");
	 }
    } else {
	 extendToDim = scomplexDim;
    }

    if (filtrationDim < 0) {
    	 mexErrMsgTxt("Filtration may not be empty.");
    } else if (nontrivialFiltrationDim > scomplexDim) {
	 mexErrMsgTxt("Filtration must be of dim less than that of the complex.");
    }

    for (int i = 0; i <= filtrationDim; ++i) {
	 const mxArray * simplexFiltration = mxGetCell(prhs[1], i);
	 int numSimplices = scomplex->getNumSimplices(i);

	 if (!mxIsDouble(simplexFiltration) || mxIsComplex(simplexFiltration)) {
	   mexErrMsgTxt("Filtrations must take real values.");
	 } else if (mxGetNumberOfElements(simplexFiltration) != numSimplices) {
	   mexErrMsgTxt("Filtrations arrays must be the lengths of the chranks.");
	 }
    }
  }

  // allocate the result
  plhs[0] = mxCreateCellMatrix(1, extendToDim + 1);

  // copy the old data
  for (int i = 0; i <= std::min(extendToDim, nontrivialFiltrationDim); ++i) {
    mxSetCell(plhs[0], i, mxDuplicateArray(mxGetCell(prhs[1], i)));
  }

  // if we are doing more than just truncating the filtration, continue
  if (extendToDim > nontrivialFiltrationDim) {
    // create a simplex->filtration value filtration for the highest 
    // dimension of the filtration
    Sgi::hash_map<const Simplex_t *, double, HandleHash<const Simplex_t *> > filtration;
    
    const mxArray * simplexFiltration 
	 = mxGetCell(prhs[1], nontrivialFiltrationDim);
    const double * filtrationData = mxGetPr(simplexFiltration);
    SimplicialComplex_t::ConstSimplexIterator it 
	 = scomplex->simplicesBegin(filtrationDim);
    SimplicialComplex_t::ConstSimplexIterator end
	 = scomplex->simplicesEnd(filtrationDim);

    while (it != end) {
	 const Simplex_t * simplex = *it++;
	 
	 filtration[simplex] = *filtrationData++;
    }
  

    // fill in the filtration for higher simplices
    for (int i = nontrivialFiltrationDim + 1; i <= extendToDim; ++i) {
	 mxArray * simplexFiltration 
	   = mxCreateDoubleMatrix(1, scomplex->getNumSimplices(i), mxREAL);
	 double * simplexFiltrationData = mxGetPr(simplexFiltration);
	 
	 it = scomplex->simplicesBegin(i);
	 end = scomplex->simplicesEnd(i);
	 
	 while (it != end) {
	   const Simplex_t * simplex = *it++;
	   Simplex_t::ConstFaceIterator face_it = simplex->facesBegin();
	   Simplex_t::ConstFaceIterator face_end = simplex->facesEnd();
	   double value = -mxGetInf();
	   
	   while (face_it != face_end) {
		const Simplex_t * face = *face_it++;
		
		value = std::max(value, filtration[face]);
	   }
	   
	   filtration[simplex] = value;
	   *simplexFiltrationData++ = value;
	 }
	 
	 mxSetCell(plhs[0], i, simplexFiltration);
    }
  }
  
}

}

