function [H, W] = homology2(C, k, vcover1, vcover2, vcover3)

%HOMOLOGY2   mod 2 homology of a simplicial complex
%   [H, W] = HOMOLOGY2(C, k) produces matrices represeting the mod 2
%   homology of the simplicial complex C.
%
%   Input arguments:
%      C : plex object
%      k : non-negative integer
%
%   Output arguments:
%      H : ck-by-bk homology basis matrix
%      W : bk-by-ck chain projection matrix
%
%   Here ck is the number of k-simplices in C, and bk is the rank of
%   the k-th mod 2 homology group.
%
%   The columns of H give a basis for a complement of image(d_k) in
%   kernel(d_{k-1}). This is the space of representatives.
%
%   The matrix W gives a map from kernel(d_{k-1}) to the space of
%   representatives, in terms of the basis H. The map satisfies:
%
%        W * H   = I
%        W * d_k = 0
%
%   If c is any closed k-chain (represented as a column vector) then (H *
%   W * c) is the unique k-chain in the space of representatives which is
%   homologous to c.
%
%   The following variants organise the calculation according to the
%   geometric sparse algorithm of Carlsson and de Silva.
%
%   [H, W] = HOMOLOGY2(C, k, vcover) uses the geometric sparse
%   LU-reduction algorithm with the given vertex cover.
%
%   [H, W] = HOMOLOGY2(C, k, vcover1, vcover2) uses the geometric sparse
%   LU-reduction algorithm twice iteratively with the given vertex covers.
%
%   [H, W] = HOMOLOGY2(C, k, vcover1, vcover2, vcover3) uses the geometric
%   sparse LU-reduction algorithm three times iteratively with the given
%   vertex covers.
%
%   See also BETTI, HARMONIC, BETTI2.
%
%Plex version 2.0 by Patrick Perry, Vin de Silva and contributors. See
%PX_PLEXINFO for credits and licensing information. [2005-May-18]

error(nargchk(2, 5, nargin));

if (nargout > 2)
  error('Too many output arguments.')
end
if ~isnumeric(k) || (length(k) ~=1) || (k ~= floor(k)) ...
      || (isinf(k))
  error('Dimension must be an integer.')
end

if (nargin > 2)
  cover1_1 = cover(C, vcover1, k);
  cover2_1 = cover(C, vcover1, k + 1);
end
if (nargin > 3)
  cover1_2 = cover(C, vcover2, k);
  cover2_2 = cover(C, vcover2, k + 1);
end
if (nargin > 4)
  cover1_3 = cover(C, vcover3, k);
  cover2_3 = cover(C, vcover3, k + 1);
end

% let d2 = boundary_{k+1}
d2 = boundary(C,k+1);

% let d1 = boundary_{k}
d1 = boundary(C,k);

% compute the kernel of d1
if (nargin == 2)
  [ker_d1, P] = px_mod2null(d1);
elseif (nargin == 3)
  [ker_d1, P] = px_mod2null(d1, cover1_1);
elseif (nargin == 4)
  [ker_d1, P] = px_mod2null(d1, cover1_1, cover1_2);
elseif (nargin == 5)
  [ker_d1, P] = px_mod2null(d1, cover1_1, cover1_2, cover1_3);
end

% compute the image of d2
% permute the rows of im_d2 to be in the same order as ker_d1
if (nargin == 2)
  im_d2 = P' * px_mod2col(d2);
elseif (nargin == 3)
  [im_d2] = P' * px_mod2col(d2, cover2_1);
elseif (nargin == 4)
  [im_d2] = P' * px_mod2col(d2, cover2_1, cover2_2);
elseif (nargin == 5)
  [im_d2] = P' * px_mod2col(d2, cover2_1, cover2_2, cover2_3);
end
  
% compute the rank of d1
rank_d1 = size(d1, 2) - size(ker_d1, 2);

% at this point ker_d1 is of the form:
%
%  XXXXX
%  XXXXX
%  1
%   1
%    1
%     1
%      1
%
% we want to choose a subset of the columns as a basis for the
% homology group.

% we initialize a logical selection mask to include every column 
homologyCycles = ones(1, size(ker_d1, 2));

% now, for each cycle in im_d2, we remove a homology cycle

% for the next calculation we ignore the block at the top of im_d2
im0_d2 = im_d2((rank_d1+1):end,:);

% for each cycle in the kernel
for k = 1:size(ker_d1, 2)

  % see if any boundary cycles have an entry in position k
  S = find(im0_d2(k,:));
  
  % if one does,
  if S
    % remove homology cycle k from the mask
    homologyCycles(k) = 0;

    % subtract that homology cycle from all other cycles in the image
    % which also have an entry in position k    
    im0_d2(:,S) = mod(im0_d2(:,S) - ...
                      repmat(im0_d2(:,S(1)), [1 length(S)]), ...
                      2);
  end
end

% this is the chosen set of homology cycles
H = ker_d1(:, homologyCycles ~= 0);
h = size(H,2);

if (nargout == 2)
  % Now we must compute the h-by-c1 matrix W so as to satisfy:
  %
  %   W * H  = I
  %   W * d2 = 0
  %
  % We do this by solving the block equation:
  % 
  %   [   H'   ] [   ]     [ I ]
  %   [ im_d2' ] [ W']  =  [ 0 ]
  %              [   ]
  % 
  % We solve this A*W = B problem by computing px_mod2null([A,B]). This is
  % a hack. We do not at this stage attempt to incorporate geometric
  % sparse techniques.
  
  Wsolve = px_mod2null([[H'; im_d2'], [speye(h); sparse(size(im_d2,2), h)]]);
  
  % assuming this has been solved sensibly, then the bottom-right h-by-h
  % block of Wsolve is an identity matrix.
  
  % just checking
  if (isequal(speye(h), Wsolve((end-h+1: end), (end-h+1: end))))
    W = Wsolve((1: end-h), (end-h+1: end))';
  else
    error('Doesn''t work.')
    % actually we should be able to work around this
  end 
end

% return to the natural permutation
H = P * H;
if (nargout == 2)
  W = W * P';
end
