/* persistence.cc
 * --------------
 * Compute persistence of s simplicial complex.
 *
 * Written by Patrick Perry
 * Original Version on February 25, 2004
 *
 * $Id: persistence.cc,v 1.3 2005/03/30 23:55:45 silva Exp $
 * $Log: persistence.cc,v $
 * Revision 1.3  2005/03/30 23:55:45  silva
 *
 *
 * Maximum dimension of homology computation is at most (and defaults to) one
 * less than the dimension of the filtration.
 *
 * Revision 1.2  2005/03/20 01:00:12  patperry
 * Added simplex pair functionality.
 *
 * Revision 1.1.1.1  2004/09/25 22:04:09  patperry
 * Imported sources
 *
 * Revision 1.14  2004/06/10 20:30:03  patperry
 * Fixed dimension bug
 *
 * Revision 1.13  2004/05/21 21:37:24  patperry
 * Cleaned up Filtration/Persistence interface
 *
 * Revision 1.12  2004/04/21 07:24:20  patperry
 * Allow computing persistence up to specified dim
 *
 * Revision 1.11  2004/04/21 00:13:18  patperry
 * Bugfix in wfilt, persistence.  Added landmarkd.
 *
 * Revision 1.10  2004/03/14 07:52:48  patperry
 * fixed compile bugs
 *
 * Revision 1.9  2004/03/14 07:30:34  patperry
 * Handle relative complexes in persistence
 *
 * Revision 1.8  2004/03/01 08:43:12  patperry
 * Persistence compile bugs.
 *
 * Revision 1.7  2004/03/01 07:59:12  patperry
 * Persistence output
 *
 * Revision 1.6  2004/03/01 06:55:49  patperry
 * Persistence fixes
 *
 * Revision 1.5  2004/02/27 08:02:22  patperry
 * Persistence fixes
 *
 * Revision 1.4  2004/02/26 11:09:54  patperry
 * persistence changes
 *
 * Revision 1.3  2004/02/26 10:17:44  patperry
 * Compile bugs
 *
 * Revision 1.2  2004/02/26 09:43:52  patperry
 * fixed bugs in filtration
 *
 * Revision 1.1  2004/02/26 08:53:47  patperry
 * Initial version of plex persistence
 *
 */


#include <iostream>
#include <limits>
#include "PlexFiltration.h"
#include "PlexObject.h"
#include "PlexPersistence.h"
#include "MxUtility.h"

extern "C" {
#include "mex.h"
#include "matrix.h"
}


extern "C" {

/**
 * pairs = persistence (P, F)
 */
void 
mexFunction (int nlhs, mxArray *plhs[], int nrhs, const mxArray *prhs[])
{
  const SimplicialComplex_t * scomplex;
  int maxDim;
  int filtrationDim;

  // check input and output
  if ((nrhs != 2) && (nrhs != 3)) {
    mexErrMsgTxt("Two or three inputs required.");
  } else if (nlhs > 2) {
    mexErrMsgTxt("Too many output arguments.");
  } else if (!IsPlexObject(prhs[0])) {
    mexErrMsgTxt("First argument must be a valid plex object.");
  } else if (!mxIsCell(prhs[1])) {
    mexErrMsgTxt("Second argument must be a cell array.");
  } else if ((nrhs == 3) && !IsScalarMxArray(prhs[2])) {
    mexErrMsgTxt("Third argument must be a scalar.");
  } else {
    scomplex = GetSimplicialComplex(prhs[0]);
    filtrationDim = mxGetNumberOfElements(prhs[1]) - 1;

    if (nrhs == 3) {
	 maxDim = GetIntFromMxArray(prhs[2]);
	 if (maxDim >= filtrationDim) {
	   mexErrMsgTxt("Third argument must be less than filtration dimension.");
	 }
    } else {
	 maxDim = filtrationDim - 1;
    }

    if (!IsValidFiltration(*scomplex, prhs[1])) {
	 mexErrMsgTxt("Second argument is not a valid filtration.");
    }
  }

  // import the filtration
  Filtration_t * filtration = ImportFiltration(*scomplex, prhs[1], maxDim + 1);

  // compute the persistence pairs
  Pivot_space_t * pairs = ComputeZ2Persistence (*filtration);
  
  // allocate space to hold the pairs cell array
  plhs[0] = ExportIntervalPairs(*scomplex, *filtration, *pairs, 
						  mxGetInf(), maxDim, false);

  // export the simplex pairs if requested
  if (nlhs > 1) {
    plhs[1] = ExportSimplexPairs(*scomplex, *pairs, maxDim);
  }

  // free the filtration and the pairs objects
  delete filtration;
  delete pairs;
}
  
}
