function [I, J] = persistence(varargin)

%PERSISTENCE   compute persistent homology
%   I = PERSISTENCE(C, F, k) returns the persistent homology intervals up
%   to dimension k for a simplicial complex C with a filtration F.
%
%   I = PERSISTENCE(C, F) takes the default value k = dim(F)-1.
%
%   [I, J] = PERSISTENCE(...) additionally returns the matching J between
%   cycle-creating and cycle-destroying simplices.
%
%   T = PERSISTENCE(..., 'table') returns more detailed information
%   in the form of a persistable object. See PERSISTABLE for details.
%
%   Input arguments:
%      C : plex object
%      F : compatible filtration
%      k : integer in the range {0, 1, ..., dim(F)-1}.
%
%   Default output arguments:
%      I : persistence intervals
%      J : simplex pairing
%
%   Alternate output argument:
%      T : persistable object (see PERSISTABLE)
%
%   See below for a detailed description of the format of the input and
%   default output arguments. First we explain the general idea.
%
%   The filtration F assigns a number ("filtration value") to each simplex
%   of C. The filtration value of a simplex is greater than or equal to
%   the filtration values of its subsimplices. Let C_R denote the
%   subcomplex of C consisting of cells with filtration value at most R. 
%   The complexes {C_R} form a nested family, and the parameter R is
%   sometimes regarded as a time parameter.
%
%   PERSISTENCE returns a set of intervals whose start- and end-points
%   correspond to filtration values of simplices in C. These intervals are
%   labelled by dimension. They encode the results of the following
%   homological calculation: for any R < R' the rank of the map
%
%      H_p(C_R) -> H_p(C_R')
%
%   in p-dimensional homology induced by the inclusion
%
%      C_R -> C_R'
%
%   is equal to the number of intervals in dimension p which span a range
%   containing [R,R'].
%
%   Each interval in p-dimensional persistent homology corresponds to a
%   homological feature created by a p-simplex and destroyed by a
%   subsequent (p+1)-simplex. The two simplices are regarded as being
%   paired with each other.
%
%   Detailed syntax:
%      F : Cell array of length dim(F)+1. Each entry F{p+1} is a vector of
%          length CHRANK(C,p). The entry F{p+1}(a) gives the filtration
%          value of the a-th p-simplex of C, where "a-th" is according to
%          the lexicographic order.
%      
%      I : Cell array of length k+1. Each entry I{p+1} is a matrix of
%          size 2-by-CHRANK(C,p). Each column I{p+1}(:,a) specifies the
%          birth and death times of a persistence interval. We only use
%          the intervals where I{p+1}(1,a) < I{p+1}(2,a), i.e. where birth
%          precedes death.
%
%      J : Cell array of length k+1. Each entry J{p+1} is a matrix of
%          size 2-by-CHRANK(C,p). The column J{p+1}(:,a) identifies the
%          simplex paired with the a-th p-simplex of C. The dimension of
%          the paired simplex (p+1 or p-1) is stored in J{p+1}(1,a), and
%          the position of the simplex in lexicographic order is stored in
%          J{p+1}(2,a). If the a-th p-simplex is not paired with another
%          simplex, then J{p+1}(:,a) is the column vector [-1;-1].
%
%   Persistence interval data can be displayed using PX_HOMOLOGYPLOT. 
%   See also COPERSISTENCE.
%
%   Persistent homology library by Lutz Kettner and Afra Zomorodian.
%
%##PLEXTAG##

if isequal(varargin{nargin}, 'table')
  [p, q, r, s] = persistence_cc(varargin{1: nargin-1});
  I = persistable(+1, p, q, r, s, varargin{2});
  if (nargout == 2)
    error('Too many output arguments.')
  end
  return

else
  [p, q] = persistence_cc(varargin{:});
  [I, J] = local_intervals(+1, p, q, varargin{2});
  return
  
end

%--------------------------------------------
function [I, J] = local_intervals(o, p, q, F)

% o = sense (+1 or -1)
% p = partner dimension
% q = partner index
% F = filtration

I = cell(size(p));
J = cell(size(p));

ix = +1; % shift from dimension a to index
for a = (0: length(p)-1)
  J{a +ix} = [p{a +ix}; q{a +ix}];
  I{a +ix} = [F{a +ix}; repmat([o*inf], size(p{a +ix}))];

  pairedup = (p{a +ix} == a+1);
  paireddown = (p{a +ix} == a-1);

  if ((a+1 +ix) <= length(p)+1) & (any(pairedup))
    I{a +ix}(2, pairedup) = F{a+1 +ix}(q{a +ix}(pairedup));
  end
  if ((a-1 +ix) >= 1) & (any(paireddown))
    I{a +ix}(2, paireddown) = F{a-1 +ix}(q{a +ix}(paireddown));  
  end

  if (o == -1)
    % reverse order of rows so that cohomology intervals come out
    % positively oriented
    I{a} = flipud(I{a});
  end
end
