/* plex.cc
 * -------
 * Interface to C++ plex constructor.
 *
 * $Id: plex.cc,v 1.1.1.1 2005/03/18 23:36:56 silva Exp $
 * $Log: plex.cc,v $
 * Revision 1.1.1.1  2005/03/18 23:36:56  silva
 * Imported Sources
 *
 * Revision 1.1.1.1  2004/09/25 22:04:09  patperry
 * Imported sources
 *
 * Revision 1.10  2004/06/21 17:16:25  patperry
 * Changed vertex label convention for construction with graph.
 *
 * Revision 1.9  2004/04/20 08:16:55  patperry
 * Initial version of wfilt
 *
 * Revision 1.8  2004/02/26 02:34:04  patperry
 * Fixed bugs in PlexObject
 *
 * Revision 1.7  2004/02/25 13:00:45  patperry
 * added new PlexObject interface
 *
 * Revision 1.6  2004/02/09 06:30:44  patperry
 * Got rid of MatlabAllocator.  A few small bugfixes.
 *
 * Revision 1.5  2004/01/23 04:36:39  patperry
 * Cleaned up MATLAB support code.
 *
 * Revision 1.4  2004/01/15 07:16:51  patperry
 * moved support to matlab-support
 *
 * Revision 1.3  2003/11/18 08:57:38  patperry
 * Added support code.
 *
 * Revision 1.2  2003/11/15 04:47:05  patperry
 * fixed a memory leak
 *
 * Revision 1.1.1.1  2003/11/14 10:08:56  patperry
 * Initial version.
 *
 */

#include <iostream>
#include "PlexObject.h"
#include "FullMatrix.h"
#include "CSCSparseMatrix.h"
#include "MxSupport.h"
#include "Iota.h"

extern "C" {
#include "mex.h"
#include "matrix.h"
}

extern "C" {

/**
 * Takes a graph and creates a simplicial complex from it.  If no
 * graph is passed-in, an empty complex is created.
 */
void 
mexFunction (int nlhs, mxArray *plhs[], int nrhs, const mxArray *prhs[])
{
  SimplicialComplex_t * scomplex;

  // Check for proper number of arguments.
  if (nrhs > 1) {
    mexErrMsgTxt("One or no inputs required.");
  } else if (nlhs > 1) {
    mexErrMsgTxt("Too many output arguments.");
  }

  // get a pointer for the result
  plhs[0] = AllocPlexObject();
  
  // if an argument is passed in, used that for the graph
  if (nrhs == 1) {
    // the input must be a square matrix
    if (mxGetM(prhs[0]) != mxGetN(prhs[0])) {
	 mexErrMsgTxt("Input must be a square matrix.");
    }

    std::vector<int> vertexLabels(mxGetM(prhs[0]));
    Sgi::iota(vertexLabels.begin(), vertexLabels.end(), 1);
    
    // handle sparse and full input differently
    if (mxIsSparse(prhs[0])) {
	 CSCSparseMatrix<Mod2> * graph
	   = getSparseMod2Matrix(prhs[0]);
	 
	 scomplex = new SimplicialComplex_t(*graph, vertexLabels.begin());
	 
	 delete graph;
	 
    } else {
	 FullMatrix<Mod2> * graph
	   = getFullMod2Matrix(prhs[0]);
	 
	 scomplex = new  SimplicialComplex_t(*graph, vertexLabels.begin());
	 
	 delete graph;
    }

	// if no argument is passed in, return an empty simplicial complex
  } else {
    scomplex = new SimplicialComplex_t();
  }

  SetSimplicialComplex(plhs[0], scomplex);
}

}
