function [R, C_out, S_out] = px_alpharadius(X);

%PX_ALPHARADIUS -- compute the alpha-radius of a set of Euclidean points
%   R         = PX_ALPHARADIUS(X)
%   [R, C]    = PX_ALPHARADIUS(X)
%   [R, C, S] = PX_ALPHARADIUS(X)
%
%   Input argument
%      X   d-by-k matrix defining k points in R^n
%
%   Output arguments
%      R   alpha-radius
%      C   alpha-center
%      S   vertices of the dominant cell
%
%   The alpha-ball of a set of points is the smallest Euclidean ball
%   containing all the points. Its center C lies in the convex hull of the
%   subset of points meeting the boundary of the alpha-ball. The dominant
%   cell is defined as the span of these points.
%
%   An initial step is to identify which points lie on the boundary of the
%   convex hull, using CONVHULLN.
%
%   See also PX_CECH, PX_BETARADIUS.
%
%Plex Metric Data Toolbox version 2.0 by Vin de Silva, Patrick Perry and
%contributors. See PX_PLEXINFO for credits and licensing information.
%Released with Plex version 2.0. [2005-May-18]

%[2004-May-10] Vin de Silva. Department of Mathematics, Stanford.

if (nargin ~= 1)
  error('Exactly one input argument required.')
end

[d,k] = size(X);

convhulltest = 1;
switch convhulltest
 case 0
  Vlist = (1: k);
  
 case 1
  try
    Klist = convhulln(X');
    Vlist = unique(Klist(:))';
  
  catch
    Vlist = (1: k);
  end
end

[R, C, S] = local_alpharadius(X(:,Vlist));

if (nargout > 1)
  C_out = C;
end

if (nargout > 2)
  S_out = Vlist(S);
end

return

%---------------------------------------------------------------
function [R_out, C_out, S_out] = local_alpharadius(X)

[d,k] = size(X);

% we express the problem in "P*q = r" format, where q is the vector of
% convex coefficients determining the sought point.

P = [ ones(1, k); 2 * diff(X') * X ];
q = [ 1; diff(sum(X.^2,1)') ];

%%%%%%%%%%%%%%%%%%%
faceorder = (1: k);
%%%%%%%%%%%%%%%%%%%

if (rank(P) == k)
  
  L = (P \ q);
  
  if all(L >= 0)
    
    C_out = X * L;
    S_out = (1: k);
    
    R_out = norm(C_out - X(:,1), 2);
  
    return; % under these circumstances, return

  else
    % reorganise the sub-tree search for faster success (I hope!)
    [foo,faceorder] = sort(L);
    clear foo
    
  end  
end

% ELSE, test the faces one by one and recurse.
% Obviously this isn't very good. What can we do?

done = 0;

%for a = (1: k);
for aa = (1: k);
  a = faceorder(aa);
  
  % compute alpha radius for face 'a'
  sublist = [(1: a-1), (a+1: k)];
  [R, C, S_sub] = local_alpharadius(X(:, sublist));
  S = sublist(S_sub);
  
  % is the center good enough? test against the remaining point
  if (norm((C - X(:,a)),2) <= R)
    done = 1;
    break % ESCAPE the FOR loop!
    
  %else
    %keyboard
  end

end

if done
  R_out = R;
  C_out = C;
  S_out = S;
  
else
  error('We failed.')
end

