function B = px_barycenter(F, X, ix)

% PX_BARYCENTER -- compute barycenters
%
%  B = PX_BARYCENTER(F, X)
% 
%  Input arguments:  
%
%   F : k-by-M matrix : M sets of k indices into {1, 2, ..., N}
%   X : d-by-N matrix : coordinates for N points in R^d
%
%   F and X together specify M sets of k points in R^d. The j-th set is
%   given by X(:, F(:,j)).
%
%  Output argument:
%
%   B : d-by-M matrix : coordinates in R^d of the barycenters of the M
%                       k-tuples of points specified by F and X
%           
%
%  Vectorization choice: an optional third input argument, ix, specifies
%  which variable occurs in the FOR loop. Options: ix = 'k','d','M'.
%
%  B = PX_BARYCENTER(F, X, ix);
%
%  For example, if M is large, choose ix = 'k' or 'd'. The default is
%  ix = 'k'.
%
%Plex Metric Data Toolbox version 2.0 by Vin de Silva, Patrick Perry and
%contributors. See PX_PLEXINFO for credits and licensing information.
%Released with Plex version 2.0. [2005-May-18]

nargchk(2, 3, nargin);

method = 'k'; % default
%
% method = 'k';  % slow when k is very large
% method = 'd';  % slow when d is very large
% method = 'M';  % slow when M is very large
%

if (nargin == 3)
  if ~ismember(ix, {'k', 'd', 'M'})
    warning(sprintf('Unrecognized option. Using ix = ''%s''.', method))
  else
    method = ix;
  end
end

[k,M] = size(F);
[d,N] = size(X);

switch method
 
 case 'k'
  B = zeros(d,M);
  for j = (1: k)
    B = B + X(:, F(j,:));
  end
  B = B ./ k;
  
 case 'd'
  B = zeros(d,M);
  for j = (1: d)
    xj = X(j, :);
    B(j, :) = mean(xj(F), 1);
  end
  
 case 'M'
  B = zeros(d, 0);
  for j = (1: M)
    B = [B, mean(X(:,F(:,j)), 2)];
  end

end
