function W = px_betaradius(X, V)

%PX_BETARADIUS -- compute the beta-radius for a list of simplices
%   W = PX_BETARADIUS(X, V)
%
%   Input arguments
%      X: d-by-k matrix, defining k points in R^d
%      V: p-by-c matrix of integer indices. Each column defines a
%         (p-1)-simplex with vertices in (1:k). This is realised as
%         a geometric simplex whose vertices are the corresponding
%         vertices of X.
%
%   Output argument:
%      W:  vector of length c. The i-th entry is the beta radius of
%          the i-th geometric simplex specified by V,X.
%
%   The beta-radius of a set of Euclidean points is defined (for the
%   present purpose) as follows. If the points are affinely independent
%   and the circumcenter of the vertices is contained in the interior of
%   the simplex they span, then the beta radius is defined to be the
%   circumradius of the points. Otherwise, the beta-radius is -inf.
%
%   The beta-radius is used by PX_CECH to compute the Cech complex.
%
%   See also PX_CECH, PX_ALPHARADIUS.
%
%Plex Metric Data Toolbox version 2.0 by Vin de Silva, Patrick Perry and
%contributors. See PX_PLEXINFO for credits and licensing information.
%Released with Plex version 2.0. [2005-May-18]

%[2004-May-17] Vin de Silva. Department of Mathematics, Stanford.

%-------------------------------------------------------
if (nargin ~= 2)
  error('Exactly two input arguments required.')
end

[d, k] = size(X);
[p, c] = size(V);

if any(~ismember(V, (1: k)))
  error('Indices must be integers in range (1:k).')
end

%-------------------------------------------------------
W = zeros(1, c);
for aa = (1: c)
  
  Xaa = X(:, V(:, aa)); % current cell

  % express the problem of locating the circumcenter in 
  % "P * q = r" format, where q is a vector of convex
  % coefficients.
  P = [ ones(1, p); 2 * diff(Xaa') * Xaa ];
  q = [ 1; diff(sum(Xaa.^2,1)') ];
  
  if (rank(P) == p) % simplex is not degenerate
    L = (P \ q);

    if all(L > 0) % solution is in interior of simplex
      W(aa) = norm(Xaa * L  - Xaa(:,1), 2);
      
    else % solution is not in interior
      W(aa) = - inf; 
    end
    
  else % simplex is degenerate
    W(aa) = -inf;
  end
  
end

%-------------------------------------------------------
% done!
%-------------------------------------------------------

