function [C,F] = px_cech(X, k, R)

%PX_CECH  Build a Cech complex.
%   [C,F] = PX_CECH(X, k, R)
%
%   Input arguments:
%      X : d-by-N matrix representing N points in R^d
%      R : scalar, possibly inf.
%      k : non-negative integer, possibly inf.
%
%   Output arguments:
%      C : plex object: k-skeleton of Cech complex with vertex set X
%          and radius R        
%      F : cell array of length dim(C)+1: F{i} is a vector of length
%          CHRANK(C,i) whose entries give the values of R at which
%          each cell enters the Cech complex
%
%   A simplex with vertices [x0, ..., xp] enters the Cech complex at
%   time R if the balls of radius R centered at the points xi meet in
%   exactly one point.
% 
%   All input arguments are required. The user is cautioned that the
%   construction can very easily lead to extremely large complexes.
%
%   See also PX_RIPS, PX_ALPHARADIUS, PX_BETARADIUS.
%
%Plex Metric Data Toolbox version 2.0 by Vin de Silva, Patrick Perry and
%contributors. See PX_PLEXINFO for credits and licensing information.
%Released with Plex version 2.0. [2005-May-18]

% [2004-Sep-15] Vin de Silva, Department of Mathematics, Stanford.

%--------------------------------------------------------
% verify input parameters
%--------------------------------------------------------
error(nargchk(3,3,nargin));
err = local_checkinputs(X, R, k);
if err
  error(err);
end

%--------------------------------------------------------
% initialise
%--------------------------------------------------------
[d, N] = size(X);
k = min(k, N-1);

C = attach(plex, (1: N));
if (nargout >= 2)
  F = {zeros(1,N)};
end

%--------------------------------------------------------
% main loop
%--------------------------------------------------------
for a = (1: k)

  C = expand(C, a-1, a);
  Ca = simplices(C, a);
  
  %------------------------------------------
  % determine the beta radii of the new faces
  %------------------------------------------
  if (a == 1) % treat this case separately

    D = px_euclid(X);
    I = sub2ind([N N], Ca(1,:), Ca(2,:));
    Ba = D(I)/2;
    
    clear D I
  
  else

    Ba = px_betaradius(X, Ca);
    
  end

  %-------------------------------------
  % remove extraneous cells and update F
  %-------------------------------------
  keep = (Ba <= R);
  
  C = excise(C, Ca(:, ~keep));
  
  if (dim(C) == a) & (nargout >= 2)
    F = extend(C, F);
    F{a+1} = max(F{a+1}, Ba(keep));
  end
  
  %-----------------------------------------
  % exit main loop if there are no new cells
  %-----------------------------------------
  if (dim(C) < a) 
    break
  end
    
end

%--------------------------------------------------------
% finished
%--------------------------------------------------------
return
  
%--------------------------------------------------------
% local functions
%--------------------------------------------------------
function err = local_checkinputs(X, R, k)
if ~isnumeric(X) | (size(size(X)) ~=2)
  err = 'X must be a numeric matrix.';

elseif ~isnumeric(R) | (length(R) > 1)
  err = 'R must be a numeric scalar.';
  
elseif ~isnumeric(k) | (length(k) > 1) | (k ~= abs(floor(k)))
  err = 'k must be a non-negative integer or inf.';
  
else
  err = 0;
  
end

%--------------------------------------------------------

