function D = px_metric(X,Y,p)

%PX_METRIC -- computes the L^p metric between sets of vectors.
%
%   D = PX_METRIC(x,y) returns the L^p distance between
%   vectors x and y.
%
%   D = PX_METRIC(X,Y) returns an m-by-n matrix of distances,
%   where X is a d-by-m array of m vectors in R^d, and Y
%   is a d-by-n array of n vectors in R^d.
%
%   The exponent p may be any real number >=1. In addition,
%   the values p=Inf and p=0 both compute the supremum
%   norm. The default is p=2.
%
%   When p=2, the fast routine PX_EUCLID is called. See M-file
%   px_euclid.m for appropriate credits.
%
%Plex Metric Data Toolbox version 2.0 by Vin de Silva, Patrick Perry and
%contributors. See PX_PLEXINFO for credits and licensing information.
%Released with Plex version 2.0. [2005-May-18]

%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
nargchk(2,3,nargin);

if (nargin < 3)
  p = 2;
end

if size(X,1) ~= size(Y,1)
  error('Vectors should be of same dimensionality');
end

m = size(X,2);
n = size(Y,2);
d = size(X,1);

%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%

if (p == 2)
  D = px_euclid(X,Y);	% call the fast routine!
  
elseif (p == 0) | (p == inf)
  D = zeros(m,n);
  for k = 1:d
    D = max(D, diffarr(X,Y,k));
  end
  
elseif (p >= 1)
  D = zeros(m,n);
  for k = 1:d
    D = D + diffarr(X,Y,k).^p;
  end
  D = D.^(1/p);
  
else
  error('Inadmissible exponent p.')
end

%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
function L = diffarr(X,Y,k)
% this constructs the matrix of differences along dimension k

m = size(X,2);
n = size(Y,2);

L = abs(repmat(X(k,:)',[1 n]) - repmat(Y(k,:), [m 1]));

%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
