function [P, Ha_out] = px_mouse(varargin)

%PX_MOUSE   input a selection of 2D points by mouse-click
%   X = PX_MOUSE(n) allow the user to enter a sequence of points in the
%   plane by mouse-click. The current axis is cleared and the points are
%   plotted there as they are selected. The process stops when n points
%   have been entered or at the first right-click.
%
%   [X, Ha] = PX_MOUSE(n) additionally returns the handle of the axis in
%   which the points are selected and plotted.
%
%   Selected points must lie within the current axis range. The default is
%   [0 1 0 1]. Points clicked outside the range are coerced into range.
%
%   X = PX_MOUSE(n, Ha) lets the user specify the axis in which the points
%   are selected. The selected points are cleared from the plot after
%   selection is complete.
%
%   [X, ...] = PX_MOUSE is synonymous with [X, ...] = PX_MOUSE(inf).
%
%   Arguments:
%      n  : positive integer or inf
%      Ha : axis handle
%      X  : 2-by-n1 matrix representing n1 points in 2D (n1 is the
%           actual number of points entered).
%
%Plex Metric Data Toolbox version 2.0 by Vin de Silva, Patrick Perry and
%contributors. See PX_PLEXINFO for credits and licensing information.
%Released with Plex version 2.0. [2005-May-18]

error(nargchk(0,2,nargin));
if (nargin == 0) 
  % default
  n = inf;
else
  % validate first input
  [n, err] = local_check_n(varargin{1});
  error(err);
end
if (nargin == 2)
  % validate second input
  [Ha, err] = local_check_Ha(varargin{2});
  error(err);
else
  % use current axis 
  Ha = cla('reset');
  axis equal;
  axis([0, 1, 0, 1]);
end

HaHeld = ishold(Ha);
HaRange = axis;

hold on

P =[];   % data points
Ho = []; % plot point object

while (n > 0)

  % input next point
  [x, y, z] = ginput(1);

  if isempty(z)
    % 'z = []' causes 'switch z' to crash
    z = -1;
  end

  % enforce axis limits
  x = min(max(x,HaRange(1)), HaRange(2));
  y = min(max(y,HaRange(3)), HaRange(4));
  
  switch z
   case 1
    % left button: record and plot new point
    Ho = [Ho, plot(x, y, 'b+')];
    P = [P, [x;y]];
    n = n-1;

   case 3
    % right-click: terminate
    n = 0;
  end

end

% revert to original HOLD condition of axes;
switch HaHeld
 case 0
  set(Ha, 'NextPlot', 'replace')
 case 1
  set(Ha, 'NextPlot', 'add')
end

% delete plotted points from original axis
if (nargin == 2)
  delete(Ho);
end

% return Ha if requested
if (nargout == 2)
  Ha_out= Ha;
end

return;

%-----------------------------------------------------
% local functions
%-----------------------------------------------------
function [n, err] = local_check_n(n)

err = [];
if ~isnumeric(n) || ~isequal(length(n(:)), 1) ...
      || ~isequal(n, floor(n)) || ~(n > 0)
  err = 'n must be a positive integer or inf.';
end

return;
%-----------------------------------------------------
function [Ha, err] = local_check_Ha(Ha);

err = [];
if ~isnumeric(Ha) || ~isequal(length(Ha(:)), 1) ...
      || ~ishandle(Ha) || ~isequal(get(Ha,'Type'), 'axes')
  err = 'Ha must be a valid axis handle.';
end

return;
%-----------------------------------------------------
