function [H,Q] = px_prune(G,P)

%PX_PRUNE   first-order graph pruning function
%
% If G is the incidence matrix of a graph, then H = PX_PRUNE(G) represents
% the graph obtained from by removing all "dangling vertices".
%
% [H, Q] = PX_PRUNE(G,P) simultaneously provides a reduction of the d-by-n
% array of vectors V, which represents the vertices of G. This is
% especially useful in the context of the PX_VIEWGRAPH command.
%
%Plex Metric Data Toolbox version 2.0 by Vin de Silva, Patrick Perry and
%contributors. See PX_PLEXINFO for credits and licensing information.
%Released with Plex version 2.0. [2005-May-18]

% old code: VdS, summer 2000.

nargchk(1,2,nargin);

if ~isequal(G,G')
  error('The graph matrix G must be symmetric.')
end
n = length(G);

% we begin: make sure to remove ones from diagonal
H0 = logical(G);
H0(1: n+1: n^1) = logical(0);

% remember the reductions so we can apply them to P
bigkeeplist = 1:n;

% repeat the pruning process until nothing further happens
done=0;
while ~done

  % which vertices have only one neighbour?
  shortlist = (sum(H0) == 1);

  % make sure we don't remove both parts of an isolated pair;
  % which edges have no neighbours?	
  htable=kron(shortlist,shortlist') & H0;

  % which vertices should we avoid deleting? in each isolated pair
  % we keep the larger-numbered vertex
  misslist = any(htable & triu(H0));
  
  % which vertices should we keep, after all?
  keeplist = ~shortlist | misslist;

  % now we can trim the graph
  H = H0(keeplist,keeplist);
  
  % and also keep track of what we have done
  bigkeeplist=bigkeeplist(keeplist);
  
  done = isequal(H,H0);
  H0=H;
  
end

% finally prune the matrix P
if (nargin == 2)
  Q=P(:,bigkeeplist);
end
