function [C,F] = px_rips(X, k, R)

%PX_RIPS  Build a Rips complex.
%   [C,F] = PX_RIPS(X, k, R)
%
%   Input arguments:
%      X : d-by-N matrix representing N points in R^d
%      R : scalar, possibly inf.
%      k : non-negative integer, possibly inf.
%
%   Output arguments:
%      C : plex object: k-skeleton of Rips complex with vertex set X
%          and radius R        
%      F : cell array of length dim(C)+1: F{i} is a vector of length
%          CHRANK(C,i) whose entries give the values of R at which
%          each cell enters the Rips complex
%
%   A simplex with vertices [x0, ..., xp] enters the Rips complex at
%   time R equal to its maximum edge length.
%
%   All input arguments are required. The user is cautioned that the
%   construction can very easily lead to extremely large complexes.
%
%   See also PX_CECH.
%
%Plex Metric Data Toolbox version 2.0 by Vin de Silva, Patrick Perry and
%contributors. See PX_PLEXINFO for credits and licensing information.
%Released with Plex version 2.0. [2005-May-18]

% [2005-Mar-09] Vin de Silva, Department of Mathematics, Stanford.

%--------------------------------------------------------
% verify input parameters
%--------------------------------------------------------
error(nargchk(3,3,nargin));
err = local_checkinputs(X, R, k);
if err
  error(err);
end

%--------------------------------------------------------
% initialise
%--------------------------------------------------------
[d, N] = size(X);
k = min(k, N-1);

if (nargout >= 2)
  F = {zeros(1,N)};
end

%--------------------------------------------------------
% main
%--------------------------------------------------------

% build 1-skeleton
D = px_euclid(X);
C = plex(D <= R);

Ca = simplices(C, 1);
I = sub2ind([N N], Ca(1,:), Ca(2,:));

if (nargout >= 2)
  F = {zeros(1,N), D(I)};
end
clear D I Ca

% extend to k skeleton
C = expand(C, 1, k);
if (nargout >= 2)
  F = extend(C, F, k);
end

%--------------------------------------------------------
% finished
%--------------------------------------------------------
return
  
%--------------------------------------------------------
% local functions
%--------------------------------------------------------
function err = local_checkinputs(X, R, k)
if ~isnumeric(X) | (size(size(X)) ~=2)
  err = 'X must be a numeric matrix.';

elseif ~isnumeric(R) | (length(R) > 1)
  err = 'R must be a numeric scalar.';
  
elseif ~isnumeric(k) | (length(k) > 1) | (k ~= abs(floor(k)))
  err = 'k must be a non-negative integer or inf.';
  
else
  err = 0;
  
end

%--------------------------------------------------------

