function [C, F] = px_strong(DL, k, R) 

%PX_STRONG -- build the strong witness complex in Plex
%
%   [C, F] = PX_STRONG(DL, k)
%   [C, F] = PX_STRONG(DL, k, R)
%
%   Input arguments:
%      DL   n-by-N matrix of landmark-to-data distances
%      k    maximum dimension of complex
%      R    maximum value of witness parameter
%
%   Output arguments:
%      C    k-skeleton of the strong witness complex with parameter R
%      F    persistence time-codes for the strong witness complex
%
%   The output arguments are PERSISTENCE-ready. The following sequence is
%   typical:
%
%   >> [C, F] = px_strong(DL, k, R);
%   >> I = persistence(C, F, k-1);
%
%   Warning: the number of cells in C is [n,1]+[n,2]+...+[n,k+1] where the
%   square brackets denote binomial coefficients. The user is cautioned to
%   keep this appropriately bounded.
%
%Plex Metric Data Toolbox version 2.0 by Vin de Silva, Patrick Perry and
%contributors. See PX_PLEXINFO for credits and licensing information.
%Released with Plex version 2.0. [2005-May-18]

% [2004-Jun-25] Vin de Silva, Department of Mathematics, Stanford.
% [2004-Sep-17] Added R parameter.

%--------------------------------------------------------
% verify input parameters
%--------------------------------------------------------
nargchk(2,3,nargin);
if (nargin == 2)
  R = inf;
end

err = local_checkinputs(DL, R, k);
if err
  error(err);
end

%--------------------------------------------------------
% initialise
%--------------------------------------------------------
[n,N] = size(DL);

%--------------------------------------------------------
% construct complex C
%--------------------------------------------------------
if (R == inf)
  C = expand(attach(plex,(1:n)),k);
else
  C = px_witness(DL, 1, R, k);
end

%--------------------------------------------------------
% construct filtration F
%--------------------------------------------------------
if (nargout == 2)
  F = px_wfilt(C, DL, 1, k);
end
  
%--------------------------------------------------------
% finished
%--------------------------------------------------------
return
   
%--------------------------------------------------------
% local functions
%--------------------------------------------------------
function err = local_checkinputs(DL, R, k)
if ~isnumeric(DL) | (size(size(DL)) ~=2)
  err = 'DL must be a numeric matrix.';
 
elseif ~isnumeric(R) | (length(R) > 1)
  err = 'R must be a numeric scalar';
   
elseif ~isnumeric(k) | (length(k) > 1) | (k ~= abs(floor(k)))
  err = 'k must be a non-negative integer or inf.';
   
else
  err = 0;
   
end
 
%--------------------------------------------------------
 
