function [C, F_out] = px_weak(DL, k, R) 

% PX_WEAK -- build the weak witness complex in Plex
%
% [C, F] = PX_WEAK(DL, k)
% [C, F] = PX_WEAK(DL, k, R)
%
% Input arguments:
%
%  DL   n-by-N matrix of landmark-to-data distances
%  k    maximum dimension of complex
%  R    maximum value of witness parameter
%
% Output arguments:
%
%  C    k-skeleton of the weak witness complex with parameter R
%  F    persistence time-codes for the weak witness complex
%
% The output arguments are PERSISTENCE-ready. The following sequence
% is typical:
%
% >> [C, F] = px_weak(DL, k, R);
% >> I = persistence(C, F, k-1);
%
% Warning: the number of cells in C is [n,1]+[n,2]+...+[n,k+1] where the
% square brackets denote binomial coefficients. The user is cautioned to
% keep this appropriately bounded.
%
%Plex Metric Data Toolbox version 2.0 by Vin de Silva, Patrick Perry and
%contributors. See PX_PLEXINFO for credits and licensing information.
%Released with Plex version 2.0. [2005-May-18]

% [2004-Jun-25] Vin de Silva, Department of Mathematics, Stanford. 
% [2004-Sep-17] Added R parameter.

%--------------------------------------------------------
% verify input parameters
%--------------------------------------------------------
error(nargchk(2,3,nargin));
if (nargin == 2)
  R = inf;
end

error(local_checkinputs(DL, R, k));

%--------------------------------------------------------
% initialise
%--------------------------------------------------------
[n,N] = size(DL);
k = min(k, n-1);

%--------------------------------------------------------
% main loop
%--------------------------------------------------------
C = plex;
F = {};
for a = (0: k)
  
  %---------------------------------------
  % extend C,F to the current dimension aa
  %---------------------------------------
  if (a == 0)
    C = attach(plex, (1: n));
    F{a+1} = zeros(1, n);
  
  else
    C = expand(C, a-1, a);

	 % disp('asd3g4'); keyboard; 
	 
	 F = extend(C, F);

	 % disp('done extend'); 
	 
	 
  end
    
  %--------------------------------------------------
  % determine filtration timecodes in the aa-skeleton
  %--------------------------------------------------
  
  %  FF = px_wfilt(C, DL, a+1, a);
  FF = px_witnessnu(DL, a+1, C, a);

  %HACK!!!
  if (length(F) < a+1)
    F{a+1} = [];
	 disp('hack called!!');
  else
    % I'd prefer just to have this line by itself
    % but EXTEND isn't working correctly
    F{a+1} = max(F{a+1}, FF{a+1});
  end
  clear FF
  
  %------------------------
  % remove extraneous cells
  %------------------------
  keep = (F{a+1} <= R);
  Ca = simplices(C, a);
  
  % disp('sfd33'); keyboard; 
  
  C = excise(C, Ca(:, ~keep));

  % disp('done excise'); 
  
  F{a+1} = F{a+1}(:, keep);

  %-----------------------------------------
  % exit main loop if there are no new cells
  %-----------------------------------------
  if (dim(C) < a)
    break
  end
  
end

%--------------------------------------------------------
% output results
%--------------------------------------------------------
if (nargout == 2)
  F_out = F;
end

%--------------------------------------------------------
% finished
%--------------------------------------------------------
return

%--------------------------------------------------------
% local functions
%--------------------------------------------------------
function err = local_checkinputs(DL, R, k)
if ~isnumeric(DL) || (length(size(DL)) ~=2)
  err = 'DL must be a numeric matrix.';
 
elseif ~isnumeric(R) || (length(R) > 1)
  err = 'R must be a numeric scalar.';
   
elseif ~isnumeric(k) || (length(k) > 1) || (k ~= abs(floor(k)))
  err = 'k must be a non-negative integer or inf.';
   
else
  err = [];
   
end

%--------------------------------------------------------

