/* wfilt.cc
 * --------
 * Filter a complex according to the witness radius.
 *
 * Written by Patrick Perry
 * Original Version on April 19, 2004
 *
 * $Id: px_wfilt.cc,v 1.1.1.1 2005/03/18 23:37:02 silva Exp $
 * $Log: px_wfilt.cc,v $
 * Revision 1.1.1.1  2005/03/18 23:37:02  silva
 * Imported Sources
 *
 * Revision 1.5  2005/03/04 21:38:37  patperry
 * Fixed bug in WitnessFilter
 *
 * Revision 1.4  2005/02/11 23:55:04  patperry
 * Fixed vertex labling in wfilt
 *
 * Revision 1.3  2004/11/25 08:38:09  silva
 *
 * Error message typo.
 *
 * Revision 1.2  2004/10/16 22:11:09  silva
 *
 * px_witness, px_wfilt treat k=inf case correctly.
 *
 * Revision 1.1.1.1  2004/09/25 22:04:09  patperry
 * Imported sources
 *
 * Revision 1.4  2004/05/23 20:19:01  patperry
 * Cleaned up Filter interface.
 *
 * Revision 1.3  2004/04/21 07:24:20  patperry
 * Allow computing persistence up to specified dim
 *
 * Revision 1.2  2004/04/21 00:13:18  patperry
 * Bugfix in wfilt, persistence.  Added landmarkd.
 *
 * Revision 1.1  2004/04/20 08:16:55  patperry
 * Initial version of wfilt
 *
 */

#include <algorithm>
#include <vector>
#include "MxUtility.h"
#include "PlexFiltration.h"
#include "PlexObject.h"
#include "WitnessFilter.h"


extern "C" {

void 
mexFunction (int nlhs, mxArray *plhs[], int nrhs, const mxArray *prhs[])
{
  const SimplicialComplex_t * scomplex;
  mxArray * distances;
  double * distanceData;
  int numLandmarks;
  int numPoints;
  int nu;
  int maxDim;

  ConstSimplexIterator_t verticesBegin;
  ConstSimplexIterator_t verticesEnd;
  int maxVertexLabel;

  // Check for proper number of arguments.
  if ((nrhs != 3) && (nrhs != 4)) {
    mexErrMsgTxt("Three or four inputs required.");
  } else if (nlhs > 1) {
    mexErrMsgTxt("Too many output arguments.");
  } else if (!IsPlexObject(prhs[0])) {
    mexErrMsgTxt("First argument must be a plex object.");
  } else if ((mxIsSparse(prhs[1])) || !mxIsDouble(prhs[1])) {
    mexErrMsgTxt("Second argument must be a full double matrix.");
  } else if (!IsScalarMxArray(prhs[2])) {
    mexErrMsgTxt("Third argument must be a scalar.");
  } else if ((nrhs == 4) && !IsScalarMxArray(prhs[3])) {
    mexErrMsgTxt("Fourth argument must be a scalar.");
  } else {
    scomplex = GetSimplicialComplex(prhs[0]);

    distances = mxDuplicateArray(prhs[1]);
    distanceData = mxGetPr(distances);
    numLandmarks = mxGetM(distances);
    numPoints = mxGetN(distances);

    nu = GetIntFromMxArray(prhs[2]);
    
    if ((nrhs == 4) && (mxIsFinite(mxGetScalar(prhs[3])))) {
      maxDim = std::min(scomplex->getDimension(), GetIntFromMxArray(prhs[3]));
    } else {
      maxDim = scomplex->getDimension();
    }
    
    verticesBegin = scomplex->simplicesBegin(0);
    verticesEnd = scomplex->simplicesEnd(0);

    while (verticesBegin != verticesEnd) {
	 maxVertexLabel = *((*verticesBegin++)->getLabel());
    }

    if (scomplex->getNumSimplices(0) > 0) {
	 if ((numLandmarks != scomplex->getNumSimplices(0))
		&& (numLandmarks != (scomplex->getNumSimplices(0)
						 - scomplex->getNumDegenerateSimplices(0)))
		&& (maxVertexLabel > numLandmarks)) {
	   mexErrMsgTxt("Number of landmarks does not match with number of vertices.");
	 }
    }

    if (nu < 0) {
	 mexErrMsgTxt("Third argument must be non-negative.");
    } else if (nu > numLandmarks) {
	 nu = numLandmarks;
    }

    if (maxDim < 0) {
	 mexErrMsgTxt("Fourth argument must be non-negative.");
    }
  }

  WitnessFilter<SimplexHandle_t, double>
    filter(scomplex->simplicesBegin(0), scomplex->simplicesEnd(0),
		 distanceData, distanceData + numLandmarks * numPoints, 
		 numLandmarks, nu, 1);
  plhs[0] = ExportFiltration(*scomplex, filter, maxDim);
}

}
