/* witness.cc
 * ----------
 * Create the witness complex from a distance matrix.
 *
 * Written by Patrick Perry
 * Original Version on May 8, 2004
 *
 * $Id: px_witness.cc,v 1.2 2005/11/02 01:59:52 patperry Exp $
 * $Log: px_witness.cc,v $
 * Revision 1.2  2005/11/02 01:59:52  patperry
 * The last (?) gcc4.0 fixes\!\!\!
 *
 * Revision 1.1.1.1  2005/03/18 23:37:03  silva
 * Imported Sources
 *
 * Revision 1.4  2005/02/11 22:59:31  patperry
 * Fixed witness vertex labeling
 *
 * Revision 1.3  2004/10/16 22:11:09  silva
 *
 * px_witness, px_wfilt treat k=inf case correctly.
 *
 * Revision 1.2  2004/09/28 23:24:52  patperry
 * Changed behavior for 0-simplices in witness complex.
 *
 * Revision 1.3  2004/09/16 09:00:00  patperry
 * Simplified C++ witness
 *
 * Revision 1.2  2004/05/19 04:15:07  patperry
 * Added doc for witness
 *
 * Revision 1.1  2004/05/10 22:47:17  patperry
 * Initial version of witness
 *
 *
 */

#include <algorithm>
#include <vector>
#include "PlexObject.h"
#include "MxUtility.h"
#include "PackedMatrixColumnIterator.h"
#include "WitnessComplex.h"

extern "C" {
#include "matrix.h"
#include "mex.h"


void 
mexFunction (int nlhs, mxArray *plhs[], int nrhs, const mxArray *prhs[])
{
  SimplicialComplex_t * scomplex;
  const mxArray * distances;
  double * distanceData;
  int numLandmarks;
  int numPoints;
  int nu;
  double R;
  int maxDim;

  // Check for proper number of arguments.
  if ((nrhs != 3) && (nrhs != 4)) {
    mexErrMsgTxt("Three or four inputs required.");
  } else if (nlhs > 2) {
    mexErrMsgTxt("Too many output arguments.");
  } else if ((mxIsSparse(prhs[0])) || !mxIsDouble(prhs[0])) {
    mexErrMsgTxt("First argument must be a full double matrix.");
  } else if (!IsScalarMxArray(prhs[1])) {
    mexErrMsgTxt("Second argument must be a scalar.");
  } else if (!IsScalarMxArray(prhs[2])) {
    mexErrMsgTxt("Third argument must be a scalar.");
  } else if ((nrhs == 4) && !IsScalarMxArray(prhs[2])) {
    mexErrMsgTxt("Fourth argument must be a scalar.");
  } else {
    distances = prhs[0]; 
    distanceData = mxGetPr(distances);
    numLandmarks = mxGetM(distances);
    numPoints = mxGetN(distances);

    nu = GetIntFromMxArray(prhs[1]);
    R = mxGetScalar(prhs[2]);

    if ((nrhs == 4) && (mxIsFinite(mxGetScalar(prhs[3])))) {
      maxDim = std::min(numLandmarks - 1, GetIntFromMxArray(prhs[3]));
    }
    else {
      maxDim = numLandmarks - 1;
    }
    
    if (nu < 0) {
	 mexErrMsgTxt("Second argument must be non-negative.");
    } else if (nu > numLandmarks) {
	 nu = numLandmarks;
    }

    if (maxDim < 0) {
	 mexErrMsgTxt("Fourth argument must be non-negative.");
    }
  }

  PackedMatrixColumnIterator<double> distancesBegin(numLandmarks, numPoints,
										  0, 0, distanceData);
  PackedMatrixColumnIterator<double> distancesEnd(numLandmarks, numPoints,
										0, 0, distanceData 
										+ numLandmarks * numPoints);
  scomplex 
    = new WitnessComplex<>(distancesBegin, distancesEnd, nu, R, maxDim, 1);


  // store the result
  mexCallMATLAB(1, plhs, 0, NULL, "plex");
  SetSimplicialComplex(plhs[0], scomplex);
}

}
