function [out1, out2] = px_witnessnu(DL, nu, in1, in2)

%PX_WITNESSNU -- normalised witness complex
%   C      = PX_WITNESSNU(DL, nu, R, k)
%   [C, F] = PX_WITNESSNU(DL, nu, R, k)
%   F      = PX_WITNESSNU(DL, nu, C)
%   F      = PX_WITNESSNU(DL, nu, C, k)
%
%   PX_WITNESSNU constructs, as a Plex object, the normalised witness
%   complex defined by the landmark-to-data distance matrix DL and
%   normalisation parameter nu.
%
%   Input arguments:
%      DL : landmark-to-data distances (n-by-N matrix)
%      nu : normalisation parameter    (non-negative integer)
%      R  : maximum witness radius     (non-negative real or inf)
%      k  : restrict to k-skeleton     (non-negative integer or inf)
%      C  : restrict to cells in C     (Plex object)
%
%   Output arguments:
%      C  : normalised witness complex (Plex object)
%      F  : associated filtration      (see PERSISTENCE)
%
%   The output arguments are PERSISTENCE-ready. The following sequence is
%   typical:
%
%   >> [C, F] = px_witnessnu(DL, nu, R, k);
%   >> I = persistence(C, F, k-1);
%
%   C = PX_WITNESSNU(DL, nu, R, k). This returns the k-skeleton of the
%   normalised witness complex with normalisation parameter nu and witness
%   radius R. The vertices of C are labelled {1, 2, ..., n} to correspond
%   with the rows of DL.
%
%   [C, F] = PX_WITNESSNU(DL, nu, R, k). The second output argument
%   returns a filtration F compatible with C. For each cell of C it gives
%   the minimum radius at which that cell appears in the normalised
%   witness complex.
%
%   F = PX_WITNESSNU(DL, nu, C). This computes filtration values for all
%   the cells of C.
%
%   F = PX_WITNESSNU(DL, nu, C, k). This computes filtration values for
%   all the cells of C up to the k-skeleton.
%
%   The n rows of DL are associated with the vertices of C according to
%   one of two schemes, whichever is applicable:
%
%      (i) If C has n vertices, these are associated with the rows in
%      increasing order.
%
%      (ii) If C has at most n vertices and the vertex labels are integers
%      in the range {1, ..., n}, then the vertex labelled 'i' is
%      associated with the i-th row of DL.
%    
%   Note that schemes (i) and (ii) agree in the situation where both
%   are applicable. No other possibilities are permitted.
%
%   The normalised witness complex is defined as follows:
%
%   [nu = 0]: the simplex with vertex set I = [i0, ..., ip] belongs to C
%   if and only if there exists j in {1, .., N} such that the condition
%   (D(i,j) <= R) is true for all i in I.
%
%   [nu > 0]: let f(j) be the nu-th smallest entry of the vector D(:, j). 
%   In other words, f(j) gives the distance between the j-th data point
%   and its nu-th nearest landmark neighbour. The simplex with vertex set
%   I = [i0, ..., ip] belongs to C if there exists j in {1, ..., N} such
%   that the condition (D(i,j) <= R + f(j)) is true for all i in I.
%
%   Note that these definitions apply to cells of all dimensions,
%   including the 0-cells. This corrects the situation in previous
%   versions, whereby all vertices would enter the filtration at time 0.
%
%   See also PX_WEAK, PX_STRONG (which both depend on PX_WITNESSNU) and
%   PX_CECH.
%
%Plex Metric Data Toolbox version 2.0 by Vin de Silva, Patrick Perry and
%contributors. See PX_PLEXINFO for credits and licensing information.
%Released with Plex version 2.0. [2005-May-18]

%-------------------------------------------------------
% verify input parameters
%-------------------------------------------------------
error(nargchk(3,4,nargin));

% check first three inputs and determine mode
[err, mode] = local_checkinputs(DL, nu, in1);
error(err);

% mode 1: C is an output argument
% mode 2: C is an input argument

switch mode
 case 1
  if (nargin == 3)
    error('Both R and k must be specified explicitly.')
  end
  R = in1;
  k = in2;
  
 case 2
  C = in1;
  if (nargin == 3)
    k = dim(C);
  else
    k = in2;
    % in this case it doesn't make sense to specify k=inf except as a
    % place-holder for k=dim(C)
    if isequal(k, inf)
      k = dim(C);
    end
  end
end

% check fourth input
error(local_checkinputk(k));

%-------------------------------------------------------
% initialise
%-------------------------------------------------------
[n, N] = size(DL);

%-------------------------------------------------------
% compute C
%-------------------------------------------------------
if (mode == 1)
  C = px_witness(DL, nu, R, k);
end

%-------------------------------------------------------
% compute F
%-------------------------------------------------------
if (mode == 2) | (nargout == 2)
  [DLrows, err] = local_checkvertices(simplices(C,0), n);
  error(err);
  F = px_wfilt(C, DL(DLrows,:), nu, k);
  % HACK-ALERT!!!
  % Add trailing empty matrices [] to make sure F is defined up to
  % dimension k
  if (length(F) > dim(C)+1)
    disp('Tell Vin to remove trailing empties hack');
  elseif (k+1 > length(F))
    Ftrail = cell(1, (k+1) - length(F));
    for a = (1: length(Ftrail))
      Ftrail{a} = [];
    end
    F = [F, Ftrail];
    
  end
end

%-------------------------------------------------------
% output results
%-------------------------------------------------------
switch mode
  
 case 1
  out1 = C;
  if (nargout == 2)
    out2 = F;
  end
  
 case 2
  out1 = F;
  
end

%-------------------------------------------------------
% finished
%-------------------------------------------------------
return

%-------------------------------------------------------
% local functions
%-------------------------------------------------------
function [err,mode] = local_checkinputs(DL, nu, in1)

mode = 0;
err = [];

%-------------
% check DL, nu
%-------------
if ~isnumeric(DL) || (length(size(DL)) ~=2)
  err = 'DL must be a numeric matrix.';
  
elseif ~isnumeric(nu) || (length(nu) > 1) || ...
      (nu ~= abs(floor(nu))) || isinf(nu);
  err = 'nu must be a non-negative integer.';
  
else
  %-----------------------------------------------------
  % no error in DL, nu: now check in1 and determine mode
  %-----------------------------------------------------
  
  if isnumeric(in1) && (length(in1) == 1) && (in1 >= 0)
    % third input argument is R
    mode = 1;
  elseif isa(in1, 'plex')
    % third input argument is C
    mode = 2;    
  else
    err = sprintf('%s %s', ...
		  'Third input must be non-negative real (or inf)', ...
		  'or a plex object.');
  end
end

return;

%-------------------------------------------------------
%-------------------------------------------------------
function err = local_checkinputk(k)

if ~isnumeric(k) || (length(k) > 1) || (k ~= abs(floor(k)))
  err = 'k must be a non-negative integer or inf.';
else
  err = [];
end

return;

%-------------------------------------------------------
%-------------------------------------------------------
function [DLrows, err] = local_checkvertices(Clist, n);

if (length(Clist) == n)
  DLrows = (1: n);
  err = [];

elseif (length(Clist) < n) & all(ismember(Clist, (1: n)))
  DLrows = Clist;
  err = [];

else
  DLrows = [];
  err = 'Vertices of C cannot be matched with rows of DL.';
  
end

return;

%-------------------------------------------------------
%-------------------------------------------------------
