/* px_gsmod2lu_compact.cc
 * ----------------------
 * Find the (mod2) LU-decomposition of a matrix using the geometric
 * sparse solver.
 *
 * $Id: px_gsmod2lu_compact.cc,v 1.1.1.1 2005/03/18 23:37:06 silva Exp $
 * $Log: px_gsmod2lu_compact.cc,v $
 * Revision 1.1.1.1  2005/03/18 23:37:06  silva
 * Imported Sources
 *
 * Revision 1.1.1.1  2004/09/25 22:04:09  patperry
 * Imported sources
 *
 * Revision 1.1  2004/09/15 22:22:16  silva
 * Renamed with px_ prefix for Plex 2.0 release.
 *
 * Revision 1.5  2004/03/17 04:57:25  patperry
 * Fixed multiple cover reductions
 *
 * Revision 1.4  2004/02/09 06:30:44  patperry
 * Got rid of MatlabAllocator.  A few small bugfixes.
 *
 * Revision 1.3  2004/01/23 04:36:37  patperry
 * Cleaned up MATLAB support code.
 *
 * Revision 1.2  2004/01/15 07:27:09  patperry
 * added global makefile
 *
 * Revision 1.1  2004/01/14 04:52:14  patperry
 * Added gsmod2lu matlab mex file.
 *
 *
 */

#include <iostream>
#include "Mod2.h"
#include "CSCSparseMatrix.h"
#include "CRCovering.h"
#include "NerveComplex.h"
#include "GeomSparseMatrix.h"
#include "LUBlockMatrix.h"
#include "LUFullMatrix.h"
#include "MxSupport.h"


extern "C" {
#include "mex.h"
#include "matrix.h"
}

extern "C" {


void 
mexFunction (int nlhs, mxArray *plhs[], int nrhs, const mxArray *prhs[])
{
  // Check for proper number of arguments.
  if (nrhs < 2) {
    mexErrMsgTxt("Two inputs required.");
  } else if (nlhs != 3) {
    mexErrMsgTxt("Three output arguments required.");
  } else if (!mxIsSparse(prhs[0])) {
    mexErrMsgTxt("Matrix argument must be sparse.");
  } else if (mxIsSparse(prhs[1]) 
		   || (mxGetClassID(prhs[1]) != mxLOGICAL_CLASS)
		   || (mxGetM(prhs[1]) != (mxGetN(prhs[0])))) {
    mexErrMsgTxt("Column cover in incorrect format.");
  }
    
  std::vector<GeomSparseMatrix<Mod2> *> gsmatrixStack;
  GeomSparseMatrix<Mod2> * gsMatrix;
  CSCSparseMatrix<Mod2> * inputMatrix
    = getSparseMod2Matrix(prhs[0]);
  
  // for each cover
  for (int cover = 1; cover < nrhs; ++cover) {
    // create a CRCovering object and a nerve complex
    CRCovering * crCover;
    NerveComplex * nerve;

    if (cover == 1) {
	 crCover = getCRCovering(prhs[cover], *inputMatrix);
    } else {
	 crCover = getCRCovering(prhs[cover], *inputMatrix, 
						gsMatrix->getQ(), gsMatrix->getNumPivots());
    }

    nerve = new NerveComplex(*crCover);    

    mexPrintf("About to create geomsparsematrix...\n");
    // create the geometric sparse matrix and reduce it
    gsMatrix = new GeomSparseMatrix<Mod2>(*inputMatrix, *nerve);
    mexPrintf("Reducing...\n");
    gsMatrix->reduce();
    mexPrintf("done.\n");

    // push the matrix onto the stack
    gsmatrixStack.push_back(gsMatrix);
    
    // free the original input matrix on the first iteration
    if (cover == 1) {
	 delete inputMatrix;
    }
    
    // set the input matrix to be the lower right block
    inputMatrix = gsMatrix->getLowerRightBlock();
    
    // free the nerve and cover memory
    delete crCover;
    delete nerve;
  }
  
  // reduce the last lower-right block using a standard algorithm
  LUFullMatrix<Mod2> lowerRight(*inputMatrix);
  lowerRight.reduce();
  
  // combine the four blocks in the innermost level
  CSCSparseMatrix<Mod2> * lowerRightReduction
    = gsMatrix->combineBlocks(lowerRight, lowerRight.getP(), 
						lowerRight.getQ());
  
  // iteratively combine the lower right blocks, starting in and moving out
  for (int cover = nrhs - 2; cover >= 1; --cover) {
    // use the result from the previous combination as the lower right
    GeomSparseMatrix<Mod2> * lowerRight = gsMatrix;
    
    // the stack contains the lower right.  we pop this value and take the
    // new top of the stack
    gsmatrixStack.pop_back();
    gsMatrix = gsmatrixStack.back();
    
    // now, we combine the blocks
    CSCSparseMatrix<Mod2> * newLowerRightReduction 
	 = gsMatrix->combineBlocks(*lowerRightReduction, lowerRight->getP(), 
						  lowerRight->getQ());
    
    // delete the lower-right gsMatrix and lower right reduction
    delete lowerRight;
    delete lowerRightReduction;
    
    // set the new reduction to be the next lower-right reduction
    lowerRightReduction = newLowerRightReduction;
  }
  
  // copy the fully-reduced matrix into the output
  plhs[0] = getSparseMatlabMatrix(*lowerRightReduction);
  
  // copy the row permutation into the output
  plhs[1] = getMatlabArray(gsMatrix->getP(), 
					  gsMatrix->getP() 
					  + lowerRightReduction->getNumRows());
  
  // copy the column permutation into the output
  plhs[2] = getMatlabArray(gsMatrix->getQ(),
					  gsMatrix->getQ() 
					  + lowerRightReduction->getNumColumns());
  
}

}
