/* plexgc.cc
 * ---------
 * PLEX garbage collector.
 * 
 * Written by Patrick Perry
 * Original Version on June 21, 2004
 *
 * $Id: px_plexgc.cc,v 1.2 2005/07/21 02:54:20 patperry Exp $
 * $Log: px_plexgc.cc,v $
 * Revision 1.2  2005/07/21 02:54:20  patperry
 * Added 64-bit compatibility.
 *
 * Revision 1.1.1.1  2005/03/18 23:36:44  silva
 * Imported Sources
 *
 * Revision 1.1.1.1  2004/09/25 22:04:09  patperry
 * Imported sources
 *
 * Revision 1.1  2004/06/28 22:03:47  patperry
 * Added Plex Garbage Collector
 *
 */


#include <iostream>
#include <set>
#include <string>
#include "PlexObject.h"

extern "C" {
#include "matrix.h"
#include "mex.h"


static bool IsCalledFromBase ();
static const std::string CreateBaseToken ();
static void DestroyBaseToken (const std::string& baseTokenName);



void
mexFunction (int nlhs, mxArray *plhs[], int nrhs, const mxArray *prhs[])
{
  if (nrhs != 0) {
    mexErrMsgTxt("No input arguments required.");
  } else if (nlhs != 0) {
    mexErrMsgTxt("No output arguments required.");
  } else if (!IsCalledFromBase()) {
    mexErrMsgTxt("Garbage collector can only be called from base workspace.");
  }

  mxArray * whoBaseArgs[2] = { mxCreateString("base"), mxCreateString("who") };
  mxArray * whoGlobalArgs[1] = { mxCreateString("global") };
  std::set<SimplicialComplex_t *> activeComplexes;

  for (int command = 0; command < 2; ++command) {
    const mxArray * variable;
    mxArray * variableList;
  
    if (command == 0) {
	 mexCallMATLAB(1, &variableList, 2, whoBaseArgs, "evalin");
    } else {
	 mexCallMATLAB(1, &variableList, 1, whoGlobalArgs, "who");
    }
  
    for (int i = 0; i < mxGetNumberOfElements(variableList); ++i) {
	 const mxArray * variableName = mxGetCell(variableList, i);
	 char * name = mxArrayToString(variableName);
	 if (command == 0) {
	   variable = mexGetVariablePtr("base", name); 
	 } else {
	   variable = mexGetVariablePtr("global", name); 
	 }

	 if (IsPlexObject(variable)) {
	   //std::cout << "   " << name << " is an active plex object\n";
	   SimplicialComplex_t * scomplex = GetSimplicialComplex(variable);
	   activeComplexes.insert(scomplex);
	 } else {
	   //std::cout << "   " << name << " is not a plex object\n";
	 }
	 
	 mxFree(name);
    }
  }

  CollectManagedComplexGarbage(activeComplexes.begin(), 
						 activeComplexes.end());
}


bool IsCalledFromBase ()
{
  std::string baseTokenName = CreateBaseToken();
  const mxArray * baseToken = mexGetVariablePtr("base", baseTokenName.c_str());

  mxArray * callerWhoArgs[2] = { mxCreateString("caller"), 
						   mxCreateString("who") };
  mxArray * variableList;
  bool result = false;

  mexCallMATLAB(1, &variableList, 2, callerWhoArgs, "evalin");
  
  const mxArray * lastVariableName 
    = mxGetCell(variableList, mxGetNumberOfElements(variableList) - 1);
  char * name = mxArrayToString(lastVariableName);
  const mxArray * lastVariable = mexGetVariablePtr("caller", name); 
    
  mxFree(name);

  result = (lastVariable == baseToken);

  DestroyBaseToken(baseTokenName);
  
  return result;
}


const std::string CreateBaseToken ()
{
  mxArray * baseWhoArgs[2] = { mxCreateString("base"), mxCreateString("who") };
  mxArray * baseVariableList;

  mexCallMATLAB(1, &baseVariableList, 2, baseWhoArgs, "evalin");

  mxArray * baseTokenName;
  std::string result;

  if (!mxIsEmpty(baseVariableList)) {
    mxArray * lastVariable 
	 = mxGetCell(baseVariableList, 
			   mxGetNumberOfElements(baseVariableList) - 1);
			 
    int lastNameLength = mxGetNumberOfElements(lastVariable);
    char nameBuffer[lastNameLength + 2];
    
    mxGetNChars(lastVariable, nameBuffer, lastNameLength);
    nameBuffer[lastNameLength] = 'X';
    nameBuffer[lastNameLength + 1] = '\0';
    
    baseTokenName = mxCreateStringFromNChars(nameBuffer, lastNameLength + 1);
    result = std::string(nameBuffer);
  } else {
    baseTokenName = mxCreateStringFromNChars("X", 1);
    result = "X";
  }
									
  mxArray * assignBaseArgs[3] = { mxCreateString("base"), 
						    baseTokenName,
						    mxCreateDoubleScalar(0) };

  mexCallMATLAB(0, NULL, 3, assignBaseArgs, "assignin");

  return result;
}


void DestroyBaseToken (const std::string& baseTokenName)
{
  std::string clearCommand = "clear ";

  clearCommand.append(baseTokenName);
  
  mxArray * clearBaseArgs[2] = { mxCreateString("base"), 
						    mxCreateString(clearCommand.c_str()) };
  
  mexCallMATLAB(0, NULL, 2, clearBaseArgs, "evalin");
}

}
