/* MxSupport.cc
 * -------------
 * Implements MxSupport.h
 *
 * $Id: MxSupport.cc,v 1.2 2005/07/21 07:45:52 patperry Exp $
 * $Log: MxSupport.cc,v $
 * Revision 1.2  2005/07/21 07:45:52  patperry
 * More 64-bit fixes.
 *
 * Revision 1.1.1.1  2005/03/18 23:36:36  silva
 * Imported Sources
 *
 * Revision 1.1.1.1  2004/09/25 22:04:09  patperry
 * Imported sources
 *
 * Revision 1.4  2004/04/15 19:41:29  patperry
 * fixed bug in getMod2Matrix
 *
 * Revision 1.3  2004/02/09 06:34:36  patperry
 * Lots of bugfixes.
 *
 * Revision 1.2  2004/01/26 02:52:44  patperry
 * Added new functionality.
 *
 * Revision 1.1  2004/01/15 07:15:43  patperry
 * Imported support.{c,h}, changed file extensions
 *
 * Revision 1.1  2003/11/18 08:57:38  patperry
 * Added support code.
 *
 *
 */

#include <algorithm>
#include <cmath>
#include <limits>
#include <memory>
#include "CSCSparseMatrix.h"
#include "FullMatrix.h"
#include "MxSupport.h"

/**
 * Casts a double to an unsigned short.  Raises an error if the value is out
 * of range.
 *
 * \param value the double value.
 * \returns the unsigned short value.
 */
static unsigned short getUnsignedShortFromDouble (double value);

/**
 * Casts a long to an unsigned short.  Raises an error if the value is out
 * of range.
 *
 * \param value the long value.
 * \returns the unsigned short value.
 */
static unsigned short getUnsignedShortFromInt (long value);

/**
* Casts a long to an unsigned short.  Raises an error if the value is out
 * of range.
 *
 * \param value the long value.
 * \returns the unsigned short value.
 */
static unsigned short getUnsignedShortFromUint (unsigned long value);


FullMatrix<double> *
getFullDoubleMatrix (const mxArray * array)
{
  FullMatrix<double> * result = NULL;
  
	// validate the data type
  if (!mxIsDouble(array)) {
    mexErrMsgTxt("Input must be of type \"double\".");
  } else if (mxIsComplex(array)) {
    mexErrMsgTxt("Input must be real.");
  } else if (mxIsSparse(array)) {
    mexErrMsgTxt("Input must not be sparse.");
  } else {
    int m = mxGetM(array);
    int n = mxGetN(array);
    const double * pr = mxGetPr(array);
    
    result = new FullMatrix<double>(pr, m, n);
  }
  
  return result;
}


CSCSparseMatrix<double> *
getSparseDoubleMatrix (const mxArray * array)
{
  CSCSparseMatrix<double> * result = NULL;
  
  // validate the data type
  if (!mxIsDouble(array)) {
    mexErrMsgTxt("Input must be of type \"double\".");
  } else if (mxIsComplex(array)) {
    mexErrMsgTxt("Input must be real.");
  } else if (!mxIsSparse(array)) {
    mexErrMsgTxt("Input must be sparse.");
  } else {
    int m = mxGetM(array);
    int n = mxGetN(array);
    int nzmax = mxGetNzmax(array);
    const double * pr = mxGetPr(array);
    const int * ir = mxGetIr(array);
    const int * jc = mxGetJc(array);
    
    result = new CSCSparseMatrix<double>(nzmax, pr, ir, jc, m, n);
  }
	
  return result;
}


FullMatrix<Mod2> *
getFullMod2Matrix (const mxArray * array)
{
  FullMatrix<Mod2> * result = NULL;
  
  // validate the data type
  if (!mxIsDouble(array) && !mxIsLogical(array)) {
    mexErrMsgTxt("Input must be of type \"double\" or \"logical\".");
  } else if (mxIsComplex(array)) {
    mexErrMsgTxt("Input must be real.");
  } else if (mxIsSparse(array)) {
    mexErrMsgTxt("Input must not be sparse.");
  } else {
    int m = mxGetM(array);
    int n = mxGetN(array);
    Mod2 mod2Pr[m * n];

    if (mxIsDouble(array)) {
	 const double * doublePr = mxGetPr(array);
    
	 std::copy(doublePr, doublePr + m * n, mod2Pr);
    } else {
	 const mxLogical * logicalPr = mxGetLogicals(array);

	 std::copy(logicalPr, logicalPr + m * n, mod2Pr);
    }

    result = new FullMatrix<Mod2>(mod2Pr, m, n);
  }
  
  return result;
}


CSCSparseMatrix<Mod2> *
getSparseMod2Matrix (const mxArray * array)
{
  CSCSparseMatrix<Mod2> * result = NULL;
  
  // validate the data type
  if (!mxIsDouble(array) && !mxIsLogical(array)) {
    mexErrMsgTxt("Input must be of type \"double\" or \"logical\".");
  } else if (mxIsComplex(array)) {
    mexErrMsgTxt("Input must be real.");
  } else if (!mxIsSparse(array)) {
    mexErrMsgTxt("Input must be sparse.");
  } else {
    int m = mxGetM(array);
    int n = mxGetN(array);
    int nzmax = mxGetNzmax(array);
    const int * ir = mxGetIr(array);
    const int * jc = mxGetJc(array);
    Mod2 mod2Pr[nzmax];

    if (mxIsDouble(array)) {
	 const double * doublePr = mxGetPr(array);

	 std::uninitialized_copy(doublePr, doublePr + nzmax, mod2Pr);
    } else {
	 const mxLogical * logicalPr = mxGetLogicals(array);

	 std::uninitialized_copy(logicalPr, logicalPr + nzmax, mod2Pr);
    }
    
    result = new CSCSparseMatrix<Mod2>(nzmax, mod2Pr, ir, jc, m, n);
  }
  
  return result;
}


unsigned short getUnsignedShortFromMxArray (const mxArray * array)
{
  double doubleValue;
  long intValue;
  unsigned long uintValue;
  unsigned short result;
  bool error = true;

  if ((mxGetM(array) == 1) 
	 && (mxGetN(array) == 1)
	 && (!mxIsComplex(array))) {
    switch(mxGetClassID(array)) {
    case mxDOUBLE_CLASS:
	 doubleValue = *static_cast<double *>(mxGetData(array));
	 result = getUnsignedShortFromDouble(doubleValue);
	 error = false;
	 break;
    case mxSINGLE_CLASS:
	 doubleValue = *static_cast<float *>(mxGetData(array));
	 result = getUnsignedShortFromDouble(doubleValue);
	 error = false;
	 break;
    case mxINT8_CLASS:
	 intValue = *static_cast<char *>(mxGetData(array));
	 result = getUnsignedShortFromInt(intValue);
	 error = false;
	 break;
    case mxINT16_CLASS:
	 intValue = *static_cast<short *>(mxGetData(array));
	 result = getUnsignedShortFromInt(intValue);
	 error = false;
	 break;
    case mxINT32_CLASS:
	 intValue = *static_cast<int *>(mxGetData(array));
	 result = getUnsignedShortFromInt(intValue);
	 error = false;
	 break;
    case mxINT64_CLASS:
	 intValue = *static_cast<long *>(mxGetData(array));
	 result = getUnsignedShortFromInt(intValue);
	 error = false;
	 break;
    case mxUINT8_CLASS:
	 uintValue = *static_cast<unsigned char *>(mxGetData(array));
	 result = getUnsignedShortFromUint(uintValue);
	 error = false;
	 break;
    case mxUINT16_CLASS:
	 uintValue = *static_cast<unsigned short *>(mxGetData(array));
	 result = getUnsignedShortFromUint(uintValue);
	 error = false;
	 break;
    case mxUINT32_CLASS:
	 uintValue = *static_cast<unsigned int *>(mxGetData(array));
	 result = getUnsignedShortFromUint(uintValue);
	 error = false;
	 break;
    case mxUINT64_CLASS:
	 uintValue = *static_cast<unsigned long *>(mxGetData(array));
	 result = getUnsignedShortFromUint(uintValue);
	 error = false;
	 break;
    default:
	 result = 0;
	 error = true;
    }
  }
  
  if (error) {
    mexErrMsgTxt("Input must be a non-negative short integer.");
  }

  return result;
}


static unsigned short getUnsignedShortFromDouble (double value)
{
  unsigned short result = 0;

  if ((value == std::floor(value))
	 && (value <= std::numeric_limits<unsigned short>::max())) {
    result = static_cast<unsigned short>(value);
  } else {
    mexErrMsgTxt("Input must be a non-negative short integer.");
  }

  return result;
}


static unsigned short getUnsignedShortFromInt (long value)
{
  unsigned short result = 0;

  if ((value >= 0)
	 && (value <= std::numeric_limits<unsigned short>::max())) {
    result = static_cast<unsigned short>(value);
  } else {
    mexErrMsgTxt("Input must be a non-negative short integer.");
  }

  return result;
}    


static unsigned short getUnsignedShortFromUint (unsigned long value)
{
  unsigned short result = 0;

  if (value <= std::numeric_limits<unsigned short>::max()) {
    result = static_cast<unsigned short>(value);
  } else {
    mexErrMsgTxt("Input must be a non-negative short integer.");
  }

  return result;
}    

