/* MxSupport.h
 * -----------
 * Support code for plex interfaces to simplicial complexes.
 *
 * $Id: MxSupport.h,v 1.2 2005/07/21 07:45:52 patperry Exp $
 * $Log: MxSupport.h,v $
 * Revision 1.2  2005/07/21 07:45:52  patperry
 * More 64-bit fixes.
 *
 * Revision 1.1.1.1  2005/03/18 23:36:37  silva
 * Imported Sources
 *
 * Revision 1.1.1.1  2004/09/25 22:04:09  patperry
 * Imported sources
 *
 * Revision 1.4  2004/03/17 04:57:25  patperry
 * Fixed multiple cover reductions
 *
 * Revision 1.3  2004/02/09 06:34:36  patperry
 * Lots of bugfixes.
 *
 * Revision 1.2  2004/01/23 04:36:44  patperry
 * Cleaned up MATLAB support code.
 *
 * Revision 1.1  2004/01/15 07:15:43  patperry
 * Imported support.{c,h}, changed file extensions
 *
 * Revision 1.1  2003/11/18 08:57:38  patperry
 * Added support code.
 *
 *
 */

#ifndef _MXSUPPORT_H
#define _MXSUPPORT_H

#include "CRCovering.h"
#include "CSCSparseMatrix.h"
#include "FullMatrix.h"
#include "Matrix.h"
#include "RowColumnIterator.h"
#include "Mod2.h"
#include "SimplicialComplex.h"

extern "C" {
#include "/matlabdir/extern/include/mex.h"
#include "/matlabdir/extern/include/matrix.h"
}


/**
 * Gets a real full matrix from an mxArray.  The mxArray must be full and 
 * double.  An error is raised if the array parameter is not a double matrix.  
 *
 * \param array the input matrix.
 * \returns a new FullMatrix object initialized from the passed-in array.
 *					The client is responsible for destroying the returned object.
 */
FullMatrix<double> *
getFullDoubleMatrix (const mxArray * array);

/**
 * Gets a real sparse matrix from an mxArray.  The mxArray must be sparse
 * and real.  An error is raised if this condition is not met.
 * 
 * \param array the input matrix.
 * \returns a CSCSparseMatrix with the data of the matrix.  This must be
 *					destroyed by the client.
 */
CSCSparseMatrix<double> *
getSparseDoubleMatrix (const mxArray * array);

/**
 * Gets a full Mod2 matrix from an mxArray.  The mxArray must be full and
 * contain double data.
 *
 * \param array the input matrix.
 * \returns a new Matrix object initialized from the passed-in array.
 * The client is responsible for destroying the returned object.
 */
FullMatrix<Mod2> *
getFullMod2Matrix (const mxArray * array);

/**
 * Gets a sparse Mod2 matrix from an mxArray.  The mxArray must be sparse and
 * contain double data.
 *
 * \param array the input matrix.
 * \returns a new Matrix object initialized from the passed-in array.
 * The client is responsible for destroying the returned object.
 */
CSCSparseMatrix<Mod2> * 
getSparseMod2Matrix (const mxArray * array);

/**
 * Given a custum matrix object, returns a full double matlab matrix object.
 *
 * \param input the input matrix.
 * \returns a mxArray with the same data as the matrix.
 */
template <class Element_t, class ColumnIterator_t, class Alloc>
mxArray * 
getFullMatlabMatrix (const Matrix<Element_t, ColumnIterator_t, Alloc>& input)
{
	int m = input.getNumRows();
	int n = input.getNumColumns();
	mxArray * result =  mxCreateNumericMatrix(m, n, mxDOUBLE_CLASS, mxREAL);
	double * pr = mxGetPr(result);
		
	for (int j = 0; j < n; ++j) {
	  //pr = std::copy(input.getColumnBegin(j), input.getColumnEnd(j), pr);
	  for (int i = 0; i < m; ++i) {
	    *pr++ = static_cast<double>(input.getElement(i, j));
	  }
	}
	
	return result;
}


/**
* Given a custum sparse matrix object, returns a sparse double matlab 
 * matrix object.
 *
 * \param input the input matrix.
 * \returns a mxArray with the same data as the matrix.
 */
template <class Element_t, class Alloc>
mxArray *  
getSparseMatlabMatrix (const CSCSparseMatrix<Element_t, Alloc>& input)
{
	int m = input.getNumRows();
	int n = input.getNumColumns();
	int nnz = input.getNumNonzeroElements();
	const Element_t * pr = input.getDataPointer();
	const int * ir = input.getRowIndexPointer();
	const int * jc = input.getColumnOffsetPointer();
	
	mxArray * result =  mxCreateSparse(m, n, nnz, mxREAL);
	std::copy(pr, pr + nnz, mxGetPr(result));
	std::copy(ir, ir + nnz, mxGetIr(result));
	std::copy(jc, jc + n + 1, mxGetJc(result));


	return result;
}


/**
 * Takes data in an array and casts it to a MATLAB double array.
 *
 * \param begin an iterator pointing to the first element in the array
 * \param end an iterator pointing to the end of the array
 */
template <class InputIterator>
mxArray * 
getMatlabArray (InputIterator begin, InputIterator end)
{
	int size = std::distance(begin, end);
	mxArray * result = mxCreateNumericMatrix(1, size, mxDOUBLE_CLASS, mxREAL);
	double * pr = mxGetPr(result);
	
	while (begin != end) {
	  *pr++ = static_cast<double>(*begin++);
	}
	//std::copy(begin, end, pr);

	return result;

}
	

/**
 * Takes the column cover data stored in the mxArray and a matrix, and 
 * returns a CRCover object.  The client is responsible for freeing the 
 * returned object.
 *
 * \param cover the column cover.  This is an (n x p) logical matrix, where
 * n is the number of columns, and p is the number of column sets.
 * \param matrix the matrix to compute the row cover from.
 * \returns a column cover object.
 */
template <class Element_t, class ColumnIterator_t, class Alloc>
CRCovering *
getCRCovering (const mxArray * cover, 
			const Matrix<Element_t, ColumnIterator_t, Alloc>& matrix,
			const int * permutation = NULL, int numPivots = 0)
{
  mxLogical * pr = mxGetLogicals(cover);
  int numColumns = mxGetM(cover);
  int numSets = mxGetN(cover);
  std::vector<int> columnIndices;
  std::vector<int> setIndices;
  CRCovering * result = NULL;
  
  // extract the cover data
  for (int set = 0; set < numSets; ++set) {
    for (int column = 0; column < numColumns; ++column) {
	 if (*pr++) {
	   int columnIndex = column;

	   if (permutation) {
		columnIndex = permutation[column];
	   }

	   if (columnIndex > numPivots) {
		columnIndices.push_back(columnIndex - numPivots);
		setIndices.push_back(set);
	   }
	 }
    }
  }
  
  // create the cover
  result = new CRCovering(columnIndices.begin(), setIndices.begin(),
					 columnIndices.size(), numSets, matrix);
  
  return result;
}

/**
 * Gets an unsigned short from a mxArray.  This is a simple static cast from
 * the scalar value.  An error is raised if the array is not 1x1 and numeric.
 *
 * \param array a mxArray asumed to be a scalar.
 * \returns the cast to unsigned short of the scalar.
 */
unsigned short getUnsignedShortFromMxArray (const mxArray * array);


#endif // _MXSUPPORT_H
