/* PlexFiltration.h
 * ----------------
 * Convenience functions for going between MATLAB and C++ filtrations.
 *
 * Written by Patrick Perry
 * Original Version on May 20, 2004
 *
 * $Id: PlexFiltration.h,v 1.2 2005/05/03 22:34:14 patperry Exp $
 * $Log: PlexFiltration.h,v $
 * Revision 1.2  2005/05/03 22:34:14  patperry
 * Fixed intersect and union bugs
 *
 * Revision 1.1.1.1  2005/03/18 23:36:37  silva
 * Imported Sources
 *
 * Revision 1.1.1.1  2004/09/25 22:04:09  patperry
 * Imported sources
 *
 * Revision 1.3  2004/05/28 05:45:48  patperry
 * Implemented union of filtrations
 *
 * Revision 1.2  2004/05/23 20:19:01  patperry
 * Cleaned up Filter interface.
 *
 * Revision 1.1  2004/05/21 21:37:24  patperry
 * Cleaned up Filtration/Persistence interface
 *
 */

#ifndef _PLEXFILTRATION_H
#define _PLEXFILTRATION_H


extern "C" {
#include "/matlabdir/extern/include/mex.h"
#include "/matlabdir/extern/include/matrix.h"
}

#include "Filtration.h"
#include "PlexObject.h"

//! The type of simplex handle
typedef const Simplex_t * SimplexHandle_t;

//! The numeric type for filtration values
typedef double FiltrationValue_t;

//! The data structure for filtrations in C++
typedef Filtration<SimplexHandle_t, FiltrationValue_t> Filtration_t;

/**
 * Indicates whether or not an mxArray stores filtration data for a
 * given simplicial complex
 *
 * \param scomplex the simplicial complex supposedly filtered by the data.
 * \param data the mxArray to validate as filtration data.
 * \returns true if data is a valid filtration of scomplex.
 */
bool IsValidFiltration (const SimplicialComplex_t& scomplex,
				    const mxArray * data);

/**
 * Import a filtration stored in an mxArray.
 *
 * \param scomplex the simplicial complex filtered by the data.
 * \param data the raw filtration data stored in an mxArray
 * \param maxDimension the maximum simplex dimension to include
 * \returns a new filtration object.  The client must free the object
 *          using the delete operator.
 */
Filtration_t * ImportFiltration (const SimplicialComplex_t& scomplex,
						   const mxArray * data, int maxDimension);

/**
 * Export a filtration to an mxArray.
 * 
 * \param scomplex the simplicial complex the filtration object filters.
 * \param filtrationFn the filtration function.
 * \param maxDimension the maximum simplex dimension to include
 * \returns a newly allocated mxArray storing the raw filtration data.
 */
template <class UnaryFunction>
mxArray * ExportFiltration (const SimplicialComplex_t& scomplex,
					   UnaryFunction filtrationFn, int maxDimension)
{
  mxArray * result = NULL;

  result = mxCreateCellMatrix(1, maxDimension + 1);

  for (int i = 0; i <= maxDimension; ++i) {
    int numSimplices = scomplex.getNumSimplices(i);

    // allocate space for the filtration values of dimension-i simplices
    mxArray * simplexFiltration 
	 = mxCreateDoubleMatrix(1, numSimplices, mxREAL);
    double * pr = mxGetPr(simplexFiltration);

    // copy the values to the allocated mxArray
    SimplicialComplex_t::ConstSimplexIterator it 
	 = scomplex.simplicesBegin(i);
    SimplicialComplex_t::ConstSimplexIterator end
	 = scomplex.simplicesEnd(i);

    while (it != end) {
	 SimplexHandle_t simplex = *it++;
	 FiltrationValue_t value = filtrationFn(simplex);

	 *pr++ = value;
    }

    // set the corresponding entry in the cell array
    mxSetCell(result, i, simplexFiltration);
  }
					    
  return result;
}


#endif // _PLEXFILTRATION_H



