/* PlexObject.c
 * ------------
 * Implements PlexObject.h
 *
 * Written by Patrick Perry
 * Original Version on February 25 2004
 *
 * $Id: PlexObject.cc,v 1.4 2005/07/21 09:26:01 patperry Exp $
 * $Log: PlexObject.cc,v $
 * Revision 1.4  2005/07/21 09:26:01  patperry
 * 64-bit bugfix.
 *
 * Revision 1.3  2005/07/21 07:45:52  patperry
 * More 64-bit fixes.
 *
 * Revision 1.2  2005/07/21 02:54:20  patperry
 * Added 64-bit compatibility.
 *
 * Revision 1.1.1.1  2005/03/18 23:36:38  silva
 * Imported Sources
 *
 * Revision 1.4  2004/09/29 21:38:05  patperry
 * changed manager to px_plexmanager
 *
 * Revision 1.3  2004/09/29 19:03:28  patperry
 * Fixed compile bug in PlexObject.cc
 *
 * Revision 1.2  2004/09/28 23:40:02  patperry
 * Added plexmanager and fixed gc
 *
 * Revision 1.1.1.1  2004/09/25 22:04:09  patperry
 * Imported sources
 *
 * Revision 1.3  2004/06/28 22:03:47  patperry
 * Added Plex Garbage Collector
 *
 * Revision 1.2  2004/03/09 09:12:46  patperry
 * Implemented nbhood
 *
 * Revision 1.1  2004/02/26 02:34:04  patperry
 * Fixed bugs in PlexObject
 *
 * Revision 1.1  2004/02/25 13:00:45  patperry
 * added new PlexObject interface
 *
 */

#include <algorithm>
#include <cstdlib>
#include "PlexObject.h"

static const char * PLEX_OBJECT_TAG = "plex";
static const char * HANDLE_FIELD_TAG = "handle";
static const char * PLEX_FIELDS[] = { HANDLE_FIELD_TAG };
static const int NUM_PLEX_FIELDS = 1;


/**
 * Get a pointer to the colection of managed complexes.
 *
 * \returns a pointer to the collection of managed complexes.
 */
static ManagedComplexCollection * GetManagedComplexesPtr ();


mxArray * 
AllocPlexObject ()
{
  mxArray * result = mxCreateStructMatrix(1, 1, NUM_PLEX_FIELDS, PLEX_FIELDS);
  mxArray * scomplexHandle = mxCreateNumericMatrix(1, 1, mxINT32_CLASS, 
										 mxREAL);
  mxArray * setClassArgs[2];
											
  mexMakeMemoryPersistent(scomplexHandle);

  // set the appropriate field in the object
  mxSetField(result, 0, HANDLE_FIELD_TAG, scomplexHandle);

  // set the class type
  setClassArgs[0] = result;
  setClassArgs[1] = mxCreateString(PLEX_OBJECT_TAG);
  mexCallMATLAB(1, &result, 2, setClassArgs, "class");
  
  // clean up all temporary memory used
  mxDestroyArray(setClassArgs[1]);
  
  return result;
}


void 
FreePlexObject (mxArray * plexObject)
{
  mxArray * scomplexHandle = NULL;
  
  if (IsPlexObject(plexObject)
	 && (scomplexHandle = mxGetField(plexObject, 0, HANDLE_FIELD_TAG))) {
    mxDestroyArray(scomplexHandle);
    mxDestroyArray(plexObject);
  } else {
    mexErrMsgTxt("Input must be a valid plex object.");
  }
}


bool
IsPlexObject (const mxArray * array) 
{
  return ((array != NULL) && (mxIsClass(array, PLEX_OBJECT_TAG)));
}


SimplicialComplex_t * 
GetSimplicialComplex (const mxArray * plexObject)
{
  mxArray * scomplexHandleField = NULL;
  int handle;
  SimplicialComplex_t * result = NULL;
  
  if (IsPlexObject(plexObject)
	 && (scomplexHandleField = mxGetField(plexObject, 0, HANDLE_FIELD_TAG))) {
    ManagedComplexCollection * managedComplexes = GetManagedComplexesPtr();

    handle = *reinterpret_cast<int32_t *>(mxGetData(scomplexHandleField));
    result = (*managedComplexes)[handle];
  } else {
    mexErrMsgTxt("Input must be a valid plex object.");
  }

  return result;
}


void 
SetSimplicialComplex (mxArray * plexObject, SimplicialComplex_t * scomplex)
{
  mxArray * scomplexHandle = NULL;
  ManagedComplexCollection * managedComplexes = GetManagedComplexesPtr();
  ManagedComplexIterator position;

  if (IsPlexObject(plexObject)
	 && (scomplexHandle = mxGetField(plexObject, 0, HANDLE_FIELD_TAG))) {

    ManagedComplexIterator begin = managedComplexes->begin();
    ManagedComplexIterator end = managedComplexes->end();

    if (!managedComplexes->empty()) {
	 position = std::find(begin, end, scomplex);
    }

    if ((managedComplexes->empty())
	   || (position == end)) {
	 managedComplexes->push_back(scomplex);
	 position = begin + managedComplexes->size() - 1;
    }

    *reinterpret_cast<int32_t *>(mxGetData(scomplexHandle)) 
	 = static_cast<int32_t>(std::distance(begin, position));
								   
  } else {
    mexErrMsgTxt("Input must be a valid plex object.");
  }
}


ManagedComplexIterator 
GetManagedComplexesBegin ()
{
  ManagedComplexCollection * managedComplexes = GetManagedComplexesPtr();
  return managedComplexes->begin();
}


ManagedComplexIterator 
GetManagedComplexesEnd ()
{
  ManagedComplexCollection * managedComplexes = GetManagedComplexesPtr();
  return managedComplexes->end();
}


void 
StopManagingComplex (SimplicialComplex_t * scomplex)
{
  if (scomplex) {
    ManagedComplexCollection * managedComplexes = GetManagedComplexesPtr();
    ManagedComplexIterator pos 
	 = std::find(managedComplexes->begin(), managedComplexes->end(), 
			   scomplex);

    if (pos != managedComplexes->end()) {
	 *pos = NULL;
    }
  }
}


ManagedComplexCollection * 
GetManagedComplexesPtr ()
{
  ManagedComplexCollection * result = 0;
  mxArray * matlabPtr;
  mxArray * dummyArg;

  mexCallMATLAB(1, &matlabPtr, 0, &dummyArg, "px_plexmanager");

  result = reinterpret_cast<ManagedComplexCollection *>
    (*reinterpret_cast<std::size_t *>(mxGetData(matlabPtr)));

  return result;
}
