/* PlexObject.h
 * ------------
 * This defines an interface to the matlab sorage of a plex object.
 *
 * Written by Patrick Perry
 * Original version on 2/25/04
 *
 * $Id: PlexObject.h,v 1.3 2005/10/24 16:20:25 patperry Exp $
 * $Log: PlexObject.h,v $
 * Revision 1.3  2005/10/24 16:20:25  patperry
 * Hells (literally) of compile fixes for gcc4.0
 *
 * Revision 1.2  2005/07/21 02:54:20  patperry
 * Added 64-bit compatibility.
 *
 * Revision 1.1.1.1  2005/03/18 23:36:38  silva
 * Imported Sources
 *
 * Revision 1.2  2005/02/11 23:55:04  patperry
 * Fixed vertex labling in wfilt
 *
 * Revision 1.1.1.1  2004/09/25 22:04:09  patperry
 * Imported sources
 *
 * Revision 1.4  2004/06/28 22:03:47  patperry
 * Added Plex Garbage Collector
 *
 * Revision 1.3  2004/02/26 06:49:24  patperry
 * Converted mexfiles to use new scomplex interface
 *
 * Revision 1.2  2004/02/26 02:34:04  patperry
 * Fixed bugs in PlexObject
 *
 * Revision 1.1  2004/02/25 13:00:45  patperry
 * added new PlexObject interface
 *
 */

#ifndef _PLEXOBJECT_H
#define _PLEXOBJECT_H

extern "C" {
#include "/matlabdir/extern/include/mex.h"
#include "/matlabdir/extern/include/matrix.h"
}


#include <iterator>
#include <memory>
#include <vector>
#include "SimplicialComplex.h"


//! The allocator type for plex objects
typedef std::allocator<int> Alloc_t;

//! The simplicial complex type stored in a plex object.
typedef SimplicialComplex<Alloc_t> SimplicialComplex_t;

//! The simplex type
typedef SimplicialComplex<Alloc_t>::Simplex Simplex_t;

//! The simplex iterator types
typedef SimplicialComplex<Alloc_t>::ConstSimplexIterator ConstSimplexIterator_t;
typedef SimplicialComplex<Alloc_t>::SimplexIterator SimplexIterator_t;


/**
 * Allocates a plex object.
 *
 * \returns a new plex object.
 */
mxArray * AllocPlexObject ();

/**
 * Frees the memory in a plex object.
 *
 * \param plexObject the plex object.
 */
void FreePlexObject (mxArray * plexObject);

/**
 * Indicates whether or not an mxArray points to a plex object.
 *
 * \param array the mxArray to test.
 * \returns true if the array is a plex object.
 */
bool IsPlexObject (const mxArray * array);

/**
 * Get the simplicial complex pointer stored in a plex object.
 *
 * \param plexObject the plex object.
 * \returns the simplicial complex pointer in the passed-in object.
 */
SimplicialComplex_t * GetSimplicialComplex (const mxArray * plexObject);

/**
 * Set the simplicial complex pointer in a plex object.
 *
 * \param plexObject the plex object.
 * \param scomplex the simplicial complex pointer to use.
 */
void SetSimplicialComplex (mxArray * plexObject, 
					  SimplicialComplex_t * scomplex);


//! The managed complex collection type
typedef std::vector<SimplicialComplex_t *> ManagedComplexCollection;

//! The iterator type for the managed complex collection
typedef ManagedComplexCollection::iterator ManagedComplexIterator;


/**
 * Get an iterator to the begin of the managed complexes collection.
 *
 * \returns begin of the managed complex collection.
 */
ManagedComplexIterator GetManagedComplexesBegin ();

/**
 * Get an iterator to the end of the managed complexes collection.
 *
 * \returns end of the managed complex collection.
 */
ManagedComplexIterator GetManagedComplexesEnd ();

/**
 * Don't manage the given simplicial complex.  If the complex isn't being
 * managed already, this does nothing.
 *
 * \param scomplex the complex to stop managing.
 */
void StopManagingComplex (SimplicialComplex_t * scomplex);

/**
 * Frees the simplicial complex objects not in the given collection.
 *
 * \param retainBegin the begin of the "retain" collection.
 * \param retainEnd the end of the "retain" collection.
 */
template<class InputIterator>
void CollectManagedComplexGarbage(InputIterator retainBegin,
						    InputIterator retainEnd)
{
  std::vector<SimplicialComplex_t *> garbage;
  std::back_insert_iterator<std::vector<SimplicialComplex_t *> > ii(garbage);  

  std::set_difference(GetManagedComplexesBegin(), GetManagedComplexesEnd(),
				  retainBegin, retainEnd, ii);

  std::vector<SimplicialComplex_t *>::iterator begin = garbage.begin();
  std::vector<SimplicialComplex_t *>::iterator end = garbage.end();

  while (begin != end) {
    SimplicialComplex_t * scomplex = *begin++;

    if (scomplex) {
	 StopManagingComplex(scomplex);
	 delete scomplex;
    }
  }
}


#endif // _PLEXOBJECT_H

