/* PlexPersistence.cc
 * ------------------
 * Implements PlexPersistence.h
 *
 * Written by Patrick Perry
 * Original Version on May 21, 2004
 *
 * $Id: PlexPersistence.cc,v 1.2 2005/03/20 01:00:12 patperry Exp $
 * $Log: PlexPersistence.cc,v $
 * Revision 1.2  2005/03/20 01:00:12  patperry
 * Added simplex pair functionality.
 *
 * Revision 1.2  2005/03/04 23:46:29  patperry
 * Added copersistence.
 *
 * Revision 1.1.1.1  2004/09/25 22:04:09  patperry
 * Imported sources
 *
 * Revision 1.1  2004/05/21 21:37:24  patperry
 * Cleaned up Filtration/Persistence interface
 *
 */

#include <algorithm>
#include <iterator>
#include "PlexPersistence.h"


Pivot_space_t * 
ComputeZ2Persistence (const Filtration_t& filtration)
{
  Pivot_space_t * result = persistence(filtration.simplicesBegin(), 
							    filtration.simplicesEnd(),
							    Z2_coefficients(), 
							    Our_storage(), 
							    Pairs_only(), 
							    BoundaryOperator_t(filtration));

  return result;
}


Pivot_space_t * 
ComputeZ2Copersistence (const Filtration_t& filtration)
{
  Pivot_space_t * result = persistence(filtration.simplicesRbegin(), 
							    filtration.simplicesRend(),
							    Z2_coefficients(), 
							    Our_storage(), 
							    Pairs_only(), 
							    CoboundaryOperator_t(filtration));

  return result;
}


mxArray * 
ExportIntervalPairs (const SimplicialComplex_t& scomplex,
				 const Filtration_t& filtration,
				 const Pivot_space_t& pairs, 
				 double unpairedValue,  int maxDimension, 
				 bool reverseOrder)
{
  // allocate space to hold the pairs cell array
  mxArray * result = mxCreateCellMatrix(1, maxDimension + 1);

  // fill in the pairs for each dimension
  for (int i = 0; i <= maxDimension; ++i) {
    // allocate space for the pairs of dimension i
    mxArray * simplexPairs 
	 = mxCreateDoubleMatrix(2, scomplex.getNumSimplices(i)
					    - scomplex.getNumDegenerateSimplices(i), mxREAL);
    double * pairsData = mxGetPr(simplexPairs);

    // extract the begin and end of each simplex
    SimplicialComplex_t::ConstSimplexIterator it = scomplex.simplicesBegin(i);
    SimplicialComplex_t::ConstSimplexIterator end = scomplex.simplicesEnd(i);

    while (it != end) {
	 SimplexHandle_t simplex = *it++;

	 if (!simplex->isDegenerate()) {
	   double first = filtration.getFiltrationValue(simplex);
	   double second = unpairedValue;
	   Pivot_space_t::Pivot_handle p 
		= const_cast<Pivot_space_t&>(pairs).pivot_handle(simplex);
	   
	   if (const_cast<Pivot_space_t&>(pairs).is_paired(p)) {
		SimplexHandle_t paired 
		  = (const_cast<Pivot_space_t&>(pairs).paired_with(p))->simplex();
		second = filtration.getFiltrationValue(paired);
	   }
	   
	   if (reverseOrder) {
		std::swap(first, second);
	   }

	   *pairsData++ = first;
	   *pairsData++ = second;
	 }
    }

    mxSetCell(result, i, simplexPairs);
  }  

  return result;
}


mxArray * 
ExportSimplexPairs (const SimplicialComplex_t& scomplex,
				const Pivot_space_t& pairs, int maxDimension)
{
  // allocate space to hold the pairs cell array
  mxArray * result = mxCreateCellMatrix(1, maxDimension + 1);

  // fill in the pairs for each dimension
  for (int i = 0; i <= maxDimension; ++i) {
    // allocate space for the pairs of dimension i
    mxArray * simplexPairs 
	 = mxCreateNumericMatrix(2, scomplex.getNumSimplices(i)
						- scomplex.getNumDegenerateSimplices(i), 
						mxINT32_CLASS, mxREAL);
    int * pairsData = static_cast<int *>(mxGetData(simplexPairs));

    // extract the begin and end of each simplex
    SimplicialComplex_t::ConstSimplexIterator it = scomplex.simplicesBegin(i);
    SimplicialComplex_t::ConstSimplexIterator end = scomplex.simplicesEnd(i);

    while (it != end) {
	 SimplexHandle_t simplex = *it++;

	 if (!simplex->isDegenerate()) {
	   int dimension = -1;
	   int index = -1;
	   
	   Pivot_space_t::Pivot_handle p 
		= const_cast<Pivot_space_t&>(pairs).pivot_handle(simplex);
	   
	   if (const_cast<Pivot_space_t&>(pairs).is_paired(p)) {
		SimplexHandle_t paired 
		  = (const_cast<Pivot_space_t&>(pairs).paired_with(p))->simplex();

		dimension = paired->getDimension();
		index = std::distance(scomplex.simplicesBegin(dimension),
						  scomplex.findSimplex(dimension, 
										   paired->getLabel())) + 1;
	   }
	   
	   *pairsData++ = dimension;
	   *pairsData++ = index;;
	 }
    }

    mxSetCell(result, i, simplexPairs);
  }  

  return result;
}
				
