/* PlexPersistence.h
 * -----------------
 * Provides Plex interface to persistence functions.
 *
 * Written by Patrick Perry
 * Original Version on May 21, 2004 
 *
 * $Id: PlexPersistence.h,v 1.2 2005/03/20 01:00:12 patperry Exp $
 * $Log: PlexPersistence.h,v $
 * Revision 1.2  2005/03/20 01:00:12  patperry
 * Added simplex pair functionality.
 *
 * Revision 1.2  2005/03/04 23:46:29  patperry
 * Added copersistence.
 *
 * Revision 1.1.1.1  2004/09/25 22:04:09  patperry
 * Imported sources
 *
 * Revision 1.1  2004/05/21 21:37:24  patperry
 * Cleaned up Filtration/Persistence interface
 *
 */

#ifndef _PLEXPERSISTENCE_H
#define _PLEXPERSISTENCE_H

#include "persistence.h"
#include "FilteredBoundaryOperator.h"
#include "FilteredCoboundaryOperator.h"
#include "PlexFiltration.h"
#include "PlexObject.h"


//! The coefficient type
typedef int Coeff_t;

//! The boundary operator type
typedef FilteredBoundaryOperator<SimplexHandle_t, FiltrationValue_t,
                                 Coeff_t> BoundaryOperator_t;

//! The coboundary operator type
typedef FilteredCoboundaryOperator<SimplexHandle_t, FiltrationValue_t,
                                 Coeff_t> CoboundaryOperator_t;

//! The persistence result type
typedef Our_storage::Pivot_space<SimplexHandle_t, Pairs_only,
						   Z2_coefficients> Pivot_space_t;

/**
 * Compute the persistence of a given filtration.
 * 
 * \param filtration the filtration object.
 * \returns the persistence result.  This must be freed by the delete
 * operator.
 */
Pivot_space_t * ComputeZ2Persistence (const Filtration_t& filtration);

/**
 * Compute the copersistence of a given filtration.
 * 
 * \param filtration the filtration object.
 * \returns the copersistence result.  This must be freed by the delete
 * operator.
 */
Pivot_space_t * ComputeZ2Copersistence (const Filtration_t& filtration);

/**
 * Export persistence interval pairs to an mxArray
 * 
 * \param scomplex the simplicial complex in the filtration.
 * \param filtration the filtration that gave the persistence result.
 * \param pairs the result of a persistence calculation.
 * \param unpairedValue the value to put in for unpaired intervals.
 * \param maxDimension the maximum dimension to include in the result.
 * \param reverseOrder indicates whether or not to swap the ordering of
 *      the appearance codes (useful for copersistence)
 * \returns an mxArray storing the interval pairs.
 */
mxArray * ExportIntervalPairs (const SimplicialComplex_t& scomplex,
						 const Filtration_t& filtration,
						 const Pivot_space_t& pairs, 
						 double unpairedValue,
						 int maxDimension, bool reverseOrder = false);

/**
 * Export persistence simplex pairs to an mxArray
 *
 * \param scomplex the simplical complex.
 * \param pairs the result of the persistence calculation.
 * \param maxDimension the maximum dimension to include in the result.
 *
 * \returns an mxArray giving the dimensions and the indices of the simplex 
 *      pairs.
 */
mxArray * ExportSimplexPairs (const SimplicialComplex_t& scomplex,
						const Pivot_space_t& pairs,
						int maxDimension);

#endif // _PLEXPERSISTENCE_H
