/* VertexSet.h
 * -----------
 * This defines some utility functions for getting the labels of a set
 * of vertices input by the user in MATLAB.
 *
 * Written by Patrick Perry
 * Original Version on March 8, 2004
 *
 * $Id: VertexSet.h,v 1.1.1.1 2005/03/18 23:36:39 silva Exp $
 * $Log: VertexSet.h,v $
 * Revision 1.1.1.1  2005/03/18 23:36:39  silva
 * Imported Sources
 *
 * Revision 1.1.1.1  2004/09/25 22:04:09  patperry
 * Imported sources
 *
 * Revision 1.2  2004/03/09 09:12:46  patperry
 * Implemented nbhood
 *
 * Revision 1.1  2004/03/09 07:58:55  patperry
 * Initial version of VertexSet code.
 *
 */

#ifndef _VERTEXSET_H
#define _VERTEXSET_H

extern "C" {
#include "mex.h"
#include "matrix.h"
}

#include "PlexObject.h"

/** 
 * Indicates whether or not the given mxArray is a valid vertex set
 * description.
 *
 * \param array the mxArray to test.
 * \returns true if the mxArray is a valid vertex set description.
 */
bool IsValidVertexSet (const mxArray * array);


/**
 * Given an mxArray with any description of the vertex set, determine the
 * type of the description and determine the result in the appropriate way
 *
 * \param array the mxArray holding the raw description.
 * \param scomplex the simplicial complex to determine the vertex set in
 *        reference to.
 * \param result the output iterator for the resulting set.  This will write
 *        integers to the passed-in iterator.
 * \returns result after the vertex set has been written to it.
 */
template <class OutputIterator>
OutputIterator
GetVertexSetFromMxArray (const mxArray * array, 
					const SimplicialComplex_t * scomplex,
					OutputIterator result);


/**
 * Given an mxArray representing an array of vertices specified explicitly
 * by their labels, this extracts the labels and writes them to an 
 * OutputIterator.
 *
 * \param array the mxArray holding the array of vertex labels.
 * \param result the output iterator for the resulting set.  This will write
 *        integers to the passed-in iterator.
 * \returns result after the vertex set has been written to it.
 */
template <class OutputIterator>
OutputIterator
GetVertexSetFromList (const mxArray * array, OutputIterator result);
				  


/**
 * Given a logical mxArray of length the number of vertices a simplicial
 * complex, extracts the appropriate list of vertices from the logical 
 * array and writes them to an OutputIterator.
 *
 * \param scomplex the simplicial complex to determine the vertex set in
 *        reference to.
 * \param result the output iterator for the resulting set.  This will write
 *        integers to the passed-in iterator.
 * \returns result after the vertex set has been written to it.
 */
template <class OutputIterator>
OutputIterator
GetVertexSetFromLogicalLabelArray (const mxArray * array, 
							OutputIterator result);


/**
 * Given a logical mxArray of length (v_max + 1), referencing the vertex 
 * labels [0, 1, ..., v_max], extracts the appropriate list of vertices
 * and writes them to an OutputIterator.
 *
 * \param result the output iterator for the resulting set.  This will write
 *        integers to the passed-in iterator.
 * \returns result after the vertex set has been written to it.
 */
template <class OutputIterator>
OutputIterator
GetVertexSetFromLogicalLabelArray (const mxArray * array, 
							OutputIterator result);
						    

// get compiler to see function definitions
#include "VertexSet.cc"


#endif // _VERTEXSET_H
