// boundary.h
// October 28, 2003
// Lutz Kettner and Afra Zomorodian

#ifndef _BOUNDARY_H_
#define _BOUNDARY_H_

#include <iterator>

// Class:  Boundary
// ****************
// Contains a boundary and allows for pivot elimination
// using second boundary.  The latter must have the same
// pivot and have pivot coefficient equal to 1.

template <class Chain>
class Boundary {
protected:
  Chain boundary_;
public:
  typedef Boundary< Chain>                         Self;
  typedef typename Chain::Field                    Field;
  typedef typename Chain::Pivot_handle             Pivot_handle;
  typedef typename Chain::const_iterator           const_iterator;
  typedef typename Chain::iterator                 iterator;
  typedef typename Chain::size_type                size_type;
  typedef typename Chain::Compare_wrto_filtration  Compare_wrto_filtration;
  typedef typename Chain::reference                reference;
  typedef typename Chain::const_reference          const_reference;

  // Constructors
  Boundary() {}
  Boundary(size_type n) : boundary_(n) {}
  Boundary(Chain& c)    : boundary_(c) {}

  // operator=
  template< class Chain2>
  Self& operator=(const Boundary< Chain2> &b)
  {
    boundary_ = b.boundary();
    return *this;
  }

  // extend some chain operations out
  const Chain&    boundary()  const { return boundary_;           }
  void            clear()           { boundary_.clear();          }
  bool            empty()     const { return boundary_.empty();   }
  size_type       size()      const { return boundary_.size();    }
  iterator        pivot()           { return boundary_.end()-1;   }
  const_iterator  pivot()     const { return boundary_.end()-1;   }
  void            print()     const { boundary_.print();          }
  void            push_back(const std::pair< Pivot_handle, Field> &p)
  { 
      boundary_.push_back( p); 
  }
  void            pop_pivot()       { boundary_.pop_back();       }

  template< class Boundary2>
  void set_boundary( const Boundary2& boundary) 
  {
    boundary_ = boundary.boundary();
  }
  
  // eliminate_pivot:  
  // boundary2 must have same pivot and coefficient equal to 1.
  // we assume the pivot is not stored in boundary2, but is
  // rather implied.  therefore, we eliminate the pivot in 
  // boundary at the end to maintain this invariant.
  template< class Boundary2>
  void eliminate_pivot( const Boundary2& boundary2,
			Compare_wrto_filtration handle_compare = 
			Compare_wrto_filtration())
  {
    // me = me - mypivotcoeff * boundary2 (which has pivotcoeff = 1)
    const typename Chain::Field& pivot_coeff = boundary_.coeff(pivot());
    boundary_.multiply_and_add( boundary2.boundary(), -pivot_coeff, 
				handle_compare);
    // delete pivot
    // note:  this would've happened in multiply_and_add if we had
    // explicit pivots
    pop_pivot();
  }
  
  void normalize() 
  {
    if( empty()) return;
    const Field& pivot_coeff = boundary_.coeff(pivot());
    if( pivot_coeff != Chain::one) {
      Field inverse = Chain::one / pivot_coeff;
      boundary_.multiply( inverse);
    }
  }
};

// Class:  Manifold
// ****************
// Also contains the immersed manifold whose boundary
// it stores.

template <class Chain>
class Manifold : public Boundary< Chain> {
 protected:
  using Boundary< Chain>::boundary_;
  Chain manifold_;
public:
  typedef Manifold< Chain>           Self;
  typedef typename Chain::Field                    Field;
  typedef typename Chain::Pivot_handle             Pivot_handle;
  typedef typename Chain::const_iterator           const_iterator;
  typedef typename Chain::iterator                 iterator;
  typedef typename Chain::size_type                size_type;
  typedef typename Chain::Compare_wrto_filtration  Compare_wrto_filtration;
  typedef typename Chain::reference                reference;
  typedef typename Chain::const_reference          const_reference;

  // Constructor from chains
  Manifold( ) {}
  Manifold( size_type n) : 
    Boundary< Chain>( n), manifold_( n) {}
  Manifold( Chain& b, Chain &m) : 
    Boundary< Chain>( b), manifold_( m) {}
  
  // operator=
  template< class Chain2>
  Self& operator=(const Manifold< Chain2> &b)
  {
    boundary_ = b.boundary();
    manifold_ = b.manifold();
    return *this;
  }

  template< class Boundary2>
  void set_boundary( const Boundary2& boundary) 
  {
    boundary_ = boundary.boundary();
    manifold_ = boundary.manifold();
  }

  // Access manifold
  const Chain& manifold() const     { return manifold_; }

  void clear()
  { 
    boundary_.clear();  
    manifold_.clear();
  }
  
  void print() const 
  { 
    boundary_.print(); 
    std::cout << "| ";
    manifold_.print();
  }

  void push_back_manifold( const std::pair< Pivot_handle, Field> &p)
  { 
    manifold_.push_back( p); 
  }

  template< class Boundary2>
  void set_manifold( const Boundary2& boundary) 
  {
    manifold_ = boundary.manifold();
  }

  // eliminate_pivot:  
  // boundary2 must have same pivot and coefficient equal to 1.
  // also does same operations to manifold chain
  template< class BoundaryWithManifold>
  void eliminate_pivot( const BoundaryWithManifold &boundary2,
			Compare_wrto_filtration handle_compare =
			Compare_wrto_filtration())
  {
    const typename Chain::Field& pivot_coeff = boundary_.coeff(this->pivot());
    boundary_.multiply_and_add( boundary2.boundary(), -pivot_coeff, 
				handle_compare);
    manifold_.multiply_and_add( boundary2.manifold(), -pivot_coeff,
				handle_compare);
    // delete pivot 
    // note:  this would've happened in multiply_and_add if we had
    // explicit pivots
    this->pop_pivot();
  }

  void normalize() 
  {
    if( this->empty()) return;
    const Field& pivot_coeff = boundary_.coeff(this->pivot());
    if( pivot_coeff != Chain::one) {
      Field inverse = Chain::one / pivot_coeff;
      boundary_.multiply( inverse);
      manifold_.multiply( inverse);
    }
  }
};

// Class:  Boundary_with_cycle
// ***************************
// Also contains the basis cycle chosen by the 
// algorithm

template <class Chain>
class Boundary_with_cycle : public Boundary< Chain> {
 protected:
  Chain cycle_;
  using Boundary< Chain>::boundary_;
public:
  typedef Boundary_with_cycle< Chain>              Self;
  typedef typename Chain::Field                    Field;
  typedef typename Chain::Pivot_handle             Pivot_handle;
  typedef typename Chain::const_iterator           const_iterator;
  typedef typename Chain::iterator                 iterator;
  typedef typename Chain::size_type                size_type;
  typedef typename Chain::Compare_wrto_filtration  Compare_wrto_filtration;
  typedef typename Chain::reference                reference;
  typedef typename Chain::const_reference          const_reference;

  // Constructor from chains
  Boundary_with_cycle( ) {}
  Boundary_with_cycle( size_type n) : 
    Boundary< Chain>( n), cycle_( n) {}
  Boundary_with_cycle( Chain& b, Chain &c) : 
    Boundary< Chain>( b), cycle_( c) {}
  
  // operator=
  template< class Chain2>
  Self& operator=(const Boundary_with_cycle< Chain2> &b)
  {
    boundary_ = b.boundary();
    cycle_    = b.cycle();
    return *this;
  }

  void push_back_cycle( const std::pair< Pivot_handle, Field> &p)
  { 
    cycle_.push_back( p); 
  }

  // Access cycle
  const Chain& cycle() const     { return cycle_; }

  void clear()
  { 
    boundary_.clear();  
    cycle_.clear();
  }
  
  void print() const 
  { 
    boundary_.print(); 
    std::cout << "[";
    cycle_.print();
    std::cout << "]";
  }

  template< class Boundary2>
  void set_cycle( const Boundary2& boundary) 
  {    
    cycle_ = boundary.cycle();
  }

  // eliminate_pivot:  
  // boundary2 must have same pivot and coefficient equal to 1.
  // note:  no need to pass in pivot as it is unique and shared
  // so we ask handle for paired.
  template< class BoundaryWithCycle>
  void eliminate_pivot( const BoundaryWithCycle &boundary2,
			Compare_wrto_filtration handle_compare =
			Compare_wrto_filtration())
  {
    const typename Chain::Field& pivot_coeff = boundary_.coeff( this->pivot());
    boundary_.multiply_and_add( boundary2.boundary(), -pivot_coeff, 
				handle_compare);
    // we need the pivot's paired with here.  how do we get it?
    push_back_cycle( std::make_pair( boundary_.handle( this->pivot())
				     ->paired_with(), 		
				     -pivot_coeff));
    // delete pivot 
    // note:  this would've happened in multiply_and_add if we had
    // explicit pivots
    this->pop_pivot();
  }
  
  void normalize() 
  {
    if( this->empty()) return;
    const Field& pivot_coeff = boundary_.coeff(this->pivot());
    if( pivot_coeff != Chain::one) {
      Field inverse = Chain::one / pivot_coeff;
      boundary_.multiply( inverse);
      cycle_.multiply( inverse);
    }
  }
};

// Class:  Manifold_with_cycle
// ***************************
// Contains boundary, manifold, and cycle.  in an ideal language
// this would derive naturally from both manifold and boundary_with_cycle

template <class Chain>
class Manifold_with_cycle : public Manifold< Chain> {
 protected:
  Chain cycle_;
  using Manifold< Chain>::boundary_;
  using Manifold< Chain>::manifold_;
public:
  typedef Manifold_with_cycle< Chain>              Self;
  typedef typename Chain::Field                    Field;
  typedef typename Chain::Pivot_handle             Pivot_handle;
  typedef typename Chain::const_iterator           const_iterator;
  typedef typename Chain::iterator                 iterator;
  typedef typename Chain::size_type                size_type;
  typedef typename Chain::Compare_wrto_filtration  Compare_wrto_filtration;
  typedef typename Chain::reference                reference;
  typedef typename Chain::const_reference          const_reference;

  // Constructor from chains
  Manifold_with_cycle( ) {}
  Manifold_with_cycle( size_type n) : 
    Manifold< Chain>( n), cycle_( n) {}
  Manifold_with_cycle( Chain& b, Chain &c) : 
    Manifold< Chain>( b), cycle_( c) {}
  
  // operator=
  // could also call manifold's and then add cycle
  // maybe fix this later to be more "elegant"
  template< class Chain2>
  Self& operator=(const Manifold_with_cycle< Chain2> &b)
  {
    boundary_ = b.boundary();
    manifold_ = b.manifold();
    cycle_    = b.cycle();
    return *this;
  }

  void push_back_cycle( const std::pair< Pivot_handle, Field> &p)
  { 
    cycle_.push_back( p); 
  }

  // Access cycle
  const Chain& cycle() const     { return cycle_; }

  void clear()
  { 
    Manifold< Chain>::clear();
    cycle_.clear();
  }
  
  void print() const 
  { 
    Manifold< Chain>::print();
    std::cout << "[";
    cycle_.print();
    std::cout << "]";
  }

  template< class Boundary2>
  void set_cycle( const Boundary2& boundary) 
  {    
    cycle_ = boundary.cycle();
  }

  // eliminate_pivot:  
  // boundary2 must have same pivot and coefficient equal to 1.
  // note:  no need to pass in pivot as it is unique and shared
  // so we ask handle for paired.
  template< class BoundaryWithCycle>
  void eliminate_pivot( const BoundaryWithCycle &boundary2,
			Compare_wrto_filtration handle_compare =
			Compare_wrto_filtration())
  {
    const typename Chain::Field& pivot_coeff = boundary_.coeff( this->pivot());
    boundary_.multiply_and_add( boundary2.boundary(), -pivot_coeff, 
				handle_compare);
    manifold_.multiply_and_add( boundary2.manifold(), -pivot_coeff, 
				handle_compare);
    // we need the pivot's paired with here.  how do we get it?
    push_back_cycle( std::make_pair( boundary_.handle( this->pivot())
				     ->paired_with(), 		
				     -pivot_coeff));
    // delete pivot 
    // note:  this would've happened in multiply_and_add if we had
    // explicit pivots
    this->pop_pivot();
  }
  
  void normalize() 
  {
    if( this->empty()) return;
    const Field& pivot_coeff = boundary_.coeff(this->pivot());
    if( pivot_coeff != Chain::one) {
      Field inverse = Chain::one / pivot_coeff;
      boundary_.multiply( inverse);
      manifold_.multiply( inverse);
      cycle_.multiply( inverse);
    }
  }
};

#endif // _BOUNDARY_H_
