// persistence.h
// October 31, 2003
// Lutz Kettner and Afra Zomorodian

#ifndef _PERSISTENCE_H_
#define _PERSISTENCE_H_

#include <vector>
#include <iterator>
#include <iostream>

// #include <CGAL/Unique_hash_map.h>

// Lots of trouble just to get a hashtable.  Code taken from
// http://gcc.gnu.org/onlinedocs/libstdc++/faq/index.html#5_4
#ifdef __GNUC__
#if __GNUC__ < 3
  #include <hash_map.h>
  namespace Sgi { using ::hash_map }; 
#else
  #include <ext/hash_map>
  #if ((__GNUC__ == 3) && (__GNUC_MINOR__ == 0))
    namespace Sgi = std;
  #else 
    namespace Sgi = ::__gnu_cxx;
  #endif
#endif
#else
  #include <hash_map>
  namespace Sgi = std;
#endif 

template <class Handle>
class HandleHash
{
 public:
  
  std::size_t operator() (Handle key) const
  {
    return reinterpret_cast<std::size_t>(&(*key));
  }
};

#include "chain.h"
#include "ext_chain.h"
#include "boundary.h"
#include "pivot.h"

// **********************************************************************
// Coefficient Policy                                                   *
// **********************************************************************
// Note:  policy allows chains to be defined

class Z2_coefficients {
public:
  typedef int Field;
  template <class Pivot_handle, class CompareWrtoFiltration>
    struct Bind {
      typedef ::Chain_Z2< Pivot_handle, CompareWrtoFiltration>     Chain;
      typedef ::Ext_Chain_Z2< Pivot_handle, CompareWrtoFiltration> Ext_Chain;
    };
};

template < class Field_>
class Field_coefficients {
public:
  typedef Field_ Field;
  template <class Pivot_handle, class CompareWrtoFiltration>
  struct Bind {
    typedef ::Chain_field< Pivot_handle, Field, 
			       CompareWrtoFiltration> Chain;
    typedef ::Ext_Chain_field< Pivot_handle, Field, 
			       CompareWrtoFiltration> Ext_Chain;
  };
};

// **********************************************************************
// Storage Policies                                                     *
// **********************************************************************

// Our storage, provided by us
class Our_storage {
  public:
  template < class SimplexHandle, class OutputPolicy, 
	     class CoefficientPolicy>
  class Pivot_space {
  public:
    typedef SimplexHandle                                Simplex_handle;
    typedef OutputPolicy                                 Output_policy;
    typedef ::Pivot < SimplexHandle, OutputPolicy,
		      CoefficientPolicy>                 Pivot;
    typedef typename Pivot::Pivot_handle                 Pivot_handle;
    typedef typename Pivot::CompareWrtoFiltration        CompareWrtoFiltration;
    typedef std::size_t                                  size_type;
    typedef typename Pivot::Boundary                     Boundary;
    typedef typename Pivot::Ext_Boundary                 Ext_Boundary;
    // defined positive and negative infinities
    Pivot_handle                                         pos_inf;
    Pivot_handle                                         neg_inf;
  private:
    // private members
    std::vector< Pivot> pivots;
    //CGAL::Unique_hash_map< Simplex_handle, Pivot_handle> hash;
    Sgi::hash_map< Simplex_handle, Pivot_handle, 
			    HandleHash<const Simplex_handle> > hash;
    typename std::vector< Pivot>::iterator               next;
  public:    
    Pivot_space( size_type n) {
      pivots.reserve( n+1);
      pivots.insert( pivots.begin(), n+1, Pivot());
      pos_inf = pivots.end();
      neg_inf = pivots.begin();
      next = neg_inf + 1;
      //hash = CGAL::Unique_hash_map< Simplex_handle, Pivot_handle>(neg_inf);
      hash = Sgi::hash_map< Simplex_handle, Pivot_handle, 
	HandleHash<const Simplex_handle> >();
      pivots[0].set_base(next);  // for printing capability
    }

    Pivot_handle pivot_handle( Simplex_handle s) 
    { 
      //      Pivot_handle& pivot = hash[s];
      Pivot_handle& pivot 
	= (*((hash.insert(std::pair<Simplex_handle, 
			  Pivot_handle>(s, neg_inf)))
	     .first)).second;      
      if ( pivot == neg_inf) {
 	pivot = next++;
 	pivot->simplex() = s;
      }
      return pivot;
    }

    Pivot_handle pivot_handle_from_index( size_type i)
    {
      return neg_inf + i + 1;
    }

    size_type pivot_index_from_handle( Pivot_handle p)
    {
      return p - neg_inf - 1;
    }

    Boundary& boundary( Pivot_handle p)        { return p->boundary();       }
    Pivot_handle& paired_with( Pivot_handle p) { return p->paired_with();    }
    bool is_paired( Pivot_handle p) { 
      return 
	p->paired_with() != pos_inf &&
	p->paired_with() != neg_inf;
    }
    bool is_positive( Pivot_handle p)          { return p->is_positive();    }
    void set_positive( Pivot_handle p)         { p->paired_with() = pos_inf; }
    void set_negative( Pivot_handle p)         { p->paired_with() = neg_inf; }
  };
};

// Example user storage, with mostly empty functions
class User_storage {
  template < class SimplexHandle, class Boundary_>
  class Pivot_space {
  public:
    typedef SimplexHandle Pivot_handle;
    typedef SimplexHandle Simplex_handle;
    typedef Boundary_     Boundary;
    typedef typename std::iterator_traits< Simplex_handle>
      ::size_type size_type;

    Pivot_space( size_type n)                           {}
    Pivot_handle pivot_handle( Simplex_handle s)        { return s; }
    Pivot_handle pivot_handle_from_index( size_type i)  {}
    Boundary& boundary( Pivot_handle p)                 {}
    Pivot_handle& paired_with( Pivot_handle p)          {}
    bool is_positive ( Pivot_handle p)                  {}
    typedef std::less< Pivot_handle>                    CompareWrtoFiltration;
  };
};

// **********************************************************************
// Output Policies                                                      *
// **********************************************************************

// 1. Pairs_only:  get only the pairs
class Pairs_only {
public:
  template < class Chain>
  struct Bind {
    typedef ::Boundary< Chain> Boundary;
  };
  template< class PivotHandle, class Boundary, class Field>
  void Initialize_descriptions( PivotHandle p, Boundary& b) {}

  template< class Boundary, class Boundary2>
  void Set_boundary( Boundary& b1, Boundary2& b2)
  {
    b1 = b2;
  }

  template< class Boundary, class Boundary2>
  void Set_cycle( Boundary& b1, Boundary2& b2) {}
};

// 2.  Pairs_and_boundary:  get pairs and the basis element
// computed from the boundary
class Pairs_and_boundary {
public:
  template < class Chain>
  struct Bind {
    typedef ::Boundary< Chain> Boundary;
  };
  template< class PivotHandle, class Boundary, class Field>
  void Initialize_descriptions( PivotHandle p, Boundary& b) {}

  template< class Boundary, class Boundary2>
  void Set_boundary( Boundary& b1, Boundary2& b2)
  {    
    b1 = b2;
  }

  template< class Boundary, class Boundary2>
  void Set_cycle( Boundary& b1, Boundary2& b2) {}
};

// 3.  Pairs_and_manifold:  get pairs and the basis element
// computed from the boundary, along with the manifold
// that turned it into a boundary.
class Pairs_and_manifold {
public:
  template <class Chain>
  struct Bind {
    typedef ::Manifold< Chain> Boundary;
  };
  template< class PivotHandle, class Boundary, class Field>
  void Initialize_descriptions( PivotHandle p, Boundary& b)
  {
    typedef PivotHandle Pivot_handle;
    b.push_back_manifold( std::pair< Pivot_handle, Field>
			  (p, b.boundary().one));
  }

  template< class Boundary, class Boundary2>
  void Set_boundary( Boundary& b1, Boundary2& b2)
  {    
    b1 = b2;
  }

  template< class Boundary, class Boundary2>
  void Set_cycle( Boundary& b1, Boundary2& b2) {}
};

// 4.  Pairs_and_cycle:  get pairs and the basis element
// chosen at the time cycle was created.
class Pairs_and_cycle {
public:
  template < class Chain>
  struct Bind {
    typedef ::Boundary_with_cycle< Chain> Boundary;
  };
  template< class PivotHandle, class Boundary, class Field>
  void Initialize_descriptions( PivotHandle p, Boundary& b) {}

  template< class Boundary, class Boundary2>
  void Set_boundary( Boundary& b1, Boundary2& b2)
  {
    b1.set_boundary(b2);
  }

  template< class Boundary, class Boundary2>
  void Set_cycle( Boundary& b1, Boundary2& b2) 
  {
    b1.set_cycle(b2);
  }
};

// 5.  Pairs_and_boundary_and_cycle:  pairs and both descriptions for
// the basis element.
class Pairs_and_boundary_and_cycle {
public:
  template < class Chain>
  struct Bind {
    typedef ::Boundary_with_cycle< Chain> Boundary;
  };
  template< class PivotHandle, class Boundary, class Field>
  void Initialize_descriptions( PivotHandle p, Boundary& b) {}

  template< class Boundary, class Boundary2>
  void Set_boundary( Boundary& b1, Boundary2& b2)
  {
    b1.set_boundary(b2);
  }

  template< class Boundary, class Boundary2>
  void Set_cycle( Boundary& b1, Boundary2& b2) 
  {
    b1.set_cycle(b2);
  }
};

// 6.  Pairs_and_manifold_and_cycle:  pairs and both descriptions for
// the basis element, and the manifold that turned the boundary into one.
class Pairs_and_manifold_and_cycle {
public:
  template < class Chain>
  struct Bind {
    typedef ::Manifold_with_cycle< Chain> Boundary;
  };
  template< class PivotHandle, class Boundary, class Field>
  void Initialize_descriptions( PivotHandle p, Boundary& b)
  {
    typedef PivotHandle Pivot_handle;
    b.push_back_manifold( std::pair< Pivot_handle, Field>
			  (p, b.boundary().one));
  }

  template< class Boundary, class Boundary2>
  void Set_boundary( Boundary& b1, Boundary2& b2)
  {
    b1.set_boundary(b2);
  }

  template< class Boundary, class Boundary2>
  void Set_cycle( Boundary& b1, Boundary2& b2) 
  {
    b1.set_cycle(b2);
  }
};

// **********************************************************************
// Boundary Operator                                                    *
// **********************************************************************

class Boundary_handle_tag {};
class Boundary_index_tag  {};

// Example Boundary Operator
// MUST return a pair type, the second type being the coefficient type.
// class BoundaryOperator {
// public:
//   typedef Boundary_handle_tag rep_tag;
//   typedef int (*Simplex_handle)[4];
//   typedef std::pair< int, int> value_type;
//   template <class OutputIterator>
//   OutputIterator operator()(SimplexHandle, OutputIterator out) {}
//};

// Boundary_output_transformer
template <class BoundaryOperator, class PivotSpace, class OutputPolicy>
class Boundary_output_transformer {
public:
  // types
  typedef Boundary_output_transformer<BoundaryOperator, 
    PivotSpace, OutputPolicy>                   Self;
  typedef std::output_iterator_tag              iterator_category;
  typedef void                                  difference_type;
  typedef void                                  pointer;
  typedef void                                  reference;
  typedef typename PivotSpace::Pivot_handle     Pivot_handle;
  typedef typename PivotSpace::Ext_Boundary     Ext_Boundary;
  typedef typename BoundaryOperator::rep_tag    rep_type;
  typedef typename BoundaryOperator::value_type value_type;
protected:
  Ext_Boundary*                    boundary;
  PivotSpace*                      pivot_space;
public:
  void action( value_type v, Boundary_handle_tag, Pairs_only)
  {
    Pivot_handle handle = pivot_space->pivot_handle( v.first);
    if(pivot_space->is_positive( handle)) {
      boundary->push_back( std::make_pair( handle, v.second));
    }
  }

  void action( value_type v, Boundary_index_tag, Pairs_only)
  {
    Pivot_handle handle = pivot_space->pivot_handle_from_index( v.first);
    if(pivot_space->is_positive( handle)) {
      boundary->push_back( std::make_pair( handle, v.second));
    }
  }

  void action( value_type v, Boundary_handle_tag, Pairs_and_cycle)
  {
    Pivot_handle handle = pivot_space->pivot_handle( v.first);
    if(pivot_space->is_positive( handle)) {
      boundary->push_back( std::make_pair( handle, v.second));
    }
  }

  void action( value_type v, Boundary_index_tag, Pairs_and_cycle)
  {
    Pivot_handle handle = pivot_space->pivot_handle_from_index( v.first);
    if(pivot_space->is_positive( handle)) {
      boundary->push_back( std::make_pair( handle, v.second));
    }
  }

  template< class OutputPolicy_>
  void action( value_type v, Boundary_handle_tag, OutputPolicy_)
  {
    Pivot_handle handle = pivot_space->pivot_handle( v.first);
    boundary->push_back( std::make_pair( handle, v.second));
  }

  template< class OutputPolicy_>
  void action( value_type v, Boundary_index_tag, OutputPolicy_)
  {
    Pivot_handle handle = pivot_space->pivot_handle_from_index( v.first);
    boundary->push_back( std::make_pair( handle, v.second));
  }
  
  explicit Boundary_output_transformer(Ext_Boundary *b, PivotSpace *p)
    : boundary( b), pivot_space( p) {}
  Self& operator=( value_type p) { 
    action(p, rep_type(), OutputPolicy());
    return *this;
  }
  Self& operator*()     { return *this; }
  Self& operator++()    { return *this; }
  Self& operator++(int) { return *this; }
};


// **********************************************************************
// Persistence Function                                                 *
// **********************************************************************

template <class ForwardIterator,
          class BoundaryOperator,
	  class CoefficientPolicy, 
          class StoragePolicy,
	  class OutputPolicy>
typename StoragePolicy::template 
Pivot_space< //ForwardIterator,
typename std::iterator_traits<ForwardIterator>::value_type, 
		   OutputPolicy, CoefficientPolicy>*
persistence( ForwardIterator first, 
		  ForwardIterator last,
		  CoefficientPolicy coefficient_policy,
		  StoragePolicy storage_policy,
		  OutputPolicy  output_policy,
                  BoundaryOperator boundary_operator)
{
  // types
  typedef typename std::iterator_traits<ForwardIterator>::value_type Simplex_handle;
  //  typedef ForwardIterator                      Simplex_handle;
  typedef std::size_t                          size_type;
  typedef typename StoragePolicy::template
    Pivot_space<Simplex_handle, OutputPolicy, CoefficientPolicy>
                                               Pivot_space;
  typedef typename Pivot_space::Pivot_handle   Pivot_handle;
  typedef typename Pivot_space::Boundary       Boundary;
  typedef typename Pivot_space::Ext_Boundary   Ext_Boundary;
  typedef typename Boundary::iterator          Output_iterator;
  typedef Boundary_output_transformer 
    < BoundaryOperator, Pivot_space, OutputPolicy> Transformer;
  typedef typename CoefficientPolicy::Field    Field;
 
  // allocate space
  size_type n = std::distance( first, last); 
  Pivot_space *pivots = new Pivot_space( n);  
  Ext_Boundary b( n);
  Transformer transformer( &b, pivots);

  // process each boundary in turn 
  //Pivot_handle p = pivots->pivot_handle_from_index( 0);
  for( ForwardIterator it = first; it != last; ++it) {
    Simplex_handle simplex = *it;
    // get simplex into our world:
    // 1.  hash table way:
    const Pivot_handle p = pivots->pivot_handle(simplex);

    // 2.  non-hash table way.  we need to set simplex
    // we already have the right pivot_handle from for loop
    //p->simplex() = simplex;
    
    // get boundary
    boundary_operator( simplex, transformer);
    
    // set up descriptions, if needed.
    output_policy.template 
      Initialize_descriptions< Pivot_handle, Ext_Boundary, Field>( p, b);
    
    // process boundary
    process_boundary( pivots, p, b, output_policy);
    // get rid of it
    b.clear();    
  }
  
  // output
  return pivots;
}

// process_boundary
// note on testing positive's boundary:
// we test if positive is already paired with somebody
// while in C, the test is something like 
// (positive's boundary == NULL), running empty() on it does not work
// as the it may still be empty for Pairs_only even though
// positive _is_ paired.  hence the current test
template < class PivotSpace, class OutputPolicy>
void process_boundary( PivotSpace* pivots, 
		       typename PivotSpace::Pivot_handle simplex,
		       typename PivotSpace::Ext_Boundary &boundary,
		       OutputPolicy output_policy)
{
  typedef PivotSpace                         Pivot_space;
  typedef typename Pivot_space::Boundary     Boundary;
  typedef typename Pivot_space::Ext_Boundary Ext_Boundary;
  typedef typename Pivot_space::Pivot_handle Pivot_handle;

  while( !boundary.empty()) {
    //boundary.boundary().print(); std::cout << std::endl;
    const Pivot_handle positive = 
      boundary.boundary().handle( boundary.pivot());
    if( !pivots->is_paired( positive)) {
      // normalize _first_ and remove pivot (coeff 1 is implied)  
      boundary.normalize();
      boundary.pop_pivot();
      //std::cout << "storing ";
      //boundary.boundary().print(); std::cout << std::endl;
      // pivots->boundary( positive) = boundary;
      output_policy.template 
	Set_boundary< Boundary, Ext_Boundary>( pivots->boundary( positive),
					       boundary);
      pivots->paired_with( positive) = simplex;
      pivots->paired_with( simplex)  = positive;
      return;
    }
    // Collision -- eliminate pivot!
    const Boundary& stored_boundary = pivots->boundary( positive);
    //std::cout << "stored is ";
    //stored_boundary.boundary().print(); std::cout << std::endl;
    boundary.eliminate_pivot( stored_boundary);
  }
  // empty boundary means positive simplex
  pivots->set_positive( simplex);
  // no need to normalize, but must pop "pivot"
  output_policy.template
    Set_cycle< Boundary, Ext_Boundary>( pivots->boundary( simplex),
					boundary);
}

#endif // _PERSISTENCE_H_
