/* Chain.cc
 * --------
 * Implements interface defined in Chain.h
 *
 * Written by Patrick Perry
 * Original Version on October 15, 2003
 *
 * $Id: Chain.cc,v 1.1.1.1 2005/03/18 23:37:11 silva Exp $
 * $Log: Chain.cc,v $
 * Revision 1.1.1.1  2005/03/18 23:37:11  silva
 * Imported Sources
 *
 * Revision 1.1.1.1  2004/09/25 22:04:08  patperry
 * Imported sources
 *
 * Revision 1.2  2004/01/15 07:10:33  patperry
 * Changed file extensions.
 *
 * Revision 1.1  2003/10/16 08:42:15  patperry
 * Added a Chain data structure.
 *
 *
 */

#include <algorithm>


template <class CoefficientType, class Alloc>
Chain<CoefficientType, Alloc>::
Chain (unsigned short dimension)
  : dimension(dimension)
{
}


template <class CoefficientType, class Alloc>
template <class SimplexInputIterator, class CoefficientInputIterator>
Chain<CoefficientType, Alloc>::
Chain (SimplexInputIterator simplicesBegin, 
	  SimplexInputIterator simplicesEnd,
	  CoefficientInputIterator coefficientsBegin)
  : dimension((*simplicesBegin)->getDimension())
{
  CoefficientInputIterator coefficient = coefficientsBegin;

  for (SimplexInputIterator simplex = simplicesBegin;
	  simplex != simplicesEnd; ++simplex, ++coefficient) {
    if (*coefficient != 0) {
	 SimplexCoefficientPair simplexCoefficientPair 
	   = { *simplex, *coefficient };
	 typename Chain::ChainIterator position 
	   = std::lower_bound(simplices.begin(), simplices.end(), 
					  simplexCoefficientPair);
	 
	 if ((position != simplices.end()) 
		&& (position->simplex == simplexCoefficientPair.simplex)) {
	   position->coefficient += simplexCoefficientPair.coefficient;
	 } else {
	   simplices.insert(position, simplexCoefficientPair);
	 }
    }
  }
}


template <class CoefficientType, class Alloc>
Chain<CoefficientType, Alloc>::
~Chain ()
{
}


template <class CoefficientType, class Alloc>
unsigned short 
Chain<CoefficientType, Alloc>::
getDimension () const
{
  return dimension;
}


template <class CoefficientType, class Alloc>
unsigned int 
Chain<CoefficientType, Alloc>::
getLength () const
{
  return simplices.size();
}


template <class CoefficientType, class Alloc> 
typename Chain<CoefficientType, Alloc>::Coefficient 
Chain<CoefficientType, Alloc>::
getSimplexCoefficient (const Chain::Simplex * simplex) const
{
  Chain::Coefficient result = 0;
  SimplexCoefficientPair simplexCoefficientPair = { simplex, 0 };
  typename Chain::ChainIterator position 
    = std::lower_bound(simplices.begin(), simplices.end(), position);

  if ((position != simplices.end()) && (position->simplex == simplex)) {
    result = position->coefficient;
  } 

  return result;
}


template <class CoefficientType, class Alloc> 
typename Chain<CoefficientType, Alloc>::ChainIterator
Chain<CoefficientType, Alloc>::
simplicesBegin ()
{
  return simplices.begin();
}


template <class CoefficientType, class Alloc> 
typename Chain<CoefficientType, Alloc>::ChainIterator
Chain<CoefficientType, Alloc>::
simplicesEnd ()
{
  return simplices.end();
}


template <class CoefficientType, class Alloc> 
typename Chain<CoefficientType, Alloc>::ConstChainIterator
Chain<CoefficientType, Alloc>::
simplicesBegin () const
{
  return simplices.begin();
}


template <class CoefficientType, class Alloc> 
typename Chain<CoefficientType, Alloc>::ConstChainIterator
Chain<CoefficientType, Alloc>::
simplicesEnd () const
{
  return simplices.end();
}


template <class CoefficientType, class Alloc> 
Chain<CoefficientType, Alloc>&
Chain<CoefficientType, Alloc>::
operator+= (const Chain& c)
{
  ChainIterator position = simplices.begin();
  ConstChainIterator otherPosition = c.simplices.begin();

  while (otherPosition != c.simplices.end()) {
    position = std::lower_bound(position, simplices.end(), *otherPosition);
    
    if ((position != simplices.end())
	   && (position->simplex == otherPosition->simplex)) {
	 position->coefficient += otherPosition->coefficient;

	 if (position->coefficient == 0) {
	   ChainIterator nextPosition = position;

	   ++nextPosition;
	   simplices.erase(position);
	   position = nextPosition;
	 }
    } else {
	 simplices.insert(position, *otherPosition);
	 ++position;
    }

    ++otherPosition;

    if (position == simplices.end()) {
	 break;
    }
  }
  
  if (otherPosition != c.simplices.end()) {
    simplices.insert(position, otherPosition, c.simplices.end());
  }

  return *this;
}


template <class CoefficientType, class Alloc> 
Chain<CoefficientType, Alloc>&
Chain<CoefficientType, Alloc>::
operator-= (const Chain& c)
{
  ChainIterator position = simplices.begin();
  ConstChainIterator otherPosition = c.simplices.begin();

  while (otherPosition != c.simplices.end()) {
    position = std::lower_bound(position, simplices.end(), *otherPosition);
    
    if (position->simplex == otherPosition->simplex) {
	 position->coefficient -= otherPosition->coefficient;

	 if (position->coefficient == 0) {
	   ChainIterator nextPosition = position;

	   ++nextPosition;
	   simplices.erase(position);
	   position = nextPosition;
	 }
    } else {
	 simplices.insert(position, *otherPosition);
	 std::advance(position, 2);
    }

    ++otherPosition;

    if (position == simplices.end()) {
	 break;
    }
  }
  
  if (otherPosition != c.simplices.end()) {
    simplices.insert(position, otherPosition, c.simplices.end());
  }

  return *this;
}


template <class CoefficientType, class Alloc> 
Chain<CoefficientType, Alloc>&
Chain<CoefficientType, Alloc>::
operator*= (const CoefficientType k)
{
  if (k == 0) {
    simplices.clear();
  } else {
    for (ChainIterator it = simplices.begin(); it != simplices.end(); ++it) {
	 it->coefficient *= k;
    }
  }

  return *this;
}


template <class CoefficientType, class Alloc> 
std::ostream& 
Chain<CoefficientType, Alloc>::
put (std::ostream& s) const
{

  s << "(";
  if (!simplices.empty()) {
    ConstChainIterator it = simplices.begin();
    
    s << " " << it->coefficient << " * " << *(it->simplex);
    ++it;

    while (it != simplices.end()) {
	 s << " + " << it->coefficient << " * " << *(it->simplex);
	 ++it;
    }
  }
  s << " )";

  return s;
}


template <class CoefficientType, class Alloc> 
Chain<CoefficientType, Alloc> 
operator+ (const Chain<CoefficientType, Alloc>& a, 
		 const Chain<CoefficientType, Alloc>& b)
{
  Chain<CoefficientType, Alloc> r = a;
  r += b;

  return r;
}


template <class CoefficientType, class Alloc> 
Chain<CoefficientType, Alloc> 
operator- (const Chain<CoefficientType, Alloc>& a, 
		 const Chain<CoefficientType, Alloc>& b)
{
  Chain<CoefficientType, Alloc> r = a;
  r -= b;

  return r;
}


template <class CoefficientType, class Alloc> 
Chain<CoefficientType, Alloc> 
operator* (const CoefficientType k, const Chain<CoefficientType, Alloc>& c)
{
  Chain<CoefficientType, Alloc> r = c;
  r *= k;

  return r;
}


template <class CoefficientType, class Alloc> 
std::ostream& 
operator<< (std::ostream& s, const Chain<CoefficientType, Alloc>& chain)
{
  return chain.put(s);
}
