/* Chain.h
 * -------
 * Defines a class for representing a chain of simplices.
 *
 * Written by Patrick Perry
 * Original Version on October 15, 2003
 *
 * $Id: Chain.h,v 1.2 2005/10/24 16:19:56 patperry Exp $
 * $Log: Chain.h,v $
 * Revision 1.2  2005/10/24 16:19:56  patperry
 * Hells (literally) of compile fixes for gcc4.0
 *
 * Revision 1.1.1.1  2005/03/18 23:37:12  silva
 * Imported Sources
 *
 * Revision 1.1.1.1  2004/09/25 22:04:08  patperry
 * Imported sources
 *
 * Revision 1.1  2004/01/15 07:10:07  patperry
 * Changed file extension.
 *
 * Revision 1.2  2003/10/16 08:47:48  patperry
 * Documentation fixes.
 *
 * Revision 1.1  2003/10/16 08:42:15  patperry
 * Added a Chain data structure.
 *
 *
 */

#ifndef _CHAIN_H
#define _CHAIN_H

#include <memory>
#include <list>
#include <utility>
#include <ostream>
#include "Simplex.h"

/**
 * Class for holding a chain.  A chain is defined as a linear combination
 * of simplices from a simplicial complex.
 */
template <class CoefficientType, class Alloc = std::allocator<CoefficientType> >
class Chain 
{
public:

  /**
   * The coefficient type.
   */
  typedef CoefficientType Coefficient;

  /**
   * The simplex type.
   */
  typedef Simplex<Alloc> Simplex;

  /**
   * The type for holding a simplex and its coefficient in the chain.
   */
  struct SimplexCoefficientPair
  {
    //! The simplex.
    const Simplex * simplex;

    //! The coefficient.
    Coefficient coefficient;

    /**
	* Less-than comparator.
	* 
	* \param other the object to compare to.
	* \returns (this < other)
	*/
    bool operator< (const SimplexCoefficientPair& other)
    {
	 return ((*simplex) < *(other.simplex));
    }
  };

  /**
   * The type for iterating through the chain.  Dereferencing a ChainIterator
   * gives a SimplexCoefficentPair.
   */
  typedef typename 
    std::list<SimplexCoefficientPair, 
		    typename Alloc::template rebind<SimplexCoefficientPair>::other>
    ::iterator ChainIterator;

  /**
   * Const iterator for iterating through a chain.  Dereferencing a
   * ConstChainIterator gives a const SimplexCoefficentPair.
   */
  typedef typename 
    std::list<SimplexCoefficientPair, 
		    typename Alloc::template rebind<SimplexCoefficientPair>::other>
    ::const_iterator ConstChainIterator;

private:

  //! the dimension of the chain
  const unsigned short dimension;

  //! the actual chain
  std::list<SimplexCoefficientPair, 
		  typename Alloc::template rebind<SimplexCoefficientPair>::other>
  simplices;
  
public: 

  /**
   * Makes an empty chain of a given dimension.
   *
   * \param dimension the dimension of the chain.
   */
  Chain (unsigned short dimension);

  /**
   * Makes a chain with the given simplices and coefficients.
   * The input ranges must be of the same size, and the simplices
   * must be of the same dimension.
   *
   * \param simplicesBegin the begin of the simplices to be in the chain.
   * \param simplicesEnd the end of the simplices to be in the chain.
   * \param coefficientsBegin the begin of the coefficients.
   */
  template <class SimplexInputIterator, class CoefficientInputIterator>
  Chain (SimplexInputIterator simplicesBegin, 
	    SimplexInputIterator simplicesEnd,
	    CoefficientInputIterator coefficientsBegin);

  /**
   * Frees all resources associated with the chain.
   */
  virtual ~Chain ();

  /**
   * Returns the dimension of the chain.
   *
   * \return the dimension.
   */
  unsigned short getDimension () const;

  /**
   * Returns the length of the chain (the number of simplices in it).
   *
   * \return the length.
   */
  unsigned int getLength () const;

  /**
   * Returns the coefficient of the given simplex.  If the simplex
   * isn't in the chain, this returns 0.
   *
   * \param simplex the simplex to get the coefficent of.
   * \returns the coefficent.
   */
  CoefficientType getSimplexCoefficient (const Simplex * simplex) const;
  
  /**
   * Returns an iterator to the begin of the chain.
   *
   * \returns a ChainIterator to the begin of the chain.
   */
  ChainIterator simplicesBegin ();

  /**
   * Returns an iterator to the end of the chain.
   *
   * \returns a ChainIterator to the end of the chain.
   */
  ChainIterator simplicesEnd ();

  /**
   * Returns a const iterator to the begin of the chain.
   *
   * \returns a ConstChainIterator to the begin of the chain.
   */
  ConstChainIterator simplicesBegin () const;

  /**
   * Returns a const iterator to the end of the chain.
   *
   * \returns a ConstChainIterator to the end of the chain.
   */
  ConstChainIterator simplicesEnd () const;

  /**
    * Add c to this, and return reference to this.
    *
    * \param c another chain.
    * \returns the result (this + c).
    */
  Chain& operator+= (const Chain& c);

  /**
    * Subtract c from this, and return reference to this.
    *
    * \param c another chain.
    * \returns the result (this - c).
    */
  Chain& operator-= (const Chain& c);

  /**
   * Multiply this by given scalar, and return reference to this.
   *
   * \param k a scalar.
   * \returns the result (k * this).
   */
  Chain& operator*= (const CoefficientType k);

  /**
   * Write the chain to an output stream.
   *
   * \param s an output stream.
   * \returns the output stream.
   */
  virtual std::ostream& put (std::ostream& s) const;

};


/**
 * Add two chains together, and return the result.
 *
 * \param a the first chain.
 * \param b the second chain.
 * \returns the result (a + b).
 */
template <class CoefficientType, class Alloc>
Chain<CoefficientType, Alloc> 
operator+ (const Chain<CoefficientType, Alloc>& a,
		 const Chain<CoefficientType, Alloc>& b);


/**
 * Find the difference of two chains.
 *
 * \param a the first chain.
 * \param b the second chain.
 * \returns the result (a - b).
 */
template <class CoefficientType, class Alloc>
Chain<CoefficientType, Alloc> 
operator- (const Chain<CoefficientType, Alloc>& a,
		 const Chain<CoefficientType, Alloc>& b);

		 
/** 
 * Multiply a chain by a scalar, and return the result.
 *
 * \param k a scalar.
 * \param c a chain.
 * \returns the result (k * c).
 */
template <class CoefficientType, class Alloc> 
Chain<CoefficientType, Alloc> 
operator* (const CoefficientType k, const Chain<CoefficientType, Alloc>& c);
		 

/**
 * Overloaded output stream operator.
 *
 * \param s an output stream.
 * \param chain a chain.
 * \returns the output stream.
 */
template <class CoefficientType, class Alloc>
std::ostream& operator<< (std::ostream& s, 
					 const Chain<CoefficientType, Alloc>& chain);


// get compiler to see function definitions:
#include "Chain.cc"


#endif // _CHAIN_H
