/* Filtration.h
 * ------------
 * Written by Patrick Perry
 * Original Version on February 9, 2004
 *
 * $Id: Filtration.cc,v 1.1.1.1 2005/03/18 23:37:14 silva Exp $
 * $Log: Filtration.cc,v $
 * Revision 1.1.1.1  2005/03/18 23:37:14  silva
 * Imported Sources
 *
 * Revision 1.2  2005/03/04 23:46:29  patperry
 * Added copersistence.
 *
 * Revision 1.1.1.1  2004/09/25 22:04:08  patperry
 * Imported sources
 *
 * Revision 1.10  2004/05/28 05:45:48  patperry
 * Implemented union of filtrations
 *
 * Revision 1.9  2004/05/23 20:19:01  patperry
 * Cleaned up Filter interface.
 *
 * Revision 1.8  2004/05/05 22:29:15  patperry
 * Fixed bugs in filtration/persistence.
 *
 * Revision 1.7  2004/04/21 00:12:01  patperry
 * Bugfix for Filtration
 *
 * Revision 1.6  2004/03/14 07:52:51  patperry
 * fixed compile bugs
 *
 * Revision 1.5  2004/03/14 07:30:34  patperry
 * Handle relative complexes in persistence
 *
 * Revision 1.4  2004/03/01 08:43:12  patperry
 * Persistence compile bugs.
 *
 * Revision 1.3  2004/03/01 07:59:12  patperry
 * Persistence output
 *
 * Revision 1.2  2004/02/26 09:43:52  patperry
 * fixed bugs in filtration
 *
 * Revision 1.1  2004/02/10 05:18:59  patperry
 * Added the Filtration class.
 *
 */


#include <utility>


template <class SimplexHandle_t, class FiltrationValue_t, class Alloc>
template <class InputIterator1, class InputIterator2>
Filtration<SimplexHandle_t, FiltrationValue_t, Alloc>::
Filtration (InputIterator1 simplicesBegin, InputIterator1 simplicesEnd,
		  InputIterator2 valuesBegin, InputIterator2 valuesEnd)
{
  std::multimap<Value, SimplexHandle> sorter;

  while (simplicesBegin != simplicesEnd) {
    SimplexHandle simplex = *simplicesBegin++;
    Value value = *valuesBegin++;

    sorter.insert(std::pair<Value, SimplexHandle>(value, simplex));
  }

  extractSortedData(sorter);
}


template <class SimplexHandle_t, class FiltrationValue_t, class Alloc>
template <class InputIterator, class UnaryFunction>
Filtration<SimplexHandle_t, FiltrationValue_t, Alloc>::
Filtration (InputIterator simplicesBegin, InputIterator simplicesEnd,
		  UnaryFunction filtrationFn)
{
  std::multimap<Value, SimplexHandle> sorter;

  while (simplicesBegin != simplicesEnd) {
    SimplexHandle simplex = *simplicesBegin++;
    Value value = filtrationFn(simplex);

    sorter.insert(std::pair<Value, SimplexHandle>(value, simplex));
  }
  
  extractSortedData(sorter);
}


template <class SimplexHandle_t, class FiltrationValue_t, class Alloc>
template <class InputIterator>
Filtration<SimplexHandle_t, FiltrationValue_t, Alloc>::
Filtration (InputIterator begin, InputIterator end)
{
  filteredSimplices.insert(begin, end);
  filtrationValues.reserve(filteredSimplices.size());

  for (int i = 0; i < filteredSimplices.size(); ++i) {
    filtrationFn[filteredSimplices[i]] = i;
    filtrationValues[i] = i;
  }
}


template <class SimplexHandle_t, class FiltrationValue_t, class Alloc>
typename Filtration<SimplexHandle_t, FiltrationValue_t, Alloc>::SimplexIterator
Filtration<SimplexHandle_t, FiltrationValue_t, Alloc>::
simplicesBegin () const
{
  return filteredSimplices.begin();
}


template <class SimplexHandle_t, class FiltrationValue_t, class Alloc>
typename Filtration<SimplexHandle_t, FiltrationValue_t, Alloc>::SimplexIterator
Filtration<SimplexHandle_t, FiltrationValue_t, Alloc>::
simplicesEnd () const
{
  return filteredSimplices.end();
}


template <class SimplexHandle_t, class FiltrationValue_t, class Alloc>
typename Filtration<SimplexHandle_t, FiltrationValue_t, Alloc>::SimplexReverseIterator
Filtration<SimplexHandle_t, FiltrationValue_t, Alloc>::
simplicesRbegin () const
{
  return filteredSimplices.rbegin();
}


template <class SimplexHandle_t, class FiltrationValue_t, class Alloc>
typename Filtration<SimplexHandle_t, FiltrationValue_t, Alloc>::SimplexReverseIterator
Filtration<SimplexHandle_t, FiltrationValue_t, Alloc>::
simplicesRend () const
{
  return filteredSimplices.rend();
}


template <class SimplexHandle_t, class FiltrationValue_t, class Alloc>
typename Filtration<SimplexHandle_t, FiltrationValue_t, Alloc>::ValueIterator
Filtration<SimplexHandle_t, FiltrationValue_t, Alloc>::
valuesBegin () const
{
  return filtrationValues.begin();
}


template <class SimplexHandle_t, class FiltrationValue_t, class Alloc>
typename Filtration<SimplexHandle_t, FiltrationValue_t, Alloc>::ValueIterator
Filtration<SimplexHandle_t, FiltrationValue_t, Alloc>::
valuesEnd () const
{
  return filtrationValues.end();
}

template <class SimplexHandle_t, class FiltrationValue_t, class Alloc>
typename Filtration<SimplexHandle_t, FiltrationValue_t, Alloc>::ValueReverseIterator
Filtration<SimplexHandle_t, FiltrationValue_t, Alloc>::
valuesRbegin () const
{
  return filtrationValues.rbegin();
}


template <class SimplexHandle_t, class FiltrationValue_t, class Alloc>
typename Filtration<SimplexHandle_t, FiltrationValue_t, Alloc>::ValueReverseIterator
Filtration<SimplexHandle_t, FiltrationValue_t, Alloc>::
valuesRend () const
{
  return filtrationValues.rend();
}


template <class SimplexHandle_t, class FiltrationValue_t, class Alloc>
int
Filtration<SimplexHandle_t, FiltrationValue_t, Alloc>::
getFiltrationPosition (SimplexIterator simplex) const
{
  return simplex - filteredSimplices.begin();
}


template <class SimplexHandle_t, class FiltrationValue_t, class Alloc>
int
Filtration<SimplexHandle_t, FiltrationValue_t, Alloc>::
getFiltrationPosition (SimplexHandle simplex) const
{
  int result = -1;
  typename Hash_t::const_iterator pos = filtrationFn.find(simplex);
  
  if (pos != filtrationFn.end()) {
    result = pos->second;
  }
  
  return result;
}


template <class SimplexHandle_t, class FiltrationValue_t, class Alloc>
typename Filtration<SimplexHandle_t, FiltrationValue_t, Alloc>::Value
Filtration<SimplexHandle_t, FiltrationValue_t, Alloc>::
getFiltrationValue (SimplexIterator simplex) const
{
  return filtrationValues[simplex - filteredSimplices.begin()];
}


template <class SimplexHandle_t, class FiltrationValue_t, class Alloc>
typename Filtration<SimplexHandle_t, FiltrationValue_t, Alloc>::Value
Filtration<SimplexHandle_t, FiltrationValue_t, Alloc>::
getFiltrationValue (SimplexHandle simplex) const
{
  Value result = static_cast<Value>(0);

  int position = getFiltrationPosition(simplex);
  if (position >= 0) {
    result = filtrationValues[position];
  }

  return result;
}


template <class SimplexHandle_t, class FiltrationValue_t, class Alloc>
typename Filtration<SimplexHandle_t, FiltrationValue_t, Alloc>::Value
Filtration<SimplexHandle_t, FiltrationValue_t, Alloc>::
operator() (SimplexHandle simplex) const
{
  return getFiltrationValue(simplex);
}


template <class SimplexHandle_t, class FiltrationValue_t, class Alloc>
void
Filtration<SimplexHandle_t, FiltrationValue_t, Alloc>::
extractSortedData (std::multimap<Value, SimplexHandle>& sorter)
{
  typename std::multimap<Value, SimplexHandle>::iterator it = sorter.begin();
  typename std::multimap<Value, SimplexHandle>::iterator end = sorter.end();
  filteredSimplices.reserve(sorter.size());
  filtrationValues.reserve(sorter.size());
  int position = 0;

  while (it != end) {
    const Value value = it->first;
    typename std::multimap<Value, SimplexHandle>::iterator next_it 
	 = sorter.upper_bound(value);
    std::multimap<int, SimplexHandle> dimensionSorter;

    // sort all simplices with the same value by their dimension
    while (it != next_it) {
	 SimplexHandle simplex = it->second;
	 int dimension = simplex->getDimension();
	 
	 dimensionSorter.insert(std::pair<int, SimplexHandle>
					    (dimension, simplex));
	 ++it;
    }

    typename std::multimap<int, SimplexHandle>::iterator dim_it 
	 = dimensionSorter.begin();
    typename std::multimap<int, SimplexHandle>::iterator dim_end
	 = dimensionSorter.end();

    while (dim_it != dim_end) {
	 filtrationValues.push_back(value);
	 filteredSimplices.push_back(dim_it->second);
	 filtrationFn[dim_it->second] = position++;
	 ++dim_it;
    }
  }
}		    

