/* Filtration.h
 * ------------
 * Written by Patrick Perry
 * Original Version on February 9, 2004
 *
 * $Id: Filtration.h,v 1.2 2005/10/24 16:19:56 patperry Exp $
 * $Log: Filtration.h,v $
 * Revision 1.2  2005/10/24 16:19:56  patperry
 * Hells (literally) of compile fixes for gcc4.0
 *
 * Revision 1.1.1.1  2005/03/18 23:37:14  silva
 * Imported Sources
 *
 * Revision 1.2  2005/03/04 23:46:29  patperry
 * Added copersistence.
 *
 * Revision 1.1.1.1  2004/09/25 22:04:08  patperry
 * Imported sources
 *
 * Revision 1.10  2004/05/28 05:45:48  patperry
 * Implemented union of filtrations
 *
 * Revision 1.9  2004/05/23 20:19:01  patperry
 * Cleaned up Filter interface.
 *
 * Revision 1.8  2004/05/05 22:29:15  patperry
 * Fixed bugs in filtration/persistence.
 *
 * Revision 1.7  2004/03/14 07:52:51  patperry
 * fixed compile bugs
 *
 * Revision 1.6  2004/03/14 07:30:34  patperry
 * Handle relative complexes in persistence
 *
 * Revision 1.5  2004/03/01 08:43:12  patperry
 * Persistence compile bugs.
 *
 * Revision 1.4  2004/03/01 07:59:12  patperry
 * Persistence output
 *
 * Revision 1.3  2004/02/26 09:43:52  patperry
 * fixed bugs in filtration
 *
 * Revision 1.2  2004/02/26 07:39:42  patperry
 * small design changes
 *
 * Revision 1.1  2004/02/10 05:18:59  patperry
 * Added the Filtration class.
 *
 */

#ifndef _FILTRATION_H
#define _FILTRATION_H

#include <map>
#include <memory>
#include <vector>

// Lots of trouble just to get a hashtable.  Code taken from
// http://gcc.gnu.org/onlinedocs/libstdc++/faq/index.html#5_4
#ifdef __GNUC__
#if __GNUC__ < 3
  #include <hash_map.h>
  namespace Sgi { using ::hash_map }; 
#else
  #include <ext/hash_map>
  #if ((__GNUC__ == 3) && (__GNUC_MINOR__ == 0))
    namespace Sgi = std;
  #else 
    namespace Sgi = ::__gnu_cxx;
  #endif
#endif
#else
  #include <hash_map>
  namespace Sgi = std;
#endif 


/**
 * Class for representing a filtration.  It represents a function from
 * a set of simplices to an ordered set.  The class stores the simplices
 * in increasing order according to the filtration, along with their
 * filtration values.
 */
template <class SimplexHandle_t, class FiltrationValue_t, 
		class Alloc = std::allocator<int> > 
class Filtration
  : public std::unary_function<SimplexHandle_t, FiltrationValue_t>
{
 public:

  //! The simplex handle type
  typedef SimplexHandle_t SimplexHandle;

  //! The values the filtration takes
  typedef FiltrationValue_t Value;

  //! The collection type for simplices
  typedef std::vector<SimplexHandle, 
				  typename Alloc::template rebind<SimplexHandle>::other> 
    FilteredSimplexCollection;

  //! The collection type for values
  typedef std::vector<Value, 
				  typename Alloc::template rebind<Value>::other> 
    FiltrationValueCollection;

  //! Iterator for filtered simplices
  typedef typename FilteredSimplexCollection::const_iterator SimplexIterator;

  //! Iterator for filtration values
  typedef typename FiltrationValueCollection::const_iterator ValueIterator;

  //! Reverse iterator for simplices
  typedef typename FilteredSimplexCollection::const_reverse_iterator 
    SimplexReverseIterator;

  typedef typename FiltrationValueCollection::const_reverse_iterator
  ValueReverseIterator;


 private:

  //! The collection of filtered simplices
  FilteredSimplexCollection filteredSimplices;
  
  //! The collection of filtration values
  FiltrationValueCollection filtrationValues;

  /**
   * type for hashing a simplex.
   */
  struct HashSimplex
  {
    std::size_t operator() (SimplexHandle x) const
    {
	 return reinterpret_cast<std::size_t>(x);
    }
  };

  //! the hash map type
 typedef Sgi::hash_map<SimplexHandle, int, HashSimplex, 
				   std::equal_to<SimplexHandle>, Alloc> Hash_t;
 
  //! a hashtable mapping simplices in the domain to filtration positions
  Hash_t filtrationFn;

 public:

  /**
   * Constructor.  Takes an ordered collection of simplices and a 
   * collection of filtration values.  The simplices are filtered according
   * to the values in the filtration value collection.
   *
   * \param simplicesBegin the begin of the simplex collection.
   * \param simplicesEnd the end of the simplices collection.
   * \param valuesBegin the begin of the filtration values.
   */
  template <class InputIterator1, class InputIterator2>
  Filtration (InputIterator1 simplicesBegin, InputIterator1 simplicesEnd,
		    InputIterator2 valuesBegin, InputIterator2 valuesEnd);


  /**
   * Constructor.  Takes a collection of simplices and a filtration
   * function.  The simplices are filtered according to the filtration
   * function.
   *
   * \param simplicesBegin the begin of the simplex collection.
   * \param simplicesEnd the end of the simplices collection.
   * \param filtrationFn the function used to compute the filtration.
   */
  template <class InputIterator, class UnaryFunction>
  Filtration (InputIterator simplicesBegin, InputIterator simplicesEnd,
		    UnaryFunction filtrationFn);

  /**
   * Constructor.  Takes an ordered collection of simplices and filters 
   * them in that order.  The filtration value of a simplex is its index 
   * in the collection.
   *
   * \param begin an iterator to the first simplex in the filtration.
   * \param end an iterator to the end of the simplex filtration.
   */
  template <class InputIterator>
  Filtration (InputIterator begin, InputIterator end);

  /**
   * Get an iterator to the begin of the simplices in filtraion order.
   *
   * \returns the begin of the filtration.
   */
  SimplexIterator simplicesBegin () const;

  /**
   * Get an iterator to the end of the simplices in filtration order.
   *
   * \returns the end of the filtration.
   */
  SimplexIterator simplicesEnd () const;

  /**
   * Get an iterator to the rbegin of the simplices in filtraion order.
   *
   * \returns the rbegin of the filtration.
   */
  SimplexReverseIterator simplicesRbegin () const;

  /**
   * Get an iterator to the rend of the simplices in filtration order.
   *
   * \returns the rend of the filtration.
   */
  SimplexReverseIterator simplicesRend () const;

  /**
   * Get an iterator to the begin of the filtration value collection.
   *
   * \returns the begin of the filtration values.
   */
  ValueIterator valuesBegin () const;

  /**
   * Get an iterator to the end of the filtration value collection.
   *
   * \returns the end of the filtration values.
   */
  ValueIterator valuesEnd () const;

  /**
   * Get an iterator to the rbegin of the filtration value collection.
   *
   * \returns the rbegin of the filtration values.
   */
  ValueReverseIterator valuesRbegin () const;

  /**
   * Get an iterator to the rend of the filtration value collection.
   *
   * \returns the rend of the filtration values.
   */
  ValueReverseIterator valuesRend () const;

  /**
   * Get the ordinal filtration value from an iterator to the simplex.  This
   * is a constant-time operation.
   * 
   * \param simplex an iterator to a simplex in the filtration.
   * \returns the position in the filtration
   */ 
  int getFiltrationPosition (SimplexIterator simplex) const;

  /**
   * Get the ordinal filtration value from a handle to a simplex.  This is a
   * constant-time operation, but it involves hashing the simplex.
   *
   * \param simplex the simplex to get the filtration value of.
   * \returns the position in the filtration
   */
  int getFiltrationPosition (SimplexHandle simplex) const;


  /**
   * Get the filtration value from an iterator to the simplex.  This
   * is a constant-time operation.
   * 
   * \param simplex an iterator to a simplex in the filtration.
   * \returns the filtration value.
   */ 
  Value getFiltrationValue (SimplexIterator simplex) const;

  /**
   * Get the filtration value from a handle to a simplex.  This is a
   * constant-time operation, but it involves hashing the simplex.
   *
   * \param simplex the simplex to get the filtration value of.
   * \returns the filtration value.
   */
  Value getFiltrationValue (SimplexHandle simplex) const;

  /**
   * Allows the filtration object to act like a unary function 
   * mapping simplex to value.
   * \param simplex the simplex to get the filtration value of.
   * \returns the filtration value.
   */
  Value operator() (SimplexHandle simplex) const;

 private:

  /**
   * Take a multimap sorting the filtration value/simplex pairs, and insert
   * the data into the filtration member arrays, making sure simplices with
   * the same filtration value are sorted by dimension.
   *
   * \param sorter the multimap sorting the simplices by value.
   */
  void extractSortedData (std::multimap<Value, SimplexHandle>& sorter);
};


// get the compiler to see the function definitions
#include "Filtration.cc"


#endif // _FILTRATION_H
