/* PackedArrayIterator.h
 * ---------------------
 * Written by Patrick Perry
 * Original Version on July 5, 2004
 *
 * $Id: PackedArrayIterator.h,v 1.2 2005/10/24 16:19:56 patperry Exp $
 * $Log: PackedArrayIterator.h,v $
 * Revision 1.2  2005/10/24 16:19:56  patperry
 * Hells (literally) of compile fixes for gcc4.0
 *
 * Revision 1.1.1.1  2005/03/18 23:37:17  silva
 * Imported Sources
 *
 * Revision 1.1.1.1  2004/09/25 22:04:08  patperry
 * Imported sources
 *
 * Revision 1.1  2004/07/07 23:03:06  patperry
 * Added packed array iterator.
 *
 */

#ifndef _PACKEDARRAYITERATOR_H
#define _PACKEDARRAYITERATOR_H

#include <iterator>
#include <memory>


template <class Element_t, class Alloc = std::allocator<int> >
class PackedArrayIterator 
  : public std::iterator<std::random_access_iterator_tag, Element_t *>
{

 private:

  //! the dimension of the points
  int dimension;

  //! the position in the array
  Element_t * position;

 public:

  /** 
   * Constructor.  Makes an iterator pointing into a dense point array
   * storing points of a given dimension at the given position.
   *
   * \param dimension the dimension of the points in the array.
   * \param position the point in the array to point to.
   */
  PackedArrayIterator (int dimension, Element_t * position);

  /**
   * Default Constructor.
   */
  PackedArrayIterator ();

  /**
   * Destructor.
   */
  virtual ~PackedArrayIterator ();

  /**
   * De-refernce operator.
   *
   * \returns the value pointed to by the iterator.
   */
  Element_t * operator* ();

  /**
   * Pre-increment operator.  Increments the iterator.
   *
   * \returns the iterator at (this + 1).
   */
  PackedArrayIterator& operator++ ();

  /**
   * Post-increment operator.
   *
   * \param post dummy argument.
   * \returns this iterator before incrementing.
   */
  PackedArrayIterator operator++ (int post);

  /**
   * Pre-decrement operator.  Decrements the iterator.
   *
   * \returns the iterator at (this - 1).
   */
  PackedArrayIterator& operator-- ();

  /**
   * Post-decrement operator.
   *
   * \param post dummy argument.
   * \returns this iterator before decrementing.
   */
  PackedArrayIterator operator-- (int post);

  /**
   * Forward-advance operator.  Increments the iterator by n.
   *
   * \param n an integer.
   * \returns this after it has been incremented.
   */
  PackedArrayIterator& operator+= (int n);

  /**
   * Backward-reverse operator.  Decrements the iterator by n.
   *
   * \param n an integer.
   * \returns this after it has been decremented.
   */
  PackedArrayIterator& operator-= (int n);

  /**
   * Distance operator.
   *
   * \param x another iterator.
   * \returns the distance between this and x.
   */
  int operator- (const PackedArrayIterator& x) const;


  /**
   * Index operator.
   *
   * \param n an integer.
   * \returns the value at this[n].
   */
  Element_t * operator[] (int n) const;

  /**
   * Equals comparator.
   *
   * \param x iterator to compare this to.
   * \returns (*this) == x.
   */
  bool operator== (const PackedArrayIterator& x) const;

  /**
   * Not-Equals comparator.
   *
   * \param x iterator to compare this to.
   * \returns (*this) != x.
   */
  bool operator!= (const PackedArrayIterator& x) const;

  /**
   * Less-than comparator.
   *
   * \param x iterator to compare this to.
   * \returns (*this) < x.
   */
  bool operator< (const PackedArrayIterator& x) const;

};


/**
 * Iterator addition.
 *
 * \param n an integer.
 * \param i an iterator.
 * \returns the iterator at i + n.
 */
template<class Element_t, class Alloc>
PackedArrayIterator<Element_t, Alloc> 
operator+ (int n, const PackedArrayIterator<Element_t, Alloc>& i);


/**
 * Iterator addition.
 *
 * \param i an iterator.
 * \param n an integer.
 * \returns the iterator at n + i.
 */
template<class Element_t, class Alloc>
PackedArrayIterator<Element_t, Alloc> 
operator+ (const PackedArrayIterator<Element_t, Alloc>& i, int n);


/**
 * Iterator subtraction.
 *
 * \param i an iterator.
 * \param n an integer.
 * \returns the iterator at i - n.
 */
template<class Element_t, class Alloc>
PackedArrayIterator<Element_t, Alloc> 
operator- (const PackedArrayIterator<Element_t, Alloc>& i, int n);


// get compiler to see function definitions
#include "PackedArrayIterator.cc"


#endif // _PACKEDARRAYITERATOR_H
