/* PackedMatrixColumnIterator.h
 * ----------------------------
 * Written by Patrick Perry
 * Original Version on July 28, 2004
 *
 * $Id: PackedMatrixColumnIterator.h,v 1.2 2005/10/24 16:19:56 patperry Exp $
 * $Log: PackedMatrixColumnIterator.h,v $
 * Revision 1.2  2005/10/24 16:19:56  patperry
 * Hells (literally) of compile fixes for gcc4.0
 *
 * Revision 1.1.1.1  2005/03/18 23:37:17  silva
 * Imported Sources
 *
 * Revision 1.1.1.1  2004/09/25 22:04:08  patperry
 * Imported sources
 *
 * Revision 1.1  2004/09/16 09:00:24  patperry
 * Added C++ Witness
 *
 */

#ifndef _PACKEDMATRIXCOLUMNITERATOR_H
#define _PACKEDMATRIXCOLUMNITERATOR_H

#include <cstdlib>
#include <iterator>
#include <memory>

template <class Element_t, class Alloc>
class PackedMatrixColumnIterator;
#include "PackedMatrixRowIterator.h"


/**
 * Implements an IndexedRandomAccessColumnIterator for a matrix
 * stored as a standard C array in column major form.  See the uBLAS
 * documentation for information on the IndexedRandomAccessColumnIterator
 * concept.
 */
template <class Element_t, class Alloc = std::allocator<int> >
class PackedMatrixColumnIterator 
  : public std::iterator<std::random_access_iterator_tag, Element_t>
{
 public:

  typedef std::size_t size_type;
  typedef PackedMatrixRowIterator<Element_t, Alloc> dual_iterator_type;
  typedef std::reverse_iterator<dual_iterator_type> dual_reverse_iterator_type;

 private:

  //! the number of rows in the matrix
  size_type numRows;

  //! the number of columns in the matrix
  size_type numColumns;

  //! the row index;
  size_type row;

  //! the column index;
  size_type column;

  //! the position in the matrix
  Element_t * position;

 public:

  /** 
   * Constructor.  Makes an iterator pointing into a dense point array
   * storing points of a given dimension at the given position.
   *
   * \param numRows the number of rows in the matrix.
   * \param numColumns the number of columns in the matrix.
   * \param row the current row.
   * \param column the current column
   * \param position the element in the array to point to.
   */
  PackedMatrixColumnIterator (size_type numRows, size_type numColumns,
						size_type row, size_type column,
						Element_t * position);

  /**
   * Default Constructor.
   */
  PackedMatrixColumnIterator ();

  /**
   * Destructor.
   */
  virtual ~PackedMatrixColumnIterator ();

  /**
   * De-refernce operator.
   *
   * \returns the value pointed to by the iterator.
   */
  Element_t operator* ();

  /**
   * Pre-increment operator.  Increments the iterator.
   *
   * \returns the iterator at (this + 1).
   */
  PackedMatrixColumnIterator& operator++ ();

  /**
   * Post-increment operator.
   *
   * \param post dummy argument.
   * \returns this iterator before incrementing.
   */
  PackedMatrixColumnIterator operator++ (int post);

  /**
   * Pre-decrement operator.  Decrements the iterator.
   *
   * \returns the iterator at (this - 1).
   */
  PackedMatrixColumnIterator& operator-- ();

  /**
   * Post-decrement operator.
   *
   * \param post dummy argument.
   * \returns this iterator before decrementing.
   */
  PackedMatrixColumnIterator operator-- (int post);

  /**
   * Forward-advance operator.  Increments the iterator by n.
   *
   * \param n an integer.
   * \returns this after it has been incremented.
   */
  PackedMatrixColumnIterator& operator+= (int n);

  /**
   * Backward-reverse operator.  Decrements the iterator by n.
   *
   * \param n an integer.
   * \returns this after it has been decremented.
   */
  PackedMatrixColumnIterator& operator-= (int n);

  /**
   * Distance operator.
   *
   * \param x another iterator.
   * \returns the distance between this and x.
   */
  int operator- (const PackedMatrixColumnIterator& x) const;

  /**
   * Index operator.
   *
   * \param n an integer.
   * \returns the value at this[n].
   */
  Element_t operator[] (int n) const;

  /**
   * Equals comparator.
   *
   * \param x iterator to compare this to.
   * \returns (*this) == x.
   */
  bool operator== (const PackedMatrixColumnIterator& x) const;

  /**
   * Not-Equals comparator.
   *
   * \param x iterator to compare this to.
   * \returns (*this) != x.
   */
  bool operator!= (const PackedMatrixColumnIterator& x) const;

  /**
   * Less-than comparator.
   *
   * \param x iterator to compare this to.
   * \returns (*this) < x.
   */
  bool operator< (const PackedMatrixColumnIterator& x) const;

  /**
   * First (row) index of the current position.
   *
   * \returns the row index.
   */
  size_type index1 () const;

  /**
   * Second (column) index of the current position.
   *
   * \returns the column index.
   */
  size_type index2 () const;

  /**
   * Get a row iterator to the begin of the current row.
   *
   * \returns the begin of the row.
   */
  dual_iterator_type begin () const;

  /**
   * Get a row iterator to the end of the current row.
   *
   * \returns the end of the row.
   */
  dual_iterator_type end () const;

  /**
   * Get a reverse row iterator to the rbegin of the current row.
   *
   * \returns the rbegin of the row.
   */
  dual_reverse_iterator_type rbegin () const;

  /**
   * Get a reverse row iterator to the rend of the current row.
   *
   * \returns the rend of the row.
   */
  dual_reverse_iterator_type rend () const;

};


/**
 * Iterator addition.
 *
 * \param n an integer.
 * \param i an iterator.
 * \returns the iterator at i + n.
 */
template<class Element_t, class Alloc>
PackedMatrixColumnIterator<Element_t, Alloc> 
operator+ (int n, const PackedMatrixColumnIterator<Element_t, Alloc>& i);


/**
 * Iterator addition.
 *
 * \param i an iterator.
 * \param n an integer.
 * \returns the iterator at n + i.
 */
template<class Element_t, class Alloc>
PackedMatrixColumnIterator<Element_t, Alloc> 
operator+ (const PackedMatrixColumnIterator<Element_t, Alloc>& i, int n);


/**
 * Iterator subtraction.
 *
 * \param i an iterator.
 * \param n an integer.
 * \returns the iterator at i - n.
 */
template<class Element_t, class Alloc>
PackedMatrixColumnIterator<Element_t, Alloc> 
operator- (const PackedMatrixColumnIterator<Element_t, Alloc>& i, int n);


#include "PackedMatrixColumnIterator.cc"


#endif // _PACKEDMATRIXCOLUMNITERATOR_H
