/* PointSet.h
 * ----------
 * Written by Patrick Perry
 * Original Version on June 30, 2004
 *
 * $Id: PointSet.h,v 1.2 2005/10/24 16:19:56 patperry Exp $
 * $Log: PointSet.h,v $
 * Revision 1.2  2005/10/24 16:19:56  patperry
 * Hells (literally) of compile fixes for gcc4.0
 *
 * Revision 1.1.1.1  2005/03/18 23:37:19  silva
 * Imported Sources
 *
 * Revision 1.1.1.1  2004/09/25 22:04:08  patperry
 * Imported sources
 *
 * Revision 1.1  2004/07/07 23:05:39  patperry
 * Added PointSet
 *
 */

#ifndef _POINTSET_H
#define _POINTSET_H

#include <memory>
#include <utility>
#include <vector>
#include <ANN/ANN.h>


template <class Alloc = std::allocator<int> >
class PointSet
{

public:

  //! the point type
  typedef const double * Point;

private:

  //! point allocator
  typedef typename Alloc::template rebind<Point>::other PointAlloc;

public:

  //! the iterator-to-point type
  typedef typename std::vector<Point, PointAlloc>::const_iterator iterator;

  //! the output type for nearest-neighbor queries
  typedef std::pair<Point, double> NeighborQueryResult;
  
 private:

  //! the dimension of points in the set
  int dimension;

  //! the collection of points
  std::vector<Point, PointAlloc> points;

  //! the tree storing the points
  mutable ANNbd_tree searchTree;

  //! indicates whether or not to free the points upon deletion
  bool freePoints;

 public:

  /**
   * Constructor.  Builds a point set from a collection of Points.
   *
   * \param dimension the dimension of the points in the set.
   * \param begin an iterator to the begin of the point collection.
   * \param end an iterator to the end of the point collection.
   * \param freePoints a flag indicating whether or not the destructor
   *        of the PointSet should call delete on each Point in the
   *        collection.
   */
  template <class InputIterator>
  PointSet (int dimension, InputIterator begin, InputIterator end,
		  bool freePoints = false);

  /**
   * Destructor.
   */
  virtual ~PointSet ();

  /**
   * Get the dimension of the points in the set.
   *
   * \returns the dimension.
   */
  int getDimension () const;

  /**
   * Get the size of the set.
   *
   * \returns the number of points.
   */
  int getSize() const;

  /**
   * Get an iterator to the begin of the point set.
   *
   * \returns an iterator to begin.
   */
  iterator begin () const;

  /**
   * Get an iterator to the end of the point set.
   *
   * \returns an iterator to end.
   */
  iterator end () const;

  /**
   * Get the nearest neighbors to a point.
   *
   * \param query the query point.
   * \param numNeighbors the number of neighbors to get.  This number
   *    must be less than or equal to the number of points in the set.
   * \param errorBound the error bound of the neighbor query.  From the
   *    ANN manual: "the ith nearest neighbor is a (1 + errorBound)
   *    approximation to the true ith nearest neighbor.  That is, the true
   *    (not squared) distance to this point may exceed the true distance to
   *    the real ith nearest neighbor of the query point by a factor of 
   *    (1 + errorBound)."  errorBound may be any non-negative value.
   * \param neighbors an output iterator capable of holding numNeighbors
   *    NeighborQueryResults.
   *
   * \returns neighbors + numNeighbors
   *         
   */
  template <class OutputIterator>
  OutputIterator getNearestNeighbors (Point query, int numNeighbors, 
							   double errorBound, 
							   OutputIterator neighbors) const;

};

// get compiler to see function definitions
#include "PointSet.cc"

#endif // _POINTSET_H
