/* Simplex.cc
 * ----------
 * Implements the interface defined in Simplex.h
 *
 * Written by Patrick Perry
 * Original Version on 2/25/2003
 *
 * $Id: Simplex.cc,v 1.2 2005/10/24 16:19:56 patperry Exp $
 * $Log: Simplex.cc,v $
 * Revision 1.2  2005/10/24 16:19:56  patperry
 * Hells (literally) of compile fixes for gcc4.0
 *
 * Revision 1.1.1.1  2005/03/18 23:37:20  silva
 * Imported Sources
 *
 * Revision 1.2  2005/03/04 23:46:29  patperry
 * Added copersistence.
 *
 * Revision 1.1.1.1  2004/09/25 22:04:08  patperry
 * Imported sources
 *
 * Revision 1.15  2004/03/15 08:15:31  patperry
 * fixed some simplex identification stuff
 *
 * Revision 1.14  2004/03/13 06:57:04  patperry
 * fixed compile bug
 *
 * Revision 1.13  2004/03/13 06:38:19  patperry
 * added initial version of relative complex code
 *
 * Revision 1.12  2004/03/13 05:43:12  patperry
 * added initial simplicial set code.
 *
 * Revision 1.11  2004/02/25 09:27:59  patperry
 * Added ConstSimplexIterators
 *
 * Revision 1.10  2004/02/25 00:45:52  patperry
 * Fixed RippsExpand.
 *
 * Revision 1.9  2004/02/24 23:48:40  patperry
 * Fixed bugs in Simplex destructor
 *
 * Revision 1.8  2004/02/24 13:30:39  patperry
 * Radically restructured Simplex/Simplicial Complex
 *
 * Revision 1.7  2004/01/28 09:55:43  patperry
 * Handled degenerate lable sorting
 *
 * Revision 1.6  2004/01/25 14:13:23  patperry
 * Added function to sort simplex label and determine its sign.
 *
 * Revision 1.5  2004/01/15 07:10:33  patperry
 * Changed file extensions.
 *
 * Revision 1.4  2003/11/16 07:38:26  patperry
 * fixed bug in simplicesEnd for vertices
 *
 * Revision 1.3  2003/10/10 22:08:24  patperry
 * Fixed problems with addSimplex.
 * Added a removeSimplex function to SimpicialComplex.
 *
 * Revision 1.2  2003/10/10 11:09:03  patperry
 * Added better interface of inserting simplices into a simplicial complex.  Added
 * simplex iterators.
 *
 * Revision 1.1.1.1  2003/10/09 03:24:46  patperry
 * Imported Sources
 *
 */

#include <algorithm>
#include <cassert>
#include <iterator>
#include <map>
#include <memory>
#include <utility>
#include <iostream>

// Lots of trouble just to get a SGI extensions.  Code taken from
// http://gcc.gnu.org/onlinedocs/libstdc++/faq/index.html#5_4
#ifdef __GNUC__
#if __GNUC__ < 3
  #include <algo.h>
  namespace Sgi { using ::hash_map }; 
#else
    #include <ext/algorithm>
  #if ((__GNUC__ == 3) && (__GNUC_MINOR__ == 0))
    namespace Sgi = std;
  #else 
    namespace Sgi = ::__gnu_cxx;
  #endif
#endif
#else
  #include <algorithm>
  namespace Sgi = std;
#endif 


template <class Alloc>
bool 
Simplex<Alloc>::
simplexPointerLess (const Simplex * first, 
				const Simplex * second)
{
  return (*first < *second);
}


template <class Alloc> 
template <class RandomAccessIterator>
int 
Simplex<Alloc>::
sortLabel (int dimension, RandomAccessIterator label)
{
  std::vector<std::pair<int, int> > sortingVector;
  int permutation[dimension + 1];
  int i;
  int numTranspositions = 0;
  int result;

  // sort the label
  for (i = 0; i <= dimension; ++i) {
    sortingVector.push_back(std::pair<int, int>(label[i], i));
  }
  std::sort(sortingVector.begin(), sortingVector.end());

  // copy the sorted label, and the permutation
  i = 0;
  for (std::vector<std::pair<int, int> >::iterator it = sortingVector.begin();
	  it != sortingVector.end(); ++it) {
    label[i] = it->first;
    permutation[i] = it->second;
    ++i;
  }

  // simulate the transpositions to put the permutation back
  // in order
  for (i = 0; i <= dimension; ++i) {
    if ((permutation[i] - i) > 0) {
	 for (int j = 1; j <= (permutation[i] - i); ++j) {
	   permutation[i + j] += 1;
	 }

	 numTranspositions += (permutation[i] - i);
    }
  }

  // determine the sign of the permutation
  if ((numTranspositions % 2) == 0) {
    result = 1;
  } else {
    result = -1;
  }

  return result;
}


template <class Alloc>
template <class InputIterator1, class InputIterator2>
Simplex<Alloc>::
Simplex (SimplicialComplex& scomplex,
	    int dimension, InputIterator1 label, InputIterator2 boundary)
  : scomplex(&scomplex),
    dimension(dimension),
    faces(dimension + 1),
    skeleton(NULL)
{
  for (int i = 0; i <= dimension; ++i) {
    (this->label).push_back(*label++);
    faces[i] = *boundary++;
    faces[i]->addCoface(this);
  }

  if (dimension >= 0) {
    scomplex.incrementSimplexCount(dimension);
  }

  //std::cout << "Constructing: " << *this << "(" << this << ")\n";
}


template <class Alloc>
template <class InputIterator>
Simplex<Alloc>::
Simplex (int dimension, InputIterator label) 
  : scomplex(NULL),
    dimension(dimension),
    skeleton(NULL)
{
  std::back_insert_iterator<std::vector<int, Alloc> > 
    label_inserter(this->label);

  Sgi::copy_n(label, dimension + 1, label_inserter);
}


template <class Alloc> 
Simplex<Alloc>::
Simplex ()
{
}

template <class Alloc> 
Simplex<Alloc>::
~Simplex ()
{
  if (scomplex != NULL) {
    //std::cout << "Deleting: " << *this << "(" << this << ")\n";
    
    FaceIterator face_it = facesBegin();
    FaceIterator face_end = facesEnd();
    
    while (face_it != face_end) {
	 (*face_it++)->removeCoface(dimension, getLabel());
    }
    
    int numCofaces = getNumCofaces();
    Simplex * localCofaceArray[numCofaces];
    std::copy(cofacesBegin(), cofacesEnd(), localCofaceArray);
    cofaces.clear();
    
    Simplex ** coface_it = localCofaceArray;
    Simplex ** coface_end = localCofaceArray + numCofaces;
    
    while (coface_it != coface_end) {
	 SimplicialComplex::SimplexAlloc.destroy(*coface_it);
	 SimplicialComplex::SimplexAlloc.deallocate(*coface_it, 1);

	 ++coface_it;
    }
    

    if (dimension >= 0) {
	 scomplex->decrementSimplexCount(dimension);
    }
  }
}


template <class Alloc> 
int
Simplex<Alloc>::
getDimension () const
{
  return dimension;
}


template <class Alloc> 
typename Simplex<Alloc>::LabelIterator
Simplex<Alloc>::
getLabel () const
{
  return label.begin();
}


template <class Alloc>
int
Simplex<Alloc>::
getNumCofaces () const
{
  return cofaces.size();
}


template <class Alloc>
typename Simplex<Alloc>::FaceIterator
Simplex<Alloc>::
facesBegin ()
{
  return faces.begin();
}


template <class Alloc>
typename Simplex<Alloc>::ConstFaceIterator
Simplex<Alloc>::
facesBegin () const
{
  return faces.begin();
}


template <class Alloc>
typename Simplex<Alloc>::FaceIterator
Simplex<Alloc>::
facesEnd ()
{
  return faces.end();
}


template <class Alloc>
typename Simplex<Alloc>::ConstFaceIterator
Simplex<Alloc>::
facesEnd () const
{
  return faces.end();
}


template <class Alloc>
typename Simplex<Alloc>::CofaceIterator
Simplex<Alloc>::
cofacesBegin () 
{
  sortCofaces();

  return cofaces.begin();
}


template <class Alloc>
typename Simplex<Alloc>::ConstCofaceIterator
Simplex<Alloc>::
cofacesBegin () const
{
  const_cast<Simplex *>(this)->sortCofaces();

  return cofaces.begin();
}


template <class Alloc>
typename Simplex<Alloc>::CofaceIterator
Simplex<Alloc>::
cofacesEnd ()
{
  sortCofaces();

  return cofaces.end();
}


template <class Alloc>
typename Simplex<Alloc>::ConstCofaceIterator
Simplex<Alloc>::
cofacesEnd () const
{
  const_cast<Simplex *>(this)->sortCofaces();

  return cofaces.end();
}


template <class Alloc>
Simplex<Alloc> *
Simplex<Alloc>::
getFace (int i)
{
  assert(i >= 0);
  assert(i <= dimension);

  return faces[i];
}


template <class Alloc>
const Simplex<Alloc> *
Simplex<Alloc>::
getFace (int i) const
{
  assert(i >= 0);
  assert(i <= dimension);

  return faces[i];
}


template <class Alloc>
template <class InputIterator>
typename Simplex<Alloc>::CofaceIterator 
Simplex<Alloc>::
findCoface (int dimension, InputIterator label)
{
  assert(dimension == (this->dimension + 1));

  sortCofaces();

  Simplex query(dimension, label);
  CofaceIterator result 
    = std::lower_bound(cofaces.begin(), cofaces.end(), &query,
				   simplexPointerLess);

  if (result != cofaces.end()) {
    if (query < **result) {
	 result = cofaces.end();
    }
  }

  return result;
}


template <class Alloc>
template <class InputIterator>
typename Simplex<Alloc>::ConstCofaceIterator 
Simplex<Alloc>::
findCoface (int dimension, InputIterator label) const
{
  assert(dimension == (this->dimension + 1));

  const_cast<Simplex *>(this)->sortCofaces();

  Simplex query(dimension, label);
  ConstCofaceIterator result 
    = std::lower_bound(cofaces.begin(), cofaces.end(), &query,
				   simplexPointerLess);

  if (result != cofaces.end()) {
    if (query < **result) {
	 result = cofaces.end();
    }
  }

  return result;
}


template <class Alloc>
bool 
Simplex<Alloc>::
isDegenerate () const
{
  bool result = (this->skeleton != NULL);

  return result;
}


template <class Alloc>
void 
Simplex<Alloc>::
identifyWith (Simplex * simplex)
{
  Simplex * skeleton = getSkeleton();
  Simplex * identifyWithSkeleton = simplex->getSkeleton();

  assert(identifyWithSkeleton->getDimension() <= 0);

  // collapsing only has effect if the target is not the same as this
  if (skeleton != identifyWithSkeleton) {

    // for non-identified simplices, just set skeleton to be the target
    if (this->skeleton == NULL) {
	 this->skeleton = identifyWithSkeleton;

	 // identify all faces with the target
	 if (getDimension() > 0) {
	   FaceIterator face_it = facesBegin();
	   FaceIterator face_end = facesEnd();
	 
	   while (face_it != face_end) {
		(*face_it++)->identifyWith(simplex);
	   }
	 }

    // for identified simplices, identify the skeleton with the target
    } else {
	 skeleton->identifyWith(identifyWithSkeleton);

	 // no need to identify faces, since they have the skeleton.
    }
  }
}


template <class Alloc>
Simplex<Alloc> *
Simplex<Alloc>::
getSkeleton ()
{
  Simplex * result = this;

  if (skeleton != NULL) {
    result = skeleton->getSkeleton();
    skeleton = result;
  }

  return result;  
}


template <class Alloc>
const Simplex<Alloc> *
Simplex<Alloc>::
getSkeleton () const
{
  const Simplex * result = this;

  if (skeleton != NULL) {
    result = skeleton->getSkeleton();
    skeleton = result;
  }

  return result;  
}


template <class Alloc> 
std::ostream& 
Simplex<Alloc>::
put (std::ostream& s) const
{
  s << "[ ";
  if (dimension >= 0) {
    std::copy(label.begin(), label.end(), std::ostream_iterator<int>(s, " "));
  }
  s << "]";

  return s;
}


template <class Alloc> 
void
Simplex<Alloc>::
addCoface (const Simplex * coface)
{
  assert(coface != NULL);
  assert (findCoface(coface->getDimension(), coface->getLabel())
		== cofacesEnd());

  cofaces.push_back(const_cast<Simplex *>(coface));
  cofacesAreSorted = false;
}


template <class Alloc>
template <class InputIterator>
void 
Simplex<Alloc>::
removeCoface (int dimension, InputIterator label)
{
  assert (dimension == this->dimension + 1);
  
  CofaceIterator pos = findCoface(dimension, label);

  if (pos != cofacesEnd()) {
    removeCoface(pos);
  }
}


template <class Alloc>
typename Simplex<Alloc>::CofaceIterator
Simplex<Alloc>::
removeCoface (CofaceIterator pos)
{
  assert(pos != cofaces.end());
  
  return cofaces.erase(pos);
}


template <class Alloc>
void 
Simplex<Alloc>::
sortCofaces ()
{
  if (!cofaces.empty()) {
    if (!cofacesAreSorted) {
	 std::sort(cofaces.begin(), cofaces.end(), simplexPointerLess);
    }

    cofacesAreSorted = true;
  }
}


template<class Alloc>
bool 
operator< (const Simplex<Alloc>& a, const Simplex<Alloc>& b)
{
  unsigned short dimension = a.getDimension();

  assert(dimension == b.getDimension());

  return std::lexicographical_compare(a.getLabel(), 
							   a.getLabel() + dimension + 1,
							   b.getLabel(),
							   b.getLabel() + dimension + 1);
}


template<class Alloc>
bool 
operator== (const Simplex<Alloc>& a, const Simplex<Alloc>& b)
{
  return !((a < b) || (b < a));
}



template <class Alloc>
std::ostream& 
operator<< (std::ostream& s, const Simplex<Alloc>& simplex) 
{
  return simplex.put(s);
}

