/* Simplex.h
 * ---------
 * Written by Patrick Perry
 * Original Version on 10/2/2003
 *
 * $Id: Simplex.h,v 1.2 2005/10/24 16:19:56 patperry Exp $
 * $Log: Simplex.h,v $
 * Revision 1.2  2005/10/24 16:19:56  patperry
 * Hells (literally) of compile fixes for gcc4.0
 *
 * Revision 1.1.1.1  2005/03/18 23:37:21  silva
 * Imported Sources
 *
 * Revision 1.2  2005/03/04 23:46:29  patperry
 * Added copersistence.
 *
 * Revision 1.1.1.1  2004/09/25 22:04:08  patperry
 * Imported sources
 *
 * Revision 1.10  2004/03/15 08:15:31  patperry
 * fixed some simplex identification stuff
 *
 * Revision 1.9  2004/03/13 06:57:04  patperry
 * fixed compile bug
 *
 * Revision 1.8  2004/03/13 06:38:19  patperry
 * added initial version of relative complex code
 *
 * Revision 1.7  2004/03/13 05:43:12  patperry
 * added initial simplicial set code.
 *
 * Revision 1.6  2004/02/25 09:49:19  patperry
 * Const bug.
 *
 * Revision 1.5  2004/02/25 09:27:59  patperry
 * Added ConstSimplexIterators
 *
 * Revision 1.4  2004/02/24 13:30:39  patperry
 * Radically restructured Simplex/Simplicial Complex
 *
 * Revision 1.3  2004/02/10 05:18:59  patperry
 * Added the Filtration class.
 *
 * Revision 1.2  2004/01/25 14:13:23  patperry
 * Added function to sort simplex label and determine its sign.
 *
 * Revision 1.1  2004/01/15 07:10:33  patperry
 * Changed file extensions.
 *
 * Revision 1.4  2003/11/13 06:25:30  patperry
 * Added method to construct scomplex from a graph
 *
 * Revision 1.3  2003/10/10 22:08:24  patperry
 * Fixed problems with addSimplex.
 * Added a removeSimplex function to SimpicialComplex.
 *
 * Revision 1.2  2003/10/10 11:09:03  patperry
 * Added better interface of inserting simplices into a simplicial complex.  Added
 * simplex iterators.
 *
 * Revision 1.1.1.1  2003/10/09 03:24:46  patperry
 * Imported Sources
 *
 */

#ifndef _SIMPLEX_H
#define _SIMPLEX_H

template <class Alloc>
class Simplex;

#include <memory>
#include <vector>
#include <ostream>
#include "SimplicialComplex.h"
#include "SimplexIterator.h"


/** 
 * Class for storing an n-simplex in a simplicial complex or a
 * simplicial set.  
 *
 * The data structure is an n-dimensional version of the quad-edge
 * structure proposed by Guibas and Stolfi in "Primitives for the 
 * manipulation of general subdivisions and the computaion of Voronoi 
 * diagrams", ACM Trans. Graph 4 (1985), 74-123.  In the version of 
 * the structure proposed here, an n-simplex stores its bounding 
 * (n-1)-simplices, along with the (n+1)-simplices that have it as 
 * a boundary.
 *
 * Terminology used:
 *   face - In the language of simplicial complexes, simplex A is 
 *     a face of simplex B if A is a subset of B.  In the language 
 *     of simplicial sets, A is a face of B if A can be written as 
 *     A = d_i1(d_i2(...d_ir(B)...)), where d_ij are boundary 
 *     operators.
 *
 *   coface - This is the dual of face.  Simplex A is a coface of s
 *     simplex B if B is a face of A.
 *
 *   principal - A simplex is principal if it has no non-degenerate
 *     cofaces.
 *
 *   degenerate - A simplex is degenerate if it is a successor of
 *     some other simplex.  (A = s_i(B) for some B).  This notion
 *     only makes sense for simplicial sets.
 *
 *   skeleton - The skeleton of a simplex A is defined to be the
 *     nondegenearte simplex B such that A = s_i1(s_i2(..(s_ir(B)...)).
 *     If A itself is nondegenerate, than the skeleton of A is
 *     defined to be A.
 */
template <class Alloc = std::allocator<int> >
class Simplex
{

  // friend classes
  friend class SimplicialComplex<Alloc>;
  friend class SimplexIterator<Simplex<Alloc> *, Alloc>;
  friend class SimplexIterator<const Simplex<Alloc> *, Alloc>;

private:
  typedef std::vector<Simplex *, 
				  typename Alloc::template rebind<Simplex *>::other>
    SimplexVector;

  typedef std::vector<int, 
		    typename Alloc::template rebind<int>::other>
    IntVector;

public:

  //! Containing Simplicial Complex type
  typedef SimplicialComplex<Alloc> SimplicialComplex;
  
  //! Type for iterating through faces.
  typedef typename SimplexVector::iterator FaceIterator;
		    

  //! Const type for iterating through faces.
  typedef typename SimplexVector::const_iterator ConstFaceIterator;

  //! Type for iterating through cofaces.
  typedef typename SimplexVector::iterator CofaceIterator;

  //! Const type for iterating through cofaces.
  typedef typename SimplexVector::const_iterator ConstCofaceIterator;
                                   
  //! Type for iterating through the label
  typedef typename IntVector::const_iterator LabelIterator;

  /**
   * Compares to simplices by derefencing the pointers and using the less
   * operator.
   *
   * \param first the first simplex pointer.
   * \param second the second simplex pointer.
   * \returns the boolean (*first < *second).
   */
  static bool simplexPointerLess (const Simplex * first, 
						    const Simplex * second);

  /**
   * Sort a simplex label in place, and determine if the sorting 
   * permutation is even or odd.
   *
   * \param dimension the dimension of the simplex.
   * \param label an iterator to the label
   * \returns -1 if the permutation is odd, and 1 if the permutation
   *          is even.
   */
  template <class RandomAccessIterator>
  static int sortLabel (int dimension, RandomAccessIterator label);

private:

  //! The simplicial complex containing the simplex
  SimplicialComplex * scomplex;

  //! The dimension of the simplex.  
  const int dimension;

  //! Label of the simplex.  This is an array of length (dimension + 1).
  IntVector label;

  //! Array of faces.  This is an array of length (dimension + 1).
  SimplexVector faces;   

  //! Collection of cofaces.  This is a collection of unknown size.
  SimplexVector cofaces;
  bool cofacesAreSorted;

  //! If degenerate, this is the skeleton.  Otherwise it is null.
  mutable Simplex * skeleton;

 private:

  /**
   * Constructor.
   *
   * Creates an n-simplex with the given range of (n-l)-simplices
   * as boundary.  The constructor will alter the simplices of the 
   * boundary, as they now have *this in their stars.
   *
   * \param dimension the dimension of the simplex.
   * \param label the label of the simplex; length (dimension + 1).
   * \param boundaryBegin iterator to a pointer to the first boundary simplex.
   * There must be (dimension + 1) pointers in the boundary.
   */
  template <class InputIterator1, class InputIterator2>
  Simplex (SimplicialComplex& scomplex, 
		 int dimension, InputIterator1 label, InputIterator2 boundary);
		 
  /**
   * Constructor.
   *
   * Creates a dummy n-simplex without boundary or cofaces to be used in
   * a search query.
   *
   * \param dimension the dimension of the simplex.
   * \param label the label of the simplex; length (dimension + 1).
   */
  template <class InputIterator>
  Simplex (int dimension, InputIterator label);

  /**
   * Default Constructor.
   */
  Simplex ();

 public:

  /** 
   * Destructor.
   *
   * Deletes a simplex and all of its cofaces (but not its faces).
   */
  virtual ~Simplex ();

  /**
   * Get the dimension of the simplex.
   */
  int getDimension () const;

  /**
   * Get the label of the simplex.  This is an array of length (dimension + 1).
   *
   * \returns a LabelIterator to the label
   */
  LabelIterator getLabel () const;

  /**
   * Get the number of cofaces.
   *
   * \returns the number of cofaces of the simplex.
   */
  int getNumCofaces () const;

  /**
   * Get an iterator to the begining of the faces collection.
   *
   * \return An iterator to the begin of the faces.
   */
  FaceIterator facesBegin ();

  /**
   * Get a const iterator to the begining of the faces collection.
   *
   * \return A const iterator to the begin of the faces.
   */
  ConstFaceIterator facesBegin () const;

  /**
   * Get an iterator to the end of the faces collection.
   * 
   * \return An iterator to the end of the faces.
   */
  FaceIterator facesEnd ();

  /**
   * Get a const iterator to the end of the faces collection.
   * 
   * \return An iterator to the end of the faces.
   */
  ConstFaceIterator facesEnd () const;

  /**
   * Get an iterator to the beginning of the cofaces collection.
   *
   * \return An iterator to the begin of the cofaces.
   */
  CofaceIterator cofacesBegin ();

  /**
   * Get a const iterator to the beginning of the cofaces collection.
   *
   * \return An iterator to the begin of the cofaces.
   */
  ConstCofaceIterator cofacesBegin () const;

  /**
   * Get an iterator to the end of the cofaces collection.
   *
   * \return An iterator to the end of the cofaces.
   */
  CofaceIterator cofacesEnd ();

  /**
   * Get a const iterator to the end of the cofaces collection.
   *
   * \return An iterator to the end of the cofaces.
   */
  ConstCofaceIterator cofacesEnd () const;

  /**
   * Returns a given face of the simplex.
   *
   * \param i the index of the face.
   * \returns a reference to the face.
   */
  Simplex * getFace (int i);

  /**
   * Returns a given face of the simplex.
   *
   * \param i the index of the face.
   * \returns a const reference to the face.
   */
  const Simplex * getFace (int i) const;

  /**
   * Look for the given coface in the coface collection.
   *
   * \param dimension the dimension of the coface (this must be
   * one plus the dimension of the simplex).
   * \param label the label of the coface.
   * \returns an iterator to the coface.
   */
  template <class InputIterator>
  CofaceIterator findCoface (int dimension, InputIterator label);

  /**
   * Look for the given coface in the coface collection.
   *
   * \param dimension the dimension of the coface (this must be
   * one plus the dimension of the simplex).
   * \param label the label of the coface
   * \returns a const_iterator to the coface.
   */
  template <class InputIterator>
  ConstCofaceIterator findCoface (int dimension, InputIterator label) const;

  /**
   * Indicates whether or not this is a degenerate simplex.  A simplex
   * is degenerate if it has been identified with another simplex.
   *
   * \returns true if this is degenerate.
   */
  bool isDegenerate () const;

  /**
   * Identifies this simplex with the given simplex.  The simplex to identify
   * with can either be a vertex (dimension 0), or a nil simplex 
   * (dimension -1).  This also identifies all faces with the given simplex.
   *
   * \param simplex a 0- or (-1)-dimensional simplex.
   */
  void identifyWith (Simplex * simplex);

  /**
   * Get the skeleton of a simplex.  If the simplex has been identified
   * with another simplex this returns the representative simplex.
   *
   * \returns a pointer to the skeleton.
   */
  Simplex * getSkeleton ();
  
  /**
   * Get the skeleton of a simplex.  If the simplex has been identified
   * with another simplex this returns the representative simplex.
   *
   * \returns a const pointer to the skeleton.
   */
  const Simplex * getSkeleton () const;
  
  /**
   * Write the simplex to an output stream.
   *
   * \param s the output stream.
   * \return The passed-in output stream.
   */
  virtual std::ostream& put (std::ostream& s) const;


private:

  /**
   * Adds a simplex of dimension to the list of cofaces.  The function
   * affects (*this), but not the passed-in simplex.  Does not check to
   * see if coface is already in the cofaces collection.  Adding a coface
   * will invalidate all existing coface iterators.
   *
   * \param coface a pointer to a simplex of dimension (dimension + 1).
   */
  void addCoface (const Simplex * coface);

  /**
   * Removes the coface with the given label, if it exists in the complex.
   *
   * \param dimension the dimension of the coface (this must be
   * one plus the dimension of the simplex).
   * \param label the label of the coface.
   */
  template <class InputIterator>
  void removeCoface (int dimension, InputIterator label);

  /**
   * Removes to coface pointed to by the given iterator.
   *
   * \param pos the iterator position of the coface.
   * \returns an iterator to the coface immediately after the deleted one.
   */
  CofaceIterator removeCoface (CofaceIterator pos);

  /**
   * Sorts the cofaces of the simplex.
   */
  void sortCofaces ();

};

/**
 * Less-than operator for simplices.  Lexicographical compare on the
 * labels of two simplices.
 * 
 * \param a a simplex.
 * \param b another simplex.
 * \return The value (a < b).
 */
template<class Alloc>
bool operator< (const Simplex<Alloc>& a, const Simplex<Alloc>& b);

/**
 * Equals-than operator for simplices.  Lexicographical compare on the
 * labels of two simplices.
 * 
 * \param a a simplex.
 * \param b another simplex.
 * \return The value (a == b).
 */
template<class Alloc>
bool operator== (const Simplex<Alloc>& a, const Simplex<Alloc>& b);


/**
 * Writes a simplex to an output stream.
 * 
 * \param s an output stream.
 * \param simplex a simplex.
 * \return The passed-in output stream.
 */
template <class Alloc>
std::ostream& operator<< (std::ostream& s, const Simplex<Alloc>& simplex);


// get compiler to see function definitions:
#include "Simplex.cc"


#endif // _SIMPLEX_H
