/* SimplicialComplex.h
 * -------------------
 * Written by Patrick Perry
 * Original Version on 10/2/2003
 *
 * $Id: SimplicialComplex.h,v 1.2 2005/10/24 16:19:56 patperry Exp $
 * $Log: SimplicialComplex.h,v $
 * Revision 1.2  2005/10/24 16:19:56  patperry
 * Hells (literally) of compile fixes for gcc4.0
 *
 * Revision 1.1.1.1  2005/03/18 23:37:24  silva
 * Imported Sources
 *
 * Revision 1.1.1.1  2004/09/25 22:04:08  patperry
 * Imported sources
 *
 * Revision 1.12  2004/09/15 18:14:15  patperry
 * Added "fromDimension" functionality for expand.
 *
 * Revision 1.11  2004/06/21 17:33:35  patperry
 * Initial TODO file
 *
 * Revision 1.10  2004/06/21 17:16:51  patperry
 *
 * Generalized scomplex construction with graph.
 *
 * Revision 1.9  2004/05/10 22:47:33  patperry
 * Added addFaces.
 *
 * Revision 1.8  2004/03/18 00:00:15  patperry
 * fixed compile bug
 *
 * Revision 1.7  2004/03/13 07:21:44  patperry
 * fixed degenerate simplex counting
 *
 * Revision 1.6  2004/03/13 06:38:19  patperry
 * added initial version of relative complex code
 *
 * Revision 1.5  2004/03/13 05:43:12  patperry
 * added initial simplicial set code.
 *
 * Revision 1.4  2004/02/25 09:27:59  patperry
 * Added ConstSimplexIterators
 *
 * Revision 1.3  2004/02/24 13:30:39  patperry
 * Radically restructured Simplex/Simplicial Complex
 *
 * Revision 1.2  2004/01/30 23:57:29  patperry
 * added neighborhood functionality
 *
 * Revision 1.1  2004/01/15 07:10:33  patperry
 * Changed file extensions.
 *
 * Revision 1.11  2003/11/18 06:46:35  patperry
 * added intersect and union.
 *
 * Revision 1.10  2003/11/18 06:23:33  patperry
 * Added k-skeleton constructor.
 *
 * Revision 1.9  2003/11/18 00:33:03  patperry
 * Added copy constructor and assignment operator.
 *
 * Revision 1.8  2003/11/14 10:18:33  patperry
 * Added getDimension method.
 *
 * Revision 1.7  2003/11/13 09:07:05  patperry
 * Added Ripps expand.
 *
 * Revision 1.6  2003/11/13 06:25:30  patperry
 * Added method to construct scomplex from a graph
 *
 * Revision 1.5  2003/10/16 08:42:15  patperry
 * Added a Chain data structure.
 *
 * Revision 1.4  2003/10/10 23:03:22  patperry
 * Documentation fix.
 *
 * Revision 1.3  2003/10/10 22:08:24  patperry
 * Fixed problems with addSimplex.
 * Added a removeSimplex function to SimpicialComplex.
 *
 * Revision 1.2  2003/10/10 11:09:03  patperry
 * Added better interface of inserting simplices into a simplicial complex.  Added
 * simplex iterators.
 *
 * Revision 1.1.1.1  2003/10/09 03:24:46  patperry
 * Imported Sources
 *
 */
 
#ifndef _SIMPLICIALCOMPLEX_H
#define _SIMPLICIALCOMPLEX_H

template <class Alloc>
class SimplicialComplex;


#include <memory>
#include <map>
#include <vector>
#include <ostream>
#include <functional>
#include <algorithm>
#include <iterator>
#include "Matrix.h"
#include "Simplex.h"
#include "SimplexIterator.h"


/**
 * Class for representing a simplicial complex.
 */
template <class Alloc = std::allocator<int> >
class SimplicialComplex 
{
  // friend classes
  friend class SimplexIterator<Simplex<Alloc> *, Alloc>;
  friend class SimplexIterator<const Simplex<Alloc> *, Alloc>;
  friend class Simplex<Alloc>;

public:

  /**
   * The simplex type stored in this simplicial complex.
   */
  typedef Simplex<Alloc> Simplex;

  /**
   * Const iterator for simplices.
   */
  typedef SimplexIterator<const Simplex *, Alloc> ConstSimplexIterator;

  /**
   * Iterator for simplices.
   */
  typedef SimplexIterator<Simplex *, Alloc> SimplexIterator;


protected:

  //! Memory allocator for simplices.
  static typename Alloc::template rebind<Simplex>::other SimplexAlloc;

private:

  //! an array storing the number of simplices of each dimension
  std::vector<int, typename Alloc::template rebind<int>::other> numSimplices;
    
  //! This is a "(-1)-dimensional" simplex that serves as a face for every
  //! vertex.
  Simplex nilSimplex;


public: 

  /**
   * Constructor.
   *
   * Creates a simplicial complex containing no simplices.
   */
  SimplicialComplex ();

  /**
   * Copy Constructor.  
   *
   * \param scomplex complex to initialize this with.
   */
  SimplicialComplex (const SimplicialComplex& scomplex);

  /**
   * Constructor.
   *
   * Takes a graph represented by a symmetric matrix and makes a
   * simplicial comlex out of it.  There are n vertices in the
   * resulting simplicial complex, where n is the number of rows
   * in the graph, and there is an edge between vertices i and j
   * if graph(i,j) is nonzero.
   *
   * \param graph the symmetric (n x n) matrix representing the 
   *        1-skeleton.
   * \param vertexLabelsBegin an integer iterator to the begin of
   *        the vertex label collection.  There must be n labels
   *        in the collection.
   */
  template <class Element_t, class ColumnIterator_t, class Alloc1, 
		  class InputIterator>
  SimplicialComplex (const Matrix<Element_t, ColumnIterator_t, Alloc1>& graph,
				 InputIterator vertexLabelsBegin);

  /**
   * Destructor.
   *
   * Deletes all simplices associated with the simplicial set.
   */
  virtual ~SimplicialComplex ();

  /**
   * Assignment operator.
   */
  SimplicialComplex& operator= (const SimplicialComplex& scomplex);

  /**
   * Returns the dimension of the complex.
   *
   * \returns the dimension of the highest-dimensional simplex.  If
   *          the complex is empty, this returns -1.
   */
  int getDimension () const;

  /**
   * Returns the number of simplices of the given dimension.
   *
   * \param dimension the dimension of simplex to query.
   * \returns the number of simplices of that dimension.
   */
  int getNumSimplices (int dimension) const;

  /**
   * Returns the total number of simplices.
   *
   * \returns the total number of simplices.
   */
  int getNumSimplices () const;

  /**
   * Returns the number of degenerate simplices of the given dimension.
   *
   * \param dimension the dimension of simplex to query.
   * \returns the number of degenerate simplices of that dimension.
   */
  int getNumDegenerateSimplices (int dimension) const;

  /**
   * Returns the total number of degenerate simplices.
   *
   * \returns the total number of degenerate simplices.
   */
  int getNumDegenerateSimplices () const;

  /**
   * Returns a SimplexIterator to the first simplex of the given dimension.
   * dimension must be less than or equal to the dimension of the complex.
   * 
   * \param dimension the dimension of the simplex collection
   * \returns an iterator to the begin of the simplices of that dimension.
   */
  SimplexIterator simplicesBegin (int dimension);
  ConstSimplexIterator simplicesBegin (int dimension) const;

  /**
   * Returns a SimplexIterator to the end of the simplices of the given
   * dimension. dimension must be less than or equal to the dimension of 
   * the complex.
   * 
   * \param dimension the dimension of the simplex collection
   * \returns an iterator to the end of the simplices of that dimension.
   */
  SimplexIterator simplicesEnd (int dimension);
  ConstSimplexIterator simplicesEnd (int dimension) const;

  /**
   * Adds the given vertex to the simplicial complex, if is not already
   * present.
   *
   * \param label the label of the vertexl
   * \returns an iterator to the vertexl
   */
  SimplexIterator addVertex (int label);

  /**
   * Adds the given Simplex to the simplicial complex, if it is not
   * already present.  This also adds the faces of the simplex if
   * they are not already present.
   *
   * \param dimension the dimension of the simplex to be added.  
   * \param label the label of the simplex to be added.  This must be 
   * sorted in ascending order.
   * \returns an iterator to the simplex.
   */
  template <class InputIterator>
  SimplexIterator addSimplex (int dimension, InputIterator label);

  /**
   * Adds all of the k-faces of a given face to the simplicial complex.
   * This does not actually add the simplex unless k >= dimension.
   *
   * \param dimension the dimension of the simplex.
   * \param label the label of the simplex.
   * \param k the dimension of face to add.
   * \returns void.
   */
  template <class InputIterator>
  void addFaces (int dimension, InputIterator label, int k);

  /**
   * Removes the given simplex from the simplicial complex.  If the simplex
   * is not in the complex, this does nothing.  The function also removes
   * all cofaces of the simplex.
   *
   * \param dimension the dimension of the simplex to be removed.
   * \param label the label of the simplex to be removed.  This must be
   * sorted in ascending order.
   * \returns void.
   */
  template <class InputIterator>
  void removeSimplex (int dimension, InputIterator label);

  /**
   * Removes the simplex pointed to by the given simplex iterator.
   *
   * \param pos an iterator to the simplex that is to be removed.
   * \returns an iterator to the next simplex in the collection.
   */
  SimplexIterator removeSimplex (SimplexIterator pos);

  /**
   * Looks for a simplex in the simplicial complex and returns an iterator
   * to it.
   *
   * \param dimension the dimension of the simplex.
   * \param label the label of the simplex to be found.  This must be
   * sorted in ascending order.
   * \returns an iterator to the queried simplex.
   */
  template <class InputIterator>
  SimplexIterator findSimplex (int dimension, InputIterator label);

  template <class InputIterator>
  ConstSimplexIterator findSimplex (int dimension, InputIterator label) const;

  /**
   * Expands the complex to be of the given dimension.  To go up one
   * dimension, simplices are added whenever their boundary simplices
   * already exist.  To go up multiple dimensions, the dimensions is
   * expanded one level at a time.  
   *
   * \param fromDimension the dimension to start expanding at
   * \param toDimension the dimension to expand to.
   * \returns void.
   */
  void rippsExpand (int fromDimension, int toDimension);

  /**
   * Adds the given simplex to the complex if all its faces already
   * exist.
   *
   * \param dimension the dimension of the simplex
   * \param label the label of the potential simplex (an int array
   *        of length dimension + 1)
   * \returns an indicator of whether or not the simplex was added.
   */
  template <class InputIterator>
  bool rippsExpandSimplex (int dimension, InputIterator label);

  /**
   * Intersect the simplicial complex with another simplicial complex
   * with a compatible vertex labeling.
   *
   * \param scomplex the simplicial complex to intersect with.
   */
  void intersectComplex (const SimplicialComplex& scomplex);

  /**
   * Union the simplicial complex with another simplicial complex
   * with a compatible vertex labeling.
   *
   * \param scomplex the simplicial complex to union with.
   */
  void unionComplex (const SimplicialComplex& scomplex);

  /**
   * Returns a new cmoplex that is the k-skeleton of this.
   *
   * \param k the highest dimension of simplex to copy.
   * \returns a new SimplicialComplex object.  Thus must be freed by
   * the client with delete.
   */
  SimplicialComplex * getSkeleton (int k) const;

  /**
   * Returns the neighborhood (closed star) of a set of vertices in 
   * the complex.
   *
   * \param verticesBegin an integer iterator to the label of the first vertex.
   * \param verticesEnd an iterator to the end of the label collection.
   * \returns a pointer to a new simplicial complex.  This must be freed by
   * the client with delete.
   */
  template <class InputIterator>
  SimplicialComplex * getNeighborhood (InputIterator verticesBegin, 
							    InputIterator verticesEnd) const;

  /**
   * Returns the restriction of the complex to a set of vertices.  A simplex
   * is present in the restriction iff all it is in the original complex and
   * all its vertices are in the passed-in set.
   *
   * \param verticesBegin an integer iterator to the label of the first vertex.
   * \param verticesEnd an iterator to the end of the label collection.
   * \returns a pointer to a new simplicial complex.  This must be freed by
   * the client with delete.
   */
  template <class InputIterator>
  SimplicialComplex * getRestriction (InputIterator verticesBegin, 
							   InputIterator verticesEnd) const;
  
  /**
   * Make this simplicial complex the relative complex (this, subcomplex).
   *
   * \param subcomplex a subcomplex of this simplicial complex.  All simplices
   * in subcomplex that are also in this are made degenerate.  Simplices of 
   * subcomplex that are not in this are ignored.
   */
  void relativeBy (const SimplicialComplex * subcomplex);

  /**
   * Make this simplicial complex the relative complex where simplices
   * with all vertices outside the given set are made degenerate.
   *
   * \param verticesBegin an integer iterator to the label of the first vertex.
   * \param verticesEnd an iterator to the end of the label collection.
   */
  template <class InputIterator>
  void relativeTo (InputIterator verticesBegin, InputIterator verticesEnd);

  /** 
   * Write the simplicial to an output stream.
   *
   * \param s the output stream.
   * \return The passed-in output stream.
   */
  virtual std::ostream& put (std::ostream& s) const;

 private:

  /**
   * Increments the count of the number of simplices at a given dimension.
   *
   * \param dimension the dimension to increment the simplex count of.
   * \param num the number to increment the count by (default is 1).
   */
  void incrementSimplexCount (int dimension, int num = 1);

  /**
   * Decrements the count of the number of simplices at a given dimension.
   *
   * \param dimension the dimension to increment the simplex count of.
   * \param num the number to decrement the count by (default = 1).
   */
  void decrementSimplexCount (int dimension, int num = 1);

};


/**
 * Write a simplicial complex to an output stream.
 * 
 * \param s an output stream.
 * \param complex a simplicial complex.
 * \return The passed-in output stream.
 */
template <class Alloc>
std::ostream& operator<< (std::ostream& s, 
					 const SimplicialComplex<Alloc>& complex);


// get compiler to see function definitions:
#include "SimplicialComplex.cc"


#endif // _SIMPLICIALCOMPLEX_H
