/* WitnessFilter.cc
 * ----------------
 * Implements WitnessFilter.h
 *
 * Written by Patrick Perry
 * Original Version on May 22, 2004
 *
 * $Id: WitnessFilter.cc,v 1.2 2005/10/24 16:19:56 patperry Exp $
 * $Log: WitnessFilter.cc,v $
 * Revision 1.2  2005/10/24 16:19:56  patperry
 * Hells (literally) of compile fixes for gcc4.0
 *
 * Revision 1.1.1.1  2005/03/18 23:37:26  silva
 * Imported Sources
 *
 * Revision 1.8  2005/03/04 21:53:10  patperry
 * Removed debug printfs
 *
 * Revision 1.7  2005/03/04 21:48:46  patperry
 * Fixed label bug in WitnessFilter
 *
 * Revision 1.6  2005/03/04 21:38:37  patperry
 * Fixed bug in WitnessFilter
 *
 * Revision 1.5  2005/02/11 23:55:04  patperry
 * Fixed vertex labling in wfilt
 *
 * Revision 1.4  2004/11/17 22:01:59  patperry
 * Added filtration on vertices
 *
 * Revision 1.3  2004/10/13 22:21:23  silva
 *
 * Reversed change that allowed for non-zero filtration values for vertices.
 * This change was causing an as-yet-untraced crash. We do want to reinstate
 * it at some point.
 *
 * Revision 1.2  2004/09/28 23:24:52  patperry
 * Changed behavior for 0-simplices in witness complex.
 *
 * Revision 1.1.1.1  2004/09/25 22:04:08  patperry
 * Imported sources
 *
 * Revision 1.1  2004/05/28 05:59:14  patperry
 * initial version of WitnessFilter
 *
 */

#include <algorithm>


template <class SimplexHandle_t, class FiltrationValue_t, class Alloc>
template <class InputIterator1, class InputIterator2>
WitnessFilter<SimplexHandle_t, FiltrationValue_t, Alloc>::
WitnessFilter (InputIterator1 verticesBegin, InputIterator1 verticesEnd,
			InputIterator2 distancesBegin, InputIterator2 distancesEnd,
			int numLandmarks, int nu, int firstRow)
  : numLandmarks(numLandmarks), distances(distancesBegin, distancesEnd), 
    firstRow(firstRow)
{
  int index = 0;

  while (verticesBegin != verticesEnd) {
    SimplexHandle_t vertex = *verticesBegin++;
    vertexToIndex[*(vertex->getLabel())] = index++;
  }

  numPoints = distances.size() / numLandmarks;
  
  // perform the necessary adjustments if there is a nu parameter
  if (nu > 0) {
    // for each point
    for (int j = 0; j < numPoints; ++j) {
	 // compute m_j
	 FiltrationValue_t m_j = 0;
	 typename std::vector<FiltrationValue_t> 
	   column_j(distances.begin() + j * numLandmarks,
			  distances.begin() + (j + 1) * numLandmarks);
	 typename std::vector<FiltrationValue_t>::iterator nu_th 
	   = column_j.begin() + (nu - 1);

	 std::nth_element(column_j.begin(), nu_th, column_j.end());
	 m_j = *nu_th;

	 // now, subtract m_j from all entries in the jth column
	 for (int i = 0; i < numLandmarks; ++i) {
	   typename FiltrationValueVector_t::iterator element_ij 
		= distances.begin() + j * numLandmarks + i;
	   
	   // set negative entries to zero
	   *element_ij = std::max(0.0, *element_ij - m_j);
	 }
    }
  }
}


template <class SimplexHandle_t, class FiltrationValue_t, class Alloc>
WitnessFilter<SimplexHandle_t, FiltrationValue_t, Alloc>::
~WitnessFilter ()
{
}


template <class SimplexHandle_t, class FiltrationValue_t, class Alloc>
FiltrationValue_t
WitnessFilter<SimplexHandle_t, FiltrationValue_t, Alloc>::
operator() (SimplexHandle_t simplex) const
{
  FiltrationValue_t result = 0;
  int dimension = simplex->getDimension();

  if (dimension > -1) {
    int labelIndices[dimension + 1];
    typename Simplex<Alloc>::LabelIterator label = simplex->getLabel();
  
    // determine the row indices of each vertex in the simplex
    for (int i = 0; i <= dimension; ++i) {
	 labelIndices[i] = getVertexIndex(*label++);
//(vertexToIndex.find(*label++))->second;
    }

    typename FiltrationValueVector_t::const_iterator column_i 
	 = distances.begin();
    FiltrationValue_t R;
    
    // loop over the columns of the distance matrix, letting R be the min
    // of the max value on the vertices
    for (int i = 0; i < numPoints; ++i) {
	 FiltrationValue_t R_new = column_i[labelIndices[0]];
	 
	 for (int j = 1; j <= dimension; ++j) {
	   R_new = std::max(R_new, column_i[labelIndices[j]]);
	 }
	 
	 if ((i == 0) || (R_new < R)) {
	   R = R_new;
	 }
	 
	 column_i += numLandmarks;
    }
  
    result = R;
  }

  return result;
}


template <class SimplexHandle_t, class FiltrationValue_t, class Alloc>
int
WitnessFilter<SimplexHandle_t, FiltrationValue_t, Alloc>::
getVertexIndex (int vertexLabel) const
{
  int result = vertexLabel - firstRow;

  if (vertexToIndex.size() == numLandmarks) {
    result = vertexToIndex.find(vertexLabel)->second;
  }

  //std::cout << "Vertex " << vertexLabel << " corresponds to row "
  //		  << result << "\n";

  return result;
}
